//---------------------------------------------------------------------------
#include "Common.h"
#pragma hdrstop

#include "DisplayClass.h"
//---------------------------------------------------------------------------
#pragma comment(lib, "setupapi.lib")
//---------------------------------------------------------------------------
unsigned char *DisplayClass::CopyData;
PropertiesClass DisplayClass::CopyProperties;
//---------------------------------------------------------------------------
DisplayClass::DisplayClass()
{
	int Size = MAX_EDID_BLOCKS * 128;

	if (Size < 256)
		Size = 256;

	ActiveData = new unsigned char[Size];
	OverrideData = new unsigned char[Size];
	ResetData = new unsigned char[Size];

	std::memset(ActiveData, 0, Size);
	std::memset(OverrideData, 0, Size);
	std::memset(ResetData, 0, Size);

	DeviceID = new char[TEXTSIZE];
	InstanceID = new char[TEXTSIZE];
	DriverName = new char[TEXTSIZE];

	DeviceID[0] = 0;
	InstanceID[0] = 0;
	DriverName[0] = 0;

	Active = false;
	Override = false;
	Matched = false;
	Deleted = false;
	Restart = false;

	DetailedResolutionList = new DetailedResolutionListClass(4);
	StandardResolutionList = new StandardResolutionListClass(32);
	EstablishedResolutionList = new EstablishedResolutionListClass;
	ExtensionBlockList = new ExtensionBlockListClass(MAX_EDID_EXTENSION_BLOCKS);
}
//---------------------------------------------------------------------------
bool DisplayClass::GetText(char *Text, int TextSize, bool GetStatus)
{
	char OverrideID[TEXTSIZE];
	char OverrideName[TEXTSIZE];
	const char *Name;
	const char *Status;

	OverrideProperties.GetDeviceID(OverrideID, TEXTSIZE);
	OverrideProperties.GetName(OverrideName, TEXTSIZE);

	if (OverrideName[0] != 0)
		Name = OverrideName;
	else if (DriverName[0] != 0)
		Name = DriverName;
	else
		Name = "Unknown Display";

	if (!GetStatus)
		Status = "";
	else if (Deleted)
		Status = " (delete)";
	else if (Active && Restart)
		Status = " (restart required)";
	else if (Active && Override)
		Status = " (active)*";
	else if (Active)
		Status = " (active)";
	else if (Override)
		Status = "*";
	else
		Status = "";

	std::snprintf(Text, TextSize, "%s - %s%s", OverrideID, Name, Status);
	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::Load(const char *NewDeviceID, const char *NewInstanceID)
{
	if (std::snprintf(DeviceID, TEXTSIZE, "%s", NewDeviceID) >= TEXTSIZE)
		return false;

	if (std::snprintf(InstanceID, TEXTSIZE, "%s", NewInstanceID) >= TEXTSIZE)
		return false;

	if (!LoadDriverName())
		return false;

	Active = LoadActiveStatus();
	Override = LoadOverrideData();

	if (!LoadActiveData())
	{
		if (std::strcmp(DeviceID, "Default_Monitor") == 0)
		{
			if (!Active && !Override)
				return false;
		}
		else
		{
			if (!Override)
				return false;
		}

		if (!LoadDefaultData())
			return false;
	}

	Matched = false;
	Deleted = false;
	Restart = false;
	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::MatchFullActive(EDIDListClass &EDIDList)
{
	if (Active && !Matched)
		Matched = EDIDList.MatchFullActive(ActiveData);

	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::MatchFullOverride(EDIDListClass &EDIDList)
{
	if (Active && Override && !Matched)
		Matched = EDIDList.MatchFullOverride(ActiveData, OverrideData);

	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::MatchPartialOverride(EDIDListClass &EDIDList)
{
	if (Active && Override && !Matched)
		Matched = EDIDList.MatchPartialOverride(ActiveData, OverrideData);

	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::MatchFirstActive(EDIDListClass &EDIDList)
{
	if (Active && !Matched)
		Matched = EDIDList.MatchFirstActive(ActiveData);

	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::Init()
{
	int Block;

	if (Override)
	{
		if (std::memcmp(ActiveData, OverrideData, 128) != 0)
			Restart = true;

		for (Block = 1; Block < MAX_EDID_BLOCKS; Block++)
			if (ActiveData[Block * 128] != 0 && OverrideData[Block * 128] != 0)
				if (std::memcmp(&ActiveData[Block * 128], &OverrideData[Block * 128], 128) != 0)
					Restart = true;
	}
	else
	{
		std::memcpy(OverrideData, ActiveData, MAX_EDID_BLOCKS * 128);
		OverrideProperties = ActiveProperties;
	}

	FixData(OverrideData, Override);
	DisplayInit();
	DisplayWrite();
	OverrideProperties.FixIncludes(DetailedResolutions()->GetSlotsLeft());
	OverrideProperties.Write(OverrideData);
	CalculateChecksums();
	std::memcpy(ResetData, OverrideData, MAX_EDID_BLOCKS * 128);
	ResetProperties = OverrideProperties;
	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::LoadDriverName()
{
	HKEY Key;
	char Path[TEXTSIZE];
	unsigned char Buffer[256];
	unsigned long BufferSize;
	int Index;
	const unsigned char *Text;

	if (std::snprintf(Path, TEXTSIZE, "SYSTEM\\CurrentControlSet\\Enum\\DISPLAY\\%s\\%s", DeviceID, InstanceID) >= TEXTSIZE)
		return false;

	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, Path, 0, KEY_QUERY_VALUE, &Key) != ERROR_SUCCESS)
		return false;

	BufferSize = sizeof Buffer - sizeof *Buffer;

	if (RegQueryValueEx(Key, "DeviceDesc", NULL, NULL, Buffer, &BufferSize) != ERROR_SUCCESS)
	{
		RegCloseKey(Key);
		return false;
	}

	RegCloseKey(Key);
	Buffer[BufferSize] = 0;

	for (Text = Buffer; *Text; Text++)
	{
		if (*Text == ';')
		{
			Text++;
			break;
		}
	}

	// Generic Non-PnP Monitor
	// ThinkPad Display 1920x1080
	// Wide viewing angle & High density FlexView Display
	for (Index = 0; Index < 29 && Text[Index] != 0; Index++)
		DriverName[Index] = Text[Index];

	if (Index == 29)
	{
		DriverName[26] = '.';
		DriverName[27] = '.';
		DriverName[28] = '.';
	}

	DriverName[Index] = 0;
	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::LoadActiveStatus()
{
	const GUID GUID_DEVINTERFACE_MONITOR = {0xE6F07B5F, 0xEE97, 0x4A90, {0xB0, 0x76, 0x33, 0xF5, 0x7B, 0xF4, 0xEA, 0xA7}};
	char Path[TEXTSIZE];
	HDEVINFO Devices;
	SP_DEVINFO_DATA Device;
	bool Status = false;

	if (std::snprintf(Path, TEXTSIZE, "DISPLAY\\%s\\%s", DeviceID, InstanceID) >= TEXTSIZE)
		return false;

	Devices = SetupDiGetClassDevs(&GUID_DEVINTERFACE_MONITOR, Path, NULL, DIGCF_DEVICEINTERFACE | DIGCF_PRESENT);

	if (Devices == INVALID_HANDLE_VALUE)
		return false;

	Device.cbSize = sizeof Device;

	if (SetupDiEnumDeviceInfo(Devices, 0, &Device))
		Status = true;

	SetupDiDestroyDeviceInfoList(Devices);
	return Status;
}
//---------------------------------------------------------------------------
bool DisplayClass::LoadOverrideData()
{
	HKEY Key;
	char Path[TEXTSIZE];
	unsigned char Buffer[256];
	unsigned long BufferSize;
	int Block;
	bool Status = false;

	std::memset(OverrideData, 0, MAX_EDID_BLOCKS * 128);

	if (std::snprintf(Path, TEXTSIZE, "SYSTEM\\CurrentControlSet\\Enum\\DISPLAY\\%s\\%s\\Device Parameters\\EDID_OVERRIDE", DeviceID, InstanceID) >= TEXTSIZE)
		return false;

	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, Path, 0, KEY_QUERY_VALUE, &Key) != ERROR_SUCCESS)
		return false;

	BufferSize = sizeof Buffer;

	if (RegQueryValueEx(Key, "0", NULL, NULL, Buffer, &BufferSize) == ERROR_SUCCESS && BufferSize >= 128 && IsValidData(Buffer))
	{
		std::memcpy(OverrideData, Buffer, 128);
		OverrideProperties.LoadBackupData(Key);
		OverrideProperties.SetResetID(DeviceID);
		OverrideProperties.Read(OverrideData, true);
		Status = true;

		for (Block = 1; Block < MAX_EDID_BLOCKS; Block++)
		{
			std::snprintf(Path, TEXTSIZE, "%d", Block);
			BufferSize = sizeof Buffer;

			if (RegQueryValueEx(Key, Path, NULL, NULL, Buffer, &BufferSize) == ERROR_SUCCESS && BufferSize >= 128)
				std::memcpy(&OverrideData[Block * 128], Buffer, 128);
		}
	}

	RegCloseKey(Key);
	return Status;
}
//---------------------------------------------------------------------------
bool DisplayClass::LoadActiveData()
{
	HKEY Key;
	char Path[TEXTSIZE];
	unsigned char Buffer[32768];
	unsigned long BufferSize;
	bool Status = false;

	std::memset(ActiveData, 0, MAX_EDID_BLOCKS * 128);

	if (std::snprintf(Path, TEXTSIZE, "SYSTEM\\CurrentControlSet\\Enum\\DISPLAY\\%s\\%s\\Device Parameters", DeviceID, InstanceID) >= TEXTSIZE)
		return false;

	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, Path, 0, KEY_QUERY_VALUE, &Key) != ERROR_SUCCESS)
		return false;

	BufferSize = sizeof Buffer;

	if (RegQueryValueEx(Key, "EDID", NULL, NULL, Buffer, &BufferSize) == ERROR_SUCCESS && BufferSize >= 128 && IsValidData(Buffer))
	{
		if (BufferSize > MAX_EDID_BLOCKS * 128)
			BufferSize = MAX_EDID_BLOCKS * 128;

		std::memcpy(ActiveData, Buffer, BufferSize / 128 * 128);
		ActiveProperties.SetResetID(DeviceID);
		ActiveProperties.Read(ActiveData, false);
		Status = true;
	}

	RegCloseKey(Key);
	return Status;
}
//---------------------------------------------------------------------------
bool DisplayClass::LoadDefaultData()
{
	HKEY Key;
	char Path[TEXTSIZE];

	std::memset(ActiveData, 0, MAX_EDID_BLOCKS * 128);

	if (std::snprintf(Path, TEXTSIZE, "SYSTEM\\CurrentControlSet\\Enum\\DISPLAY\\%s\\%s\\Device Parameters", DeviceID, InstanceID) >= TEXTSIZE)
		return false;

	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, Path, 0, KEY_QUERY_VALUE, &Key) != ERROR_SUCCESS)
		return false;

	RegCloseKey(Key);
	std::memcpy(ActiveData, "\x00\xFF\xFF\xFF\xFF\xFF\xFF\x00\x41\xD0\xFF\x09\x00\x00\x00\x00\x00\x00\x01\x03\x08\x00\x00\x78\xEA\xEE\x95\xA3\x54\x4C\x99\x26\x0F\x50\x54", 35);
	ActiveProperties.SetResetID(DeviceID);
	ActiveProperties.Read(ActiveData, false);
	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::GetDeviceID(char *Text, int TextSize)
{
	if (std::snprintf(Text, TextSize, "%s", DeviceID) >= TextSize)
		return false;

	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::GetInstanceID(char *Text, int TextSize)
{
	if (std::snprintf(Text, TextSize, "%s", InstanceID) >= TextSize)
		return false;

	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::GetDriverName(char *Text, int TextSize)
{
	if (std::snprintf(Text, TextSize, "%s", DriverName) >= TextSize)
		return false;

	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::GetActiveStatus()
{
	return Active;
}
//---------------------------------------------------------------------------
bool DisplayClass::GetOverrideStatus()
{
	return Override;
}
//---------------------------------------------------------------------------
bool DisplayClass::IsValidData(const unsigned char *Data)
{
	if (std::memcmp(Data, "\x00\xFF\xFF\xFF\xFF\xFF\xFF\x00", 8) != 0)
		return false;

	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::FixData(unsigned char *Data, bool Override)
{
	unsigned char CompareData[127];

	if (!IsValidData(Data))
		return false;

	std::memset(&Data[12], 0, 4);

	if (Data[18] != 1 || Data[19] != 4)
	{
		Data[18] = 1;
		Data[19] = 3;

		if (Data[20] & 128)
			Data[20] = 128;
	}

	Data[24] |= 2;

	if (Override)
	{
		if (Data[126] == 1)
		{
			std::memset(CompareData, 0, 127);
			CompareData[0] = 0x02;
			CompareData[1] = 0x03;

			if (std::memcmp(&Data[128], CompareData, 127) == 0)
				Data[126] = 0;
		}
	}
	else
	{
		if (Data[126] == 1)
		{
			std::memset(CompareData, 0xFF, 127);

			if (std::memcmp(&Data[128], CompareData, 127) == 0)
				Data[126] = 0;
		}
	}

	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::Import(const char *FileName, bool Complete)
{
	std::FILE *File = std::fopen(FileName, "rb");

	if (!File)
		return false;

	DisplayWrite();
	OverrideData[126] = 0;

	if (!ImportFile(File, Complete, "(b)"))
	if (!ImportFile(File, Complete, "(hhhhhhhhhhhhhhhh)"))
	if (!ImportFile(File, Complete, "l(hhhhhhhhhhhhhhhh)"))
	if (!ImportFile(File, Complete, "(xcxcxcxcxcxcxcxcxcxcxcxcxcxcxcxc)"))
	if (!ImportFile(File, Complete, "l(xcxcxcxcxcxcxcxcxcxcxcxcxcxcxcxc)"))
	if (!ImportFile(File, Complete, "lll(oshhhhhhhhhhhhhhhh)"))
	if (!ImportFile(File, Complete, "lll(sshhhhhhhhhh)"))
	if (!ImportINF(File, Complete))
	if (!ImportEXE(File, Complete))
	{
		std::fclose(File);
		return false;
	}

	std::fclose(File);
	FixData(OverrideData, true);
	DisplayRead();
	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::ImportFile(std::FILE *File, bool Complete, const char *Format)
{
	PropertiesClass Properties;
	unsigned char Buffer[128];
	int Index;
	int Block;
	const char *Byte;
	const char *Start;
	int Hex;
	char Text[256];
	bool Valid = false;
	bool Status = false;

	rewind(File);
	Index = 0;
	Block = 0;

	for (Byte = Format; Block < MAX_EDID_BLOCKS; Byte++)
	{
		switch (*Byte)
		{
			case '(':
				Start = Byte;
				break;

			case ')':
				Byte = Start;
				break;

			case 'b':
				if (std::fread(&Buffer[Index], 1, 128, File) != 128)
					return Status;

				Index += 128;
				break;

			case 'c':
				if (std::fgetc(File) == EOF)
					return Status;

				break;

			case 'h':
				if (std::fscanf(File, "%2x", &Hex) != 1)
					return Status;

				Buffer[Index] = Hex;
				Index++;
				break;

			case 'l':
				if (!std::fgets(Text, 256, File))
					return Status;

				break;

			case 'o':
				if (std::fscanf(File, "%x", &Hex) != 1)
					return Status;

				if (Hex % 128 != Index)
					return Status;

				if (Hex / 128 < Block)
					return Status;

				if (Index == 0)
					Block = Hex / 128;

				break;

			case 's':
				if (std::fscanf(File, "%255s", Text) != 1)
					return Status;

				break;

			case 'x':
				if (std::fscanf(File, "%x", &Hex) != 1)
					return Status;

				if (Hex > 255)
					return Status;

				Buffer[Index] = Hex;
				Index++;
				break;

			default:
				return Status;
		}

		if (Index != 128)
			continue;

		if (Block == 0)
		{
			if (!IsValidData(Buffer))
				return Status;

			Valid = true;

			if (Complete)
				std::memcpy(OverrideData, Buffer, 128);
			else
				std::memcpy(&OverrideData[35], &Buffer[35], 93);

			if (!Status)
				std::memset(&OverrideData[128], 0, MAX_EDID_EXTENSION_BLOCKS * 128);

			Properties = OverrideProperties;
			Properties.Read(Buffer, true);
			Properties.ResetIncludes();

			if (Complete)
				OverrideProperties = Properties;
			else
				OverrideProperties.Import(Properties);

			Status = true;
		}
		else
		{
			if (!Valid && OverrideData[126] < Block)
				OverrideData[126] = Block;

			if (!Status)
				std::memset(&OverrideData[128], 0, MAX_EDID_EXTENSION_BLOCKS * 128);

			std::memcpy(&OverrideData[Block * 128], Buffer, 128);
			Status = true;
		}

		Index = 0;
		Block++;
	}

	return Status;
}
//---------------------------------------------------------------------------
bool DisplayClass::ImportINF(std::FILE *File, bool Complete)
{
	PropertiesClass Properties;
	unsigned char Buffer[128];
	char Line[1024];
	const char *Text;
	int Block;
	int Hex;
	int Index;
	bool Valid = false;
	bool Status = false;

	rewind(File);

	while (std::fgets(Line, 1024, File))
	{
		if (std::strchr(Line, '\x7F'))
			return Status;

		Text = std::strtok(Line, ", \"\r\n");

		if (!Text)
			continue;

		if (stricmp(Text, "HKR") != 0)
			continue;

		Text = std::strtok(NULL, ", \"\r\n");

		if (!Text)
			continue;

		if (stricmp(Text, "EDID_OVERRIDE") != 0)
			continue;

		Text = std::strtok(NULL, ", \"\r\n");

		if (!Text)
			continue;

		if (std::sscanf(Text, "%d", &Block) != 1)
			continue;

		if (Block < 0 || Block >= MAX_EDID_BLOCKS)
			continue;

		Text = std::strtok(NULL, ", \"\r\n");

		if (!Text)
			continue;

		if (std::sscanf(Text, "%x", &Hex) != 1)
			continue;

		if (Hex != 1)
			continue;

		for (Index = 0; Index < 128; Index++)
		{
			Text = std::strtok(NULL, ", \"\r\n");

			if (!Text)
				break;

			if (std::sscanf(Text, "%x", &Hex) != 1)
				break;

			if (Hex > 255)
				break;

			Buffer[Index] = Hex;
		}

		if (Index != 128)
			return Status;

		Text = std::strtok(NULL, ", \"\r\n");

		if (Text)
			return Status;

		if (Block == 0)
		{
			if (!IsValidData(Buffer))
				return Status;

			Valid = true;

			if (Complete)
				std::memcpy(OverrideData, Buffer, 128);
			else
				std::memcpy(&OverrideData[35], &Buffer[35], 93);

			if (!Status)
				std::memset(&OverrideData[128], 0, MAX_EDID_EXTENSION_BLOCKS * 128);

			Properties = OverrideProperties;
			Properties.Read(Buffer, true);
			Properties.ResetIncludes();

			if (Complete)
				OverrideProperties = Properties;
			else
				OverrideProperties.Import(Properties);

			Status = true;
		}
		else
		{
			if (!Valid && OverrideData[126] < Block)
				OverrideData[126] = Block;

			if (!Status)
				std::memset(&OverrideData[128], 0, MAX_EDID_EXTENSION_BLOCKS * 128);

			std::memcpy(&OverrideData[Block * 128], Buffer, 128);
			Status = true;
		}
	}

	return Status;
}
//---------------------------------------------------------------------------
bool DisplayClass::ImportEXE(std::FILE *File, bool Complete)
{
	PropertiesClass Properties;
	const unsigned char Magic[] = {0x71, 0x42, 0x12, 0x83, 0x54, 0x24, 0x95, 0x66};
	const int DataSize = sizeof Magic + MAX_EDID_BLOCKS * 128;
	const int ReadSize = 4096;
	unsigned char Buffer[DataSize + ReadSize];
	int Size;
	int Index;

	rewind(File);
	Size = std::fread(&Buffer[DataSize], 1, ReadSize, File);
	Index = DataSize;

	if (Size < DataSize || memcmp(&Buffer[DataSize], "MZ", 2) != 0)
		return false;

	while (Size > 0)
	{
		while (Index <= Size)
		{
			if (memcmp(&Buffer[Index], Magic, sizeof Magic) == 0)
			{
				Index += sizeof Magic;

				if (!IsValidData(&Buffer[Index]))
					continue;

				if (Complete)
					std::memcpy(OverrideData, &Buffer[Index], DataSize);
				else
					std::memcpy(&OverrideData[35], &Buffer[Index + 35], DataSize - 35);

				Properties = OverrideProperties;
				Properties.Read(&Buffer[Index], true);
				Properties.ResetIncludes();

				if (Complete)
					OverrideProperties = Properties;
				else
					OverrideProperties.Import(Properties);

				return true;
			}

			Index++;
		}

		std::memcpy(Buffer, &Buffer[ReadSize], DataSize);
		Size = std::fread(&Buffer[DataSize], 1, ReadSize, File);
		Index = 1;
	}

	return false;
}
//---------------------------------------------------------------------------
const char *DisplayClass::GetImportFormats()
{
	return "EDID Files (*.bin, *.dat, *.inf)\0*.bin;*.dat;*.inf\0TXT Files (*.txt)\0*.txt\0CSV Files (*.csv)\0*.csv\0EXE Files (*.exe)\0*.exe\0All Files (*.*)\0*.*\0\0";
}
//---------------------------------------------------------------------------
const char *DisplayClass::GetExportFormats()
{
	return "BIN File (*.bin)\0*.bin\0DAT File (*.dat)\0*.dat\0INF File (*.inf)\0*.inf\0TXT File (*.txt)\0*.txt\0CSV File (*.csv)\0*.csv\0EXE File (*.exe)\0*.exe\0\0";
}
//---------------------------------------------------------------------------
bool DisplayClass::Export(const char *FileName, int FileType, const char *Name)
{
	std::FILE *File = std::fopen(FileName, "wb");

	if (!File)
		return false;

	if (!ExportFile(File, FileType, Name))
	{
		std::fclose(File);
		return false;
	}

	if (std::fclose(File) != 0)
		return false;

	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::ExportFile(std::FILE *File, int FileType, const char *Name)
{
	unsigned char *ExportData;
	unsigned Count;
	unsigned Line;
	unsigned Index;
	char OverrideID[TEXTSIZE];

	DisplayWrite();
	OverrideProperties.Write(OverrideData);
	CalculateChecksums();

	if (!Override && std::memcmp(OverrideData, ResetData, MAX_EDID_BLOCKS * 128) == 0 && OverrideProperties == ResetProperties)
		ExportData = ActiveData;
	else
		ExportData = OverrideData;

	for (Count = MAX_EDID_EXTENSION_BLOCKS; Count > 0; Count--)
		if (ExportData[Count * 128] != 0)
			break;

	Count++;

	switch (FileType)
	{
		case 1:
			Count *= 128;

			if (std::fwrite(ExportData, 1, Count, File) != Count)
				return false;

			return true;

		case 2:
			Count *= 8;

			if (std::fprintf(File, "EDID BYTES:\r\n") <= 0)
				return false;

			if (std::fprintf(File, "0x %*s 00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F\r\n", Count <= 16 ? 1 : 3, "") <= 0)
				return false;

			if (std::fprintf(File, "%*s   ------------------------------------------------\r\n", Count <= 16 ? 1 : 3, "") <= 0)
				return false;

			for (Line = 0; Line < Count; Line++)
			{
				if (std::fprintf(File, "%0*X0 |", Count <= 16 ? 1 : 3, Line) <= 0)
					return false;

				for (Index = 0; Index < 16; Index++)
					if (std::fprintf(File, " %02X", *(ExportData++)) <= 0)
						return false;

				if (std::fprintf(File, "\r\n") <= 0)
					return false;
			}

			return true;

		case 3:
			if (ExportData == OverrideData && ExportData[126] == 0 && OverrideProperties.GetExtensions())
			{
				Count++;
				ExportData[126]++;
				ExportData[127]--;
				ExtensionBlockList->WriteBlankExtension(&ExportData[128], 128);
			}

			if (std::fprintf(File,
				"[Version]\r\n"
				"Signature=\"$Windows NT$\"\r\n"
				"Class=Monitor\r\n"
				"ClassGUID={4d36e96e-e325-11ce-bfc1-08002be10318}\r\n"
				"Provider=%%MFR%%\r\n"
				"\r\n"
				"[Manufacturer]\r\n"
				"%%MFR%%=Monitor,NTx86,NTamd64\r\n"
				) <= 0)
				return false;

			OverrideProperties.GetDeviceID(OverrideID, TEXTSIZE);

			if (std::fprintf(File,
				"\r\n"
				"[Monitor.NTx86]\r\n"
				"%%MODEL%%=Monitor.Install,MONITOR\\%s\r\n", OverrideID
				) <= 0)
				return false;

			if (std::fprintf(File,
				"\r\n"
				"[Monitor.NTamd64]\r\n"
				"%%MODEL%%=Monitor.Install,MONITOR\\%s\r\n", OverrideID
				) <= 0)
				return false;

			if (std::fprintf(File,
				"\r\n"
				"[Monitor.Install]\r\n"
				"DelReg=Monitor.DelReg\r\n"
				"\r\n"
				"[Monitor.Install.HW]\r\n"
				"DelReg=Monitor.DelReg\r\n"
				"AddReg=Monitor.AddReg\r\n"
				"\r\n"
				"[Monitor.DelReg]\r\n"
				"HKR,EDID_OVERRIDE\r\n"
				"HKR,MODES\r\n"
				"HKR,,DPMS\r\n"
				"HKR,,ICMProfile\r\n"
				"HKR,,MaxResolution\r\n"
				"HKR,,PreferredMode\r\n"
				"\r\n"
				"[Monitor.AddReg]\r\n"
				) <= 0)
				return false;

			for (Line = 0; Line < Count; Line++)
			{
				if (Line > 0 && *ExportData == 0)
				{
					ExportData += 128;
					continue;
				}

				if (std::fprintf(File, "HKR,EDID_OVERRIDE,\"%d\",0x01", Line) <= 0)
					return false;

				for (Index = 0; Index < 128; Index++)
					if (std::fprintf(File, ",0x%02X", *(ExportData++)) <= 0)
						return false;

				if (std::fprintf(File, "\r\n") <= 0)
					return false;
			}

			if (std::fprintf(File,
				"\r\n"
				"[Strings]\r\n"
				"MFR=\"EDID Override\"\r\n"
				"MODEL=\"%s\"\r\n", Name
				) <= 0)
				return false;

			return true;

		case 4:
			Count *= 8;

			for (Line = 0; Line < Count; Line++)
			{
				if (std::fprintf(File, "%02X", *(ExportData++)) <= 0)
					return false;

				for (Index = 1; Index < 16; Index++)
					if (std::fprintf(File, " %02X", *(ExportData++)) <= 0)
						return false;

				if (std::fprintf(File, "\r\n") <= 0)
					return false;
			}

			return true;

		case 5:
			for (Line = 0; Line < Count; Line++)
			{
				if (std::fprintf(File, "%02X", *(ExportData++)) <= 0)
					return false;

				for (Index = 1; Index < 128; Index++)
					if (std::fprintf(File, ",%02X", *(ExportData++)) <= 0)
						return false;

				if (std::fprintf(File, "\r\n") <= 0)
					return false;
			}

			return true;

		case 6:
			unsigned char FileData[] =
			{
				0x4D, 0x5A, 0x90, 0x00, 0x03, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0x00, 0x00, 0xB8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xF0, 0x00, 0x00, 0x00, 0x0E, 0x1F, 0xBA, 0x0E, 0x00, 0xB4, 0x09, 0xCD, 0x21, 0xB8, 0x01, 0x4C, 0xCD, 0x21, 0x54, 0x68, 0x69, 0x73, 0x20, 0x70, 0x72, 0x6F, 0x67, 0x72, 0x61, 0x6D, 0x20, 0x63, 0x61, 0x6E, 0x6E, 0x6F, 0x74, 0x20, 0x62, 0x65, 0x20, 0x72, 0x75, 0x6E, 0x20, 0x69, 0x6E, 0x20, 0x44, 0x4F, 0x53, 0x20, 0x6D, 0x6F, 0x64, 0x65, 0x2E, 0x0D, 0x0D, 0x0A, 0x24, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x78, 0xEC, 0x13, 0xCA, 0x3C, 0x8D, 0x7D, 0x99, 0x3C, 0x8D, 0x7D, 0x99, 0x3C, 0x8D, 0x7D, 0x99, 0x27, 0x10, 0xD6, 0x99, 0x2B, 0x8D, 0x7D, 0x99, 0x27, 0x10, 0xE3, 0x99, 0x32, 0x8D, 0x7D, 0x99, 0x27, 0x10, 0xD7, 0x99, 0x6B, 0x8D, 0x7D, 0x99, 0x35, 0xF5, 0xEE, 0x99, 0x3B, 0x8D, 0x7D, 0x99, 0x3C, 0x8D, 0x7C, 0x99, 0x6B, 0x8D, 0x7D, 0x99, 0x27, 0x10, 0xD3, 0x99, 0x3E, 0x8D, 0x7D, 0x99, 0x27, 0x10, 0xE7, 0x99, 0x3D, 0x8D, 0x7D, 0x99, 0x27, 0x10, 0xE0, 0x99, 0x3D, 0x8D, 0x7D, 0x99, 0x52, 0x69, 0x63, 0x68, 0x3C, 0x8D, 0x7D, 0x99, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x50, 0x45, 0x00, 0x00, 0x4C, 0x01, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xE0, 0x00, 0x02, 0x01, 0x0B, 0x01, 0x0A, 0x00, 0x00, 0x82, 0x00, 0x00, 0x00, 0x4E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xDA, 0x26, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0xA0, 0x00, 0x00, 0x00, 0x00, 0x40, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x05, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x05, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x01, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x00, 0x40, 0x81, 0x00, 0x00, 0x10, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x74, 0xC3, 0x00, 0x00, 0x50, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0xDC, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x01, 0x00, 0xB0, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x78, 0xC0, 0x00, 0x00, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xA0, 0x00, 0x00, 0x34, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x2E, 0x74, 0x65, 0x78, 0x74, 0x00, 0x00, 0x00, 0xAC, 0x80, 0x00, 0x00,
				0x00, 0x10, 0x00, 0x00, 0x00, 0x82, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x00, 0x00, 0x60, 0x2E, 0x72, 0x64, 0x61, 0x74, 0x61, 0x00, 0x00, 0x5E, 0x2A, 0x00, 0x00, 0x00, 0xA0, 0x00, 0x00, 0x00, 0x2C, 0x00, 0x00, 0x00, 0x86, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x00, 0x00, 0x40, 0x2E, 0x64, 0x61, 0x74, 0x61, 0x00, 0x00, 0x00, 0xA0, 0x2B, 0x00, 0x00, 0x00, 0xD0, 0x00, 0x00, 0x00, 0x0E, 0x00, 0x00, 0x00, 0xB2, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x00, 0x00, 0xC0, 0x2E, 0x72, 0x73, 0x72, 0x63, 0x00, 0x00, 0x00, 0xDC, 0x04, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x06, 0x00, 0x00, 0x00, 0xC0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x00, 0x00, 0x40, 0x2E, 0x72, 0x65, 0x6C, 0x6F, 0x63, 0x00, 0x00, 0xB0, 0x0D, 0x00, 0x00, 0x00, 0x10, 0x01, 0x00, 0x00, 0x0E, 0x00, 0x00, 0x00, 0xC6, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x00, 0x00, 0x42, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
				0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x55, 0x8B, 0xEC, 0x81, 0xEC, 0x08, 0x01, 0x00, 0x00, 0xA1, 0x04, 0xD0, 0x40, 0x00, 0x33, 0xC5, 0x89, 0x45, 0xFC, 0x8B, 0x45, 0x08, 0x50, 0x68, 0xB8, 0xB9, 0x40, 0x00, 0x68, 0x58, 0xBC, 0x40, 0x00, 0x68, 0x80, 0xBC, 0x40, 0x00, 0x8D, 0x85, 0xFC, 0xFE, 0xFF, 0xFF, 0x68, 0x00, 0x01, 0x00, 0x00, 0x50, 0xE8, 0x93, 0x13, 0x00, 0x00, 0x83, 0xC4, 0x18, 0x3D, 0x00, 0x01, 0x00, 0x00, 0x7C, 0x1B, 0xA1, 0x38, 0xEA, 0x40, 0x00, 0x83, 0xC8, 0xFE, 0xA3, 0x38, 0xEA, 0x40, 0x00, 0x8B, 0x4D, 0xFC, 0x33, 0xCD, 0xE8, 0x67, 0x10, 0x00, 0x00, 0x8B, 0xE5, 0x5D, 0xC3, 0x6A, 0x00, 0x8D, 0x8D, 0xF8, 0xFE, 0xFF, 0xFF, 0x51, 0x6A, 0x00, 0x6A, 0x02, 0x6A, 0x00, 0x6A, 0x00, 0x6A, 0x00, 0x8D, 0x95, 0xFC, 0xFE, 0xFF, 0xFF, 0x52, 0x68, 0x02, 0x00, 0x00, 0x80, 0xFF, 0x15, 0x14, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x75, 0xBC, 0x83, 0x0D, 0x38, 0xEA, 0x40, 0x00, 0x01, 0x53, 0x8B, 0x1D, 0x0C, 0xA0, 0x40, 0x00, 0x56, 0x57, 0x33, 0xFF, 0xBE, 0xD8, 0xB9, 0x40, 0x00, 0x8D, 0x64, 0x24, 0x00, 0x57, 0x68, 0xAC, 0xBC, 0x40, 0x00, 0x8D, 0x85, 0xFC, 0xFE, 0xFF, 0xFF, 0x68, 0x00, 0x01, 0x00, 0x00, 0x50, 0xE8, 0x13, 0x13, 0x00, 0x00, 0x83, 0xC4, 0x10, 0x3D, 0x00, 0x01, 0x00, 0x00, 0x0F, 0x8D, 0xD4, 0x00, 0x00, 0x00, 0x85, 0xFF, 0x74, 0x49, 0x80, 0x3E, 0x00, 0x75, 0x44, 0x8B, 0x95, 0xF8, 0xFE, 0xFF, 0xFF, 0x8D, 0x8D, 0xFC, 0xFE, 0xFF, 0xFF, 0x51, 0x52, 0xFF, 0x15, 0x1C, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x74, 0x4A, 0x83, 0xC0, 0xFE, 0x83, 0xF8, 0x01, 0x76, 0x49, 0x8B, 0x85, 0xF8, 0xFE, 0xFF, 0xFF, 0xBE, 0xFE, 0xFF, 0xFF, 0xFF, 0x09, 0x35, 0x38, 0xEA, 0x40, 0x00, 0x50, 0xFF, 0x15, 0x04, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x74, 0x55, 0x09, 0x35, 0x38, 0xEA, 0x40, 0x00, 0xEB, 0x4D, 0x8B, 0x8D, 0xF8, 0xFE, 0xFF, 0xFF, 0x68, 0x80, 0x00, 0x00, 0x00, 0x56, 0x6A, 0x03, 0x6A, 0x00, 0x8D, 0x85, 0xFC, 0xFE, 0xFF, 0xFF, 0x50, 0x51, 0xFF, 0xD3, 0x85, 0xC0, 0x75, 0x45, 0x83, 0x0D, 0x38, 0xEA, 0x40, 0x00, 0x01, 0x83, 0xEE, 0x80, 0x47, 0x81, 0xFE, 0xD8, 0xBB, 0x40, 0x00, 0x0F, 0x8C, 0x59, 0xFF, 0xFF, 0xFF, 0x8B, 0x95, 0xF8, 0xFE, 0xFF, 0xFF, 0x52, 0xFF, 0x15, 0x04, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x74, 0x07, 0x83, 0x0D, 0x38, 0xEA, 0x40, 0x00, 0xFE, 0x8B, 0x4D, 0xFC, 0xA1, 0x38, 0xEA, 0x40, 0x00, 0x5F, 0x5E, 0x33, 0xCD, 0x5B, 0xE8, 0x4E, 0x0F, 0x00, 0x00, 0x8B, 0xE5, 0x5D, 0xC3, 0x8B, 0x8D, 0xF8, 0xFE, 0xFF, 0xFF, 0xBE, 0xFE, 0xFF, 0xFF, 0xFF, 0x09, 0x35, 0x38, 0xEA, 0x40, 0x00, 0x51, 0xFF, 0x15, 0x04, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x74, 0xCE, 0x09, 0x35, 0x38, 0xEA, 0x40, 0x00, 0xEB, 0xC6, 0x8B, 0x95, 0xF8, 0xFE, 0xFF, 0xFF, 0xBE, 0xFE, 0xFF, 0xFF, 0xFF, 0x09, 0x35, 0x38, 0xEA, 0x40, 0x00, 0x52, 0xFF, 0x15, 0x04, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x74, 0xAA, 0x09, 0x35, 0x38, 0xEA, 0x40, 0x00, 0xEB, 0xA2, 0xCC, 0xCC, 0xCC, 0x55, 0x8B, 0xEC, 0x81, 0xEC, 0x08, 0x01, 0x00, 0x00, 0xA1, 0x04, 0xD0, 0x40, 0x00, 0x33, 0xC5, 0x89, 0x45, 0xFC, 0x8B, 0x45, 0x08, 0x50, 0x68, 0xB8, 0xB9, 0x40, 0x00,
				0x68, 0x58, 0xBC, 0x40, 0x00, 0x68, 0xB0, 0xBC, 0x40, 0x00, 0x8D, 0x85, 0xFC, 0xFE, 0xFF, 0xFF, 0x68, 0x00, 0x01, 0x00, 0x00, 0x50, 0xE8, 0xD3, 0x11, 0x00, 0x00, 0x83, 0xC4, 0x18, 0x3D, 0x00, 0x01, 0x00, 0x00, 0x7C, 0x1B, 0xA1, 0x38, 0xEA, 0x40, 0x00, 0x83, 0xC8, 0xFE, 0xA3, 0x38, 0xEA, 0x40, 0x00, 0x8B, 0x4D, 0xFC, 0x33, 0xCD, 0xE8, 0xA7, 0x0E, 0x00, 0x00, 0x8B, 0xE5, 0x5D, 0xC3, 0x8D, 0x8D, 0xF8, 0xFE, 0xFF, 0xFF, 0x51, 0x6A, 0x02, 0x6A, 0x00, 0x8D, 0x95, 0xFC, 0xFE, 0xFF, 0xFF, 0x52, 0x68, 0x02, 0x00, 0x00, 0x80, 0xFF, 0x15, 0x18, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x74, 0x22, 0x83, 0xC0, 0xFE, 0x83, 0xF8, 0x01, 0x76, 0x07, 0x83, 0x0D, 0x38, 0xEA, 0x40, 0x00, 0xFE, 0xA1, 0x38, 0xEA, 0x40, 0x00, 0x8B, 0x4D, 0xFC, 0x33, 0xCD, 0xE8, 0x64, 0x0E, 0x00, 0x00, 0x8B, 0xE5, 0x5D, 0xC3, 0x8B, 0x85, 0xF8, 0xFE, 0xFF, 0xFF, 0x56, 0x8B, 0x35, 0x08, 0xA0, 0x40, 0x00, 0x68, 0xCC, 0xBC, 0x40, 0x00, 0x50, 0xFF, 0xD6, 0x85, 0xC0, 0x74, 0x3E, 0x83, 0xC0, 0xFE, 0x83, 0xF8, 0x01, 0x76, 0x3D, 0x8B, 0x8D, 0xF8, 0xFE, 0xFF, 0xFF, 0xBE, 0xFE, 0xFF, 0xFF, 0xFF, 0x09, 0x35, 0x38, 0xEA, 0x40, 0x00, 0x51, 0xFF, 0x15, 0x04, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x74, 0x61, 0x09, 0x35, 0x38, 0xEA, 0x40, 0x00, 0xA1, 0x38, 0xEA, 0x40, 0x00, 0x5E, 0x8B, 0x4D, 0xFC, 0x33, 0xCD, 0xE8, 0x0D, 0x0E, 0x00, 0x00, 0x8B, 0xE5, 0x5D, 0xC3, 0x83, 0x0D, 0x38, 0xEA, 0x40, 0x00, 0x01, 0x8B, 0x95, 0xF8, 0xFE, 0xFF, 0xFF, 0x68, 0xDC, 0xBC, 0x40, 0x00, 0x52, 0xFF, 0x15, 0x1C, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x74, 0x3E, 0x83, 0xC0, 0xFE, 0x83, 0xF8, 0x01, 0x76, 0x3D, 0x8B, 0x85, 0xF8, 0xFE, 0xFF, 0xFF, 0xBE, 0xFE, 0xFF, 0xFF, 0xFF, 0x09, 0x35, 0x38, 0xEA, 0x40, 0x00, 0x50, 0xFF, 0x15, 0x04, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x74, 0x06, 0x09, 0x35, 0x38, 0xEA, 0x40, 0x00, 0xA1, 0x38, 0xEA, 0x40, 0x00, 0x5E, 0x8B, 0x4D, 0xFC, 0x33, 0xCD, 0xE8, 0xB2, 0x0D, 0x00, 0x00, 0x8B, 0xE5, 0x5D, 0xC3, 0x83, 0x0D, 0x38, 0xEA, 0x40, 0x00, 0x01, 0x8B, 0x8D, 0xF8, 0xFE, 0xFF, 0xFF, 0x68, 0xE4, 0xBC, 0x40, 0x00, 0x51, 0xFF, 0xD6, 0x85, 0xC0, 0x74, 0x11, 0x83, 0xC0, 0xFE, 0x83, 0xF8, 0x01, 0x76, 0x10, 0x83, 0x0D, 0x38, 0xEA, 0x40, 0x00, 0xFE, 0xEB, 0x07, 0x83, 0x0D, 0x38, 0xEA, 0x40, 0x00, 0x01, 0x8B, 0x85, 0xF8, 0xFE, 0xFF, 0xFF, 0xE8, 0xF3, 0x0A, 0x00, 0x00, 0x8B, 0x4D, 0xFC, 0x33, 0xCD, 0x5E, 0xE8, 0x67, 0x0D, 0x00, 0x00, 0x8B, 0xE5, 0x5D, 0xC3, 0xCC, 0xCC, 0xCC, 0xCC, 0x55, 0x8B, 0xEC, 0x81, 0xEC, 0x0C, 0x01, 0x00, 0x00, 0xA1, 0x04, 0xD0, 0x40, 0x00, 0x33, 0xC5, 0x89, 0x45, 0xFC, 0x53, 0x68, 0xB8, 0xB9, 0x40, 0x00, 0x68, 0x58, 0xBC, 0x40, 0x00, 0x68, 0xF4, 0xBC, 0x40, 0x00, 0xBB, 0x00, 0x01, 0x00, 0x00, 0x8D, 0x85, 0xFC, 0xFE, 0xFF, 0xFF, 0x53, 0x50, 0xE8, 0x35, 0x10, 0x00, 0x00, 0x83, 0xC4, 0x14, 0x3B, 0xC3, 0x7C, 0x1C, 0xA1, 0x38, 0xEA, 0x40, 0x00, 0x83, 0xC8, 0xFE, 0xA3, 0x38, 0xEA, 0x40, 0x00, 0x5B, 0x8B, 0x4D, 0xFC, 0x33, 0xCD, 0xE8, 0x0B, 0x0D, 0x00, 0x00, 0x8B, 0xE5, 0x5D, 0xC3, 0x8D, 0x8D, 0xF8, 0xFE, 0xFF, 0xFF, 0x51, 0x6A, 0x08, 0x6A, 0x00, 0x8D, 0x95, 0xFC, 0xFE, 0xFF, 0xFF, 0x52, 0x68, 0x02, 0x00, 0x00, 0x80, 0xFF,
				0x15, 0x18, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x74, 0x11, 0x83, 0xC0, 0xFE, 0x83, 0xF8, 0x01, 0x0F, 0x86, 0xF7, 0x00, 0x00, 0x00, 0xE9, 0xEB, 0x00, 0x00, 0x00, 0x8B, 0x45, 0x08, 0x83, 0xE8, 0x00, 0x56, 0x57, 0x0F, 0x84, 0xA8, 0x00, 0x00, 0x00, 0x48, 0x74, 0x51, 0x48, 0x0F, 0x85, 0xC0, 0x00, 0x00, 0x00, 0x8B, 0x3D, 0x00, 0xA0, 0x40, 0x00, 0x33, 0xF6, 0x8D, 0x9B, 0x00, 0x00, 0x00, 0x00, 0x8B, 0x95, 0xF8, 0xFE, 0xFF, 0xFF, 0x6A, 0x00, 0x6A, 0x00, 0x6A, 0x00, 0x6A, 0x00, 0x8D, 0x85, 0xF4, 0xFE, 0xFF, 0xFF, 0x50, 0x8D, 0x8D, 0xFC, 0xFE, 0xFF, 0xFF, 0x51, 0x56, 0x52, 0x89, 0x9D, 0xF4, 0xFE, 0xFF, 0xFF, 0xFF, 0xD7, 0x85, 0xC0, 0x75, 0x56, 0x8D, 0x85, 0xFC, 0xFE, 0xFF, 0xFF, 0x50, 0xE8, 0x7A, 0xFD, 0xFF, 0xFF, 0x83, 0xC4, 0x04, 0x46, 0xEB, 0xC4, 0x8B, 0x3D, 0x00, 0xA0, 0x40, 0x00, 0x33, 0xF6, 0x8B, 0x85, 0xF8, 0xFE, 0xFF, 0xFF, 0x6A, 0x00, 0x6A, 0x00, 0x6A, 0x00, 0x6A, 0x00, 0x8D, 0x8D, 0xF4, 0xFE, 0xFF, 0xFF, 0x51, 0x8D, 0x95, 0xFC, 0xFE, 0xFF, 0xFF, 0x52, 0x56, 0x50, 0x89, 0x9D, 0xF4, 0xFE, 0xFF, 0xFF, 0xFF, 0xD7, 0x85, 0xC0, 0x75, 0x12, 0x8D, 0x8D, 0xFC, 0xFE, 0xFF, 0xFF, 0x51, 0xE8, 0x76, 0xFB, 0xFF, 0xFF, 0x83, 0xC4, 0x04, 0x46, 0xEB, 0xC4, 0x3D, 0x03, 0x01, 0x00, 0x00, 0x74, 0x2B, 0x83, 0x0D, 0x38, 0xEA, 0x40, 0x00, 0xFE, 0xEB, 0x22, 0x8B, 0x95, 0xF8, 0xFE, 0xFF, 0xFF, 0x52, 0x8D, 0x8D, 0xFC, 0xFE, 0xFF, 0xFF, 0x33, 0xD2, 0xE8, 0x3C, 0x09, 0x00, 0x00, 0x83, 0xC4, 0x04, 0x83, 0xF8, 0x01, 0x75, 0x06, 0x09, 0x05, 0x38, 0xEA, 0x40, 0x00, 0x8B, 0x85, 0xF8, 0xFE, 0xFF, 0xFF, 0x50, 0xFF, 0x15, 0x04, 0xA0, 0x40, 0x00, 0x5F, 0x5E, 0x85, 0xC0, 0x74, 0x07, 0x83, 0x0D, 0x38, 0xEA, 0x40, 0x00, 0xFE, 0x8B, 0x4D, 0xFC, 0xA1, 0x38, 0xEA, 0x40, 0x00, 0x33, 0xCD, 0x5B, 0xE8, 0xD3, 0x0B, 0x00, 0x00, 0x8B, 0xE5, 0x5D, 0xC3, 0x55, 0x8B, 0xEC, 0x81, 0xEC, 0x0C, 0x01, 0x00, 0x00, 0xA1, 0x04, 0xD0, 0x40, 0x00, 0x33, 0xC5, 0x89, 0x45, 0xFC, 0x8B, 0xC2, 0x56, 0x8D, 0x70, 0x01, 0x8D, 0xA4, 0x24, 0x00, 0x00, 0x00, 0x00, 0x8A, 0x08, 0x40, 0x84, 0xC9, 0x75, 0xF9, 0x2B, 0xC6, 0x83, 0xF8, 0x04, 0x75, 0x72, 0x52, 0x68, 0x00, 0xBD, 0x40, 0x00, 0x68, 0x50, 0xBD, 0x40, 0x00, 0x8D, 0x85, 0xFC, 0xFE, 0xFF, 0xFF, 0x68, 0x00, 0x01, 0x00, 0x00, 0x50, 0xE8, 0x90, 0x0E, 0x00, 0x00, 0x83, 0xC4, 0x14, 0x3D, 0x00, 0x01, 0x00, 0x00, 0x7C, 0x1C, 0xA1, 0x38, 0xEA, 0x40, 0x00, 0x83, 0xC8, 0xFE, 0xA3, 0x38, 0xEA, 0x40, 0x00, 0x5E, 0x8B, 0x4D, 0xFC, 0x33, 0xCD, 0xE8, 0x63, 0x0B, 0x00, 0x00, 0x8B, 0xE5, 0x5D, 0xC3, 0x8D, 0x8D, 0xF8, 0xFE, 0xFF, 0xFF, 0x51, 0x6A, 0x03, 0x6A, 0x00, 0x8D, 0x95, 0xFC, 0xFE, 0xFF, 0xFF, 0x52, 0x68, 0x02, 0x00, 0x00, 0x80, 0xFF, 0x15, 0x18, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x74, 0x23, 0x83, 0xC0, 0xFE, 0x83, 0xF8, 0x01, 0x76, 0x07, 0x83, 0x0D, 0x38, 0xEA, 0x40, 0x00, 0xFE, 0xA1, 0x38, 0xEA, 0x40, 0x00, 0x5E, 0x8B, 0x4D, 0xFC, 0x33, 0xCD, 0xE8, 0x1F, 0x0B, 0x00, 0x00, 0x8B, 0xE5, 0x5D, 0xC3, 0x8B, 0x95, 0xF8, 0xFE, 0xFF, 0xFF, 0x8D, 0x85, 0xF4, 0xFE, 0xFF, 0xFF, 0x50, 0x8D, 0x8D, 0xFC, 0xFE, 0xFF, 0xFF, 0x51, 0x6A, 0x00, 0x6A, 0x00, 0x68, 0x58, 0xBD, 0x40, 0x00, 0x52, 0xC7, 0x85,
				0xF4, 0xFE, 0xFF, 0xFF, 0x00, 0x01, 0x00, 0x00, 0xFF, 0x15, 0x10, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x74, 0x3E, 0x83, 0xC0, 0xFE, 0xBE, 0xFE, 0xFF, 0xFF, 0xFF, 0x83, 0xF8, 0x01, 0x76, 0x06, 0x09, 0x35, 0x38, 0xEA, 0x40, 0x00, 0x8B, 0x85, 0xF8, 0xFE, 0xFF, 0xFF, 0x50, 0xFF, 0x15, 0x04, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x74, 0x06, 0x09, 0x35, 0x38, 0xEA, 0x40, 0x00, 0xA1, 0x38, 0xEA, 0x40, 0x00, 0x5E, 0x8B, 0x4D, 0xFC, 0x33, 0xCD, 0xE8, 0xAF, 0x0A, 0x00, 0x00, 0x8B, 0xE5, 0x5D, 0xC3, 0x83, 0xBD, 0xF4, 0xFE, 0xFF, 0xFF, 0x05, 0x0F, 0x82, 0x99, 0x02, 0x00, 0x00, 0x6A, 0x05, 0x8D, 0x8D, 0xFC, 0xFE, 0xFF, 0xFF, 0x68, 0x68, 0xBD, 0x40, 0x00, 0x51, 0xE8, 0x43, 0x0D, 0x00, 0x00, 0x83, 0xC4, 0x0C, 0x85, 0xC0, 0x0F, 0x85, 0x7B, 0x02, 0x00, 0x00, 0x8B, 0x45, 0x08, 0x48, 0x53, 0x57, 0x0F, 0x84, 0x10, 0x01, 0x00, 0x00, 0x48, 0x0F, 0x85, 0xED, 0x01, 0x00, 0x00, 0x33, 0xFF, 0xEB, 0x0A, 0x8D, 0xA4, 0x24, 0x00, 0x00, 0x00, 0x00, 0x8D, 0x49, 0x00, 0x33, 0xF6, 0x0F, 0xB6, 0x05, 0xCB, 0xB9, 0x40, 0x00, 0x8A, 0x0D, 0xC9, 0xB9, 0x40, 0x00, 0x0F, 0xB6, 0x15, 0xCA, 0xB9, 0x40, 0x00, 0xC1, 0xE0, 0x08, 0x0B, 0xD0, 0xA0, 0xC8, 0xB9, 0x40, 0x00, 0x52, 0x0F, 0xB6, 0xC9, 0x0F, 0xB6, 0xD0, 0xC1, 0xE1, 0x08, 0x0B, 0xD1, 0x52, 0x56, 0x57, 0x68, 0x70, 0xBD, 0x40, 0x00, 0x8D, 0x85, 0xFC, 0xFE, 0xFF, 0xFF, 0x68, 0x00, 0x01, 0x00, 0x00, 0x50, 0xE8, 0x25, 0x0D, 0x00, 0x00, 0x83, 0xC4, 0x1C, 0x3D, 0x00, 0x01, 0x00, 0x00, 0x7D, 0x74, 0x8D, 0x8D, 0xFC, 0xFE, 0xFF, 0xFF, 0x68, 0xA0, 0xBD, 0x40, 0x00, 0x51, 0xE8, 0x8E, 0x0B, 0x00, 0x00, 0x83, 0xC4, 0x08, 0x85, 0xC0, 0x75, 0x16, 0x8B, 0x9D, 0xF8, 0xFE, 0xFF, 0xFF, 0xE8, 0xDE, 0x07, 0x00, 0x00, 0x83, 0xF8, 0x01, 0x0F, 0x85, 0xC3, 0x01, 0x00, 0x00, 0xEB, 0x2D, 0x8B, 0x85, 0xF8, 0xFE, 0xFF, 0xFF, 0x8D, 0x95, 0xFC, 0xFE, 0xFF, 0xFF, 0x52, 0x50, 0xFF, 0x15, 0x1C, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x74, 0x0E, 0x83, 0xC0, 0xFE, 0x83, 0xF8, 0x01, 0x0F, 0x87, 0x5C, 0x01, 0x00, 0x00, 0xEB, 0x07, 0x83, 0x0D, 0x38, 0xEA, 0x40, 0x00, 0x01, 0x46, 0x83, 0xFE, 0x04, 0x0F, 0x8C, 0x4E, 0xFF, 0xFF, 0xFF, 0x47, 0x83, 0xFF, 0x14, 0x0F, 0x8C, 0x42, 0xFF, 0xFF, 0xFF, 0xE9, 0x1C, 0x01, 0x00, 0x00, 0x8B, 0x8D, 0xF8, 0xFE, 0xFF, 0xFF, 0xBE, 0xFE, 0xFF, 0xFF, 0xFF, 0x09, 0x35, 0x38, 0xEA, 0x40, 0x00, 0x51, 0xFF, 0x15, 0x04, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x74, 0x06, 0x09, 0x35, 0x38, 0xEA, 0x40, 0x00, 0xA1, 0x38, 0xEA, 0x40, 0x00, 0x5F, 0x5B, 0x5E, 0x8B, 0x4D, 0xFC, 0x33, 0xCD, 0xE8, 0x68, 0x09, 0x00, 0x00, 0x8B, 0xE5, 0x5D, 0xC3, 0x33, 0xDB, 0x8D, 0x49, 0x00, 0x33, 0xFF, 0xBE, 0xD8, 0xB9, 0x40, 0x00, 0xEB, 0x07, 0x8D, 0xA4, 0x24, 0x00, 0x00, 0x00, 0x00, 0x0F, 0xB6, 0x05, 0xCB, 0xB9, 0x40, 0x00, 0x8A, 0x0D, 0xC9, 0xB9, 0x40, 0x00, 0x0F, 0xB6, 0x15, 0xCA, 0xB9, 0x40, 0x00, 0xC1, 0xE0, 0x08, 0x0B, 0xD0, 0xA0, 0xC8, 0xB9, 0x40, 0x00, 0x52, 0x0F, 0xB6, 0xC9, 0x0F, 0xB6, 0xD0, 0xC1, 0xE1, 0x08, 0x0B, 0xD1, 0x52, 0x57, 0x53, 0x68, 0x70, 0xBD, 0x40, 0x00, 0x8D, 0x85, 0xFC, 0xFE, 0xFF, 0xFF, 0x68, 0x00, 0x01, 0x00, 0x00, 0x50, 0xE8, 0x17, 0x0C, 0x00, 0x00, 0x83, 0xC4, 0x1C, 0x3D, 0x00,
				0x01, 0x00, 0x00, 0x0F, 0x8D, 0xBF, 0x00, 0x00, 0x00, 0x85, 0xFF, 0x74, 0x27, 0x80, 0x3E, 0x00, 0x75, 0x22, 0x8B, 0x95, 0xF8, 0xFE, 0xFF, 0xFF, 0x8D, 0x8D, 0xFC, 0xFE, 0xFF, 0xFF, 0x51, 0x52, 0xFF, 0x15, 0x1C, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x74, 0x2C, 0x83, 0xC0, 0xFE, 0x83, 0xF8, 0x01, 0x77, 0x73, 0xEB, 0x29, 0x8B, 0x8D, 0xF8, 0xFE, 0xFF, 0xFF, 0x68, 0x80, 0x00, 0x00, 0x00, 0x56, 0x6A, 0x03, 0x6A, 0x00, 0x8D, 0x85, 0xFC, 0xFE, 0xFF, 0xFF, 0x50, 0x51, 0xFF, 0x15, 0x0C, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x75, 0x4F, 0x83, 0x0D, 0x38, 0xEA, 0x40, 0x00, 0x01, 0x83, 0xEE, 0x80, 0x47, 0x81, 0xFE, 0xD8, 0xBB, 0x40, 0x00, 0x0F, 0x8C, 0x4B, 0xFF, 0xFF, 0xFF, 0x43, 0x83, 0xFB, 0x14, 0x0F, 0x8C, 0x31, 0xFF, 0xFF, 0xFF, 0x8B, 0x8D, 0xF8, 0xFE, 0xFF, 0xFF, 0xE8, 0x26, 0x06, 0x00, 0x00, 0x83, 0xF8, 0x01, 0x75, 0x5F, 0x8B, 0x85, 0xF8, 0xFE, 0xFF, 0xFF, 0x68, 0xE4, 0xBC, 0x40, 0x00, 0x50, 0xFF, 0x15, 0x08, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x74, 0x42, 0x83, 0xC0, 0xFE, 0x83, 0xF8, 0x01, 0x76, 0x41, 0x8B, 0x85, 0xF8, 0xFE, 0xFF, 0xFF, 0x83, 0x0D, 0x38, 0xEA, 0x40, 0x00, 0xFE, 0xE8, 0xD1, 0x05, 0x00, 0x00, 0x5F, 0x5B, 0x5E, 0x8B, 0x4D, 0xFC, 0x33, 0xCD, 0xE8, 0x43, 0x08, 0x00, 0x00, 0x8B, 0xE5, 0x5D, 0xC3, 0x8B, 0x95, 0xF8, 0xFE, 0xFF, 0xFF, 0xBE, 0xFE, 0xFF, 0xFF, 0xFF, 0x09, 0x35, 0x38, 0xEA, 0x40, 0x00, 0x52, 0xE9, 0x9E, 0xFE, 0xFF, 0xFF, 0x83, 0x0D, 0x38, 0xEA, 0x40, 0x00, 0x01, 0x8B, 0x85, 0xF8, 0xFE, 0xFF, 0xFF, 0xE8, 0x97, 0x05, 0x00, 0x00, 0x5F, 0x5B, 0x5E, 0x8B, 0x4D, 0xFC, 0x33, 0xCD, 0xE8, 0x09, 0x08, 0x00, 0x00, 0x8B, 0xE5, 0x5D, 0xC3, 0x8B, 0x8D, 0xF8, 0xFE, 0xFF, 0xFF, 0x51, 0xFF, 0x15, 0x04, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x0F, 0x84, 0xC1, 0xFC, 0xFF, 0xFF, 0x8B, 0x4D, 0xFC, 0x83, 0x0D, 0x38, 0xEA, 0x40, 0x00, 0xFE, 0xA1, 0x38, 0xEA, 0x40, 0x00, 0x33, 0xCD, 0x5E, 0xE8, 0xD9, 0x07, 0x00, 0x00, 0x8B, 0xE5, 0x5D, 0xC3, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0x55, 0x8B, 0xEC, 0x81, 0xEC, 0x0C, 0x01, 0x00, 0x00, 0xA1, 0x04, 0xD0, 0x40, 0x00, 0x33, 0xC5, 0x89, 0x45, 0xFC, 0xB9, 0xB8, 0xBD, 0x40, 0x00, 0xB8, 0xB8, 0xB9, 0x40, 0x00, 0x8D, 0x49, 0x00, 0x8A, 0x10, 0x3A, 0x11, 0x75, 0x1A, 0x84, 0xD2, 0x74, 0x12, 0x8A, 0x50, 0x01, 0x3A, 0x51, 0x01, 0x75, 0x0E, 0x83, 0xC0, 0x02, 0x83, 0xC1, 0x02, 0x84, 0xD2, 0x75, 0xE4, 0x33, 0xC0, 0xEB, 0x05, 0x1B, 0xC0, 0x83, 0xD8, 0xFF, 0x85, 0xC0, 0x0F, 0x84, 0xC1, 0x00, 0x00, 0x00, 0x8D, 0x85, 0xF8, 0xFE, 0xFF, 0xFF, 0x50, 0x6A, 0x08, 0x6A, 0x00, 0x68, 0x00, 0xBD, 0x40, 0x00, 0x68, 0x02, 0x00, 0x00, 0x80, 0xFF, 0x15, 0x18, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x74, 0x26, 0x83, 0xC0, 0xFE, 0x83, 0xF8, 0x01, 0x0F, 0x86, 0x96, 0x00, 0x00, 0x00, 0x83, 0x0D, 0x38, 0xEA, 0x40, 0x00, 0xFE, 0xA1, 0x38, 0xEA, 0x40, 0x00, 0x8B, 0x4D, 0xFC, 0x33, 0xCD, 0xE8, 0x41, 0x07, 0x00, 0x00, 0x8B, 0xE5, 0x5D, 0xC3, 0x56, 0x57, 0x8B, 0x3D, 0x00, 0xA0, 0x40, 0x00, 0x33, 0xF6, 0x8D, 0x64, 0x24, 0x00, 0x8B, 0x85, 0xF8, 0xFE, 0xFF, 0xFF, 0x6A, 0x00, 0x6A, 0x00, 0x6A, 0x00, 0x6A, 0x00, 0x8D, 0x8D, 0xF4, 0xFE, 0xFF, 0xFF, 0x51, 0x8D, 0x95, 0xFC, 0xFE, 0xFF, 0xFF, 0x52,
				0x56, 0x50, 0xC7, 0x85, 0xF4, 0xFE, 0xFF, 0xFF, 0x00, 0x01, 0x00, 0x00, 0xFF, 0xD7, 0x85, 0xC0, 0x75, 0x15, 0x8B, 0x4D, 0x08, 0x51, 0x8D, 0x95, 0xFC, 0xFE, 0xFF, 0xFF, 0xE8, 0x23, 0xFB, 0xFF, 0xFF, 0x83, 0xC4, 0x04, 0x46, 0xEB, 0xBD, 0xBE, 0xFE, 0xFF, 0xFF, 0xFF, 0x3D, 0x03, 0x01, 0x00, 0x00, 0x74, 0x06, 0x09, 0x35, 0x38, 0xEA, 0x40, 0x00, 0x8B, 0x95, 0xF8, 0xFE, 0xFF, 0xFF, 0x52, 0xFF, 0x15, 0x04, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x74, 0x06, 0x09, 0x35, 0x38, 0xEA, 0x40, 0x00, 0x5F, 0x5E, 0x8B, 0x4D, 0xFC, 0xA1, 0x38, 0xEA, 0x40, 0x00, 0x33, 0xCD, 0xE8, 0xB2, 0x06, 0x00, 0x00, 0x8B, 0xE5, 0x5D, 0xC3, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0x6A, 0x00, 0xC7, 0x05, 0x38, 0xEA, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0xE8, 0x2F, 0xF9, 0xFF, 0xFF, 0xA1, 0x38, 0xEA, 0x40, 0x00, 0x83, 0xC4, 0x04, 0x83, 0xF8, 0xFE, 0x74, 0x3D, 0x85, 0xC0, 0x74, 0x22, 0x83, 0xF8, 0x01, 0x74, 0x17, 0x6A, 0x10, 0x68, 0xC8, 0xBD, 0x40, 0x00, 0x68, 0x30, 0xBE, 0x40, 0x00, 0x6A, 0x00, 0xFF, 0x15, 0x24, 0xA1, 0x40, 0x00, 0x33, 0xC0, 0xC3, 0xB8, 0x01, 0x00, 0x00, 0x00, 0xC3, 0x6A, 0x10, 0x68, 0xC8, 0xBD, 0x40, 0x00, 0x68, 0xD8, 0xBD, 0x40, 0x00, 0x6A, 0x00, 0xFF, 0x15, 0x24, 0xA1, 0x40, 0x00, 0x33, 0xC0, 0xC3, 0x6A, 0x10, 0x68, 0xC8, 0xBD, 0x40, 0x00, 0x68, 0xF4, 0xBD, 0x40, 0x00, 0x6A, 0x00, 0xFF, 0x15, 0x24, 0xA1, 0x40, 0x00, 0x33, 0xC0, 0xC3, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0x55, 0x8B, 0xEC, 0x6A, 0x01, 0xC7, 0x05, 0x38, 0xEA, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0xE8, 0xAC, 0xF8, 0xFF, 0xFF, 0x6A, 0x01, 0xE8, 0x35, 0xFE, 0xFF, 0xFF, 0xA1, 0x38, 0xEA, 0x40, 0x00, 0x83, 0xC0, 0x02, 0x83, 0xC4, 0x08, 0x83, 0xF8, 0x03, 0x77, 0x76, 0xFF, 0x24, 0x85, 0x5C, 0x1B, 0x40, 0x00, 0x83, 0x7D, 0x08, 0x00, 0x74, 0x14, 0x6A, 0x40, 0x68, 0x40, 0xBE, 0x40, 0x00, 0x68, 0x50, 0xBE, 0x40, 0x00, 0x6A, 0x00, 0xFF, 0x15, 0x24, 0xA1, 0x40, 0x00, 0x33, 0xC0, 0x5D, 0xC3, 0x6A, 0x10, 0x68, 0x40, 0xBE, 0x40, 0x00, 0x68, 0xD8, 0xBD, 0x40, 0x00, 0x6A, 0x00, 0xFF, 0x15, 0x24, 0xA1, 0x40, 0x00, 0xB8, 0x01, 0x00, 0x00, 0x00, 0x5D, 0xC3, 0x6A, 0x10, 0x68, 0x40, 0xBE, 0x40, 0x00, 0x68, 0xA0, 0xBE, 0x40, 0x00, 0x6A, 0x00, 0xFF, 0x15, 0x24, 0xA1, 0x40, 0x00, 0xB8, 0x01, 0x00, 0x00, 0x00, 0x5D, 0xC3, 0x6A, 0x10, 0x68, 0x40, 0xBE, 0x40, 0x00, 0x68, 0xFC, 0xBE, 0x40, 0x00, 0x6A, 0x00, 0xFF, 0x15, 0x24, 0xA1, 0x40, 0x00, 0xB8, 0x01, 0x00, 0x00, 0x00, 0x5D, 0xC3, 0x6A, 0x10, 0x68, 0x40, 0xBE, 0x40, 0x00, 0x68, 0x30, 0xBE, 0x40, 0x00, 0x6A, 0x00, 0xFF, 0x15, 0x24, 0xA1, 0x40, 0x00, 0xB8, 0x01, 0x00, 0x00, 0x00, 0x5D, 0xC3, 0x26, 0x1B, 0x40, 0x00, 0x0B, 0x1B, 0x40, 0x00, 0xF0, 0x1A, 0x40, 0x00, 0xD2, 0x1A, 0x40, 0x00, 0xCC, 0xCC, 0xCC, 0xCC, 0x6A, 0x02, 0xC7, 0x05, 0x38, 0xEA, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0xE8, 0xDF, 0xF7, 0xFF, 0xFF, 0x6A, 0x02, 0xE8, 0x68, 0xFD, 0xFF, 0xFF, 0xA1, 0x38, 0xEA, 0x40, 0x00, 0x83, 0xC0, 0x02, 0x83, 0xC4, 0x08, 0x83, 0xF8, 0x03, 0x77, 0x71, 0xFF, 0x24, 0x85, 0x24, 0x1C, 0x40, 0x00, 0x85,
				0xF6, 0x74, 0x14, 0x6A, 0x40, 0x68, 0x34, 0xBF, 0x40, 0x00, 0x68, 0x40, 0xBF, 0x40, 0x00, 0x6A, 0x00, 0xFF, 0x15, 0x24, 0xA1, 0x40, 0x00, 0x33, 0xC0, 0xC3, 0x85, 0xF6, 0x74, 0x14, 0x6A, 0x40, 0x68, 0x34, 0xBF, 0x40, 0x00, 0x68, 0x90, 0xBF, 0x40, 0x00, 0x6A, 0x00, 0xFF, 0x15, 0x24, 0xA1, 0x40, 0x00, 0x33, 0xC0, 0xC3, 0x6A, 0x10, 0x68, 0x34, 0xBF, 0x40, 0x00, 0x68, 0xE0, 0xBF, 0x40, 0x00, 0x6A, 0x00, 0xFF, 0x15, 0x24, 0xA1, 0x40, 0x00, 0xB8, 0x01, 0x00, 0x00, 0x00, 0xC3, 0x6A, 0x10, 0x68, 0x34, 0xBF, 0x40, 0x00, 0x68, 0x38, 0xC0, 0x40, 0x00, 0x6A, 0x00, 0xFF, 0x15, 0x24, 0xA1, 0x40, 0x00, 0xB8, 0x01, 0x00, 0x00, 0x00, 0xC3, 0x6A, 0x10, 0x68, 0x34, 0xBF, 0x40, 0x00, 0x68, 0x30, 0xBE, 0x40, 0x00, 0x6A, 0x00, 0xFF, 0x15, 0x24, 0xA1, 0x40, 0x00, 0xB8, 0x01, 0x00, 0x00, 0x00, 0xC3, 0x90, 0xEF, 0x1B, 0x40, 0x00, 0xD5, 0x1B, 0x40, 0x00, 0xBA, 0x1B, 0x40, 0x00, 0x9F, 0x1B, 0x40, 0x00, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0x55, 0x8B, 0xEC, 0x81, 0x7D, 0x0C, 0x11, 0x01, 0x00, 0x00, 0x74, 0x06, 0x33, 0xC0, 0x5D, 0xC2, 0x10, 0x00, 0x8B, 0x45, 0x10, 0x8B, 0x4D, 0x08, 0x50, 0x51, 0xFF, 0x15, 0x28, 0xA1, 0x40, 0x00, 0xB8, 0x01, 0x00, 0x00, 0x00, 0x5D, 0xC2, 0x10, 0x00, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0x55, 0x8B, 0xEC, 0xE8, 0xA8, 0xFD, 0xFF, 0xFF, 0x85, 0xC0, 0x0F, 0x84, 0x08, 0x01, 0x00, 0x00, 0x8B, 0x4D, 0x10, 0x8A, 0x01, 0x3C, 0x2D, 0x74, 0x04, 0x3C, 0x2F, 0x75, 0x2E, 0x80, 0x79, 0x02, 0x00, 0x75, 0x28, 0x0F, 0xBE, 0x41, 0x01, 0x83, 0xC0, 0xB7, 0x83, 0xF8, 0x29, 0x77, 0x1C, 0x0F, 0xB6, 0x80, 0xAC, 0x1D, 0x40, 0x00, 0xFF, 0x24, 0x85, 0xA0, 0x1D, 0x40, 0x00, 0xB8, 0x64, 0x00, 0x00, 0x00, 0xEB, 0x09, 0xB8, 0x65, 0x00, 0x00, 0x00, 0xEB, 0x02, 0x33, 0xC0, 0x83, 0xE8, 0x64, 0x0F, 0x84, 0xCB, 0x00, 0x00, 0x00, 0x48, 0x74, 0x3C, 0x8B, 0x4D, 0x08, 0x6A, 0x00, 0x68, 0x40, 0x1C, 0x40, 0x00, 0x6A, 0x00, 0x68, 0x6C, 0xC0, 0x40, 0x00, 0x51, 0xFF, 0x15, 0x2C, 0xA1, 0x40, 0x00, 0x83, 0xE8, 0x64, 0x74, 0x11, 0x48, 0x75, 0x4B, 0x56, 0x8D, 0x70, 0x01, 0xE8, 0x7E, 0xFE, 0xFF, 0xFF, 0x5E, 0x5D, 0xC2, 0x10, 0x00, 0x6A, 0x01, 0xE8, 0xA2, 0xFD, 0xFF, 0xFF, 0x83, 0xC4, 0x04, 0x5D, 0xC2, 0x10, 0x00, 0x6A, 0x02, 0xC7, 0x05, 0x38, 0xEA, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0xE8, 0x4A, 0xF6, 0xFF, 0xFF, 0x6A, 0x02, 0xE8, 0xD3, 0xFB, 0xFF, 0xFF, 0xA1, 0x38, 0xEA, 0x40, 0x00, 0x83, 0xC0, 0x02, 0x83, 0xC4, 0x08, 0x83, 0xF8, 0x03, 0x77, 0x47, 0xFF, 0x24, 0x85, 0xD8, 0x1D, 0x40, 0x00, 0x33, 0xC0, 0x5D, 0xC2, 0x10, 0x00, 0x6A, 0x10, 0x68, 0x34, 0xBF, 0x40, 0x00, 0x68, 0xE0, 0xBF, 0x40, 0x00, 0x6A, 0x00, 0xFF, 0x15, 0x24, 0xA1, 0x40, 0x00, 0xB8, 0x01, 0x00, 0x00, 0x00, 0x5D, 0xC2, 0x10, 0x00, 0x6A, 0x10, 0x68, 0x34, 0xBF, 0x40, 0x00, 0x68, 0x38, 0xC0, 0x40, 0x00, 0x6A, 0x00, 0xFF, 0x15, 0x24, 0xA1, 0x40, 0x00, 0xB8, 0x01, 0x00, 0x00, 0x00, 0x5D, 0xC2, 0x10, 0x00, 0x6A, 0x10, 0x68, 0x34, 0xBF, 0x40, 0x00, 0x68, 0x30, 0xBE, 0x40, 0x00, 0x6A, 0x00, 0xFF, 0x15, 0x24, 0xA1, 0x40, 0x00, 0xB8, 0x01, 0x00, 0x00, 0x00, 0x5D, 0xC2, 0x10, 0x00, 0x6A, 0x00, 0xE8,
				0x08, 0xFD, 0xFF, 0xFF, 0x83, 0xC4, 0x04, 0x5D, 0xC2, 0x10, 0x00, 0x90, 0xAD, 0x1C, 0x40, 0x00, 0xB4, 0x1C, 0x40, 0x00, 0xBB, 0x1C, 0x40, 0x00, 0x00, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x01, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x00, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x01, 0x8B, 0xFF, 0x57, 0x1D, 0x40, 0x00, 0x3A, 0x1D, 0x40, 0x00, 0x34, 0x1D, 0x40, 0x00, 0x34, 0x1D, 0x40, 0x00, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0x55, 0x8B, 0xEC, 0x51, 0x6A, 0x00, 0x6A, 0x00, 0x6A, 0x00, 0x6A, 0x00, 0x8D, 0x45, 0xFC, 0x50, 0x8B, 0x45, 0x08, 0x51, 0x52, 0x50, 0xC7, 0x45, 0xFC, 0x00, 0x01, 0x00, 0x00, 0xFF, 0x15, 0x00, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x74, 0x1C, 0x3D, 0x03, 0x01, 0x00, 0x00, 0x74, 0x0F, 0xB8, 0xFE, 0xFF, 0xFF, 0xFF, 0x09, 0x05, 0x38, 0xEA, 0x40, 0x00, 0x8B, 0xE5, 0x5D, 0xC3, 0x33, 0xC0, 0x8B, 0xE5, 0x5D, 0xC3, 0xB8, 0x01, 0x00, 0x00, 0x00, 0x8B, 0xE5, 0x5D, 0xC3, 0xCC, 0xCC, 0xCC, 0xCC, 0x50, 0xFF, 0x15, 0x04, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0xA1, 0x38, 0xEA, 0x40, 0x00, 0x74, 0x08, 0x83, 0xC8, 0xFE, 0xA3, 0x38, 0xEA, 0x40, 0x00, 0xC3, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0x55, 0x8B, 0xEC, 0x51, 0x6A, 0x04, 0x8D, 0x45, 0xFC, 0x50, 0x6A, 0x04, 0x6A, 0x00, 0x68, 0x88, 0xBD, 0x40, 0x00, 0x51, 0xC7, 0x45, 0xFC, 0x01, 0x00, 0x00, 0x00, 0xFF, 0x15, 0x0C, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x75, 0x10, 0x83, 0x0D, 0x38, 0xEA, 0x40, 0x00, 0x01, 0xB8, 0x01, 0x00, 0x00, 0x00, 0x8B, 0xE5, 0x5D, 0xC3, 0xB8, 0xFE, 0xFF, 0xFF, 0xFF, 0x09, 0x05, 0x38, 0xEA, 0x40, 0x00, 0x8B, 0xE5, 0x5D, 0xC3, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0x55, 0x8B, 0xEC, 0x83, 0xEC, 0x08, 0x56, 0x57, 0xBF, 0x80, 0x00, 0x00, 0x00, 0x57, 0xE8, 0xED, 0x05, 0x00, 0x00, 0x8B, 0xF0, 0x83, 0xC4, 0x04, 0x85, 0xF6, 0x0F, 0x84, 0xE5, 0x00, 0x00, 0x00, 0x8D, 0x45, 0xFC, 0x50, 0x56, 0x8D, 0x4D, 0xF8, 0x51, 0x6A, 0x00, 0x68, 0xA0, 0xBD, 0x40, 0x00, 0x53, 0x89, 0x7D, 0xFC, 0xFF, 0x15, 0x10, 0xA0, 0x40, 0x00, 0x3D, 0xEA, 0x00, 0x00, 0x00, 0x0F, 0x87, 0xB7, 0x00, 0x00, 0x00, 0x0F, 0xB6, 0x90, 0xD4, 0x1F, 0x40, 0x00, 0xFF, 0x24, 0x95, 0xC8, 0x1F, 0x40, 0x00, 0x83, 0x7D, 0xF8, 0x03, 0x75, 0x5A, 0x39, 0x7D, 0xFC, 0x75, 0x55, 0x8B, 0xC7, 0xB9, 0xD8, 0xBB, 0x40, 0x00, 0x8B, 0xFE, 0x8B, 0x17, 0x3B, 0x11, 0x75, 0x46, 0x83, 0xE8, 0x04, 0x83, 0xC1, 0x04, 0x83, 0xC7, 0x04, 0x83, 0xF8, 0x04, 0x73, 0xEC, 0x85, 0xC0, 0x74, 0x20, 0x8A, 0x11, 0x3A, 0x17, 0x75, 0x2E, 0x83, 0xF8, 0x01, 0x76, 0x15, 0x8A, 0x51, 0x01, 0x3A, 0x57, 0x01, 0x75, 0x21, 0x83, 0xF8, 0x02, 0x76, 0x08, 0x8A, 0x41, 0x02, 0x3A, 0x47, 0x02, 0x75, 0x14, 0x56, 0xE8, 0x21, 0x05, 0x00, 0x00, 0x83, 0xC4, 0x04, 0x5F, 0xB8, 0x01, 0x00, 0x00, 0x00, 0x5E, 0x8B, 0xE5, 0x5D, 0xC3, 0x56, 0xE8, 0x0D, 0x05, 0x00, 0x00, 0x83, 0xC4, 0x04, 0x68, 0x80, 0x00, 0x00, 0x00, 0x68, 0xD8, 0xBB, 0x40, 0x00, 0x6A, 0x03, 0x6A, 0x00, 0x68, 0xA0, 0xBD, 0x40, 0x00, 0x53, 0xFF, 0x15, 0x0C, 0xA0, 0x40, 0x00, 0x85, 0xC0,
				0xB8, 0x01, 0x00, 0x00, 0x00, 0x74, 0x2B, 0xB8, 0xFE, 0xFF, 0xFF, 0xFF, 0x09, 0x05, 0x38, 0xEA, 0x40, 0x00, 0x5F, 0x5E, 0x8B, 0xE5, 0x5D, 0xC3, 0x56, 0xE8, 0xD0, 0x04, 0x00, 0x00, 0x83, 0xC4, 0x04, 0x57, 0xEB, 0xC5, 0x56, 0xE8, 0xC4, 0x04, 0x00, 0x00, 0x83, 0xC4, 0x04, 0xB8, 0xFE, 0xFF, 0xFF, 0xFF, 0x09, 0x05, 0x38, 0xEA, 0x40, 0x00, 0x5F, 0x5E, 0x8B, 0xE5, 0x5D, 0xC3, 0x8B, 0xFF, 0x03, 0x1F, 0x40, 0x00, 0xA0, 0x1F, 0x40, 0x00, 0xAC, 0x1F, 0x40, 0x00, 0x00, 0x02, 0x01, 0x01, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x01, 0x3B, 0x0D, 0x04, 0xD0, 0x40, 0x00, 0x75, 0x02, 0xF3, 0xC3, 0xE9, 0x16, 0x06, 0x00, 0x00, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x8B, 0x45, 0x08, 0x56, 0x8B, 0xF1, 0xC6, 0x46, 0x0C, 0x00, 0x85, 0xC0, 0x75, 0x63, 0xE8, 0xD1, 0x12, 0x00, 0x00, 0x89, 0x46, 0x08, 0x8B, 0x48, 0x6C, 0x89, 0x0E, 0x8B, 0x48, 0x68, 0x89, 0x4E, 0x04, 0x8B, 0x0E, 0x3B, 0x0D, 0x78, 0xD7, 0x40, 0x00, 0x74, 0x12, 0x8B, 0x0D, 0x30, 0xD5, 0x40, 0x00, 0x85, 0x48, 0x70, 0x75, 0x07, 0xE8, 0x80, 0x10, 0x00, 0x00, 0x89, 0x06, 0x8B, 0x46, 0x04, 0x3B, 0x05, 0x38, 0xD4, 0x40, 0x00, 0x74, 0x16, 0x8B, 0x46, 0x08, 0x8B, 0x0D, 0x30, 0xD5, 0x40, 0x00, 0x85, 0x48, 0x70, 0x75, 0x08, 0xE8, 0xDF, 0x08, 0x00, 0x00, 0x89, 0x46, 0x04, 0x8B, 0x46, 0x08, 0xF6, 0x40, 0x70, 0x02, 0x75, 0x14, 0x83, 0x48, 0x70, 0x02, 0xC6, 0x46, 0x0C, 0x01, 0xEB, 0x0A, 0x8B, 0x08, 0x89, 0x0E, 0x8B, 0x40, 0x04, 0x89, 0x46, 0x04, 0x8B, 0xC6, 0x5E, 0x5D, 0xC2, 0x04, 0x00, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x8B, 0x55, 0x0C, 0x56, 0x8B, 0x75, 0x08, 0x57, 0x0F, 0xB6, 0x06, 0x8D, 0x48, 0xBF, 0x46, 0x83, 0xF9, 0x19, 0x77, 0x03, 0x83, 0xC0, 0x20, 0x0F, 0xB6, 0x0A, 0x8D, 0x79, 0xBF, 0x42, 0x83, 0xFF, 0x19, 0x77,
				0x03, 0x83, 0xC1, 0x20, 0x85, 0xC0, 0x74, 0x04, 0x3B, 0xC1, 0x74, 0xDA, 0x5F, 0x2B, 0xC1, 0x5E, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x83, 0xEC, 0x10, 0x53, 0xFF, 0x75, 0x10, 0x8D, 0x4D, 0xF0, 0xE8, 0x2C, 0xFF, 0xFF, 0xFF, 0x8B, 0x5D, 0x08, 0x85, 0xDB, 0x75, 0x23, 0xE8, 0xBE, 0x17, 0x00, 0x00, 0xC7, 0x00, 0x16, 0x00, 0x00, 0x00, 0xE8, 0x61, 0x17, 0x00, 0x00, 0x38, 0x5D, 0xFC, 0x74, 0x07, 0x8B, 0x45, 0xF8, 0x83, 0x60, 0x70, 0xFD, 0xB8, 0xFF, 0xFF, 0xFF, 0x7F, 0xEB, 0x7F, 0x56, 0x8B, 0x75, 0x0C, 0x85, 0xF6, 0x75, 0x24, 0xE8, 0x93, 0x17, 0x00, 0x00, 0xC7, 0x00, 0x16, 0x00, 0x00, 0x00, 0xE8, 0x36, 0x17, 0x00, 0x00, 0x80, 0x7D, 0xFC, 0x00, 0x74, 0x07, 0x8B, 0x45, 0xF8, 0x83, 0x60, 0x70, 0xFD, 0xB8, 0xFF, 0xFF, 0xFF, 0x7F, 0xEB, 0x52, 0x8B, 0x45, 0xF0, 0x83, 0x78, 0x14, 0x00, 0x75, 0x0B, 0x56, 0x53, 0xE8, 0x4D, 0xFF, 0xFF, 0xFF, 0x59, 0x59, 0xEB, 0x31, 0x2B, 0xDE, 0x57, 0x0F, 0xB6, 0x04, 0x33, 0x8D, 0x4D, 0xF0, 0x51, 0x50, 0xE8, 0x5E, 0x14, 0x00, 0x00, 0x8B, 0xF8, 0x0F, 0xB6, 0x06, 0x8D, 0x4D, 0xF0, 0x51, 0x50, 0xE8, 0x4F, 0x14, 0x00, 0x00, 0x83, 0xC4, 0x10, 0x46, 0x85, 0xFF, 0x74, 0x04, 0x3B, 0xF8, 0x74, 0xD7, 0x2B, 0xF8, 0x8B, 0xC7, 0x5F, 0x80, 0x7D, 0xFC, 0x00, 0x74, 0x07, 0x8B, 0x4D, 0xF8, 0x83, 0x61, 0x70, 0xFD, 0x5E, 0x5B, 0xC9, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x33, 0xC0, 0x39, 0x05, 0x24, 0xE1, 0x40, 0x00, 0x75, 0x27, 0x39, 0x45, 0x08, 0x75, 0x17, 0xE8, 0x05, 0x17, 0x00, 0x00, 0xC7, 0x00, 0x16, 0x00, 0x00, 0x00, 0xE8, 0xA8, 0x16, 0x00, 0x00, 0xB8, 0xFF, 0xFF, 0xFF, 0x7F, 0x5D, 0xC3, 0x39, 0x45, 0x0C, 0x74, 0xE4, 0x5D, 0xE9, 0xD1, 0xFE, 0xFF, 0xFF, 0x50, 0xFF, 0x75, 0x0C, 0xFF, 0x75, 0x08, 0xE8, 0xFE, 0xFE, 0xFF, 0xFF, 0x83, 0xC4, 0x0C, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x83, 0xEC, 0x10, 0x83, 0x7D, 0x10, 0x00, 0x53, 0x56, 0x57, 0x0F, 0x84, 0xC6, 0x00, 0x00, 0x00, 0xFF, 0x75, 0x14, 0x8D, 0x4D, 0xF0, 0xE8, 0x19, 0xFE, 0xFF, 0xFF, 0x8B, 0x5D, 0x08, 0x85, 0xDB, 0x75, 0x27, 0xE8, 0xAB, 0x16, 0x00, 0x00, 0xC7, 0x00, 0x16, 0x00, 0x00, 0x00, 0xE8, 0x4E, 0x16, 0x00, 0x00, 0x80, 0x7D, 0xFC, 0x00, 0x74, 0x07, 0x8B, 0x45, 0xF8, 0x83, 0x60, 0x70, 0xFD, 0xB8, 0xFF, 0xFF, 0xFF, 0x7F, 0xE9, 0x8F, 0x00, 0x00, 0x00, 0x8B, 0x75, 0x0C, 0x85, 0xF6, 0x74, 0xD2, 0xBF, 0xFF, 0xFF, 0xFF, 0x7F, 0x39, 0x7D, 0x10, 0x76, 0x21, 0xE8, 0x73, 0x16, 0x00, 0x00, 0xC7, 0x00, 0x16, 0x00, 0x00, 0x00, 0xE8, 0x16, 0x16, 0x00, 0x00, 0x80, 0x7D, 0xFC, 0x00, 0x74, 0x07, 0x8B, 0x45, 0xF8, 0x83, 0x60, 0x70, 0xFD, 0x8B, 0xC7, 0xEB, 0x5D, 0x8B, 0x45, 0xF0, 0x83, 0x78, 0x14, 0x00, 0x75, 0x1C, 0xFF, 0x75, 0x10, 0x56, 0x53, 0xE8, 0x98, 0x16, 0x00, 0x00, 0x83, 0xC4, 0x0C, 0x80, 0x7D, 0xFC, 0x00, 0x74, 0x41, 0x8B, 0x4D, 0xF8, 0x83, 0x61, 0x70, 0xFD, 0xEB, 0x38, 0x2B, 0xDE, 0x0F, 0xB6, 0x04, 0x33, 0x8D, 0x4D, 0xF0, 0x51, 0x50, 0xE8, 0x31, 0x13, 0x00, 0x00, 0x8B, 0xF8, 0x0F, 0xB6, 0x06, 0x8D, 0x4D, 0xF0, 0x51, 0x50, 0xE8, 0x22, 0x13, 0x00, 0x00, 0x83, 0xC4, 0x10, 0x46, 0xFF, 0x4D, 0x10, 0x74, 0x08, 0x85, 0xFF, 0x74, 0x04, 0x3B, 0xF8, 0x74, 0xD2, 0x2B, 0xF8, 0x8B, 0xC7, 0xEB, 0xBB,
				0x33, 0xC0, 0x5F, 0x5E, 0x5B, 0xC9, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x33, 0xC0, 0x39, 0x05, 0x24, 0xE1, 0x40, 0x00, 0x75, 0x30, 0x39, 0x45, 0x08, 0x75, 0x17, 0xE8, 0xDC, 0x15, 0x00, 0x00, 0xC7, 0x00, 0x16, 0x00, 0x00, 0x00, 0xE8, 0x7F, 0x15, 0x00, 0x00, 0xB8, 0xFF, 0xFF, 0xFF, 0x7F, 0x5D, 0xC3, 0x39, 0x45, 0x0C, 0x74, 0xE4, 0x81, 0x7D, 0x10, 0xFF, 0xFF, 0xFF, 0x7F, 0x77, 0xDB, 0x5D, 0xE9, 0x0A, 0x16, 0x00, 0x00, 0x50, 0xFF, 0x75, 0x10, 0xFF, 0x75, 0x0C, 0xFF, 0x75, 0x08, 0xE8, 0xD0, 0xFE, 0xFF, 0xFF, 0x83, 0xC4, 0x10, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x83, 0xEC, 0x20, 0x53, 0x57, 0x33, 0xDB, 0x6A, 0x07, 0x33, 0xC0, 0x59, 0x8D, 0x7D, 0xE4, 0x89, 0x5D, 0xE0, 0xF3, 0xAB, 0x39, 0x5D, 0x10, 0x75, 0x15, 0xE8, 0x7F, 0x15, 0x00, 0x00, 0xC7, 0x00, 0x16, 0x00, 0x00, 0x00, 0xE8, 0x22, 0x15, 0x00, 0x00, 0x83, 0xC8, 0xFF, 0xEB, 0x75, 0x8B, 0x4D, 0x0C, 0x56, 0x8B, 0x75, 0x08, 0x3B, 0xCB, 0x74, 0x19, 0x3B, 0xF3, 0x75, 0x15, 0xE8, 0x5B, 0x15, 0x00, 0x00, 0xC7, 0x00, 0x16, 0x00, 0x00, 0x00, 0xE8, 0xFE, 0x14, 0x00, 0x00, 0x83, 0xC8, 0xFF, 0xEB, 0x50, 0xB8, 0xFF, 0xFF, 0xFF, 0x7F, 0x89, 0x45, 0xE4, 0x3B, 0xC8, 0x77, 0x03, 0x89, 0x4D, 0xE4, 0x8D, 0x45, 0x14, 0x50, 0x53, 0xFF, 0x75, 0x10, 0x8D, 0x45, 0xE0, 0x50, 0xC7, 0x45, 0xEC, 0x42, 0x00, 0x00, 0x00, 0x89, 0x75, 0xE8, 0x89, 0x75, 0xE0, 0xE8, 0xCC, 0x17, 0x00, 0x00, 0x83, 0xC4, 0x10, 0x8B, 0xF8, 0x3B, 0xF3, 0x74, 0x1A, 0xFF, 0x4D, 0xE4, 0x78, 0x07, 0x8B, 0x45, 0xE0, 0x88, 0x18, 0xEB, 0x0C, 0x8D, 0x45, 0xE0, 0x50, 0x53, 0xE8, 0xB4, 0x15, 0x00, 0x00, 0x59, 0x59, 0x8B, 0xC7, 0x5E, 0x5F, 0x5B, 0xC9, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x83, 0x7D, 0x08, 0x00, 0x74, 0x2D, 0xFF, 0x75, 0x08, 0x6A, 0x00, 0xFF, 0x35, 0x28, 0xE1, 0x40, 0x00, 0xFF, 0x15, 0x38, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x75, 0x18, 0x56, 0xE8, 0xD0, 0x14, 0x00, 0x00, 0x8B, 0xF0, 0xFF, 0x15, 0x34, 0xA0, 0x40, 0x00, 0x50, 0xE8, 0x80, 0x14, 0x00, 0x00, 0x59, 0x89, 0x06, 0x5E, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x53, 0x8B, 0x5D, 0x08, 0x83, 0xFB, 0xE0, 0x77, 0x6F, 0x56, 0x57, 0x83, 0x3D, 0x28, 0xE1, 0x40, 0x00, 0x00, 0x75, 0x18, 0xE8, 0x32, 0x28, 0x00, 0x00, 0x6A, 0x1E, 0xE8, 0x7C, 0x26, 0x00, 0x00, 0x68, 0xFF, 0x00, 0x00, 0x00, 0xE8, 0x8C, 0x23, 0x00, 0x00, 0x59, 0x59, 0x85, 0xDB, 0x74, 0x04, 0x8B, 0xC3, 0xEB, 0x03, 0x33, 0xC0, 0x40, 0x50, 0x6A, 0x00, 0xFF, 0x35, 0x28, 0xE1, 0x40, 0x00, 0xFF, 0x15, 0x3C, 0xA0, 0x40, 0x00, 0x8B, 0xF8, 0x85, 0xFF, 0x75, 0x26, 0x6A, 0x0C, 0x5E, 0x39, 0x05, 0x8C, 0xE7, 0x40, 0x00, 0x74, 0x0D, 0x53, 0xE8, 0x36, 0x28, 0x00, 0x00, 0x59, 0x85, 0xC0, 0x75, 0xA9, 0xEB, 0x07, 0xE8, 0x4E, 0x14, 0x00, 0x00, 0x89, 0x30, 0xE8, 0x47, 0x14, 0x00, 0x00, 0x89, 0x30, 0x8B, 0xC7, 0x5F, 0x5E, 0xEB, 0x14, 0x53, 0xE8, 0x15, 0x28, 0x00, 0x00, 0x59, 0xE8, 0x33, 0x14, 0x00, 0x00, 0xC7, 0x00, 0x0C, 0x00, 0x00, 0x00, 0x33, 0xC0, 0x5B, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x83, 0x3D, 0xC8, 0xDD, 0x40, 0x00, 0x01, 0x75, 0x05, 0xE8, 0xA9, 0x27, 0x00, 0x00, 0xFF, 0x75, 0x08, 0xE8, 0xF2, 0x25, 0x00, 0x00, 0x68, 0xFF, 0x00, 0x00, 0x00,
				0xE8, 0x02, 0x23, 0x00, 0x00, 0x59, 0x59, 0x5D, 0xC3, 0x6A, 0x58, 0x68, 0xE0, 0xC0, 0x40, 0x00, 0xE8, 0x57, 0x30, 0x00, 0x00, 0x8D, 0x45, 0x98, 0x50, 0xFF, 0x15, 0x48, 0xA0, 0x40, 0x00, 0x33, 0xF6, 0x39, 0x35, 0x9C, 0xFB, 0x40, 0x00, 0x75, 0x0B, 0x56, 0x56, 0x6A, 0x01, 0x56, 0xFF, 0x15, 0x44, 0xA0, 0x40, 0x00, 0xB8, 0x4D, 0x5A, 0x00, 0x00, 0x66, 0x39, 0x05, 0x00, 0x00, 0x40, 0x00, 0x74, 0x05, 0x89, 0x75, 0xE4, 0xEB, 0x36, 0xA1, 0x3C, 0x00, 0x40, 0x00, 0x81, 0xB8, 0x00, 0x00, 0x40, 0x00, 0x50, 0x45, 0x00, 0x00, 0x75, 0xEA, 0xB9, 0x0B, 0x01, 0x00, 0x00, 0x66, 0x39, 0x88, 0x18, 0x00, 0x40, 0x00, 0x75, 0xDC, 0x83, 0xB8, 0x74, 0x00, 0x40, 0x00, 0x0E, 0x76, 0xD3, 0x33, 0xC9, 0x39, 0xB0, 0xE8, 0x00, 0x40, 0x00, 0x0F, 0x95, 0xC1, 0x89, 0x4D, 0xE4, 0xE8, 0x3C, 0x22, 0x00, 0x00, 0x85, 0xC0, 0x75, 0x08, 0x6A, 0x1C, 0xE8, 0x53, 0xFF, 0xFF, 0xFF, 0x59, 0xE8, 0x09, 0x0F, 0x00, 0x00, 0x85, 0xC0, 0x75, 0x08, 0x6A, 0x10, 0xE8, 0x42, 0xFF, 0xFF, 0xFF, 0x59, 0xE8, 0x71, 0x2F, 0x00, 0x00, 0x89, 0x75, 0xFC, 0xE8, 0x24, 0x2D, 0x00, 0x00, 0x85, 0xC0, 0x79, 0x08, 0x6A, 0x1B, 0xE8, 0xF2, 0x24, 0x00, 0x00, 0x59, 0xFF, 0x15, 0x40, 0xA0, 0x40, 0x00, 0xA3, 0x98, 0xFB, 0x40, 0x00, 0xE8, 0x71, 0x2C, 0x00, 0x00, 0xA3, 0xC0, 0xDD, 0x40, 0x00, 0xE8, 0xAC, 0x2B, 0x00, 0x00, 0x85, 0xC0, 0x79, 0x08, 0x6A, 0x08, 0xE8, 0xCC, 0x24, 0x00, 0x00, 0x59, 0xE8, 0x25, 0x29, 0x00, 0x00, 0x85, 0xC0, 0x79, 0x08, 0x6A, 0x09, 0xE8, 0xBB, 0x24, 0x00, 0x00, 0x59, 0x6A, 0x01, 0xE8, 0x92, 0x22, 0x00, 0x00, 0x59, 0x3B, 0xC6, 0x74, 0x07, 0x50, 0xE8, 0xA8, 0x24, 0x00, 0x00, 0x59, 0xE8, 0x9F, 0x28, 0x00, 0x00, 0xF6, 0x45, 0xC4, 0x01, 0x74, 0x06, 0x0F, 0xB7, 0x4D, 0xC8, 0xEB, 0x03, 0x6A, 0x0A, 0x59, 0x51, 0x50, 0x56, 0x68, 0x00, 0x00, 0x40, 0x00, 0xE8, 0xE9, 0xF5, 0xFF, 0xFF, 0x89, 0x45, 0xE0, 0x39, 0x75, 0xE4, 0x75, 0x06, 0x50, 0xE8, 0x2E, 0x24, 0x00, 0x00, 0xE8, 0x55, 0x24, 0x00, 0x00, 0xEB, 0x2E, 0x8B, 0x45, 0xEC, 0x8B, 0x08, 0x8B, 0x09, 0x89, 0x4D, 0xDC, 0x50, 0x51, 0xE8, 0x13, 0x27, 0x00, 0x00, 0x59, 0x59, 0xC3, 0x8B, 0x65, 0xE8, 0x8B, 0x45, 0xDC, 0x89, 0x45, 0xE0, 0x83, 0x7D, 0xE4, 0x00, 0x75, 0x06, 0x50, 0xE8, 0x14, 0x24, 0x00, 0x00, 0xE8, 0x34, 0x24, 0x00, 0x00, 0xC7, 0x45, 0xFC, 0xFE, 0xFF, 0xFF, 0xFF, 0x8B, 0x45, 0xE0, 0xE8, 0x3C, 0x2F, 0x00, 0x00, 0xC3, 0xE8, 0xE0, 0x30, 0x00, 0x00, 0xE9, 0x89, 0xFE, 0xFF, 0xFF, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x81, 0xEC, 0x28, 0x03, 0x00, 0x00, 0xA3, 0xD8, 0xDE, 0x40, 0x00, 0x89, 0x0D, 0xD4, 0xDE, 0x40, 0x00, 0x89, 0x15, 0xD0, 0xDE, 0x40, 0x00, 0x89, 0x1D, 0xCC, 0xDE, 0x40, 0x00, 0x89, 0x35, 0xC8, 0xDE, 0x40, 0x00, 0x89, 0x3D, 0xC4, 0xDE, 0x40, 0x00, 0x66, 0x8C, 0x15, 0xF0, 0xDE, 0x40, 0x00, 0x66, 0x8C, 0x0D, 0xE4, 0xDE, 0x40, 0x00, 0x66, 0x8C, 0x1D, 0xC0, 0xDE, 0x40, 0x00, 0x66, 0x8C, 0x05, 0xBC, 0xDE, 0x40, 0x00, 0x66, 0x8C, 0x25, 0xB8, 0xDE, 0x40, 0x00, 0x66, 0x8C, 0x2D, 0xB4, 0xDE, 0x40, 0x00, 0x9C, 0x8F, 0x05, 0xE8, 0xDE, 0x40, 0x00, 0x8B, 0x45, 0x00, 0xA3, 0xDC, 0xDE, 0x40, 0x00, 0x8B, 0x45, 0x04, 0xA3, 0xE0, 0xDE, 0x40, 0x00, 0x8D, 0x45, 0x08, 0xA3, 0xEC,
				0xDE, 0x40, 0x00, 0x8B, 0x85, 0xE0, 0xFC, 0xFF, 0xFF, 0xC7, 0x05, 0x28, 0xDE, 0x40, 0x00, 0x01, 0x00, 0x01, 0x00, 0xA1, 0xE0, 0xDE, 0x40, 0x00, 0xA3, 0xDC, 0xDD, 0x40, 0x00, 0xC7, 0x05, 0xD0, 0xDD, 0x40, 0x00, 0x09, 0x04, 0x00, 0xC0, 0xC7, 0x05, 0xD4, 0xDD, 0x40, 0x00, 0x01, 0x00, 0x00, 0x00, 0xA1, 0x04, 0xD0, 0x40, 0x00, 0x89, 0x85, 0xD8, 0xFC, 0xFF, 0xFF, 0xA1, 0x08, 0xD0, 0x40, 0x00, 0x89, 0x85, 0xDC, 0xFC, 0xFF, 0xFF, 0xFF, 0x15, 0x5C, 0xA0, 0x40, 0x00, 0xA3, 0x20, 0xDE, 0x40, 0x00, 0x6A, 0x01, 0xE8, 0xA9, 0x30, 0x00, 0x00, 0x59, 0x6A, 0x00, 0xFF, 0x15, 0x58, 0xA0, 0x40, 0x00, 0x68, 0x70, 0xA1, 0x40, 0x00, 0xFF, 0x15, 0x54, 0xA0, 0x40, 0x00, 0x83, 0x3D, 0x20, 0xDE, 0x40, 0x00, 0x00, 0x75, 0x08, 0x6A, 0x01, 0xE8, 0x85, 0x30, 0x00, 0x00, 0x59, 0x68, 0x09, 0x04, 0x00, 0xC0, 0xFF, 0x15, 0x50, 0xA0, 0x40, 0x00, 0x50, 0xFF, 0x15, 0x4C, 0xA0, 0x40, 0x00, 0xC9, 0xC3, 0x2D, 0xA4, 0x03, 0x00, 0x00, 0x74, 0x22, 0x83, 0xE8, 0x04, 0x74, 0x17, 0x83, 0xE8, 0x0D, 0x74, 0x0C, 0x48, 0x74, 0x03, 0x33, 0xC0, 0xC3, 0xB8, 0x04, 0x04, 0x00, 0x00, 0xC3, 0xB8, 0x12, 0x04, 0x00, 0x00, 0xC3, 0xB8, 0x04, 0x08, 0x00, 0x00, 0xC3, 0xB8, 0x11, 0x04, 0x00, 0x00, 0xC3, 0x8B, 0xFF, 0x56, 0x57, 0x8B, 0xF0, 0x68, 0x01, 0x01, 0x00, 0x00, 0x33, 0xFF, 0x8D, 0x46, 0x1C, 0x57, 0x50, 0xE8, 0x40, 0x30, 0x00, 0x00, 0x33, 0xC0, 0x0F, 0xB7, 0xC8, 0x8B, 0xC1, 0x89, 0x7E, 0x04, 0x89, 0x7E, 0x08, 0x89, 0x7E, 0x0C, 0xC1, 0xE1, 0x10, 0x0B, 0xC1, 0x8D, 0x7E, 0x10, 0xAB, 0xAB, 0xAB, 0xB9, 0x10, 0xD0, 0x40, 0x00, 0x83, 0xC4, 0x0C, 0x8D, 0x46, 0x1C, 0x2B, 0xCE, 0xBF, 0x01, 0x01, 0x00, 0x00, 0x8A, 0x14, 0x01, 0x88, 0x10, 0x40, 0x4F, 0x75, 0xF7, 0x8D, 0x86, 0x1D, 0x01, 0x00, 0x00, 0xBE, 0x00, 0x01, 0x00, 0x00, 0x8A, 0x14, 0x08, 0x88, 0x10, 0x40, 0x4E, 0x75, 0xF7, 0x5F, 0x5E, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x81, 0xEC, 0x1C, 0x05, 0x00, 0x00, 0xA1, 0x04, 0xD0, 0x40, 0x00, 0x33, 0xC5, 0x89, 0x45, 0xFC, 0x53, 0x57, 0x8D, 0x85, 0xE8, 0xFA, 0xFF, 0xFF, 0x50, 0xFF, 0x76, 0x04, 0xFF, 0x15, 0x60, 0xA0, 0x40, 0x00, 0xBF, 0x00, 0x01, 0x00, 0x00, 0x85, 0xC0, 0x0F, 0x84, 0xFC, 0x00, 0x00, 0x00, 0x33, 0xC0, 0x88, 0x84, 0x05, 0xFC, 0xFE, 0xFF, 0xFF, 0x40, 0x3B, 0xC7, 0x72, 0xF4, 0x8A, 0x85, 0xEE, 0xFA, 0xFF, 0xFF, 0xC6, 0x85, 0xFC, 0xFE, 0xFF, 0xFF, 0x20, 0x84, 0xC0, 0x74, 0x30, 0x8D, 0x9D, 0xEF, 0xFA, 0xFF, 0xFF, 0x0F, 0xB6, 0xC8, 0x0F, 0xB6, 0x03, 0x3B, 0xC8, 0x77, 0x16, 0x2B, 0xC1, 0x40, 0x50, 0x8D, 0x94, 0x0D, 0xFC, 0xFE, 0xFF, 0xFF, 0x6A, 0x20, 0x52, 0xE8, 0x7D, 0x2F, 0x00, 0x00, 0x83, 0xC4, 0x0C, 0x8A, 0x43, 0x01, 0x83, 0xC3, 0x02, 0x84, 0xC0, 0x75, 0xD6, 0x6A, 0x00, 0xFF, 0x76, 0x0C, 0x8D, 0x85, 0xFC, 0xFA, 0xFF, 0xFF, 0xFF, 0x76, 0x04, 0x50, 0x57, 0x8D, 0x85, 0xFC, 0xFE, 0xFF, 0xFF, 0x50, 0x6A, 0x01, 0x6A, 0x00, 0xE8, 0xFE, 0x32, 0x00, 0x00, 0x33, 0xDB, 0x53, 0xFF, 0x76, 0x04, 0x8D, 0x85, 0xFC, 0xFD, 0xFF, 0xFF, 0x57, 0x50, 0x57, 0x8D, 0x85, 0xFC, 0xFE, 0xFF, 0xFF, 0x50, 0x57, 0xFF, 0x76, 0x0C, 0x53, 0xE8, 0xB1, 0x31, 0x00, 0x00, 0x83, 0xC4, 0x44, 0x53, 0xFF, 0x76, 0x04, 0x8D, 0x85, 0xFC, 0xFC, 0xFF,
				0xFF, 0x57, 0x50, 0x57, 0x8D, 0x85, 0xFC, 0xFE, 0xFF, 0xFF, 0x50, 0x68, 0x00, 0x02, 0x00, 0x00, 0xFF, 0x76, 0x0C, 0x53, 0xE8, 0x8C, 0x31, 0x00, 0x00, 0x83, 0xC4, 0x24, 0x33, 0xC0, 0x0F, 0xB7, 0x8C, 0x45, 0xFC, 0xFA, 0xFF, 0xFF, 0xF6, 0xC1, 0x01, 0x74, 0x0E, 0x80, 0x4C, 0x06, 0x1D, 0x10, 0x8A, 0x8C, 0x05, 0xFC, 0xFD, 0xFF, 0xFF, 0xEB, 0x11, 0xF6, 0xC1, 0x02, 0x74, 0x15, 0x80, 0x4C, 0x06, 0x1D, 0x20, 0x8A, 0x8C, 0x05, 0xFC, 0xFC, 0xFF, 0xFF, 0x88, 0x8C, 0x06, 0x1D, 0x01, 0x00, 0x00, 0xEB, 0x07, 0x88, 0x9C, 0x06, 0x1D, 0x01, 0x00, 0x00, 0x40, 0x3B, 0xC7, 0x72, 0xBF, 0xEB, 0x52, 0x8D, 0x86, 0x1D, 0x01, 0x00, 0x00, 0xC7, 0x85, 0xE4, 0xFA, 0xFF, 0xFF, 0x9F, 0xFF, 0xFF, 0xFF, 0x33, 0xC9, 0x29, 0x85, 0xE4, 0xFA, 0xFF, 0xFF, 0x8B, 0x95, 0xE4, 0xFA, 0xFF, 0xFF, 0x8D, 0x84, 0x0E, 0x1D, 0x01, 0x00, 0x00, 0x03, 0xD0, 0x8D, 0x5A, 0x20, 0x83, 0xFB, 0x19, 0x77, 0x0A, 0x80, 0x4C, 0x0E, 0x1D, 0x10, 0x8D, 0x51, 0x20, 0xEB, 0x0D, 0x83, 0xFA, 0x19, 0x77, 0x0C, 0x80, 0x4C, 0x0E, 0x1D, 0x20, 0x8D, 0x51, 0xE0, 0x88, 0x10, 0xEB, 0x03, 0xC6, 0x00, 0x00, 0x41, 0x3B, 0xCF, 0x72, 0xC6, 0x8B, 0x4D, 0xFC, 0x5F, 0x33, 0xCD, 0x5B, 0xE8, 0xB4, 0xF6, 0xFF, 0xFF, 0xC9, 0xC3, 0x6A, 0x0C, 0x68, 0x00, 0xC1, 0x40, 0x00, 0xE8, 0xB7, 0x2B, 0x00, 0x00, 0xE8, 0x99, 0x09, 0x00, 0x00, 0x8B, 0xF8, 0xA1, 0x30, 0xD5, 0x40, 0x00, 0x85, 0x47, 0x70, 0x74, 0x1D, 0x83, 0x7F, 0x6C, 0x00, 0x74, 0x17, 0x8B, 0x77, 0x68, 0x85, 0xF6, 0x75, 0x08, 0x6A, 0x20, 0xE8, 0xCF, 0x20, 0x00, 0x00, 0x59, 0x8B, 0xC6, 0xE8, 0xCF, 0x2B, 0x00, 0x00, 0xC3, 0x6A, 0x0D, 0xE8, 0x8A, 0x33, 0x00, 0x00, 0x59, 0x83, 0x65, 0xFC, 0x00, 0x8B, 0x77, 0x68, 0x89, 0x75, 0xE4, 0x3B, 0x35, 0x38, 0xD4, 0x40, 0x00, 0x74, 0x36, 0x85, 0xF6, 0x74, 0x1A, 0x56, 0xFF, 0x15, 0x68, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x75, 0x0F, 0x81, 0xFE, 0x10, 0xD0, 0x40, 0x00, 0x74, 0x07, 0x56, 0xE8, 0xF8, 0xF9, 0xFF, 0xFF, 0x59, 0xA1, 0x38, 0xD4, 0x40, 0x00, 0x89, 0x47, 0x68, 0x8B, 0x35, 0x38, 0xD4, 0x40, 0x00, 0x89, 0x75, 0xE4, 0x56, 0xFF, 0x15, 0x64, 0xA0, 0x40, 0x00, 0xC7, 0x45, 0xFC, 0xFE, 0xFF, 0xFF, 0xFF, 0xE8, 0x05, 0x00, 0x00, 0x00, 0xEB, 0x8E, 0x8B, 0x75, 0xE4, 0x6A, 0x0D, 0xE8, 0x50, 0x32, 0x00, 0x00, 0x59, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x83, 0xEC, 0x10, 0x53, 0x33, 0xDB, 0x53, 0x8D, 0x4D, 0xF0, 0xE8, 0x09, 0xF6, 0xFF, 0xFF, 0x89, 0x1D, 0xF4, 0xE0, 0x40, 0x00, 0x83, 0xFE, 0xFE, 0x75, 0x1E, 0xC7, 0x05, 0xF4, 0xE0, 0x40, 0x00, 0x01, 0x00, 0x00, 0x00, 0xFF, 0x15, 0x70, 0xA0, 0x40, 0x00, 0x38, 0x5D, 0xFC, 0x74, 0x45, 0x8B, 0x4D, 0xF8, 0x83, 0x61, 0x70, 0xFD, 0xEB, 0x3C, 0x83, 0xFE, 0xFD, 0x75, 0x12, 0xC7, 0x05, 0xF4, 0xE0, 0x40, 0x00, 0x01, 0x00, 0x00, 0x00, 0xFF, 0x15, 0x6C, 0xA0, 0x40, 0x00, 0xEB, 0xDB, 0x83, 0xFE, 0xFC, 0x75, 0x12, 0x8B, 0x45, 0xF0, 0x8B, 0x40, 0x04, 0xC7, 0x05, 0xF4, 0xE0, 0x40, 0x00, 0x01, 0x00, 0x00, 0x00, 0xEB, 0xC4, 0x38, 0x5D, 0xFC, 0x74, 0x07, 0x8B, 0x45, 0xF8, 0x83, 0x60, 0x70, 0xFD, 0x8B, 0xC6, 0x5B, 0xC9, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x83, 0xEC, 0x20, 0xA1, 0x04, 0xD0, 0x40, 0x00, 0x33, 0xC5, 0x89, 0x45, 0xFC, 0x53,
				0x8B, 0x5D, 0x0C, 0x56, 0x8B, 0x75, 0x08, 0x57, 0xE8, 0x64, 0xFF, 0xFF, 0xFF, 0x8B, 0xF8, 0x33, 0xF6, 0x89, 0x7D, 0x08, 0x3B, 0xFE, 0x75, 0x0E, 0x8B, 0xC3, 0xE8, 0xBA, 0xFC, 0xFF, 0xFF, 0x33, 0xC0, 0xE9, 0xA1, 0x01, 0x00, 0x00, 0x89, 0x75, 0xE4, 0x33, 0xC0, 0x39, 0xB8, 0x40, 0xD4, 0x40, 0x00, 0x0F, 0x84, 0x91, 0x00, 0x00, 0x00, 0xFF, 0x45, 0xE4, 0x83, 0xC0, 0x30, 0x3D, 0xF0, 0x00, 0x00, 0x00, 0x72, 0xE7, 0x81, 0xFF, 0xE8, 0xFD, 0x00, 0x00, 0x0F, 0x84, 0x74, 0x01, 0x00, 0x00, 0x81, 0xFF, 0xE9, 0xFD, 0x00, 0x00, 0x0F, 0x84, 0x68, 0x01, 0x00, 0x00, 0x0F, 0xB7, 0xC7, 0x50, 0xFF, 0x15, 0x74, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x0F, 0x84, 0x56, 0x01, 0x00, 0x00, 0x8D, 0x45, 0xE8, 0x50, 0x57, 0xFF, 0x15, 0x60, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x0F, 0x84, 0x37, 0x01, 0x00, 0x00, 0x68, 0x01, 0x01, 0x00, 0x00, 0x8D, 0x43, 0x1C, 0x56, 0x50, 0xE8, 0xA0, 0x2C, 0x00, 0x00, 0x33, 0xD2, 0x42, 0x83, 0xC4, 0x0C, 0x89, 0x7B, 0x04, 0x89, 0x73, 0x0C, 0x39, 0x55, 0xE8, 0x0F, 0x86, 0xFC, 0x00, 0x00, 0x00, 0x80, 0x7D, 0xEE, 0x00, 0x0F, 0x84, 0xD3, 0x00, 0x00, 0x00, 0x8D, 0x75, 0xEF, 0x8A, 0x0E, 0x84, 0xC9, 0x0F, 0x84, 0xC6, 0x00, 0x00, 0x00, 0x0F, 0xB6, 0x46, 0xFF, 0x0F, 0xB6, 0xC9, 0xE9, 0xA9, 0x00, 0x00, 0x00, 0x68, 0x01, 0x01, 0x00, 0x00, 0x8D, 0x43, 0x1C, 0x56, 0x50, 0xE8, 0x59, 0x2C, 0x00, 0x00, 0x8B, 0x4D, 0xE4, 0x83, 0xC4, 0x0C, 0x6B, 0xC9, 0x30, 0x89, 0x75, 0xE0, 0x8D, 0xB1, 0x50, 0xD4, 0x40, 0x00, 0x89, 0x75, 0xE4, 0xEB, 0x2B, 0x8A, 0x46, 0x01, 0x84, 0xC0, 0x74, 0x29, 0x0F, 0xB6, 0x3E, 0x0F, 0xB6, 0xC0, 0xEB, 0x12, 0x8B, 0x45, 0xE0, 0x8A, 0x80, 0x3C, 0xD4, 0x40, 0x00, 0x08, 0x44, 0x3B, 0x1D, 0x0F, 0xB6, 0x46, 0x01, 0x47, 0x3B, 0xF8, 0x76, 0xEA, 0x8B, 0x7D, 0x08, 0x83, 0xC6, 0x02, 0x80, 0x3E, 0x00, 0x75, 0xD0, 0x8B, 0x75, 0xE4, 0xFF, 0x45, 0xE0, 0x83, 0xC6, 0x08, 0x83, 0x7D, 0xE0, 0x04, 0x89, 0x75, 0xE4, 0x72, 0xE9, 0x8B, 0xC7, 0x89, 0x7B, 0x04, 0xC7, 0x43, 0x08, 0x01, 0x00, 0x00, 0x00, 0xE8, 0x69, 0xFB, 0xFF, 0xFF, 0x6A, 0x06, 0x89, 0x43, 0x0C, 0x8D, 0x43, 0x10, 0x8D, 0x89, 0x44, 0xD4, 0x40, 0x00, 0x5A, 0x66, 0x8B, 0x31, 0x66, 0x89, 0x30, 0x83, 0xC1, 0x02, 0x83, 0xC0, 0x02, 0x4A, 0x75, 0xF1, 0x8B, 0xF3, 0xE8, 0xD7, 0xFB, 0xFF, 0xFF, 0xE9, 0xB4, 0xFE, 0xFF, 0xFF, 0x80, 0x4C, 0x03, 0x1D, 0x04, 0x40, 0x3B, 0xC1, 0x76, 0xF6, 0x83, 0xC6, 0x02, 0x80, 0x7E, 0xFF, 0x00, 0x0F, 0x85, 0x30, 0xFF, 0xFF, 0xFF, 0x8D, 0x43, 0x1E, 0xB9, 0xFE, 0x00, 0x00, 0x00, 0x80, 0x08, 0x08, 0x40, 0x49, 0x75, 0xF9, 0x8B, 0x43, 0x04, 0xE8, 0x11, 0xFB, 0xFF, 0xFF, 0x89, 0x43, 0x0C, 0x89, 0x53, 0x08, 0xEB, 0x03, 0x89, 0x73, 0x08, 0x33, 0xC0, 0x0F, 0xB7, 0xC8, 0x8B, 0xC1, 0xC1, 0xE1, 0x10, 0x0B, 0xC1, 0x8D, 0x7B, 0x10, 0xAB, 0xAB, 0xAB, 0xEB, 0xA7, 0x39, 0x35, 0xF4, 0xE0, 0x40, 0x00, 0x0F, 0x85, 0x54, 0xFE, 0xFF, 0xFF, 0x83, 0xC8, 0xFF, 0x8B, 0x4D, 0xFC, 0x5F, 0x5E, 0x33, 0xCD, 0x5B, 0xE8, 0xAB, 0xF3, 0xFF, 0xFF, 0xC9, 0xC3, 0x6A, 0x14, 0x68, 0x20, 0xC1, 0x40, 0x00, 0xE8, 0xAE, 0x28, 0x00, 0x00, 0x83, 0x4D, 0xE0, 0xFF, 0xE8, 0x8C, 0x06, 0x00, 0x00, 0x8B, 0xF8, 0x89, 0x7D, 0xDC, 0xE8, 0xD8, 0xFC, 0xFF,
				0xFF, 0x8B, 0x5F, 0x68, 0x8B, 0x75, 0x08, 0xE8, 0x71, 0xFD, 0xFF, 0xFF, 0x89, 0x45, 0x08, 0x3B, 0x43, 0x04, 0x0F, 0x84, 0x57, 0x01, 0x00, 0x00, 0x68, 0x20, 0x02, 0x00, 0x00, 0xE8, 0xB5, 0x30, 0x00, 0x00, 0x59, 0x8B, 0xD8, 0x85, 0xDB, 0x0F, 0x84, 0x46, 0x01, 0x00, 0x00, 0xB9, 0x88, 0x00, 0x00, 0x00, 0x8B, 0x77, 0x68, 0x8B, 0xFB, 0xF3, 0xA5, 0x83, 0x23, 0x00, 0x53, 0xFF, 0x75, 0x08, 0xE8, 0xB4, 0xFD, 0xFF, 0xFF, 0x59, 0x59, 0x89, 0x45, 0xE0, 0x85, 0xC0, 0x0F, 0x85, 0xFC, 0x00, 0x00, 0x00, 0x8B, 0x75, 0xDC, 0xFF, 0x76, 0x68, 0xFF, 0x15, 0x68, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x75, 0x11, 0x8B, 0x46, 0x68, 0x3D, 0x10, 0xD0, 0x40, 0x00, 0x74, 0x07, 0x50, 0xE8, 0xD0, 0xF6, 0xFF, 0xFF, 0x59, 0x89, 0x5E, 0x68, 0x53, 0x8B, 0x3D, 0x64, 0xA0, 0x40, 0x00, 0xFF, 0xD7, 0xF6, 0x46, 0x70, 0x02, 0x0F, 0x85, 0xEA, 0x00, 0x00, 0x00, 0xF6, 0x05, 0x30, 0xD5, 0x40, 0x00, 0x01, 0x0F, 0x85, 0xDD, 0x00, 0x00, 0x00, 0x6A, 0x0D, 0xE8, 0x07, 0x30, 0x00, 0x00, 0x59, 0x83, 0x65, 0xFC, 0x00, 0x8B, 0x43, 0x04, 0xA3, 0x04, 0xE1, 0x40, 0x00, 0x8B, 0x43, 0x08, 0xA3, 0x08, 0xE1, 0x40, 0x00, 0x8B, 0x43, 0x0C, 0xA3, 0x0C, 0xE1, 0x40, 0x00, 0x33, 0xC0, 0x89, 0x45, 0xE4, 0x83, 0xF8, 0x05, 0x7D, 0x10, 0x66, 0x8B, 0x4C, 0x43, 0x10, 0x66, 0x89, 0x0C, 0x45, 0xF8, 0xE0, 0x40, 0x00, 0x40, 0xEB, 0xE8, 0x33, 0xC0, 0x89, 0x45, 0xE4, 0x3D, 0x01, 0x01, 0x00, 0x00, 0x7D, 0x0D, 0x8A, 0x4C, 0x18, 0x1C, 0x88, 0x88, 0x30, 0xD2, 0x40, 0x00, 0x40, 0xEB, 0xE9, 0x33, 0xC0, 0x89, 0x45, 0xE4, 0x3D, 0x00, 0x01, 0x00, 0x00, 0x7D, 0x10, 0x8A, 0x8C, 0x18, 0x1D, 0x01, 0x00, 0x00, 0x88, 0x88, 0x38, 0xD3, 0x40, 0x00, 0x40, 0xEB, 0xE6, 0xFF, 0x35, 0x38, 0xD4, 0x40, 0x00, 0xFF, 0x15, 0x68, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x75, 0x13, 0xA1, 0x38, 0xD4, 0x40, 0x00, 0x3D, 0x10, 0xD0, 0x40, 0x00, 0x74, 0x07, 0x50, 0xE8, 0x17, 0xF6, 0xFF, 0xFF, 0x59, 0x89, 0x1D, 0x38, 0xD4, 0x40, 0x00, 0x53, 0xFF, 0xD7, 0xC7, 0x45, 0xFC, 0xFE, 0xFF, 0xFF, 0xFF, 0xE8, 0x02, 0x00, 0x00, 0x00, 0xEB, 0x30, 0x6A, 0x0D, 0xE8, 0x81, 0x2E, 0x00, 0x00, 0x59, 0xC3, 0xEB, 0x25, 0x83, 0xF8, 0xFF, 0x75, 0x20, 0x81, 0xFB, 0x10, 0xD0, 0x40, 0x00, 0x74, 0x07, 0x53, 0xE8, 0xE1, 0xF5, 0xFF, 0xFF, 0x59, 0xE8, 0xD1, 0x0A, 0x00, 0x00, 0xC7, 0x00, 0x16, 0x00, 0x00, 0x00, 0xEB, 0x04, 0x83, 0x65, 0xE0, 0x00, 0x8B, 0x45, 0xE0, 0xE8, 0x66, 0x27, 0x00, 0x00, 0xC3, 0x83, 0x3D, 0x8C, 0xFB, 0x40, 0x00, 0x00, 0x75, 0x12, 0x6A, 0xFD, 0xE8, 0x56, 0xFE, 0xFF, 0xFF, 0x59, 0xC7, 0x05, 0x8C, 0xFB, 0x40, 0x00, 0x01, 0x00, 0x00, 0x00, 0x33, 0xC0, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x53, 0x56, 0x8B, 0x35, 0x64, 0xA0, 0x40, 0x00, 0x57, 0x8B, 0x7D, 0x08, 0x57, 0xFF, 0xD6, 0x8B, 0x87, 0xB0, 0x00, 0x00, 0x00, 0x85, 0xC0, 0x74, 0x03, 0x50, 0xFF, 0xD6, 0x8B, 0x87, 0xB8, 0x00, 0x00, 0x00, 0x85, 0xC0, 0x74, 0x03, 0x50, 0xFF, 0xD6, 0x8B, 0x87, 0xB4, 0x00, 0x00, 0x00, 0x85, 0xC0, 0x74, 0x03, 0x50, 0xFF, 0xD6, 0x8B, 0x87, 0xC0, 0x00, 0x00, 0x00, 0x85, 0xC0, 0x74, 0x03, 0x50, 0xFF, 0xD6, 0x8D, 0x5F, 0x50, 0xC7, 0x45, 0x08, 0x06, 0x00, 0x00, 0x00, 0x81, 0x7B, 0xF8, 0x34, 0xD5, 0x40, 0x00, 0x74,
				0x09, 0x8B, 0x03, 0x85, 0xC0, 0x74, 0x03, 0x50, 0xFF, 0xD6, 0x83, 0x7B, 0xFC, 0x00, 0x74, 0x0A, 0x8B, 0x43, 0x04, 0x85, 0xC0, 0x74, 0x03, 0x50, 0xFF, 0xD6, 0x83, 0xC3, 0x10, 0xFF, 0x4D, 0x08, 0x75, 0xD6, 0x8B, 0x87, 0xD4, 0x00, 0x00, 0x00, 0x05, 0xB4, 0x00, 0x00, 0x00, 0x50, 0xFF, 0xD6, 0x5F, 0x5E, 0x5B, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x57, 0x8B, 0x7D, 0x08, 0x85, 0xFF, 0x0F, 0x84, 0x83, 0x00, 0x00, 0x00, 0x53, 0x56, 0x8B, 0x35, 0x68, 0xA0, 0x40, 0x00, 0x57, 0xFF, 0xD6, 0x8B, 0x87, 0xB0, 0x00, 0x00, 0x00, 0x85, 0xC0, 0x74, 0x03, 0x50, 0xFF, 0xD6, 0x8B, 0x87, 0xB8, 0x00, 0x00, 0x00, 0x85, 0xC0, 0x74, 0x03, 0x50, 0xFF, 0xD6, 0x8B, 0x87, 0xB4, 0x00, 0x00, 0x00, 0x85, 0xC0, 0x74, 0x03, 0x50, 0xFF, 0xD6, 0x8B, 0x87, 0xC0, 0x00, 0x00, 0x00, 0x85, 0xC0, 0x74, 0x03, 0x50, 0xFF, 0xD6, 0x8D, 0x5F, 0x50, 0xC7, 0x45, 0x08, 0x06, 0x00, 0x00, 0x00, 0x81, 0x7B, 0xF8, 0x34, 0xD5, 0x40, 0x00, 0x74, 0x09, 0x8B, 0x03, 0x85, 0xC0, 0x74, 0x03, 0x50, 0xFF, 0xD6, 0x83, 0x7B, 0xFC, 0x00, 0x74, 0x0A, 0x8B, 0x43, 0x04, 0x85, 0xC0, 0x74, 0x03, 0x50, 0xFF, 0xD6, 0x83, 0xC3, 0x10, 0xFF, 0x4D, 0x08, 0x75, 0xD6, 0x8B, 0x87, 0xD4, 0x00, 0x00, 0x00, 0x05, 0xB4, 0x00, 0x00, 0x00, 0x50, 0xFF, 0xD6, 0x5E, 0x5B, 0x8B, 0xC7, 0x5F, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x53, 0x56, 0x8B, 0x75, 0x08, 0x8B, 0x86, 0xBC, 0x00, 0x00, 0x00, 0x33, 0xDB, 0x57, 0x3B, 0xC3, 0x74, 0x6F, 0x3D, 0x78, 0xDA, 0x40, 0x00, 0x74, 0x68, 0x8B, 0x86, 0xB0, 0x00, 0x00, 0x00, 0x3B, 0xC3, 0x74, 0x5E, 0x39, 0x18, 0x75, 0x5A, 0x8B, 0x86, 0xB8, 0x00, 0x00, 0x00, 0x3B, 0xC3, 0x74, 0x17, 0x39, 0x18, 0x75, 0x13, 0x50, 0xE8, 0x40, 0xF4, 0xFF, 0xFF, 0xFF, 0xB6, 0xBC, 0x00, 0x00, 0x00, 0xE8, 0x89, 0x32, 0x00, 0x00, 0x59, 0x59, 0x8B, 0x86, 0xB4, 0x00, 0x00, 0x00, 0x3B, 0xC3, 0x74, 0x17, 0x39, 0x18, 0x75, 0x13, 0x50, 0xE8, 0x1F, 0xF4, 0xFF, 0xFF, 0xFF, 0xB6, 0xBC, 0x00, 0x00, 0x00, 0xE8, 0xFF, 0x31, 0x00, 0x00, 0x59, 0x59, 0xFF, 0xB6, 0xB0, 0x00, 0x00, 0x00, 0xE8, 0x07, 0xF4, 0xFF, 0xFF, 0xFF, 0xB6, 0xBC, 0x00, 0x00, 0x00, 0xE8, 0xFC, 0xF3, 0xFF, 0xFF, 0x59, 0x59, 0x8B, 0x86, 0xC0, 0x00, 0x00, 0x00, 0x3B, 0xC3, 0x74, 0x44, 0x39, 0x18, 0x75, 0x40, 0x8B, 0x86, 0xC4, 0x00, 0x00, 0x00, 0x2D, 0xFE, 0x00, 0x00, 0x00, 0x50, 0xE8, 0xDB, 0xF3, 0xFF, 0xFF, 0x8B, 0x86, 0xCC, 0x00, 0x00, 0x00, 0xBF, 0x80, 0x00, 0x00, 0x00, 0x2B, 0xC7, 0x50, 0xE8, 0xC8, 0xF3, 0xFF, 0xFF, 0x8B, 0x86, 0xD0, 0x00, 0x00, 0x00, 0x2B, 0xC7, 0x50, 0xE8, 0xBA, 0xF3, 0xFF, 0xFF, 0xFF, 0xB6, 0xC0, 0x00, 0x00, 0x00, 0xE8, 0xAF, 0xF3, 0xFF, 0xFF, 0x83, 0xC4, 0x10, 0x8B, 0x86, 0xD4, 0x00, 0x00, 0x00, 0x3D, 0x38, 0xD5, 0x40, 0x00, 0x74, 0x1B, 0x39, 0x98, 0xB4, 0x00, 0x00, 0x00, 0x75, 0x13, 0x50, 0xE8, 0x05, 0x2E, 0x00, 0x00, 0xFF, 0xB6, 0xD4, 0x00, 0x00, 0x00, 0xE8, 0x86, 0xF3, 0xFF, 0xFF, 0x59, 0x59, 0x8D, 0x7E, 0x50, 0xC7, 0x45, 0x08, 0x06, 0x00, 0x00, 0x00, 0x81, 0x7F, 0xF8, 0x34, 0xD5, 0x40, 0x00, 0x74, 0x11, 0x8B, 0x07, 0x3B, 0xC3, 0x74, 0x0B, 0x39, 0x18, 0x75, 0x07, 0x50, 0xE8, 0x61, 0xF3, 0xFF, 0xFF, 0x59, 0x39, 0x5F, 0xFC, 0x74, 0x12, 0x8B,
				0x47, 0x04, 0x3B, 0xC3, 0x74, 0x0B, 0x39, 0x18, 0x75, 0x07, 0x50, 0xE8, 0x4A, 0xF3, 0xFF, 0xFF, 0x59, 0x83, 0xC7, 0x10, 0xFF, 0x4D, 0x08, 0x75, 0xC7, 0x56, 0xE8, 0x3B, 0xF3, 0xFF, 0xFF, 0x59, 0x5F, 0x5E, 0x5B, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x57, 0x8B, 0x7D, 0x0C, 0x85, 0xFF, 0x74, 0x3B, 0x8B, 0x45, 0x08, 0x85, 0xC0, 0x74, 0x34, 0x56, 0x8B, 0x30, 0x3B, 0xF7, 0x74, 0x28, 0x57, 0x89, 0x38, 0xE8, 0x6A, 0xFD, 0xFF, 0xFF, 0x59, 0x85, 0xF6, 0x74, 0x1B, 0x56, 0xE8, 0xEE, 0xFD, 0xFF, 0xFF, 0x83, 0x3E, 0x00, 0x59, 0x75, 0x0F, 0x81, 0xFE, 0xA0, 0xD6, 0x40, 0x00, 0x74, 0x07, 0x56, 0xE8, 0x73, 0xFE, 0xFF, 0xFF, 0x59, 0x8B, 0xC7, 0x5E, 0xEB, 0x02, 0x33, 0xC0, 0x5F, 0x5D, 0xC3, 0x6A, 0x0C, 0x68, 0x40, 0xC1, 0x40, 0x00, 0xE8, 0x36, 0x24, 0x00, 0x00, 0xE8, 0x18, 0x02, 0x00, 0x00, 0x8B, 0xF0, 0xA1, 0x30, 0xD5, 0x40, 0x00, 0x85, 0x46, 0x70, 0x74, 0x22, 0x83, 0x7E, 0x6C, 0x00, 0x74, 0x1C, 0xE8, 0x01, 0x02, 0x00, 0x00, 0x8B, 0x70, 0x6C, 0x85, 0xF6, 0x75, 0x08, 0x6A, 0x20, 0xE8, 0x49, 0x19, 0x00, 0x00, 0x59, 0x8B, 0xC6, 0xE8, 0x49, 0x24, 0x00, 0x00, 0xC3, 0x6A, 0x0C, 0xE8, 0x04, 0x2C, 0x00, 0x00, 0x59, 0x83, 0x65, 0xFC, 0x00, 0xFF, 0x35, 0x78, 0xD7, 0x40, 0x00, 0x83, 0xC6, 0x6C, 0x56, 0xE8, 0x59, 0xFF, 0xFF, 0xFF, 0x59, 0x59, 0x89, 0x45, 0xE4, 0xC7, 0x45, 0xFC, 0xFE, 0xFF, 0xFF, 0xFF, 0xE8, 0x02, 0x00, 0x00, 0x00, 0xEB, 0xBE, 0x6A, 0x0C, 0xE8, 0xFD, 0x2A, 0x00, 0x00, 0x59, 0x8B, 0x75, 0xE4, 0xC3, 0x6A, 0x00, 0xFF, 0x15, 0x78, 0xA0, 0x40, 0x00, 0xC3, 0xFF, 0x15, 0x7C, 0xA0, 0x40, 0x00, 0xC2, 0x04, 0x00, 0x8B, 0xFF, 0x56, 0xFF, 0x35, 0x80, 0xD7, 0x40, 0x00, 0xFF, 0x15, 0x80, 0xA0, 0x40, 0x00, 0x8B, 0xF0, 0x85, 0xF6, 0x75, 0x1B, 0xFF, 0x35, 0x14, 0xE1, 0x40, 0x00, 0xFF, 0x15, 0x88, 0xA0, 0x40, 0x00, 0x8B, 0xF0, 0x56, 0xFF, 0x35, 0x80, 0xD7, 0x40, 0x00, 0xFF, 0x15, 0x84, 0xA0, 0x40, 0x00, 0x8B, 0xC6, 0x5E, 0xC3, 0xA1, 0x7C, 0xD7, 0x40, 0x00, 0x83, 0xF8, 0xFF, 0x74, 0x16, 0x50, 0xFF, 0x35, 0x1C, 0xE1, 0x40, 0x00, 0xFF, 0x15, 0x88, 0xA0, 0x40, 0x00, 0xFF, 0xD0, 0x83, 0x0D, 0x7C, 0xD7, 0x40, 0x00, 0xFF, 0xA1, 0x80, 0xD7, 0x40, 0x00, 0x83, 0xF8, 0xFF, 0x74, 0x0E, 0x50, 0xFF, 0x15, 0x8C, 0xA0, 0x40, 0x00, 0x83, 0x0D, 0x80, 0xD7, 0x40, 0x00, 0xFF, 0xE9, 0x1E, 0x2A, 0x00, 0x00, 0x6A, 0x08, 0x68, 0x60, 0xC1, 0x40, 0x00, 0xE8, 0x3A, 0x23, 0x00, 0x00, 0x68, 0xC8, 0xA4, 0x40, 0x00, 0xFF, 0x15, 0x90, 0xA0, 0x40, 0x00, 0x8B, 0x75, 0x08, 0xC7, 0x46, 0x5C, 0x20, 0xB0, 0x40, 0x00, 0x83, 0x66, 0x08, 0x00, 0x33, 0xFF, 0x47, 0x89, 0x7E, 0x14, 0x89, 0x7E, 0x70, 0xC6, 0x86, 0xC8, 0x00, 0x00, 0x00, 0x43, 0xC6, 0x86, 0x4B, 0x01, 0x00, 0x00, 0x43, 0xC7, 0x46, 0x68, 0x10, 0xD0, 0x40, 0x00, 0x6A, 0x0D, 0xE8, 0x04, 0x2B, 0x00, 0x00, 0x59, 0x83, 0x65, 0xFC, 0x00, 0xFF, 0x76, 0x68, 0xFF, 0x15, 0x64, 0xA0, 0x40, 0x00, 0xC7, 0x45, 0xFC, 0xFE, 0xFF, 0xFF, 0xFF, 0xE8, 0x3E, 0x00, 0x00, 0x00, 0x6A, 0x0C, 0xE8, 0xE3, 0x2A, 0x00, 0x00, 0x59, 0x89, 0x7D, 0xFC, 0x8B, 0x45, 0x0C, 0x89, 0x46, 0x6C, 0x85, 0xC0, 0x75, 0x08, 0xA1, 0x78, 0xD7, 0x40, 0x00, 0x89, 0x46, 0x6C, 0xFF, 0x76, 0x6C, 0xE8, 0xBB,
				0xFB, 0xFF, 0xFF, 0x59, 0xC7, 0x45, 0xFC, 0xFE, 0xFF, 0xFF, 0xFF, 0xE8, 0x15, 0x00, 0x00, 0x00, 0xE8, 0xF0, 0x22, 0x00, 0x00, 0xC3, 0x33, 0xFF, 0x47, 0x8B, 0x75, 0x08, 0x6A, 0x0D, 0xE8, 0xCC, 0x29, 0x00, 0x00, 0x59, 0xC3, 0x6A, 0x0C, 0xE8, 0xC3, 0x29, 0x00, 0x00, 0x59, 0xC3, 0x8B, 0xFF, 0x56, 0x57, 0xFF, 0x15, 0x34, 0xA0, 0x40, 0x00, 0xFF, 0x35, 0x7C, 0xD7, 0x40, 0x00, 0x8B, 0xF8, 0xE8, 0xC4, 0xFE, 0xFF, 0xFF, 0xFF, 0xD0, 0x8B, 0xF0, 0x85, 0xF6, 0x75, 0x4E, 0x68, 0x14, 0x02, 0x00, 0x00, 0x6A, 0x01, 0xE8, 0xE7, 0x2A, 0x00, 0x00, 0x8B, 0xF0, 0x59, 0x59, 0x85, 0xF6, 0x74, 0x3A, 0x56, 0xFF, 0x35, 0x7C, 0xD7, 0x40, 0x00, 0xFF, 0x35, 0x18, 0xE1, 0x40, 0x00, 0xFF, 0x15, 0x88, 0xA0, 0x40, 0x00, 0xFF, 0xD0, 0x85, 0xC0, 0x74, 0x18, 0x6A, 0x00, 0x56, 0xE8, 0xF8, 0xFE, 0xFF, 0xFF, 0x59, 0x59, 0xFF, 0x15, 0x98, 0xA0, 0x40, 0x00, 0x83, 0x4E, 0x04, 0xFF, 0x89, 0x06, 0xEB, 0x09, 0x56, 0xE8, 0xCE, 0xF0, 0xFF, 0xFF, 0x59, 0x33, 0xF6, 0x57, 0xFF, 0x15, 0x94, 0xA0, 0x40, 0x00, 0x5F, 0x8B, 0xC6, 0x5E, 0xC3, 0x8B, 0xFF, 0x56, 0xE8, 0x7F, 0xFF, 0xFF, 0xFF, 0x8B, 0xF0, 0x85, 0xF6, 0x75, 0x08, 0x6A, 0x10, 0xE8, 0x41, 0x17, 0x00, 0x00, 0x59, 0x8B, 0xC6, 0x5E, 0xC3, 0x6A, 0x08, 0x68, 0x88, 0xC1, 0x40, 0x00, 0xE8, 0xF3, 0x21, 0x00, 0x00, 0x8B, 0x75, 0x08, 0x85, 0xF6, 0x0F, 0x84, 0xF8, 0x00, 0x00, 0x00, 0x8B, 0x46, 0x24, 0x85, 0xC0, 0x74, 0x07, 0x50, 0xE8, 0x81, 0xF0, 0xFF, 0xFF, 0x59, 0x8B, 0x46, 0x2C, 0x85, 0xC0, 0x74, 0x07, 0x50, 0xE8, 0x73, 0xF0, 0xFF, 0xFF, 0x59, 0x8B, 0x46, 0x34, 0x85, 0xC0, 0x74, 0x07, 0x50, 0xE8, 0x65, 0xF0, 0xFF, 0xFF, 0x59, 0x8B, 0x46, 0x3C, 0x85, 0xC0, 0x74, 0x07, 0x50, 0xE8, 0x57, 0xF0, 0xFF, 0xFF, 0x59, 0x8B, 0x46, 0x40, 0x85, 0xC0, 0x74, 0x07, 0x50, 0xE8, 0x49, 0xF0, 0xFF, 0xFF, 0x59, 0x8B, 0x46, 0x44, 0x85, 0xC0, 0x74, 0x07, 0x50, 0xE8, 0x3B, 0xF0, 0xFF, 0xFF, 0x59, 0x8B, 0x46, 0x48, 0x85, 0xC0, 0x74, 0x07, 0x50, 0xE8, 0x2D, 0xF0, 0xFF, 0xFF, 0x59, 0x8B, 0x46, 0x5C, 0x3D, 0x20, 0xB0, 0x40, 0x00, 0x74, 0x07, 0x50, 0xE8, 0x1C, 0xF0, 0xFF, 0xFF, 0x59, 0x6A, 0x0D, 0xE8, 0x76, 0x29, 0x00, 0x00, 0x59, 0x83, 0x65, 0xFC, 0x00, 0x8B, 0x7E, 0x68, 0x85, 0xFF, 0x74, 0x1A, 0x57, 0xFF, 0x15, 0x68, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x75, 0x0F, 0x81, 0xFF, 0x10, 0xD0, 0x40, 0x00, 0x74, 0x07, 0x57, 0xE8, 0xEF, 0xEF, 0xFF, 0xFF, 0x59, 0xC7, 0x45, 0xFC, 0xFE, 0xFF, 0xFF, 0xFF, 0xE8, 0x57, 0x00, 0x00, 0x00, 0x6A, 0x0C, 0xE8, 0x3D, 0x29, 0x00, 0x00, 0x59, 0xC7, 0x45, 0xFC, 0x01, 0x00, 0x00, 0x00, 0x8B, 0x7E, 0x6C, 0x85, 0xFF, 0x74, 0x23, 0x57, 0xE8, 0xAD, 0xFA, 0xFF, 0xFF, 0x59, 0x3B, 0x3D, 0x78, 0xD7, 0x40, 0x00, 0x74, 0x14, 0x81, 0xFF, 0xA0, 0xD6, 0x40, 0x00, 0x74, 0x0C, 0x83, 0x3F, 0x00, 0x75, 0x07, 0x57, 0xE8, 0x2A, 0xFB, 0xFF, 0xFF, 0x59, 0xC7, 0x45, 0xFC, 0xFE, 0xFF, 0xFF, 0xFF, 0xE8, 0x1E, 0x00, 0x00, 0x00, 0x56, 0xE8, 0x97, 0xEF, 0xFF, 0xFF, 0x59, 0xE8, 0x30, 0x21, 0x00, 0x00, 0xC2, 0x04, 0x00, 0x8B, 0x75, 0x08, 0x6A, 0x0D, 0xE8, 0x0D, 0x28, 0x00, 0x00, 0x59, 0xC3, 0x8B, 0x75, 0x08, 0x6A, 0x0C, 0xE8, 0x01, 0x28, 0x00, 0x00, 0x59, 0xC3, 0x8B, 0xFF, 0x57, 0x68,
				0xC8, 0xA4, 0x40, 0x00, 0xFF, 0x15, 0x90, 0xA0, 0x40, 0x00, 0x8B, 0xF8, 0x85, 0xFF, 0x75, 0x09, 0xE8, 0x34, 0xFD, 0xFF, 0xFF, 0x33, 0xC0, 0x5F, 0xC3, 0x56, 0x8B, 0x35, 0x9C, 0xA0, 0x40, 0x00, 0x68, 0x04, 0xA5, 0x40, 0x00, 0x57, 0xFF, 0xD6, 0x68, 0xF8, 0xA4, 0x40, 0x00, 0x57, 0xA3, 0x10, 0xE1, 0x40, 0x00, 0xFF, 0xD6, 0x68, 0xEC, 0xA4, 0x40, 0x00, 0x57, 0xA3, 0x14, 0xE1, 0x40, 0x00, 0xFF, 0xD6, 0x68, 0xE4, 0xA4, 0x40, 0x00, 0x57, 0xA3, 0x18, 0xE1, 0x40, 0x00, 0xFF, 0xD6, 0x83, 0x3D, 0x10, 0xE1, 0x40, 0x00, 0x00, 0x8B, 0x35, 0x84, 0xA0, 0x40, 0x00, 0xA3, 0x1C, 0xE1, 0x40, 0x00, 0x74, 0x16, 0x83, 0x3D, 0x14, 0xE1, 0x40, 0x00, 0x00, 0x74, 0x0D, 0x83, 0x3D, 0x18, 0xE1, 0x40, 0x00, 0x00, 0x74, 0x04, 0x85, 0xC0, 0x75, 0x24, 0xA1, 0x80, 0xA0, 0x40, 0x00, 0xA3, 0x14, 0xE1, 0x40, 0x00, 0xA1, 0x8C, 0xA0, 0x40, 0x00, 0xC7, 0x05, 0x10, 0xE1, 0x40, 0x00, 0x10, 0x32, 0x40, 0x00, 0x89, 0x35, 0x18, 0xE1, 0x40, 0x00, 0xA3, 0x1C, 0xE1, 0x40, 0x00, 0xFF, 0x15, 0x7C, 0xA0, 0x40, 0x00, 0xA3, 0x80, 0xD7, 0x40, 0x00, 0x83, 0xF8, 0xFF, 0x0F, 0x84, 0xC1, 0x00, 0x00, 0x00, 0xFF, 0x35, 0x14, 0xE1, 0x40, 0x00, 0x50, 0xFF, 0xD6, 0x85, 0xC0, 0x0F, 0x84, 0xB0, 0x00, 0x00, 0x00, 0xE8, 0xCA, 0x12, 0x00, 0x00, 0xFF, 0x35, 0x10, 0xE1, 0x40, 0x00, 0x8B, 0x35, 0x78, 0xA0, 0x40, 0x00, 0xFF, 0xD6, 0xFF, 0x35, 0x14, 0xE1, 0x40, 0x00, 0xA3, 0x10, 0xE1, 0x40, 0x00, 0xFF, 0xD6, 0xFF, 0x35, 0x18, 0xE1, 0x40, 0x00, 0xA3, 0x14, 0xE1, 0x40, 0x00, 0xFF, 0xD6, 0xFF, 0x35, 0x1C, 0xE1, 0x40, 0x00, 0xA3, 0x18, 0xE1, 0x40, 0x00, 0xFF, 0xD6, 0xA3, 0x1C, 0xE1, 0x40, 0x00, 0xE8, 0x54, 0x26, 0x00, 0x00, 0x85, 0xC0, 0x74, 0x63, 0x8B, 0x3D, 0x88, 0xA0, 0x40, 0x00, 0x68, 0xD1, 0x33, 0x40, 0x00, 0xFF, 0x35, 0x10, 0xE1, 0x40, 0x00, 0xFF, 0xD7, 0xFF, 0xD0, 0xA3, 0x7C, 0xD7, 0x40, 0x00, 0x83, 0xF8, 0xFF, 0x74, 0x44, 0x68, 0x14, 0x02, 0x00, 0x00, 0x6A, 0x01, 0xE8, 0x17, 0x28, 0x00, 0x00, 0x8B, 0xF0, 0x59, 0x59, 0x85, 0xF6, 0x74, 0x30, 0x56, 0xFF, 0x35, 0x7C, 0xD7, 0x40, 0x00, 0xFF, 0x35, 0x18, 0xE1, 0x40, 0x00, 0xFF, 0xD7, 0xFF, 0xD0, 0x85, 0xC0, 0x74, 0x1B, 0x6A, 0x00, 0x56, 0xE8, 0x2C, 0xFC, 0xFF, 0xFF, 0x59, 0x59, 0xFF, 0x15, 0x98, 0xA0, 0x40, 0x00, 0x83, 0x4E, 0x04, 0xFF, 0x89, 0x06, 0x33, 0xC0, 0x40, 0xEB, 0x07, 0xE8, 0xD7, 0xFB, 0xFF, 0xFF, 0x33, 0xC0, 0x5E, 0x5F, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x83, 0xEC, 0x18, 0x53, 0x56, 0xFF, 0x75, 0x0C, 0x8D, 0x4D, 0xE8, 0xE8, 0x3E, 0xEA, 0xFF, 0xFF, 0x8B, 0x5D, 0x08, 0xBE, 0x00, 0x01, 0x00, 0x00, 0x3B, 0xDE, 0x73, 0x54, 0x8B, 0x4D, 0xE8, 0x83, 0xB9, 0xAC, 0x00, 0x00, 0x00, 0x01, 0x7E, 0x14, 0x8D, 0x45, 0xE8, 0x50, 0x6A, 0x01, 0x53, 0xE8, 0x5F, 0x2D, 0x00, 0x00, 0x8B, 0x4D, 0xE8, 0x83, 0xC4, 0x0C, 0xEB, 0x0D, 0x8B, 0x81, 0xC8, 0x00, 0x00, 0x00, 0x0F, 0xB7, 0x04, 0x58, 0x83, 0xE0, 0x01, 0x85, 0xC0, 0x74, 0x0F, 0x8B, 0x81, 0xCC, 0x00, 0x00, 0x00, 0x0F, 0xB6, 0x04, 0x18, 0xE9, 0xA3, 0x00, 0x00, 0x00, 0x80, 0x7D, 0xF4, 0x00, 0x74, 0x07, 0x8B, 0x45, 0xF0, 0x83, 0x60, 0x70, 0xFD, 0x8B, 0xC3, 0xE9, 0x9C, 0x00, 0x00, 0x00, 0x8B, 0x45, 0xE8, 0x83, 0xB8, 0xAC, 0x00, 0x00,
				0x00, 0x01, 0x7E, 0x31, 0x89, 0x5D, 0x08, 0xC1, 0x7D, 0x08, 0x08, 0x8D, 0x45, 0xE8, 0x50, 0x8B, 0x45, 0x08, 0x25, 0xFF, 0x00, 0x00, 0x00, 0x50, 0xE8, 0xB3, 0x2C, 0x00, 0x00, 0x59, 0x59, 0x85, 0xC0, 0x74, 0x12, 0x8A, 0x45, 0x08, 0x6A, 0x02, 0x88, 0x45, 0xFC, 0x88, 0x5D, 0xFD, 0xC6, 0x45, 0xFE, 0x00, 0x59, 0xEB, 0x15, 0xE8, 0x3A, 0x02, 0x00, 0x00, 0xC7, 0x00, 0x2A, 0x00, 0x00, 0x00, 0x33, 0xC9, 0x88, 0x5D, 0xFC, 0xC6, 0x45, 0xFD, 0x00, 0x41, 0x8B, 0x45, 0xE8, 0x6A, 0x01, 0xFF, 0x70, 0x04, 0x8D, 0x55, 0xF8, 0x6A, 0x03, 0x52, 0x51, 0x8D, 0x4D, 0xFC, 0x51, 0x56, 0xFF, 0x70, 0x14, 0x8D, 0x45, 0xE8, 0x50, 0xE8, 0x8F, 0x23, 0x00, 0x00, 0x83, 0xC4, 0x24, 0x85, 0xC0, 0x0F, 0x84, 0x6F, 0xFF, 0xFF, 0xFF, 0x83, 0xF8, 0x01, 0x0F, 0xB6, 0x45, 0xF8, 0x74, 0x09, 0x0F, 0xB6, 0x4D, 0xF9, 0xC1, 0xE0, 0x08, 0x0B, 0xC1, 0x80, 0x7D, 0xF4, 0x00, 0x74, 0x07, 0x8B, 0x4D, 0xF0, 0x83, 0x61, 0x70, 0xFD, 0x5E, 0x5B, 0xC9, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x8B, 0x45, 0x08, 0xA3, 0x20, 0xE1, 0x40, 0x00, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x81, 0xEC, 0x28, 0x03, 0x00, 0x00, 0xA1, 0x04, 0xD0, 0x40, 0x00, 0x33, 0xC5, 0x89, 0x45, 0xFC, 0x53, 0x8B, 0x5D, 0x08, 0x57, 0x83, 0xFB, 0xFF, 0x74, 0x07, 0x53, 0xE8, 0x96, 0x20, 0x00, 0x00, 0x59, 0x83, 0xA5, 0xE0, 0xFC, 0xFF, 0xFF, 0x00, 0x6A, 0x4C, 0x8D, 0x85, 0xE4, 0xFC, 0xFF, 0xFF, 0x6A, 0x00, 0x50, 0xE8, 0x94, 0x20, 0x00, 0x00, 0x8D, 0x85, 0xE0, 0xFC, 0xFF, 0xFF, 0x89, 0x85, 0xD8, 0xFC, 0xFF, 0xFF, 0x8D, 0x85, 0x30, 0xFD, 0xFF, 0xFF, 0x83, 0xC4, 0x0C, 0x89, 0x85, 0xDC, 0xFC, 0xFF, 0xFF, 0x89, 0x85, 0xE0, 0xFD, 0xFF, 0xFF, 0x89, 0x8D, 0xDC, 0xFD, 0xFF, 0xFF, 0x89, 0x95, 0xD8, 0xFD, 0xFF, 0xFF, 0x89, 0x9D, 0xD4, 0xFD, 0xFF, 0xFF, 0x89, 0xB5, 0xD0, 0xFD, 0xFF, 0xFF, 0x89, 0xBD, 0xCC, 0xFD, 0xFF, 0xFF, 0x66, 0x8C, 0x95, 0xF8, 0xFD, 0xFF, 0xFF, 0x66, 0x8C, 0x8D, 0xEC, 0xFD, 0xFF, 0xFF, 0x66, 0x8C, 0x9D, 0xC8, 0xFD, 0xFF, 0xFF, 0x66, 0x8C, 0x85, 0xC4, 0xFD, 0xFF, 0xFF, 0x66, 0x8C, 0xA5, 0xC0, 0xFD, 0xFF, 0xFF, 0x66, 0x8C, 0xAD, 0xBC, 0xFD, 0xFF, 0xFF, 0x9C, 0x8F, 0x85, 0xF0, 0xFD, 0xFF, 0xFF, 0x8B, 0x45, 0x04, 0x8D, 0x4D, 0x04, 0x89, 0x8D, 0xF4, 0xFD, 0xFF, 0xFF, 0xC7, 0x85, 0x30, 0xFD, 0xFF, 0xFF, 0x01, 0x00, 0x01, 0x00, 0x89, 0x85, 0xE8, 0xFD, 0xFF, 0xFF, 0x8B, 0x49, 0xFC, 0x89, 0x8D, 0xE4, 0xFD, 0xFF, 0xFF, 0x8B, 0x4D, 0x0C, 0x89, 0x8D, 0xE0, 0xFC, 0xFF, 0xFF, 0x8B, 0x4D, 0x10, 0x89, 0x8D, 0xE4, 0xFC, 0xFF, 0xFF, 0x89, 0x85, 0xEC, 0xFC, 0xFF, 0xFF, 0xFF, 0x15, 0x5C, 0xA0, 0x40, 0x00, 0x6A, 0x00, 0x8B, 0xF8, 0xFF, 0x15, 0x58, 0xA0, 0x40, 0x00, 0x8D, 0x85, 0xD8, 0xFC, 0xFF, 0xFF, 0x50, 0xFF, 0x15, 0x54, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x75, 0x10, 0x85, 0xFF, 0x75, 0x0C, 0x83, 0xFB, 0xFF, 0x74, 0x07, 0x53, 0xE8, 0xA1, 0x1F, 0x00, 0x00, 0x59, 0x8B, 0x4D, 0xFC, 0x5F, 0x33, 0xCD, 0x5B, 0xE8, 0xF9, 0xE7, 0xFF, 0xFF, 0xC9, 0xC3, 0x8B, 0xFF, 0x56, 0x6A, 0x01, 0xBE, 0x17, 0x04, 0x00, 0xC0, 0x56, 0x6A, 0x02, 0xE8, 0xC5, 0xFE, 0xFF, 0xFF, 0x83, 0xC4, 0x0C, 0x56, 0xFF, 0x15, 0x50, 0xA0, 0x40, 0x00, 0x50, 0xFF, 0x15, 0x4C, 0xA0, 0x40, 0x00, 0x5E,
				0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0xFF, 0x35, 0x20, 0xE1, 0x40, 0x00, 0xFF, 0x15, 0x88, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x74, 0x03, 0x5D, 0xFF, 0xE0, 0xFF, 0x75, 0x18, 0xFF, 0x75, 0x14, 0xFF, 0x75, 0x10, 0xFF, 0x75, 0x0C, 0xFF, 0x75, 0x08, 0xE8, 0xAF, 0xFF, 0xFF, 0xFF, 0xCC, 0x33, 0xC0, 0x50, 0x50, 0x50, 0x50, 0x50, 0xE8, 0xC7, 0xFF, 0xFF, 0xFF, 0x83, 0xC4, 0x14, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x8B, 0x45, 0x08, 0x33, 0xC9, 0x3B, 0x04, 0xCD, 0x88, 0xD7, 0x40, 0x00, 0x74, 0x13, 0x41, 0x83, 0xF9, 0x2D, 0x72, 0xF1, 0x8D, 0x48, 0xED, 0x83, 0xF9, 0x11, 0x77, 0x0E, 0x6A, 0x0D, 0x58, 0x5D, 0xC3, 0x8B, 0x04, 0xCD, 0x8C, 0xD7, 0x40, 0x00, 0x5D, 0xC3, 0x05, 0x44, 0xFF, 0xFF, 0xFF, 0x6A, 0x0E, 0x59, 0x3B, 0xC8, 0x1B, 0xC0, 0x23, 0xC1, 0x83, 0xC0, 0x08, 0x5D, 0xC3, 0xE8, 0xCD, 0xF9, 0xFF, 0xFF, 0x85, 0xC0, 0x75, 0x06, 0xB8, 0xF0, 0xD8, 0x40, 0x00, 0xC3, 0x83, 0xC0, 0x08, 0xC3, 0xE8, 0xBA, 0xF9, 0xFF, 0xFF, 0x85, 0xC0, 0x75, 0x06, 0xB8, 0xF4, 0xD8, 0x40, 0x00, 0xC3, 0x83, 0xC0, 0x0C, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x56, 0xE8, 0xE2, 0xFF, 0xFF, 0xFF, 0x8B, 0x4D, 0x08, 0x51, 0x89, 0x08, 0xE8, 0x82, 0xFF, 0xFF, 0xFF, 0x59, 0x8B, 0xF0, 0xE8, 0xBC, 0xFF, 0xFF, 0xFF, 0x89, 0x30, 0x5E, 0x5D, 0xC3, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0x55, 0x8B, 0xEC, 0x57, 0x56, 0x53, 0x8B, 0x4D, 0x10, 0x0B, 0xC9, 0x74, 0x4D, 0x8B, 0x75, 0x08, 0x8B, 0x7D, 0x0C, 0xB7, 0x41, 0xB3, 0x5A, 0xB6, 0x20, 0x8D, 0x49, 0x00, 0x8A, 0x26, 0x0A, 0xE4, 0x8A, 0x07, 0x74, 0x27, 0x0A, 0xC0, 0x74, 0x23, 0x83, 0xC6, 0x01, 0x83, 0xC7, 0x01, 0x3A, 0xE7, 0x72, 0x06, 0x3A, 0xE3, 0x77, 0x02, 0x02, 0xE6, 0x3A, 0xC7, 0x72, 0x06, 0x3A, 0xC3, 0x77, 0x02, 0x02, 0xC6, 0x3A, 0xE0, 0x75, 0x0B, 0x83, 0xE9, 0x01, 0x75, 0xD1, 0x33, 0xC9, 0x3A, 0xE0, 0x74, 0x09, 0xB9, 0xFF, 0xFF, 0xFF, 0xFF, 0x72, 0x02, 0xF7, 0xD9, 0x8B, 0xC1, 0x5B, 0x5E, 0x5F, 0xC9, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x51, 0x56, 0x8B, 0x75, 0x0C, 0x56, 0xE8, 0xB3, 0x3B, 0x00, 0x00, 0x89, 0x45, 0x0C, 0x8B, 0x46, 0x0C, 0x59, 0xA8, 0x82, 0x75, 0x17, 0xE8, 0x2B, 0xFF, 0xFF, 0xFF, 0xC7, 0x00, 0x09, 0x00, 0x00, 0x00, 0x83, 0x4E, 0x0C, 0x20, 0x83, 0xC8, 0xFF, 0xE9, 0x2F, 0x01, 0x00, 0x00, 0xA8, 0x40, 0x74, 0x0D, 0xE8, 0x10, 0xFF, 0xFF, 0xFF, 0xC7, 0x00, 0x22, 0x00, 0x00, 0x00, 0xEB, 0xE3, 0x53, 0x33, 0xDB, 0xA8, 0x01, 0x74, 0x16, 0x89, 0x5E, 0x04, 0xA8, 0x10, 0x0F, 0x84, 0x87, 0x00, 0x00, 0x00, 0x8B, 0x4E, 0x08, 0x83, 0xE0, 0xFE, 0x89, 0x0E, 0x89, 0x46, 0x0C, 0x8B, 0x46, 0x0C, 0x83, 0xE0, 0xEF, 0x83, 0xC8, 0x02, 0x89, 0x46, 0x0C, 0x89, 0x5E, 0x04, 0x89, 0x5D, 0xFC, 0xA9, 0x0C, 0x01, 0x00, 0x00, 0x75, 0x2C, 0xE8, 0x90, 0x39, 0x00, 0x00, 0x83, 0xC0, 0x20, 0x3B, 0xF0, 0x74, 0x0C, 0xE8, 0x84, 0x39, 0x00, 0x00, 0x83, 0xC0, 0x40, 0x3B, 0xF0, 0x75, 0x0D, 0xFF, 0x75, 0x0C, 0xE8, 0x1F, 0x39, 0x00, 0x00, 0x59, 0x85, 0xC0, 0x75, 0x07, 0x56, 0xE8, 0xCB, 0x38, 0x00, 0x00, 0x59, 0xF7, 0x46, 0x0C, 0x08, 0x01, 0x00, 0x00, 0x57, 0x0F, 0x84, 0x80, 0x00, 0x00, 0x00, 0x8B, 0x46, 0x08, 0x8B, 0x3E, 0x8D, 0x48, 0x01, 0x89, 0x0E, 0x8B, 0x4E,
				0x18, 0x2B, 0xF8, 0x49, 0x89, 0x4E, 0x04, 0x3B, 0xFB, 0x7E, 0x1D, 0x57, 0x50, 0xFF, 0x75, 0x0C, 0xE8, 0xC7, 0x37, 0x00, 0x00, 0x83, 0xC4, 0x0C, 0x89, 0x45, 0xFC, 0xEB, 0x4D, 0x83, 0xC8, 0x20, 0x89, 0x46, 0x0C, 0x83, 0xC8, 0xFF, 0xEB, 0x79, 0x8B, 0x4D, 0x0C, 0x83, 0xF9, 0xFF, 0x74, 0x1B, 0x83, 0xF9, 0xFE, 0x74, 0x16, 0x8B, 0xC1, 0x83, 0xE0, 0x1F, 0x8B, 0xD1, 0xC1, 0xFA, 0x05, 0xC1, 0xE0, 0x06, 0x03, 0x04, 0x95, 0x80, 0xFA, 0x40, 0x00, 0xEB, 0x05, 0xB8, 0x08, 0xD9, 0x40, 0x00, 0xF6, 0x40, 0x04, 0x20, 0x74, 0x14, 0x6A, 0x02, 0x53, 0x53, 0x51, 0xE8, 0x95, 0x2F, 0x00, 0x00, 0x23, 0xC2, 0x83, 0xC4, 0x10, 0x83, 0xF8, 0xFF, 0x74, 0x25, 0x8B, 0x46, 0x08, 0x8A, 0x4D, 0x08, 0x88, 0x08, 0xEB, 0x16, 0x33, 0xFF, 0x47, 0x57, 0x8D, 0x45, 0x08, 0x50, 0xFF, 0x75, 0x0C, 0xE8, 0x58, 0x37, 0x00, 0x00, 0x83, 0xC4, 0x0C, 0x89, 0x45, 0xFC, 0x39, 0x7D, 0xFC, 0x74, 0x09, 0x83, 0x4E, 0x0C, 0x20, 0x83, 0xC8, 0xFF, 0xEB, 0x08, 0x8B, 0x45, 0x08, 0x25, 0xFF, 0x00, 0x00, 0x00, 0x5F, 0x5B, 0x5E, 0xC9, 0xC3, 0xF6, 0x41, 0x0C, 0x40, 0x74, 0x06, 0x83, 0x79, 0x08, 0x00, 0x74, 0x24, 0xFF, 0x49, 0x04, 0x78, 0x0B, 0x8B, 0x11, 0x88, 0x02, 0xFF, 0x01, 0x0F, 0xB6, 0xC0, 0xEB, 0x0C, 0x0F, 0xBE, 0xC0, 0x51, 0x50, 0xE8, 0x76, 0xFE, 0xFF, 0xFF, 0x59, 0x59, 0x83, 0xF8, 0xFF, 0x75, 0x03, 0x09, 0x06, 0xC3, 0xFF, 0x06, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x51, 0xF6, 0x43, 0x0C, 0x40, 0x56, 0x8B, 0xF0, 0x8B, 0x07, 0x89, 0x45, 0xFC, 0x74, 0x0D, 0x83, 0x7B, 0x08, 0x00, 0x75, 0x07, 0x8B, 0x45, 0x0C, 0x01, 0x06, 0xEB, 0x3E, 0x83, 0x27, 0x00, 0x83, 0x7D, 0x0C, 0x00, 0x7E, 0x30, 0x8B, 0x45, 0x08, 0x8A, 0x00, 0xFF, 0x4D, 0x0C, 0x8B, 0xCB, 0xE8, 0x94, 0xFF, 0xFF, 0xFF, 0xFF, 0x45, 0x08, 0x83, 0x3E, 0xFF, 0x75, 0x0E, 0x83, 0x3F, 0x2A, 0x75, 0x0F, 0x8B, 0xCB, 0xB0, 0x3F, 0xE8, 0x7E, 0xFF, 0xFF, 0xFF, 0x83, 0x7D, 0x0C, 0x00, 0x7F, 0xD5, 0x83, 0x3F, 0x00, 0x75, 0x05, 0x8B, 0x45, 0xFC, 0x89, 0x07, 0x5E, 0xC9, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x81, 0xEC, 0x80, 0x02, 0x00, 0x00, 0xA1, 0x04, 0xD0, 0x40, 0x00, 0x33, 0xC5, 0x89, 0x45, 0xFC, 0x53, 0x8B, 0x5D, 0x0C, 0x56, 0x8B, 0x75, 0x08, 0x33, 0xC0, 0x57, 0x8B, 0x7D, 0x14, 0xFF, 0x75, 0x10, 0x8D, 0x8D, 0xA4, 0xFD, 0xFF, 0xFF, 0x89, 0xB5, 0xD0, 0xFD, 0xFF, 0xFF, 0x89, 0xBD, 0xE4, 0xFD, 0xFF, 0xFF, 0x89, 0x85, 0xB4, 0xFD, 0xFF, 0xFF, 0x89, 0x85, 0xF0, 0xFD, 0xFF, 0xFF, 0x89, 0x85, 0xC8, 0xFD, 0xFF, 0xFF, 0x89, 0x85, 0xE8, 0xFD, 0xFF, 0xFF, 0x89, 0x85, 0xCC, 0xFD, 0xFF, 0xFF, 0x89, 0x85, 0xBC, 0xFD, 0xFF, 0xFF, 0x89, 0x85, 0xC4, 0xFD, 0xFF, 0xFF, 0xE8, 0x4D, 0xE4, 0xFF, 0xFF, 0xE8, 0xE6, 0xFC, 0xFF, 0xFF, 0x89, 0x85, 0x94, 0xFD, 0xFF, 0xFF, 0x85, 0xF6, 0x75, 0x2B, 0xE8, 0xD7, 0xFC, 0xFF, 0xFF, 0xC7, 0x00, 0x16, 0x00, 0x00, 0x00, 0xE8, 0x7A, 0xFC, 0xFF, 0xFF, 0x80, 0xBD, 0xB0, 0xFD, 0xFF, 0xFF, 0x00, 0x74, 0x0A, 0x8B, 0x85, 0xAC, 0xFD, 0xFF, 0xFF, 0x83, 0x60, 0x70, 0xFD, 0x83, 0xC8, 0xFF, 0xE9, 0x37, 0x0B, 0x00, 0x00, 0xF6, 0x46, 0x0C, 0x40, 0x75, 0x5E, 0x56, 0xE8, 0x1D, 0x39, 0x00, 0x00, 0x59, 0xBA, 0x08, 0xD9, 0x40, 0x00, 0x83, 0xF8, 0xFF, 0x74, 0x1B, 0x83, 0xF8,
				0xFE, 0x74, 0x16, 0x8B, 0xC8, 0x83, 0xE1, 0x1F, 0x8B, 0xF0, 0xC1, 0xFE, 0x05, 0xC1, 0xE1, 0x06, 0x03, 0x0C, 0xB5, 0x80, 0xFA, 0x40, 0x00, 0xEB, 0x02, 0x8B, 0xCA, 0xF6, 0x41, 0x24, 0x7F, 0x75, 0x9B, 0x83, 0xF8, 0xFF, 0x74, 0x19, 0x83, 0xF8, 0xFE, 0x74, 0x14, 0x8B, 0xC8, 0x83, 0xE0, 0x1F, 0xC1, 0xF9, 0x05, 0xC1, 0xE0, 0x06, 0x03, 0x04, 0x8D, 0x80, 0xFA, 0x40, 0x00, 0xEB, 0x02, 0x8B, 0xC2, 0xF6, 0x40, 0x24, 0x80, 0x0F, 0x85, 0x71, 0xFF, 0xFF, 0xFF, 0x33, 0xC9, 0x3B, 0xD9, 0x0F, 0x84, 0x67, 0xFF, 0xFF, 0xFF, 0x8A, 0x13, 0x89, 0x8D, 0xDC, 0xFD, 0xFF, 0xFF, 0x89, 0x8D, 0xD8, 0xFD, 0xFF, 0xFF, 0x89, 0x8D, 0xB8, 0xFD, 0xFF, 0xFF, 0x88, 0x95, 0xEF, 0xFD, 0xFF, 0xFF, 0x84, 0xD2, 0x0F, 0x84, 0x8E, 0x0A, 0x00, 0x00, 0x43, 0x33, 0xF6, 0x89, 0x9D, 0xC0, 0xFD, 0xFF, 0xFF, 0x39, 0xB5, 0xDC, 0xFD, 0xFF, 0xFF, 0x0F, 0x8C, 0x79, 0x0A, 0x00, 0x00, 0x8D, 0x42, 0xE0, 0x3C, 0x58, 0x77, 0x0F, 0x0F, 0xBE, 0xC2, 0x0F, 0xBE, 0x80, 0x88, 0xA5, 0x40, 0x00, 0x83, 0xE0, 0x0F, 0xEB, 0x02, 0x33, 0xC0, 0x0F, 0xBE, 0x84, 0xC1, 0xA8, 0xA5, 0x40, 0x00, 0x6A, 0x07, 0xC1, 0xF8, 0x04, 0x59, 0x89, 0x85, 0x8C, 0xFD, 0xFF, 0xFF, 0x3B, 0xC1, 0x0F, 0x87, 0x20, 0x0A, 0x00, 0x00, 0xFF, 0x24, 0x85, 0x02, 0x48, 0x40, 0x00, 0x83, 0x8D, 0xE8, 0xFD, 0xFF, 0xFF, 0xFF, 0x89, 0xB5, 0x88, 0xFD, 0xFF, 0xFF, 0x89, 0xB5, 0xBC, 0xFD, 0xFF, 0xFF, 0x89, 0xB5, 0xC8, 0xFD, 0xFF, 0xFF, 0x89, 0xB5, 0xCC, 0xFD, 0xFF, 0xFF, 0x89, 0xB5, 0xF0, 0xFD, 0xFF, 0xFF, 0x89, 0xB5, 0xC4, 0xFD, 0xFF, 0xFF, 0xE9, 0xE9, 0x09, 0x00, 0x00, 0x0F, 0xBE, 0xC2, 0x83, 0xE8, 0x20, 0x74, 0x4A, 0x83, 0xE8, 0x03, 0x74, 0x36, 0x83, 0xE8, 0x08, 0x74, 0x25, 0x48, 0x48, 0x74, 0x15, 0x83, 0xE8, 0x03, 0x0F, 0x85, 0xCA, 0x09, 0x00, 0x00, 0x83, 0x8D, 0xF0, 0xFD, 0xFF, 0xFF, 0x08, 0xE9, 0xBE, 0x09, 0x00, 0x00, 0x83, 0x8D, 0xF0, 0xFD, 0xFF, 0xFF, 0x04, 0xE9, 0xB2, 0x09, 0x00, 0x00, 0x83, 0x8D, 0xF0, 0xFD, 0xFF, 0xFF, 0x01, 0xE9, 0xA6, 0x09, 0x00, 0x00, 0x81, 0x8D, 0xF0, 0xFD, 0xFF, 0xFF, 0x80, 0x00, 0x00, 0x00, 0xE9, 0x97, 0x09, 0x00, 0x00, 0x83, 0x8D, 0xF0, 0xFD, 0xFF, 0xFF, 0x02, 0xE9, 0x8B, 0x09, 0x00, 0x00, 0x80, 0xFA, 0x2A, 0x75, 0x2C, 0x83, 0xC7, 0x04, 0x89, 0xBD, 0xE4, 0xFD, 0xFF, 0xFF, 0x8B, 0x7F, 0xFC, 0x89, 0xBD, 0xC8, 0xFD, 0xFF, 0xFF, 0x3B, 0xFE, 0x0F, 0x8D, 0x6C, 0x09, 0x00, 0x00, 0x83, 0x8D, 0xF0, 0xFD, 0xFF, 0xFF, 0x04, 0xF7, 0x9D, 0xC8, 0xFD, 0xFF, 0xFF, 0xE9, 0x5A, 0x09, 0x00, 0x00, 0x8B, 0x85, 0xC8, 0xFD, 0xFF, 0xFF, 0x6B, 0xC0, 0x0A, 0x0F, 0xBE, 0xCA, 0x8D, 0x44, 0x08, 0xD0, 0x89, 0x85, 0xC8, 0xFD, 0xFF, 0xFF, 0xE9, 0x3F, 0x09, 0x00, 0x00, 0x89, 0xB5, 0xE8, 0xFD, 0xFF, 0xFF, 0xE9, 0x34, 0x09, 0x00, 0x00, 0x80, 0xFA, 0x2A, 0x75, 0x26, 0x83, 0xC7, 0x04, 0x89, 0xBD, 0xE4, 0xFD, 0xFF, 0xFF, 0x8B, 0x7F, 0xFC, 0x89, 0xBD, 0xE8, 0xFD, 0xFF, 0xFF, 0x3B, 0xFE, 0x0F, 0x8D, 0x15, 0x09, 0x00, 0x00, 0x83, 0x8D, 0xE8, 0xFD, 0xFF, 0xFF, 0xFF, 0xE9, 0x09, 0x09, 0x00, 0x00, 0x8B, 0x85, 0xE8, 0xFD, 0xFF, 0xFF, 0x6B, 0xC0, 0x0A, 0x0F, 0xBE, 0xCA, 0x8D, 0x44, 0x08, 0xD0, 0x89, 0x85, 0xE8, 0xFD, 0xFF, 0xFF, 0xE9, 0xEE, 0x08, 0x00, 0x00, 0x80, 0xFA,
				0x49, 0x74, 0x55, 0x80, 0xFA, 0x68, 0x74, 0x44, 0x80, 0xFA, 0x6C, 0x74, 0x18, 0x80, 0xFA, 0x77, 0x0F, 0x85, 0xD6, 0x08, 0x00, 0x00, 0x81, 0x8D, 0xF0, 0xFD, 0xFF, 0xFF, 0x00, 0x08, 0x00, 0x00, 0xE9, 0xC7, 0x08, 0x00, 0x00, 0x80, 0x3B, 0x6C, 0x75, 0x16, 0x43, 0x81, 0x8D, 0xF0, 0xFD, 0xFF, 0xFF, 0x00, 0x10, 0x00, 0x00, 0x89, 0x9D, 0xC0, 0xFD, 0xFF, 0xFF, 0xE9, 0xAC, 0x08, 0x00, 0x00, 0x83, 0x8D, 0xF0, 0xFD, 0xFF, 0xFF, 0x10, 0xE9, 0xA0, 0x08, 0x00, 0x00, 0x83, 0x8D, 0xF0, 0xFD, 0xFF, 0xFF, 0x20, 0xE9, 0x94, 0x08, 0x00, 0x00, 0x8A, 0x03, 0x3C, 0x36, 0x75, 0x1E, 0x80, 0x7B, 0x01, 0x34, 0x75, 0x18, 0x83, 0xC3, 0x02, 0x81, 0x8D, 0xF0, 0xFD, 0xFF, 0xFF, 0x00, 0x80, 0x00, 0x00, 0x89, 0x9D, 0xC0, 0xFD, 0xFF, 0xFF, 0xE9, 0x70, 0x08, 0x00, 0x00, 0x3C, 0x33, 0x75, 0x1E, 0x80, 0x7B, 0x01, 0x32, 0x75, 0x18, 0x83, 0xC3, 0x02, 0x81, 0xA5, 0xF0, 0xFD, 0xFF, 0xFF, 0xFF, 0x7F, 0xFF, 0xFF, 0x89, 0x9D, 0xC0, 0xFD, 0xFF, 0xFF, 0xE9, 0x4E, 0x08, 0x00, 0x00, 0x3C, 0x64, 0x0F, 0x84, 0x46, 0x08, 0x00, 0x00, 0x3C, 0x69, 0x0F, 0x84, 0x3E, 0x08, 0x00, 0x00, 0x3C, 0x6F, 0x0F, 0x84, 0x36, 0x08, 0x00, 0x00, 0x3C, 0x75, 0x0F, 0x84, 0x2E, 0x08, 0x00, 0x00, 0x3C, 0x78, 0x0F, 0x84, 0x26, 0x08, 0x00, 0x00, 0x3C, 0x58, 0x0F, 0x84, 0x1E, 0x08, 0x00, 0x00, 0x89, 0xB5, 0x8C, 0xFD, 0xFF, 0xFF, 0x21, 0xB5, 0xC4, 0xFD, 0xFF, 0xFF, 0x8D, 0x85, 0xA4, 0xFD, 0xFF, 0xFF, 0x50, 0x0F, 0xB6, 0xC2, 0x50, 0xE8, 0x16, 0x24, 0x00, 0x00, 0x59, 0x85, 0xC0, 0x8A, 0x85, 0xEF, 0xFD, 0xFF, 0xFF, 0x59, 0x74, 0x22, 0x8B, 0x8D, 0xD0, 0xFD, 0xFF, 0xFF, 0x8D, 0xB5, 0xDC, 0xFD, 0xFF, 0xFF, 0xE8, 0xB6, 0xFB, 0xFF, 0xFF, 0x8A, 0x03, 0x43, 0x89, 0x9D, 0xC0, 0xFD, 0xFF, 0xFF, 0x84, 0xC0, 0x0F, 0x84, 0xB0, 0xFC, 0xFF, 0xFF, 0x8B, 0x8D, 0xD0, 0xFD, 0xFF, 0xFF, 0x8D, 0xB5, 0xDC, 0xFD, 0xFF, 0xFF, 0xE8, 0x94, 0xFB, 0xFF, 0xFF, 0xE9, 0xBE, 0x07, 0x00, 0x00, 0x0F, 0xBE, 0xC2, 0x83, 0xF8, 0x64, 0x0F, 0x8F, 0xB6, 0x01, 0x00, 0x00, 0x0F, 0x84, 0x58, 0x02, 0x00, 0x00, 0x83, 0xF8, 0x53, 0x0F, 0x8F, 0xBA, 0x00, 0x00, 0x00, 0x74, 0x4C, 0x83, 0xE8, 0x41, 0x74, 0x10, 0x48, 0x48, 0x74, 0x24, 0x48, 0x48, 0x74, 0x08, 0x48, 0x48, 0x0F, 0x85, 0xAB, 0x05, 0x00, 0x00, 0x80, 0xC2, 0x20, 0xC7, 0x85, 0x88, 0xFD, 0xFF, 0xFF, 0x01, 0x00, 0x00, 0x00, 0x88, 0x95, 0xEF, 0xFD, 0xFF, 0xFF, 0xE9, 0x56, 0x02, 0x00, 0x00, 0xF7, 0x85, 0xF0, 0xFD, 0xFF, 0xFF, 0x30, 0x08, 0x00, 0x00, 0x0F, 0x85, 0x9E, 0x00, 0x00, 0x00, 0x81, 0x8D, 0xF0, 0xFD, 0xFF, 0xFF, 0x00, 0x08, 0x00, 0x00, 0xE9, 0x8F, 0x00, 0x00, 0x00, 0xF7, 0x85, 0xF0, 0xFD, 0xFF, 0xFF, 0x30, 0x08, 0x00, 0x00, 0x75, 0x0A, 0x81, 0x8D, 0xF0, 0xFD, 0xFF, 0xFF, 0x00, 0x08, 0x00, 0x00, 0x8B, 0x8D, 0xE8, 0xFD, 0xFF, 0xFF, 0x83, 0xF9, 0xFF, 0x75, 0x05, 0xB9, 0xFF, 0xFF, 0xFF, 0x7F, 0x83, 0xC7, 0x04, 0xF7, 0x85, 0xF0, 0xFD, 0xFF, 0xFF, 0x10, 0x08, 0x00, 0x00, 0x89, 0xBD, 0xE4, 0xFD, 0xFF, 0xFF, 0x8B, 0x7F, 0xFC, 0x89, 0xBD, 0xE0, 0xFD, 0xFF, 0xFF, 0x0F, 0x84, 0xFE, 0x04, 0x00, 0x00, 0x85, 0xFF, 0x75, 0x0B, 0xA1, 0x04, 0xD9, 0x40, 0x00, 0x89, 0x85, 0xE0, 0xFD, 0xFF, 0xFF, 0x8B, 0x85, 0xE0, 0xFD, 0xFF, 0xFF,
				0xC7, 0x85, 0xC4, 0xFD, 0xFF, 0xFF, 0x01, 0x00, 0x00, 0x00, 0xE9, 0xCC, 0x04, 0x00, 0x00, 0x83, 0xE8, 0x58, 0x0F, 0x84, 0x22, 0x03, 0x00, 0x00, 0x48, 0x48, 0x74, 0x7F, 0x2B, 0xC1, 0x0F, 0x84, 0xB6, 0x01, 0x00, 0x00, 0x48, 0x48, 0x0F, 0x85, 0xEB, 0x04, 0x00, 0x00, 0x8B, 0xBD, 0xE4, 0xFD, 0xFF, 0xFF, 0x83, 0xC7, 0x04, 0xF7, 0x85, 0xF0, 0xFD, 0xFF, 0xFF, 0x10, 0x08, 0x00, 0x00, 0x89, 0xBD, 0xE4, 0xFD, 0xFF, 0xFF, 0x74, 0x30, 0x0F, 0xB7, 0x47, 0xFC, 0x50, 0x68, 0x00, 0x02, 0x00, 0x00, 0x8D, 0x85, 0xF4, 0xFD, 0xFF, 0xFF, 0x50, 0x8D, 0x85, 0xD8, 0xFD, 0xFF, 0xFF, 0x50, 0xE8, 0x78, 0x36, 0x00, 0x00, 0x83, 0xC4, 0x10, 0x85, 0xC0, 0x74, 0x1F, 0xC7, 0x85, 0xBC, 0xFD, 0xFF, 0xFF, 0x01, 0x00, 0x00, 0x00, 0xEB, 0x13, 0x8A, 0x47, 0xFC, 0x88, 0x85, 0xF4, 0xFD, 0xFF, 0xFF, 0xC7, 0x85, 0xD8, 0xFD, 0xFF, 0xFF, 0x01, 0x00, 0x00, 0x00, 0x8D, 0x85, 0xF4, 0xFD, 0xFF, 0xFF, 0x89, 0x85, 0xE0, 0xFD, 0xFF, 0xFF, 0xE9, 0x7C, 0x04, 0x00, 0x00, 0x8B, 0x07, 0x83, 0xC7, 0x04, 0x89, 0xBD, 0xE4, 0xFD, 0xFF, 0xFF, 0x85, 0xC0, 0x74, 0x3B, 0x8B, 0x48, 0x04, 0x85, 0xC9, 0x74, 0x34, 0xF7, 0x85, 0xF0, 0xFD, 0xFF, 0xFF, 0x00, 0x08, 0x00, 0x00, 0x0F, 0xBF, 0x00, 0x89, 0x8D, 0xE0, 0xFD, 0xFF, 0xFF, 0x74, 0x14, 0x99, 0x2B, 0xC2, 0xD1, 0xF8, 0xC7, 0x85, 0xC4, 0xFD, 0xFF, 0xFF, 0x01, 0x00, 0x00, 0x00, 0xE9, 0x37, 0x04, 0x00, 0x00, 0x21, 0xB5, 0xC4, 0xFD, 0xFF, 0xFF, 0xE9, 0x2C, 0x04, 0x00, 0x00, 0xA1, 0x00, 0xD9, 0x40, 0x00, 0x89, 0x85, 0xE0, 0xFD, 0xFF, 0xFF, 0x50, 0xE8, 0xCE, 0x27, 0x00, 0x00, 0x59, 0xE9, 0x15, 0x04, 0x00, 0x00, 0x83, 0xF8, 0x70, 0x0F, 0x8F, 0x3D, 0x02, 0x00, 0x00, 0x0F, 0x84, 0x25, 0x02, 0x00, 0x00, 0x83, 0xF8, 0x65, 0x0F, 0x8C, 0x03, 0x04, 0x00, 0x00, 0x83, 0xF8, 0x67, 0x0F, 0x8E, 0xB7, 0x00, 0x00, 0x00, 0x83, 0xF8, 0x69, 0x74, 0x7C, 0x83, 0xF8, 0x6E, 0x74, 0x2E, 0x83, 0xF8, 0x6F, 0x0F, 0x85, 0xE7, 0x03, 0x00, 0x00, 0xF6, 0x85, 0xF0, 0xFD, 0xFF, 0xFF, 0x80, 0x8B, 0xBD, 0xE4, 0xFD, 0xFF, 0xFF, 0xC7, 0x85, 0xD8, 0xFD, 0xFF, 0xFF, 0x08, 0x00, 0x00, 0x00, 0x74, 0x6C, 0x81, 0x8D, 0xF0, 0xFD, 0xFF, 0xFF, 0x00, 0x02, 0x00, 0x00, 0xEB, 0x60, 0x83, 0x85, 0xE4, 0xFD, 0xFF, 0xFF, 0x04, 0x8B, 0x85, 0xE4, 0xFD, 0xFF, 0xFF, 0x8B, 0x70, 0xFC, 0xE8, 0x07, 0x34, 0x00, 0x00, 0x85, 0xC0, 0x0F, 0x84, 0x62, 0xFA, 0xFF, 0xFF, 0xF6, 0x85, 0xF0, 0xFD, 0xFF, 0xFF, 0x20, 0x74, 0x0C, 0x66, 0x8B, 0x85, 0xDC, 0xFD, 0xFF, 0xFF, 0x66, 0x89, 0x06, 0xEB, 0x08, 0x8B, 0x85, 0xDC, 0xFD, 0xFF, 0xFF, 0x89, 0x06, 0xC7, 0x85, 0xBC, 0xFD, 0xFF, 0xFF, 0x01, 0x00, 0x00, 0x00, 0xE9, 0x3E, 0x05, 0x00, 0x00, 0x8B, 0xBD, 0xE4, 0xFD, 0xFF, 0xFF, 0x83, 0x8D, 0xF0, 0xFD, 0xFF, 0xFF, 0x40, 0xC7, 0x85, 0xD8, 0xFD, 0xFF, 0xFF, 0x0A, 0x00, 0x00, 0x00, 0x8B, 0x8D, 0xF0, 0xFD, 0xFF, 0xFF, 0xF7, 0xC1, 0x00, 0x80, 0x00, 0x00, 0x0F, 0x84, 0xE0, 0x01, 0x00, 0x00, 0x8B, 0x07, 0x8B, 0x57, 0x04, 0x83, 0xC7, 0x08, 0xE9, 0x0C, 0x02, 0x00, 0x00, 0x8B, 0xBD, 0xE4, 0xFD, 0xFF, 0xFF, 0x83, 0x8D, 0xF0, 0xFD, 0xFF, 0xFF, 0x40, 0x8D, 0x9D, 0xF4, 0xFD, 0xFF, 0xFF, 0xB8, 0x00, 0x02, 0x00, 0x00, 0x89, 0x9D, 0xE0, 0xFD, 0xFF, 0xFF, 0x89, 0x85,
				0xA0, 0xFD, 0xFF, 0xFF, 0x39, 0xB5, 0xE8, 0xFD, 0xFF, 0xFF, 0x7D, 0x0C, 0xC7, 0x85, 0xE8, 0xFD, 0xFF, 0xFF, 0x06, 0x00, 0x00, 0x00, 0xEB, 0x6A, 0x75, 0x11, 0x80, 0xFA, 0x67, 0x75, 0x63, 0xC7, 0x85, 0xE8, 0xFD, 0xFF, 0xFF, 0x01, 0x00, 0x00, 0x00, 0xEB, 0x57, 0x39, 0x85, 0xE8, 0xFD, 0xFF, 0xFF, 0x7E, 0x06, 0x89, 0x85, 0xE8, 0xFD, 0xFF, 0xFF, 0x81, 0xBD, 0xE8, 0xFD, 0xFF, 0xFF, 0xA3, 0x00, 0x00, 0x00, 0x7E, 0x3D, 0x8B, 0xB5, 0xE8, 0xFD, 0xFF, 0xFF, 0x81, 0xC6, 0x5D, 0x01, 0x00, 0x00, 0x56, 0xE8, 0x04, 0x1B, 0x00, 0x00, 0x8A, 0x95, 0xEF, 0xFD, 0xFF, 0xFF, 0x59, 0x89, 0x85, 0xB8, 0xFD, 0xFF, 0xFF, 0x85, 0xC0, 0x74, 0x10, 0x89, 0x85, 0xE0, 0xFD, 0xFF, 0xFF, 0x89, 0xB5, 0xA0, 0xFD, 0xFF, 0xFF, 0x8B, 0xD8, 0xEB, 0x0A, 0xC7, 0x85, 0xE8, 0xFD, 0xFF, 0xFF, 0xA3, 0x00, 0x00, 0x00, 0x8B, 0x07, 0x8B, 0x35, 0x88, 0xA0, 0x40, 0x00, 0x83, 0xC7, 0x08, 0x89, 0x85, 0x80, 0xFD, 0xFF, 0xFF, 0x8B, 0x47, 0xFC, 0x89, 0x85, 0x84, 0xFD, 0xFF, 0xFF, 0x8D, 0x85, 0xA4, 0xFD, 0xFF, 0xFF, 0x50, 0xFF, 0xB5, 0x88, 0xFD, 0xFF, 0xFF, 0x0F, 0xBE, 0xC2, 0xFF, 0xB5, 0xE8, 0xFD, 0xFF, 0xFF, 0x89, 0xBD, 0xE4, 0xFD, 0xFF, 0xFF, 0x50, 0xFF, 0xB5, 0xA0, 0xFD, 0xFF, 0xFF, 0x8D, 0x85, 0x80, 0xFD, 0xFF, 0xFF, 0x53, 0x50, 0xFF, 0x35, 0x78, 0xDD, 0x40, 0x00, 0xFF, 0xD6, 0xFF, 0xD0, 0x8B, 0xBD, 0xF0, 0xFD, 0xFF, 0xFF, 0x83, 0xC4, 0x1C, 0x81, 0xE7, 0x80, 0x00, 0x00, 0x00, 0x74, 0x1D, 0x83, 0xBD, 0xE8, 0xFD, 0xFF, 0xFF, 0x00, 0x75, 0x14, 0x8D, 0x85, 0xA4, 0xFD, 0xFF, 0xFF, 0x50, 0x53, 0xFF, 0x35, 0x84, 0xDD, 0x40, 0x00, 0xFF, 0xD6, 0xFF, 0xD0, 0x59, 0x59, 0x80, 0xBD, 0xEF, 0xFD, 0xFF, 0xFF, 0x67, 0x75, 0x18, 0x85, 0xFF, 0x75, 0x14, 0x8D, 0x85, 0xA4, 0xFD, 0xFF, 0xFF, 0x50, 0x53, 0xFF, 0x35, 0x80, 0xDD, 0x40, 0x00, 0xFF, 0xD6, 0xFF, 0xD0, 0x59, 0x59, 0x80, 0x3B, 0x2D, 0x75, 0x11, 0x81, 0x8D, 0xF0, 0xFD, 0xFF, 0xFF, 0x00, 0x01, 0x00, 0x00, 0x43, 0x89, 0x9D, 0xE0, 0xFD, 0xFF, 0xFF, 0x53, 0xE9, 0xC1, 0xFD, 0xFF, 0xFF, 0xC7, 0x85, 0xE8, 0xFD, 0xFF, 0xFF, 0x08, 0x00, 0x00, 0x00, 0x89, 0x8D, 0xB4, 0xFD, 0xFF, 0xFF, 0xEB, 0x2A, 0x83, 0xE8, 0x73, 0x0F, 0x84, 0x6E, 0xFC, 0xFF, 0xFF, 0x48, 0x48, 0x0F, 0x84, 0x58, 0xFE, 0xFF, 0xFF, 0x83, 0xE8, 0x03, 0x0F, 0x85, 0xBB, 0x01, 0x00, 0x00, 0x8B, 0xBD, 0xE4, 0xFD, 0xFF, 0xFF, 0xC7, 0x85, 0xB4, 0xFD, 0xFF, 0xFF, 0x27, 0x00, 0x00, 0x00, 0xF6, 0x85, 0xF0, 0xFD, 0xFF, 0xFF, 0x80, 0xC7, 0x85, 0xD8, 0xFD, 0xFF, 0xFF, 0x10, 0x00, 0x00, 0x00, 0x0F, 0x84, 0x32, 0xFE, 0xFF, 0xFF, 0x8A, 0x85, 0xB4, 0xFD, 0xFF, 0xFF, 0x04, 0x51, 0xC6, 0x85, 0xD4, 0xFD, 0xFF, 0xFF, 0x30, 0x88, 0x85, 0xD5, 0xFD, 0xFF, 0xFF, 0xC7, 0x85, 0xCC, 0xFD, 0xFF, 0xFF, 0x02, 0x00, 0x00, 0x00, 0xE9, 0x0E, 0xFE, 0xFF, 0xFF, 0xF7, 0xC1, 0x00, 0x10, 0x00, 0x00, 0x0F, 0x85, 0x14, 0xFE, 0xFF, 0xFF, 0x83, 0xC7, 0x04, 0xF6, 0xC1, 0x20, 0x74, 0x18, 0x89, 0xBD, 0xE4, 0xFD, 0xFF, 0xFF, 0xF6, 0xC1, 0x40, 0x74, 0x06, 0x0F, 0xBF, 0x47, 0xFC, 0xEB, 0x04, 0x0F, 0xB7, 0x47, 0xFC, 0x99, 0xEB, 0x13, 0x8B, 0x47, 0xFC, 0xF6, 0xC1, 0x40, 0x74, 0x03, 0x99, 0xEB, 0x02, 0x33, 0xD2, 0x89, 0xBD, 0xE4, 0xFD, 0xFF, 0xFF, 0xF6, 0xC1,
				0x40, 0x74, 0x1A, 0x85, 0xD2, 0x7F, 0x16, 0x7C, 0x04, 0x85, 0xC0, 0x73, 0x10, 0xF7, 0xD8, 0x13, 0xD6, 0xF7, 0xDA, 0x81, 0x8D, 0xF0, 0xFD, 0xFF, 0xFF, 0x00, 0x01, 0x00, 0x00, 0xF7, 0x85, 0xF0, 0xFD, 0xFF, 0xFF, 0x00, 0x90, 0x00, 0x00, 0x8B, 0xFA, 0x8B, 0xD8, 0x75, 0x02, 0x33, 0xFF, 0x39, 0xB5, 0xE8, 0xFD, 0xFF, 0xFF, 0x7D, 0x0C, 0xC7, 0x85, 0xE8, 0xFD, 0xFF, 0xFF, 0x01, 0x00, 0x00, 0x00, 0xEB, 0x1A, 0x83, 0xA5, 0xF0, 0xFD, 0xFF, 0xFF, 0xF7, 0xB8, 0x00, 0x02, 0x00, 0x00, 0x39, 0x85, 0xE8, 0xFD, 0xFF, 0xFF, 0x7E, 0x06, 0x89, 0x85, 0xE8, 0xFD, 0xFF, 0xFF, 0x8B, 0xC3, 0x0B, 0xC7, 0x75, 0x06, 0x21, 0xB5, 0xCC, 0xFD, 0xFF, 0xFF, 0x8D, 0x75, 0xF3, 0x8B, 0x85, 0xE8, 0xFD, 0xFF, 0xFF, 0xFF, 0x8D, 0xE8, 0xFD, 0xFF, 0xFF, 0x85, 0xC0, 0x7F, 0x06, 0x8B, 0xC3, 0x0B, 0xC7, 0x74, 0x2D, 0x8B, 0x85, 0xD8, 0xFD, 0xFF, 0xFF, 0x99, 0x52, 0x50, 0x57, 0x53, 0xE8, 0x8A, 0x32, 0x00, 0x00, 0x83, 0xC1, 0x30, 0x89, 0x9D, 0xA0, 0xFD, 0xFF, 0xFF, 0x8B, 0xD8, 0x8B, 0xFA, 0x83, 0xF9, 0x39, 0x7E, 0x06, 0x03, 0x8D, 0xB4, 0xFD, 0xFF, 0xFF, 0x88, 0x0E, 0x4E, 0xEB, 0xBD, 0x8D, 0x45, 0xF3, 0x2B, 0xC6, 0x46, 0xF7, 0x85, 0xF0, 0xFD, 0xFF, 0xFF, 0x00, 0x02, 0x00, 0x00, 0x89, 0x85, 0xD8, 0xFD, 0xFF, 0xFF, 0x89, 0xB5, 0xE0, 0xFD, 0xFF, 0xFF, 0x74, 0x62, 0x85, 0xC0, 0x74, 0x07, 0x8B, 0xCE, 0x80, 0x39, 0x30, 0x74, 0x57, 0xFF, 0x8D, 0xE0, 0xFD, 0xFF, 0xFF, 0x8B, 0x8D, 0xE0, 0xFD, 0xFF, 0xFF, 0xC6, 0x01, 0x30, 0x40, 0xEB, 0x3F, 0x49, 0x66, 0x39, 0x30, 0x74, 0x07, 0x83, 0xC0, 0x02, 0x85, 0xC9, 0x75, 0xF3, 0x2B, 0x85, 0xE0, 0xFD, 0xFF, 0xFF, 0xD1, 0xF8, 0xEB, 0x28, 0x85, 0xFF, 0x75, 0x0B, 0xA1, 0x00, 0xD9, 0x40, 0x00, 0x89, 0x85, 0xE0, 0xFD, 0xFF, 0xFF, 0x8B, 0x85, 0xE0, 0xFD, 0xFF, 0xFF, 0xEB, 0x07, 0x49, 0x80, 0x38, 0x00, 0x74, 0x05, 0x40, 0x85, 0xC9, 0x75, 0xF5, 0x2B, 0x85, 0xE0, 0xFD, 0xFF, 0xFF, 0x89, 0x85, 0xD8, 0xFD, 0xFF, 0xFF, 0x83, 0xBD, 0xBC, 0xFD, 0xFF, 0xFF, 0x00, 0x0F, 0x85, 0xB8, 0x01, 0x00, 0x00, 0x8B, 0x8D, 0xF0, 0xFD, 0xFF, 0xFF, 0xF6, 0xC1, 0x40, 0x74, 0x35, 0xF7, 0xC1, 0x00, 0x01, 0x00, 0x00, 0x74, 0x09, 0xC6, 0x85, 0xD4, 0xFD, 0xFF, 0xFF, 0x2D, 0xEB, 0x1A, 0xF6, 0xC1, 0x01, 0x74, 0x09, 0xC6, 0x85, 0xD4, 0xFD, 0xFF, 0xFF, 0x2B, 0xEB, 0x0C, 0xF6, 0xC1, 0x02, 0x74, 0x11, 0xC6, 0x85, 0xD4, 0xFD, 0xFF, 0xFF, 0x20, 0xC7, 0x85, 0xCC, 0xFD, 0xFF, 0xFF, 0x01, 0x00, 0x00, 0x00, 0x8B, 0x85, 0xC8, 0xFD, 0xFF, 0xFF, 0x2B, 0x85, 0xD8, 0xFD, 0xFF, 0xFF, 0x2B, 0x85, 0xCC, 0xFD, 0xFF, 0xFF, 0x89, 0x85, 0xA0, 0xFD, 0xFF, 0xFF, 0xF6, 0xC1, 0x0C, 0x75, 0x25, 0x8B, 0xF8, 0xEB, 0x1D, 0x8B, 0x8D, 0xD0, 0xFD, 0xFF, 0xFF, 0x8D, 0xB5, 0xDC, 0xFD, 0xFF, 0xFF, 0xB0, 0x20, 0x4F, 0xE8, 0x30, 0xF5, 0xFF, 0xFF, 0x83, 0xBD, 0xDC, 0xFD, 0xFF, 0xFF, 0xFF, 0x74, 0x04, 0x85, 0xFF, 0x7F, 0xDF, 0xFF, 0xB5, 0xCC, 0xFD, 0xFF, 0xFF, 0x8B, 0xBD, 0x94, 0xFD, 0xFF, 0xFF, 0x8B, 0x9D, 0xD0, 0xFD, 0xFF, 0xFF, 0x8D, 0x85, 0xD4, 0xFD, 0xFF, 0xFF, 0x50, 0x8D, 0x85, 0xDC, 0xFD, 0xFF, 0xFF, 0xE8, 0x32, 0xF5, 0xFF, 0xFF, 0xF6, 0x85, 0xF0, 0xFD, 0xFF, 0xFF, 0x08, 0x59, 0x59, 0x74, 0x2E, 0xF6, 0x85, 0xF0, 0xFD, 0xFF, 0xFF, 0x04,
				0x75, 0x25, 0x8B, 0xBD, 0xA0, 0xFD, 0xFF, 0xFF, 0xEB, 0x19, 0x8D, 0xB5, 0xDC, 0xFD, 0xFF, 0xFF, 0x8B, 0xCB, 0xB0, 0x30, 0x4F, 0xE8, 0xD3, 0xF4, 0xFF, 0xFF, 0x83, 0xBD, 0xDC, 0xFD, 0xFF, 0xFF, 0xFF, 0x74, 0x04, 0x85, 0xFF, 0x7F, 0xE3, 0x83, 0xBD, 0xC4, 0xFD, 0xFF, 0xFF, 0x00, 0x8B, 0x85, 0xD8, 0xFD, 0xFF, 0xFF, 0x74, 0x75, 0x85, 0xC0, 0x7E, 0x71, 0x8B, 0xB5, 0xE0, 0xFD, 0xFF, 0xFF, 0x89, 0x85, 0x98, 0xFD, 0xFF, 0xFF, 0x0F, 0xB7, 0x06, 0xFF, 0x8D, 0x98, 0xFD, 0xFF, 0xFF, 0x50, 0x6A, 0x06, 0x8D, 0x45, 0xF4, 0x50, 0x8D, 0x85, 0x90, 0xFD, 0xFF, 0xFF, 0x50, 0x83, 0xC6, 0x02, 0xE8, 0x9B, 0x30, 0x00, 0x00, 0x83, 0xC4, 0x10, 0x85, 0xC0, 0x75, 0x36, 0x39, 0x85, 0x90, 0xFD, 0xFF, 0xFF, 0x74, 0x2E, 0xFF, 0xB5, 0x90, 0xFD, 0xFF, 0xFF, 0x8B, 0xBD, 0x94, 0xFD, 0xFF, 0xFF, 0x8B, 0x9D, 0xD0, 0xFD, 0xFF, 0xFF, 0x8D, 0x45, 0xF4, 0x50, 0x8D, 0x85, 0xDC, 0xFD, 0xFF, 0xFF, 0xE8, 0x8B, 0xF4, 0xFF, 0xFF, 0x83, 0xBD, 0x98, 0xFD, 0xFF, 0xFF, 0x00, 0x59, 0x59, 0x75, 0xA6, 0xEB, 0x23, 0x83, 0x8D, 0xDC, 0xFD, 0xFF, 0xFF, 0xFF, 0xEB, 0x1A, 0x8B, 0xBD, 0x94, 0xFD, 0xFF, 0xFF, 0x50, 0xFF, 0xB5, 0xE0, 0xFD, 0xFF, 0xFF, 0x8D, 0x85, 0xDC, 0xFD, 0xFF, 0xFF, 0xE8, 0x5D, 0xF4, 0xFF, 0xFF, 0x59, 0x59, 0x83, 0xBD, 0xDC, 0xFD, 0xFF, 0xFF, 0x00, 0x7C, 0x32, 0xF6, 0x85, 0xF0, 0xFD, 0xFF, 0xFF, 0x04, 0x74, 0x29, 0x8B, 0xBD, 0xA0, 0xFD, 0xFF, 0xFF, 0xEB, 0x1D, 0x8B, 0x8D, 0xD0, 0xFD, 0xFF, 0xFF, 0x8D, 0xB5, 0xDC, 0xFD, 0xFF, 0xFF, 0xB0, 0x20, 0x4F, 0xE8, 0xFA, 0xF3, 0xFF, 0xFF, 0x83, 0xBD, 0xDC, 0xFD, 0xFF, 0xFF, 0xFF, 0x74, 0x04, 0x85, 0xFF, 0x7F, 0xDF, 0x83, 0xBD, 0xB8, 0xFD, 0xFF, 0xFF, 0x00, 0x74, 0x13, 0xFF, 0xB5, 0xB8, 0xFD, 0xFF, 0xFF, 0xE8, 0xCA, 0xDC, 0xFF, 0xFF, 0x83, 0xA5, 0xB8, 0xFD, 0xFF, 0xFF, 0x00, 0x59, 0x8B, 0x9D, 0xC0, 0xFD, 0xFF, 0xFF, 0x8A, 0x03, 0x88, 0x85, 0xEF, 0xFD, 0xFF, 0xFF, 0x84, 0xC0, 0x74, 0x13, 0x8B, 0x8D, 0x8C, 0xFD, 0xFF, 0xFF, 0x8B, 0xBD, 0xE4, 0xFD, 0xFF, 0xFF, 0x8A, 0xD0, 0xE9, 0x72, 0xF5, 0xFF, 0xFF, 0x80, 0xBD, 0xB0, 0xFD, 0xFF, 0xFF, 0x00, 0x74, 0x0A, 0x8B, 0x85, 0xAC, 0xFD, 0xFF, 0xFF, 0x83, 0x60, 0x70, 0xFD, 0x8B, 0x85, 0xDC, 0xFD, 0xFF, 0xFF, 0x8B, 0x4D, 0xFC, 0x5F, 0x5E, 0x33, 0xCD, 0x5B, 0xE8, 0xC0, 0xD8, 0xFF, 0xFF, 0xC9, 0xC3, 0x90, 0x9C, 0x3F, 0x40, 0x00, 0x9B, 0x3D, 0x40, 0x00, 0xCB, 0x3D, 0x40, 0x00, 0x29, 0x3E, 0x40, 0x00, 0x75, 0x3E, 0x40, 0x00, 0x80, 0x3E, 0x40, 0x00, 0xC6, 0x3E, 0x40, 0x00, 0xF6, 0x3F, 0x40, 0x00, 0x6A, 0x00, 0x68, 0x00, 0x10, 0x00, 0x00, 0x6A, 0x00, 0xFF, 0x15, 0xA0, 0xA0, 0x40, 0x00, 0x33, 0xC9, 0x85, 0xC0, 0x0F, 0x95, 0xC1, 0xA3, 0x28, 0xE1, 0x40, 0x00, 0x8B, 0xC1, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x68, 0x14, 0xA6, 0x40, 0x00, 0xFF, 0x15, 0x90, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x74, 0x15, 0x68, 0x04, 0xA6, 0x40, 0x00, 0x50, 0xFF, 0x15, 0x9C, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x74, 0x05, 0xFF, 0x75, 0x08, 0xFF, 0xD0, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0xFF, 0x75, 0x08, 0xE8, 0xC8, 0xFF, 0xFF, 0xFF, 0x59, 0xFF, 0x75, 0x08, 0xFF, 0x15, 0x2C, 0xA0, 0x40, 0x00, 0xCC, 0x6A, 0x08, 0xE8, 0x4E, 0x15, 0x00, 0x00, 0x59, 0xC3,
				0x6A, 0x08, 0xE8, 0x6C, 0x14, 0x00, 0x00, 0x59, 0xC3, 0x8B, 0xFF, 0x56, 0xE8, 0x6A, 0xE9, 0xFF, 0xFF, 0x8B, 0xF0, 0x56, 0xE8, 0x94, 0x04, 0x00, 0x00, 0x56, 0xE8, 0xE5, 0xEE, 0xFF, 0xFF, 0x56, 0xE8, 0x02, 0x32, 0x00, 0x00, 0x56, 0xE8, 0xED, 0x31, 0x00, 0x00, 0x56, 0xE8, 0xE2, 0x2F, 0x00, 0x00, 0x56, 0xE8, 0xCB, 0x2F, 0x00, 0x00, 0x83, 0xC4, 0x18, 0x5E, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x56, 0x8B, 0x75, 0x08, 0x33, 0xC0, 0xEB, 0x0F, 0x85, 0xC0, 0x75, 0x10, 0x8B, 0x0E, 0x85, 0xC9, 0x74, 0x02, 0xFF, 0xD1, 0x83, 0xC6, 0x04, 0x3B, 0x75, 0x0C, 0x72, 0xEC, 0x5E, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x83, 0x3D, 0x90, 0xFB, 0x40, 0x00, 0x00, 0x74, 0x19, 0x68, 0x90, 0xFB, 0x40, 0x00, 0xE8, 0x8C, 0x33, 0x00, 0x00, 0x59, 0x85, 0xC0, 0x74, 0x0A, 0xFF, 0x75, 0x08, 0xFF, 0x15, 0x90, 0xFB, 0x40, 0x00, 0x59, 0xE8, 0xF2, 0x2C, 0x00, 0x00, 0x68, 0x54, 0xA1, 0x40, 0x00, 0x68, 0x3C, 0xA1, 0x40, 0x00, 0xE8, 0xA1, 0xFF, 0xFF, 0xFF, 0x59, 0x59, 0x85, 0xC0, 0x75, 0x54, 0x56, 0x57, 0x68, 0x9F, 0x55, 0x40, 0x00, 0xE8, 0xAC, 0x32, 0x00, 0x00, 0xB8, 0x34, 0xA1, 0x40, 0x00, 0xBE, 0x38, 0xA1, 0x40, 0x00, 0x59, 0x8B, 0xF8, 0x3B, 0xC6, 0x73, 0x0F, 0x8B, 0x07, 0x85, 0xC0, 0x74, 0x02, 0xFF, 0xD0, 0x83, 0xC7, 0x04, 0x3B, 0xFE, 0x72, 0xF1, 0x83, 0x3D, 0x94, 0xFB, 0x40, 0x00, 0x00, 0x5F, 0x5E, 0x74, 0x1B, 0x68, 0x94, 0xFB, 0x40, 0x00, 0xE8, 0x22, 0x33, 0x00, 0x00, 0x59, 0x85, 0xC0, 0x74, 0x0C, 0x6A, 0x00, 0x6A, 0x02, 0x6A, 0x00, 0xFF, 0x15, 0x94, 0xFB, 0x40, 0x00, 0x33, 0xC0, 0x5D, 0xC3, 0x6A, 0x20, 0x68, 0xB0, 0xC1, 0x40, 0x00, 0xE8, 0x41, 0x0C, 0x00, 0x00, 0x6A, 0x08, 0xE8, 0x42, 0x14, 0x00, 0x00, 0x59, 0x83, 0x65, 0xFC, 0x00, 0x33, 0xC0, 0x40, 0x39, 0x05, 0x5C, 0xE1, 0x40, 0x00, 0x0F, 0x84, 0xD8, 0x00, 0x00, 0x00, 0xA3, 0x58, 0xE1, 0x40, 0x00, 0x8A, 0x45, 0x10, 0xA2, 0x54, 0xE1, 0x40, 0x00, 0x83, 0x7D, 0x0C, 0x00, 0x0F, 0x85, 0xA0, 0x00, 0x00, 0x00, 0xFF, 0x35, 0x88, 0xFB, 0x40, 0x00, 0x8B, 0x35, 0x88, 0xA0, 0x40, 0x00, 0xFF, 0xD6, 0x8B, 0xD8, 0x89, 0x5D, 0xD0, 0x85, 0xDB, 0x74, 0x68, 0xFF, 0x35, 0x84, 0xFB, 0x40, 0x00, 0xFF, 0xD6, 0x8B, 0xF8, 0x89, 0x7D, 0xD4, 0x89, 0x5D, 0xDC, 0x89, 0x7D, 0xD8, 0x83, 0xEF, 0x04, 0x89, 0x7D, 0xD4, 0x3B, 0xFB, 0x72, 0x4B, 0xE8, 0x0D, 0xE8, 0xFF, 0xFF, 0x39, 0x07, 0x74, 0xED, 0x3B, 0xFB, 0x72, 0x3E, 0xFF, 0x37, 0xFF, 0xD6, 0x8B, 0xD8, 0xE8, 0xFA, 0xE7, 0xFF, 0xFF, 0x89, 0x07, 0xFF, 0xD3, 0xFF, 0x35, 0x88, 0xFB, 0x40, 0x00, 0xFF, 0xD6, 0x8B, 0xD8, 0xFF, 0x35, 0x84, 0xFB, 0x40, 0x00, 0xFF, 0xD6, 0x39, 0x5D, 0xDC, 0x75, 0x05, 0x39, 0x45, 0xD8, 0x74, 0x0E, 0x89, 0x5D, 0xDC, 0x89, 0x5D, 0xD0, 0x89, 0x45, 0xD8, 0x8B, 0xF8, 0x89, 0x7D, 0xD4, 0x8B, 0x5D, 0xD0, 0xEB, 0xAB, 0xC7, 0x45, 0xE4, 0x58, 0xA1, 0x40, 0x00, 0x81, 0x7D, 0xE4, 0x64, 0xA1, 0x40, 0x00, 0x73, 0x11, 0x8B, 0x45, 0xE4, 0x8B, 0x00, 0x85, 0xC0, 0x74, 0x02, 0xFF, 0xD0, 0x83, 0x45, 0xE4, 0x04, 0xEB, 0xE6, 0xC7, 0x45, 0xE0, 0x68, 0xA1, 0x40, 0x00, 0x81, 0x7D, 0xE0, 0x6C, 0xA1, 0x40, 0x00, 0x73, 0x11, 0x8B, 0x45, 0xE0, 0x8B, 0x00, 0x85, 0xC0, 0x74, 0x02, 0xFF, 0xD0, 0x83, 0x45, 0xE0, 0x04,
				0xEB, 0xE6, 0xC7, 0x45, 0xFC, 0xFE, 0xFF, 0xFF, 0xFF, 0xE8, 0x20, 0x00, 0x00, 0x00, 0x83, 0x7D, 0x10, 0x00, 0x75, 0x29, 0xC7, 0x05, 0x5C, 0xE1, 0x40, 0x00, 0x01, 0x00, 0x00, 0x00, 0x6A, 0x08, 0xE8, 0x5A, 0x12, 0x00, 0x00, 0x59, 0xFF, 0x75, 0x08, 0xE8, 0xBD, 0xFD, 0xFF, 0xFF, 0x83, 0x7D, 0x10, 0x00, 0x74, 0x08, 0x6A, 0x08, 0xE8, 0x44, 0x12, 0x00, 0x00, 0x59, 0xC3, 0xE8, 0x53, 0x0B, 0x00, 0x00, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x6A, 0x00, 0x6A, 0x00, 0xFF, 0x75, 0x08, 0xE8, 0xAF, 0xFE, 0xFF, 0xFF, 0x83, 0xC4, 0x0C, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x6A, 0x00, 0x6A, 0x01, 0xFF, 0x75, 0x08, 0xE8, 0x99, 0xFE, 0xFF, 0xFF, 0x83, 0xC4, 0x0C, 0x5D, 0xC3, 0x6A, 0x01, 0x6A, 0x00, 0x6A, 0x00, 0xE8, 0x89, 0xFE, 0xFF, 0xFF, 0x83, 0xC4, 0x0C, 0xC3, 0x6A, 0x01, 0x6A, 0x01, 0x6A, 0x00, 0xE8, 0x7A, 0xFE, 0xFF, 0xFF, 0x83, 0xC4, 0x0C, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0xE8, 0xE9, 0x01, 0x00, 0x00, 0xFF, 0x75, 0x08, 0xE8, 0x32, 0x00, 0x00, 0x00, 0x59, 0x68, 0xFF, 0x00, 0x00, 0x00, 0xE8, 0xAF, 0xFF, 0xFF, 0xFF, 0xCC, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x33, 0xC0, 0x8B, 0x4D, 0x08, 0x3B, 0x0C, 0xC5, 0xB0, 0xAE, 0x40, 0x00, 0x74, 0x0A, 0x40, 0x83, 0xF8, 0x16, 0x72, 0xEE, 0x33, 0xC0, 0x5D, 0xC3, 0x8B, 0x04, 0xC5, 0xB4, 0xAE, 0x40, 0x00, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x81, 0xEC, 0xFC, 0x01, 0x00, 0x00, 0xA1, 0x04, 0xD0, 0x40, 0x00, 0x33, 0xC5, 0x89, 0x45, 0xFC, 0x53, 0x56, 0x8B, 0x75, 0x08, 0x57, 0x56, 0xE8, 0xB9, 0xFF, 0xFF, 0xFF, 0x8B, 0xF8, 0x33, 0xDB, 0x59, 0x89, 0xBD, 0x04, 0xFE, 0xFF, 0xFF, 0x3B, 0xFB, 0x0F, 0x84, 0x6C, 0x01, 0x00, 0x00, 0x6A, 0x03, 0xE8, 0xEC, 0x34, 0x00, 0x00, 0x59, 0x83, 0xF8, 0x01, 0x0F, 0x84, 0x07, 0x01, 0x00, 0x00, 0x6A, 0x03, 0xE8, 0xDB, 0x34, 0x00, 0x00, 0x59, 0x85, 0xC0, 0x75, 0x0D, 0x83, 0x3D, 0x00, 0xD0, 0x40, 0x00, 0x01, 0x0F, 0x84, 0xEE, 0x00, 0x00, 0x00, 0x81, 0xFE, 0xFC, 0x00, 0x00, 0x00, 0x0F, 0x84, 0x36, 0x01, 0x00, 0x00, 0x68, 0xEC, 0xAF, 0x40, 0x00, 0x68, 0x14, 0x03, 0x00, 0x00, 0xBF, 0x60, 0xE1, 0x40, 0x00, 0x57, 0xE8, 0x45, 0x34, 0x00, 0x00, 0x83, 0xC4, 0x0C, 0x85, 0xC0, 0x0F, 0x85, 0xB8, 0x00, 0x00, 0x00, 0x68, 0x04, 0x01, 0x00, 0x00, 0xBE, 0x92, 0xE1, 0x40, 0x00, 0x56, 0x53, 0x66, 0xA3, 0x9A, 0xE3, 0x40, 0x00, 0xFF, 0x15, 0xAC, 0xA0, 0x40, 0x00, 0xBB, 0xFB, 0x02, 0x00, 0x00, 0x85, 0xC0, 0x75, 0x1F, 0x68, 0xBC, 0xAF, 0x40, 0x00, 0x53, 0x56, 0xE8, 0x0D, 0x34, 0x00, 0x00, 0x83, 0xC4, 0x0C, 0x85, 0xC0, 0x74, 0x0C, 0x33, 0xC0, 0x50, 0x50, 0x50, 0x50, 0x50, 0xE8, 0xAD, 0xEC, 0xFF, 0xFF, 0x56, 0xE8, 0xD9, 0x33, 0x00, 0x00, 0x40, 0x59, 0x83, 0xF8, 0x3C, 0x76, 0x2A, 0x56, 0xE8, 0xCC, 0x33, 0x00, 0x00, 0x8D, 0x04, 0x45, 0x1C, 0xE1, 0x40, 0x00, 0x8B, 0xC8, 0x2B, 0xCE, 0x6A, 0x03, 0xD1, 0xF9, 0x68, 0xB4, 0xAF, 0x40, 0x00, 0x2B, 0xD9, 0x53, 0x50, 0xE8, 0xE2, 0x32, 0x00, 0x00, 0x83, 0xC4, 0x14, 0x85, 0xC0, 0x75, 0xBD, 0x68, 0xAC, 0xAF, 0x40, 0x00, 0xBE, 0x14, 0x03, 0x00, 0x00, 0x56, 0x57, 0xE8, 0x55, 0x32, 0x00, 0x00, 0x83, 0xC4, 0x0C, 0x85, 0xC0, 0x75, 0xA5, 0xFF, 0xB5, 0x04, 0xFE, 0xFF, 0xFF, 0x56, 0x57, 0xE8, 0x41,
				0x32, 0x00, 0x00, 0x83, 0xC4, 0x0C, 0x85, 0xC0, 0x75, 0x91, 0x68, 0x10, 0x20, 0x01, 0x00, 0x68, 0x60, 0xAF, 0x40, 0x00, 0x57, 0xE8, 0xBE, 0x30, 0x00, 0x00, 0x83, 0xC4, 0x0C, 0xEB, 0x5E, 0x53, 0x53, 0x53, 0x53, 0x53, 0xE9, 0x79, 0xFF, 0xFF, 0xFF, 0x6A, 0xF4, 0xFF, 0x15, 0xA8, 0xA0, 0x40, 0x00, 0x8B, 0xF0, 0x3B, 0xF3, 0x74, 0x46, 0x83, 0xFE, 0xFF, 0x74, 0x41, 0x33, 0xC0, 0x8A, 0x0C, 0x47, 0x88, 0x8C, 0x05, 0x08, 0xFE, 0xFF, 0xFF, 0x66, 0x39, 0x1C, 0x47, 0x74, 0x08, 0x40, 0x3D, 0xF4, 0x01, 0x00, 0x00, 0x72, 0xE8, 0x53, 0x8D, 0x85, 0x04, 0xFE, 0xFF, 0xFF, 0x50, 0x8D, 0x85, 0x08, 0xFE, 0xFF, 0xFF, 0x50, 0x88, 0x5D, 0xFB, 0xE8, 0x9F, 0x1C, 0x00, 0x00, 0x59, 0x50, 0x8D, 0x85, 0x08, 0xFE, 0xFF, 0xFF, 0x50, 0x56, 0xFF, 0x15, 0xA4, 0xA0, 0x40, 0x00, 0x8B, 0x4D, 0xFC, 0x5F, 0x5E, 0x33, 0xCD, 0x5B, 0xE8, 0xC1, 0xD3, 0xFF, 0xFF, 0xC9, 0xC3, 0x6A, 0x03, 0xE8, 0x71, 0x33, 0x00, 0x00, 0x59, 0x83, 0xF8, 0x01, 0x74, 0x15, 0x6A, 0x03, 0xE8, 0x64, 0x33, 0x00, 0x00, 0x59, 0x85, 0xC0, 0x75, 0x1F, 0x83, 0x3D, 0x00, 0xD0, 0x40, 0x00, 0x01, 0x75, 0x16, 0x68, 0xFC, 0x00, 0x00, 0x00, 0xE8, 0x25, 0xFE, 0xFF, 0xFF, 0x68, 0xFF, 0x00, 0x00, 0x00, 0xE8, 0x1B, 0xFE, 0xFF, 0xFF, 0x59, 0x59, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x8B, 0x45, 0x08, 0xA3, 0x88, 0xE7, 0x40, 0x00, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0xFF, 0x35, 0x88, 0xE7, 0x40, 0x00, 0xFF, 0x15, 0x88, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x74, 0x0F, 0xFF, 0x75, 0x08, 0xFF, 0xD0, 0x59, 0x85, 0xC0, 0x74, 0x05, 0x33, 0xC0, 0x40, 0x5D, 0xC3, 0x33, 0xC0, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x8B, 0x45, 0x08, 0x8B, 0x00, 0x81, 0x38, 0x63, 0x73, 0x6D, 0xE0, 0x75, 0x2A, 0x83, 0x78, 0x10, 0x03, 0x75, 0x24, 0x8B, 0x40, 0x14, 0x3D, 0x20, 0x05, 0x93, 0x19, 0x74, 0x15, 0x3D, 0x21, 0x05, 0x93, 0x19, 0x74, 0x0E, 0x3D, 0x22, 0x05, 0x93, 0x19, 0x74, 0x07, 0x3D, 0x00, 0x40, 0x99, 0x01, 0x75, 0x05, 0xE8, 0xA9, 0x2A, 0x00, 0x00, 0x33, 0xC0, 0x5D, 0xC2, 0x04, 0x00, 0x68, 0x70, 0x4D, 0x40, 0x00, 0xFF, 0x15, 0x58, 0xA0, 0x40, 0x00, 0x33, 0xC0, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x56, 0xE8, 0x73, 0xE5, 0xFF, 0xFF, 0x8B, 0xF0, 0x85, 0xF6, 0x0F, 0x84, 0x32, 0x01, 0x00, 0x00, 0x8B, 0x4E, 0x5C, 0x8B, 0x55, 0x08, 0x8B, 0xC1, 0x57, 0x39, 0x10, 0x74, 0x0D, 0x83, 0xC0, 0x0C, 0x8D, 0xB9, 0x90, 0x00, 0x00, 0x00, 0x3B, 0xC7, 0x72, 0xEF, 0x81, 0xC1, 0x90, 0x00, 0x00, 0x00, 0x3B, 0xC1, 0x73, 0x04, 0x39, 0x10, 0x74, 0x02, 0x33, 0xC0, 0x85, 0xC0, 0x74, 0x07, 0x8B, 0x50, 0x08, 0x85, 0xD2, 0x75, 0x07, 0x33, 0xC0, 0xE9, 0xF5, 0x00, 0x00, 0x00, 0x83, 0xFA, 0x05, 0x75, 0x0C, 0x83, 0x60, 0x08, 0x00, 0x33, 0xC0, 0x40, 0xE9, 0xE4, 0x00, 0x00, 0x00, 0x83, 0xFA, 0x01, 0x0F, 0x84, 0xD8, 0x00, 0x00, 0x00, 0x8B, 0x4D, 0x0C, 0x53, 0x8B, 0x5E, 0x60, 0x89, 0x4E, 0x60, 0x8B, 0x48, 0x04, 0x83, 0xF9, 0x08, 0x0F, 0x85, 0xB6, 0x00, 0x00, 0x00, 0x6A, 0x24, 0x59, 0x8B, 0x7E, 0x5C, 0x83, 0x64, 0x39, 0x08, 0x00, 0x83, 0xC1, 0x0C, 0x81, 0xF9, 0x90, 0x00, 0x00, 0x00, 0x7C, 0xED, 0x8B, 0x00, 0x8B, 0x7E, 0x64, 0x3D, 0x8E, 0x00, 0x00, 0xC0, 0x75, 0x09, 0xC7, 0x46, 0x64, 0x83, 0x00,
				0x00, 0x00, 0xEB, 0x7E, 0x3D, 0x90, 0x00, 0x00, 0xC0, 0x75, 0x09, 0xC7, 0x46, 0x64, 0x81, 0x00, 0x00, 0x00, 0xEB, 0x6E, 0x3D, 0x91, 0x00, 0x00, 0xC0, 0x75, 0x09, 0xC7, 0x46, 0x64, 0x84, 0x00, 0x00, 0x00, 0xEB, 0x5E, 0x3D, 0x93, 0x00, 0x00, 0xC0, 0x75, 0x09, 0xC7, 0x46, 0x64, 0x85, 0x00, 0x00, 0x00, 0xEB, 0x4E, 0x3D, 0x8D, 0x00, 0x00, 0xC0, 0x75, 0x09, 0xC7, 0x46, 0x64, 0x82, 0x00, 0x00, 0x00, 0xEB, 0x3E, 0x3D, 0x8F, 0x00, 0x00, 0xC0, 0x75, 0x09, 0xC7, 0x46, 0x64, 0x86, 0x00, 0x00, 0x00, 0xEB, 0x2E, 0x3D, 0x92, 0x00, 0x00, 0xC0, 0x75, 0x09, 0xC7, 0x46, 0x64, 0x8A, 0x00, 0x00, 0x00, 0xEB, 0x1E, 0x3D, 0xB5, 0x02, 0x00, 0xC0, 0x75, 0x09, 0xC7, 0x46, 0x64, 0x8D, 0x00, 0x00, 0x00, 0xEB, 0x0E, 0x3D, 0xB4, 0x02, 0x00, 0xC0, 0x75, 0x07, 0xC7, 0x46, 0x64, 0x8E, 0x00, 0x00, 0x00, 0xFF, 0x76, 0x64, 0x6A, 0x08, 0xFF, 0xD2, 0x59, 0x89, 0x7E, 0x64, 0xEB, 0x07, 0x83, 0x60, 0x08, 0x00, 0x51, 0xFF, 0xD2, 0x59, 0x89, 0x5E, 0x60, 0x5B, 0x83, 0xC8, 0xFF, 0x5F, 0x5E, 0x5D, 0xC3, 0x8B, 0xFF, 0x56, 0x57, 0x33, 0xFF, 0x39, 0x3D, 0x8C, 0xFB, 0x40, 0x00, 0x75, 0x05, 0xE8, 0x93, 0xDF, 0xFF, 0xFF, 0x8B, 0x35, 0x98, 0xFB, 0x40, 0x00, 0x85, 0xF6, 0x75, 0x05, 0xBE, 0x8F, 0xA5, 0x40, 0x00, 0x8A, 0x06, 0x3C, 0x20, 0x77, 0x08, 0x84, 0xC0, 0x74, 0x2E, 0x85, 0xFF, 0x74, 0x24, 0x3C, 0x22, 0x75, 0x09, 0x33, 0xC9, 0x85, 0xFF, 0x0F, 0x94, 0xC1, 0x8B, 0xF9, 0x0F, 0xB6, 0xC0, 0x50, 0xE8, 0xBA, 0x31, 0x00, 0x00, 0x59, 0x85, 0xC0, 0x74, 0x01, 0x46, 0x46, 0xEB, 0xD3, 0x3C, 0x20, 0x77, 0x07, 0x46, 0x8A, 0x06, 0x84, 0xC0, 0x75, 0xF5, 0x5F, 0x8B, 0xC6, 0x5E, 0xC3, 0xCC, 0xCC, 0xCC, 0x83, 0x3D, 0x8C, 0xFB, 0x40, 0x00, 0x00, 0x75, 0x05, 0xE8, 0x36, 0xDF, 0xFF, 0xFF, 0x56, 0x8B, 0x35, 0xC0, 0xDD, 0x40, 0x00, 0x57, 0x33, 0xFF, 0x85, 0xF6, 0x75, 0x18, 0x83, 0xC8, 0xFF, 0xE9, 0x91, 0x00, 0x00, 0x00, 0x3C, 0x3D, 0x74, 0x01, 0x47, 0x56, 0xE8, 0xE5, 0x19, 0x00, 0x00, 0x59, 0x8D, 0x74, 0x06, 0x01, 0x8A, 0x06, 0x84, 0xC0, 0x75, 0xEA, 0x6A, 0x04, 0x47, 0x57, 0xE8, 0xA1, 0x0E, 0x00, 0x00, 0x8B, 0xF8, 0x59, 0x59, 0x89, 0x3D, 0x3C, 0xE1, 0x40, 0x00, 0x85, 0xFF, 0x74, 0xCB, 0x8B, 0x35, 0xC0, 0xDD, 0x40, 0x00, 0x53, 0xEB, 0x33, 0x56, 0xE8, 0xB4, 0x19, 0x00, 0x00, 0x80, 0x3E, 0x3D, 0x59, 0x8D, 0x58, 0x01, 0x74, 0x22, 0x6A, 0x01, 0x53, 0xE8, 0x73, 0x0E, 0x00, 0x00, 0x59, 0x59, 0x89, 0x07, 0x85, 0xC0, 0x74, 0x3F, 0x56, 0x53, 0x50, 0xE8, 0x29, 0x15, 0x00, 0x00, 0x83, 0xC4, 0x0C, 0x85, 0xC0, 0x75, 0x47, 0x83, 0xC7, 0x04, 0x03, 0xF3, 0x80, 0x3E, 0x00, 0x75, 0xC8, 0xFF, 0x35, 0xC0, 0xDD, 0x40, 0x00, 0xE8, 0x6D, 0xD4, 0xFF, 0xFF, 0x83, 0x25, 0xC0, 0xDD, 0x40, 0x00, 0x00, 0x83, 0x27, 0x00, 0xC7, 0x05, 0x80, 0xFB, 0x40, 0x00, 0x01, 0x00, 0x00, 0x00, 0x33, 0xC0, 0x59, 0x5B, 0x5F, 0x5E, 0xC3, 0xFF, 0x35, 0x3C, 0xE1, 0x40, 0x00, 0xE8, 0x47, 0xD4, 0xFF, 0xFF, 0x83, 0x25, 0x3C, 0xE1, 0x40, 0x00, 0x00, 0x83, 0xC8, 0xFF, 0xEB, 0xE4, 0x33, 0xC0, 0x50, 0x50, 0x50, 0x50, 0x50, 0xE8, 0x81, 0xE8, 0xFF, 0xFF, 0xCC, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x51, 0x8B, 0x4D, 0x10, 0x53, 0x33, 0xC0, 0x56, 0x89, 0x07, 0x8B, 0xF2, 0x8B, 0x55, 0x0C,
				0xC7, 0x01, 0x01, 0x00, 0x00, 0x00, 0x39, 0x45, 0x08, 0x74, 0x09, 0x8B, 0x5D, 0x08, 0x83, 0x45, 0x08, 0x04, 0x89, 0x13, 0x89, 0x45, 0xFC, 0x80, 0x3E, 0x22, 0x75, 0x10, 0x33, 0xC0, 0x39, 0x45, 0xFC, 0xB3, 0x22, 0x0F, 0x94, 0xC0, 0x46, 0x89, 0x45, 0xFC, 0xEB, 0x3C, 0xFF, 0x07, 0x85, 0xD2, 0x74, 0x08, 0x8A, 0x06, 0x88, 0x02, 0x42, 0x89, 0x55, 0x0C, 0x8A, 0x1E, 0x0F, 0xB6, 0xC3, 0x50, 0x46, 0xE8, 0x68, 0x30, 0x00, 0x00, 0x59, 0x85, 0xC0, 0x74, 0x13, 0xFF, 0x07, 0x83, 0x7D, 0x0C, 0x00, 0x74, 0x0A, 0x8B, 0x4D, 0x0C, 0x8A, 0x06, 0xFF, 0x45, 0x0C, 0x88, 0x01, 0x46, 0x8B, 0x55, 0x0C, 0x8B, 0x4D, 0x10, 0x84, 0xDB, 0x74, 0x32, 0x83, 0x7D, 0xFC, 0x00, 0x75, 0xA9, 0x80, 0xFB, 0x20, 0x74, 0x05, 0x80, 0xFB, 0x09, 0x75, 0x9F, 0x85, 0xD2, 0x74, 0x04, 0xC6, 0x42, 0xFF, 0x00, 0x83, 0x65, 0xFC, 0x00, 0x80, 0x3E, 0x00, 0x0F, 0x84, 0xE9, 0x00, 0x00, 0x00, 0x8A, 0x06, 0x3C, 0x20, 0x74, 0x04, 0x3C, 0x09, 0x75, 0x06, 0x46, 0xEB, 0xF3, 0x4E, 0xEB, 0xE3, 0x80, 0x3E, 0x00, 0x0F, 0x84, 0xD0, 0x00, 0x00, 0x00, 0x83, 0x7D, 0x08, 0x00, 0x74, 0x09, 0x8B, 0x45, 0x08, 0x83, 0x45, 0x08, 0x04, 0x89, 0x10, 0xFF, 0x01, 0x33, 0xDB, 0x43, 0x33, 0xC9, 0xEB, 0x02, 0x46, 0x41, 0x80, 0x3E, 0x5C, 0x74, 0xF9, 0x80, 0x3E, 0x22, 0x75, 0x26, 0xF6, 0xC1, 0x01, 0x75, 0x1F, 0x83, 0x7D, 0xFC, 0x00, 0x74, 0x0C, 0x8D, 0x46, 0x01, 0x80, 0x38, 0x22, 0x75, 0x04, 0x8B, 0xF0, 0xEB, 0x0D, 0x33, 0xC0, 0x33, 0xDB, 0x39, 0x45, 0xFC, 0x0F, 0x94, 0xC0, 0x89, 0x45, 0xFC, 0xD1, 0xE9, 0x85, 0xC9, 0x74, 0x12, 0x49, 0x85, 0xD2, 0x74, 0x04, 0xC6, 0x02, 0x5C, 0x42, 0xFF, 0x07, 0x85, 0xC9, 0x75, 0xF1, 0x89, 0x55, 0x0C, 0x8A, 0x06, 0x84, 0xC0, 0x74, 0x55, 0x83, 0x7D, 0xFC, 0x00, 0x75, 0x08, 0x3C, 0x20, 0x74, 0x4B, 0x3C, 0x09, 0x74, 0x47, 0x85, 0xDB, 0x74, 0x3D, 0x0F, 0xBE, 0xC0, 0x50, 0x85, 0xD2, 0x74, 0x23, 0xE8, 0x83, 0x2F, 0x00, 0x00, 0x59, 0x85, 0xC0, 0x74, 0x0D, 0x8A, 0x06, 0x8B, 0x4D, 0x0C, 0xFF, 0x45, 0x0C, 0x88, 0x01, 0x46, 0xFF, 0x07, 0x8B, 0x4D, 0x0C, 0x8A, 0x06, 0xFF, 0x45, 0x0C, 0x88, 0x01, 0xEB, 0x0D, 0xE8, 0x60, 0x2F, 0x00, 0x00, 0x59, 0x85, 0xC0, 0x74, 0x03, 0x46, 0xFF, 0x07, 0xFF, 0x07, 0x8B, 0x55, 0x0C, 0x46, 0xE9, 0x56, 0xFF, 0xFF, 0xFF, 0x85, 0xD2, 0x74, 0x07, 0xC6, 0x02, 0x00, 0x42, 0x89, 0x55, 0x0C, 0xFF, 0x07, 0x8B, 0x4D, 0x10, 0xE9, 0x0E, 0xFF, 0xFF, 0xFF, 0x8B, 0x45, 0x08, 0x5E, 0x5B, 0x85, 0xC0, 0x74, 0x03, 0x83, 0x20, 0x00, 0xFF, 0x01, 0xC9, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x83, 0xEC, 0x0C, 0x53, 0x33, 0xDB, 0x56, 0x57, 0x39, 0x1D, 0x8C, 0xFB, 0x40, 0x00, 0x75, 0x05, 0xE8, 0xB4, 0xDC, 0xFF, 0xFF, 0x68, 0x04, 0x01, 0x00, 0x00, 0xBE, 0x90, 0xE7, 0x40, 0x00, 0x56, 0x53, 0x88, 0x1D, 0x94, 0xE8, 0x40, 0x00, 0xFF, 0x15, 0xB0, 0xA0, 0x40, 0x00, 0xA1, 0x98, 0xFB, 0x40, 0x00, 0x89, 0x35, 0x4C, 0xE1, 0x40, 0x00, 0x3B, 0xC3, 0x74, 0x07, 0x89, 0x45, 0xFC, 0x38, 0x18, 0x75, 0x03, 0x89, 0x75, 0xFC, 0x8B, 0x55, 0xFC, 0x8D, 0x45, 0xF8, 0x50, 0x53, 0x53, 0x8D, 0x7D, 0xF4, 0xE8, 0x0A, 0xFE, 0xFF, 0xFF, 0x8B, 0x45, 0xF8, 0x83, 0xC4, 0x0C, 0x3D, 0xFF, 0xFF, 0xFF, 0x3F, 0x73, 0x4A, 0x8B, 0x4D, 0xF4, 0x83, 0xF9,
				0xFF, 0x73, 0x42, 0x8B, 0xF8, 0xC1, 0xE7, 0x02, 0x8D, 0x04, 0x0F, 0x3B, 0xC1, 0x72, 0x36, 0x50, 0xE8, 0xA6, 0x0B, 0x00, 0x00, 0x8B, 0xF0, 0x59, 0x3B, 0xF3, 0x74, 0x29, 0x8B, 0x55, 0xFC, 0x8D, 0x45, 0xF8, 0x50, 0x03, 0xFE, 0x57, 0x56, 0x8D, 0x7D, 0xF4, 0xE8, 0xC9, 0xFD, 0xFF, 0xFF, 0x8B, 0x45, 0xF8, 0x83, 0xC4, 0x0C, 0x48, 0xA3, 0x30, 0xE1, 0x40, 0x00, 0x89, 0x35, 0x34, 0xE1, 0x40, 0x00, 0x33, 0xC0, 0xEB, 0x03, 0x83, 0xC8, 0xFF, 0x5F, 0x5E, 0x5B, 0xC9, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x83, 0xEC, 0x0C, 0x53, 0x56, 0xFF, 0x15, 0xBC, 0xA0, 0x40, 0x00, 0x8B, 0xD8, 0x33, 0xF6, 0x3B, 0xDE, 0x75, 0x04, 0x33, 0xC0, 0xEB, 0x77, 0x66, 0x39, 0x33, 0x74, 0x10, 0x83, 0xC0, 0x02, 0x66, 0x39, 0x30, 0x75, 0xF8, 0x83, 0xC0, 0x02, 0x66, 0x39, 0x30, 0x75, 0xF0, 0x57, 0x8B, 0x3D, 0xB8, 0xA0, 0x40, 0x00, 0x56, 0x56, 0x56, 0x2B, 0xC3, 0x56, 0xD1, 0xF8, 0x40, 0x50, 0x53, 0x56, 0x56, 0x89, 0x45, 0xF4, 0xFF, 0xD7, 0x89, 0x45, 0xF8, 0x3B, 0xC6, 0x74, 0x38, 0x50, 0xE8, 0x17, 0x0B, 0x00, 0x00, 0x59, 0x89, 0x45, 0xFC, 0x3B, 0xC6, 0x74, 0x2A, 0x56, 0x56, 0xFF, 0x75, 0xF8, 0x50, 0xFF, 0x75, 0xF4, 0x53, 0x56, 0x56, 0xFF, 0xD7, 0x85, 0xC0, 0x75, 0x0C, 0xFF, 0x75, 0xFC, 0xE8, 0x60, 0xD1, 0xFF, 0xFF, 0x59, 0x89, 0x75, 0xFC, 0x53, 0xFF, 0x15, 0xB4, 0xA0, 0x40, 0x00, 0x8B, 0x45, 0xFC, 0xEB, 0x09, 0x53, 0xFF, 0x15, 0xB4, 0xA0, 0x40, 0x00, 0x33, 0xC0, 0x5F, 0x5E, 0x5B, 0xC9, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x83, 0xEC, 0x4C, 0x56, 0x8D, 0x45, 0xB4, 0x50, 0xFF, 0x15, 0x48, 0xA0, 0x40, 0x00, 0x6A, 0x40, 0x6A, 0x20, 0x5E, 0x56, 0xE8, 0xFE, 0x0A, 0x00, 0x00, 0x59, 0x59, 0x33, 0xC9, 0x3B, 0xC1, 0x75, 0x08, 0x83, 0xC8, 0xFF, 0xE9, 0x0F, 0x02, 0x00, 0x00, 0x8D, 0x90, 0x00, 0x08, 0x00, 0x00, 0xA3, 0x80, 0xFA, 0x40, 0x00, 0x89, 0x35, 0x6C, 0xFA, 0x40, 0x00, 0x3B, 0xC2, 0x73, 0x36, 0x83, 0xC0, 0x05, 0x83, 0x48, 0xFB, 0xFF, 0x66, 0xC7, 0x40, 0xFF, 0x00, 0x0A, 0x89, 0x48, 0x03, 0x66, 0xC7, 0x40, 0x1F, 0x00, 0x0A, 0xC6, 0x40, 0x21, 0x0A, 0x89, 0x48, 0x33, 0x88, 0x48, 0x2F, 0x8B, 0x35, 0x80, 0xFA, 0x40, 0x00, 0x83, 0xC0, 0x40, 0x8D, 0x50, 0xFB, 0x81, 0xC6, 0x00, 0x08, 0x00, 0x00, 0x3B, 0xD6, 0x72, 0xCD, 0x53, 0x57, 0x66, 0x39, 0x4D, 0xE6, 0x0F, 0x84, 0x0E, 0x01, 0x00, 0x00, 0x8B, 0x45, 0xE8, 0x3B, 0xC1, 0x0F, 0x84, 0x03, 0x01, 0x00, 0x00, 0x8B, 0x18, 0x83, 0xC0, 0x04, 0x89, 0x45, 0xFC, 0x03, 0xC3, 0xBE, 0x00, 0x08, 0x00, 0x00, 0x89, 0x45, 0xF8, 0x3B, 0xDE, 0x7C, 0x02, 0x8B, 0xDE, 0x39, 0x1D, 0x6C, 0xFA, 0x40, 0x00, 0x7D, 0x6B, 0xBF, 0x84, 0xFA, 0x40, 0x00, 0x6A, 0x40, 0x6A, 0x20, 0xE8, 0x5E, 0x0A, 0x00, 0x00, 0x59, 0x59, 0x85, 0xC0, 0x74, 0x51, 0x83, 0x05, 0x6C, 0xFA, 0x40, 0x00, 0x20, 0x8D, 0x88, 0x00, 0x08, 0x00, 0x00, 0x89, 0x07, 0x3B, 0xC1, 0x73, 0x31, 0x83, 0xC0, 0x05, 0x83, 0x48, 0xFB, 0xFF, 0x83, 0x60, 0x03, 0x00, 0x80, 0x60, 0x1F, 0x80, 0x83, 0x60, 0x33, 0x00, 0x66, 0xC7, 0x40, 0xFF, 0x00, 0x0A, 0x66, 0xC7, 0x40, 0x20, 0x0A, 0x0A, 0xC6, 0x40, 0x2F, 0x00, 0x8B, 0x0F, 0x83, 0xC0, 0x40, 0x03, 0xCE, 0x8D, 0x50, 0xFB, 0x3B, 0xD1, 0x72, 0xD2, 0x83, 0xC7, 0x04, 0x39, 0x1D, 0x6C, 0xFA, 0x40,
				0x00, 0x7C, 0xA2, 0xEB, 0x06, 0x8B, 0x1D, 0x6C, 0xFA, 0x40, 0x00, 0x33, 0xFF, 0x85, 0xDB, 0x7E, 0x72, 0x8B, 0x45, 0xF8, 0x8B, 0x00, 0x83, 0xF8, 0xFF, 0x74, 0x5C, 0x83, 0xF8, 0xFE, 0x74, 0x57, 0x8B, 0x4D, 0xFC, 0x8A, 0x09, 0xF6, 0xC1, 0x01, 0x74, 0x4D, 0xF6, 0xC1, 0x08, 0x75, 0x0B, 0x50, 0xFF, 0x15, 0xC8, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x74, 0x3D, 0x8B, 0xF7, 0x83, 0xE6, 0x1F, 0x8B, 0xC7, 0xC1, 0xF8, 0x05, 0xC1, 0xE6, 0x06, 0x03, 0x34, 0x85, 0x80, 0xFA, 0x40, 0x00, 0x8B, 0x45, 0xF8, 0x8B, 0x00, 0x89, 0x06, 0x8B, 0x45, 0xFC, 0x8A, 0x00, 0x88, 0x46, 0x04, 0x68, 0xA0, 0x0F, 0x00, 0x00, 0x8D, 0x46, 0x0C, 0x50, 0xFF, 0x15, 0xC4, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x0F, 0x84, 0xBC, 0x00, 0x00, 0x00, 0xFF, 0x46, 0x08, 0x83, 0x45, 0xF8, 0x04, 0x47, 0xFF, 0x45, 0xFC, 0x3B, 0xFB, 0x7C, 0x8E, 0x33, 0xDB, 0x8B, 0xF3, 0xC1, 0xE6, 0x06, 0x03, 0x35, 0x80, 0xFA, 0x40, 0x00, 0x8B, 0x06, 0x83, 0xF8, 0xFF, 0x74, 0x0B, 0x83, 0xF8, 0xFE, 0x74, 0x06, 0x80, 0x4E, 0x04, 0x80, 0xEB, 0x71, 0xC6, 0x46, 0x04, 0x81, 0x85, 0xDB, 0x75, 0x05, 0x6A, 0xF6, 0x58, 0xEB, 0x0A, 0x8D, 0x43, 0xFF, 0xF7, 0xD8, 0x1B, 0xC0, 0x83, 0xC0, 0xF5, 0x50, 0xFF, 0x15, 0xA8, 0xA0, 0x40, 0x00, 0x8B, 0xF8, 0x83, 0xFF, 0xFF, 0x74, 0x42, 0x85, 0xFF, 0x74, 0x3E, 0x57, 0xFF, 0x15, 0xC8, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x74, 0x33, 0x25, 0xFF, 0x00, 0x00, 0x00, 0x89, 0x3E, 0x83, 0xF8, 0x02, 0x75, 0x06, 0x80, 0x4E, 0x04, 0x40, 0xEB, 0x09, 0x83, 0xF8, 0x03, 0x75, 0x04, 0x80, 0x4E, 0x04, 0x08, 0x68, 0xA0, 0x0F, 0x00, 0x00, 0x8D, 0x46, 0x0C, 0x50, 0xFF, 0x15, 0xC4, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x74, 0x2C, 0xFF, 0x46, 0x08, 0xEB, 0x0A, 0x80, 0x4E, 0x04, 0x40, 0xC7, 0x06, 0xFE, 0xFF, 0xFF, 0xFF, 0x43, 0x83, 0xFB, 0x03, 0x0F, 0x8C, 0x68, 0xFF, 0xFF, 0xFF, 0xFF, 0x35, 0x6C, 0xFA, 0x40, 0x00, 0xFF, 0x15, 0xC0, 0xA0, 0x40, 0x00, 0x33, 0xC0, 0x5F, 0x5B, 0x5E, 0xC9, 0xC3, 0x83, 0xC8, 0xFF, 0xEB, 0xF6, 0x8B, 0xFF, 0x56, 0xB8, 0xD0, 0xC0, 0x40, 0x00, 0xBE, 0xD0, 0xC0, 0x40, 0x00, 0x57, 0x8B, 0xF8, 0x3B, 0xC6, 0x73, 0x0F, 0x8B, 0x07, 0x85, 0xC0, 0x74, 0x02, 0xFF, 0xD0, 0x83, 0xC7, 0x04, 0x3B, 0xFE, 0x72, 0xF1, 0x5F, 0x5E, 0xC3, 0x8B, 0xFF, 0x56, 0xB8, 0xD8, 0xC0, 0x40, 0x00, 0xBE, 0xD8, 0xC0, 0x40, 0x00, 0x57, 0x8B, 0xF8, 0x3B, 0xC6, 0x73, 0x0F, 0x8B, 0x07, 0x85, 0xC0, 0x74, 0x02, 0xFF, 0xD0, 0x83, 0xC7, 0x04, 0x3B, 0xFE, 0x72, 0xF1, 0x5F, 0x5E, 0xC3, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0x68, 0x30, 0x56, 0x40, 0x00, 0x64, 0xFF, 0x35, 0x00, 0x00, 0x00, 0x00, 0x8B, 0x44, 0x24, 0x10, 0x89, 0x6C, 0x24, 0x10, 0x8D, 0x6C, 0x24, 0x10, 0x2B, 0xE0, 0x53, 0x56, 0x57, 0xA1, 0x04, 0xD0, 0x40, 0x00, 0x31, 0x45, 0xFC, 0x33, 0xC5, 0x50, 0x89, 0x65, 0xE8, 0xFF, 0x75, 0xF8, 0x8B, 0x45, 0xFC, 0xC7, 0x45, 0xFC, 0xFE, 0xFF, 0xFF, 0xFF, 0x89, 0x45, 0xF8, 0x8D, 0x45, 0xF0, 0x64, 0xA3, 0x00, 0x00, 0x00, 0x00, 0xC3, 0x8B, 0x4D, 0xF0, 0x64, 0x89, 0x0D, 0x00, 0x00, 0x00, 0x00, 0x59, 0x5F, 0x5F, 0x5E, 0x5B, 0x8B, 0xE5, 0x5D, 0x51, 0xC3, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x83, 0xEC, 0x18,
				0x53, 0x8B, 0x5D, 0x0C, 0x56, 0x8B, 0x73, 0x08, 0x33, 0x35, 0x04, 0xD0, 0x40, 0x00, 0x57, 0x8B, 0x06, 0xC6, 0x45, 0xFF, 0x00, 0xC7, 0x45, 0xF4, 0x01, 0x00, 0x00, 0x00, 0x8D, 0x7B, 0x10, 0x83, 0xF8, 0xFE, 0x74, 0x0D, 0x8B, 0x4E, 0x04, 0x03, 0xCF, 0x33, 0x0C, 0x38, 0xE8, 0x56, 0xCA, 0xFF, 0xFF, 0x8B, 0x4E, 0x0C, 0x8B, 0x46, 0x08, 0x03, 0xCF, 0x33, 0x0C, 0x38, 0xE8, 0x46, 0xCA, 0xFF, 0xFF, 0x8B, 0x45, 0x08, 0xF6, 0x40, 0x04, 0x66, 0x0F, 0x85, 0x19, 0x01, 0x00, 0x00, 0x8B, 0x4D, 0x10, 0x8D, 0x55, 0xE8, 0x89, 0x53, 0xFC, 0x8B, 0x5B, 0x0C, 0x89, 0x45, 0xE8, 0x89, 0x4D, 0xEC, 0x83, 0xFB, 0xFE, 0x74, 0x5F, 0x8D, 0x49, 0x00, 0x8D, 0x04, 0x5B, 0x8B, 0x4C, 0x86, 0x14, 0x8D, 0x44, 0x86, 0x10, 0x89, 0x45, 0xF0, 0x8B, 0x00, 0x89, 0x45, 0xF8, 0x85, 0xC9, 0x74, 0x14, 0x8B, 0xD7, 0xE8, 0x64, 0x2B, 0x00, 0x00, 0xC6, 0x45, 0xFF, 0x01, 0x85, 0xC0, 0x78, 0x40, 0x7F, 0x47, 0x8B, 0x45, 0xF8, 0x8B, 0xD8, 0x83, 0xF8, 0xFE, 0x75, 0xCE, 0x80, 0x7D, 0xFF, 0x00, 0x74, 0x24, 0x8B, 0x06, 0x83, 0xF8, 0xFE, 0x74, 0x0D, 0x8B, 0x4E, 0x04, 0x03, 0xCF, 0x33, 0x0C, 0x38, 0xE8, 0xD3, 0xC9, 0xFF, 0xFF, 0x8B, 0x4E, 0x0C, 0x8B, 0x56, 0x08, 0x03, 0xCF, 0x33, 0x0C, 0x3A, 0xE8, 0xC3, 0xC9, 0xFF, 0xFF, 0x8B, 0x45, 0xF4, 0x5F, 0x5E, 0x5B, 0x8B, 0xE5, 0x5D, 0xC3, 0xC7, 0x45, 0xF4, 0x00, 0x00, 0x00, 0x00, 0xEB, 0xC9, 0x8B, 0x4D, 0x08, 0x81, 0x39, 0x63, 0x73, 0x6D, 0xE0, 0x75, 0x29, 0x83, 0x3D, 0x68, 0xFA, 0x40, 0x00, 0x00, 0x74, 0x20, 0x68, 0x68, 0xFA, 0x40, 0x00, 0xE8, 0x63, 0x25, 0x00, 0x00, 0x83, 0xC4, 0x04, 0x85, 0xC0, 0x74, 0x0F, 0x8B, 0x55, 0x08, 0x6A, 0x01, 0x52, 0xFF, 0x15, 0x68, 0xFA, 0x40, 0x00, 0x83, 0xC4, 0x08, 0x8B, 0x4D, 0x0C, 0x8B, 0x55, 0x08, 0xE8, 0x04, 0x2B, 0x00, 0x00, 0x8B, 0x45, 0x0C, 0x39, 0x58, 0x0C, 0x74, 0x12, 0x68, 0x04, 0xD0, 0x40, 0x00, 0x57, 0x8B, 0xD3, 0x8B, 0xC8, 0xE8, 0x06, 0x2B, 0x00, 0x00, 0x8B, 0x45, 0x0C, 0x8B, 0x4D, 0xF8, 0x89, 0x48, 0x0C, 0x8B, 0x06, 0x83, 0xF8, 0xFE, 0x74, 0x0D, 0x8B, 0x4E, 0x04, 0x03, 0xCF, 0x33, 0x0C, 0x38, 0xE8, 0x3D, 0xC9, 0xFF, 0xFF, 0x8B, 0x4E, 0x0C, 0x8B, 0x56, 0x08, 0x03, 0xCF, 0x33, 0x0C, 0x3A, 0xE8, 0x2D, 0xC9, 0xFF, 0xFF, 0x8B, 0x45, 0xF0, 0x8B, 0x48, 0x08, 0x8B, 0xD7, 0xE8, 0x9A, 0x2A, 0x00, 0x00, 0xBA, 0xFE, 0xFF, 0xFF, 0xFF, 0x39, 0x53, 0x0C, 0x0F, 0x84, 0x4F, 0xFF, 0xFF, 0xFF, 0x68, 0x04, 0xD0, 0x40, 0x00, 0x57, 0x8B, 0xCB, 0xE8, 0xB1, 0x2A, 0x00, 0x00, 0xE9, 0x19, 0xFF, 0xFF, 0xFF, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x83, 0xEC, 0x10, 0xA1, 0x04, 0xD0, 0x40, 0x00, 0x83, 0x65, 0xF8, 0x00, 0x83, 0x65, 0xFC, 0x00, 0x53, 0x57, 0xBF, 0x4E, 0xE6, 0x40, 0xBB, 0xBB, 0x00, 0x00, 0xFF, 0xFF, 0x3B, 0xC7, 0x74, 0x0D, 0x85, 0xC3, 0x74, 0x09, 0xF7, 0xD0, 0xA3, 0x08, 0xD0, 0x40, 0x00, 0xEB, 0x65, 0x56, 0x8D, 0x45, 0xF8, 0x50, 0xFF, 0x15, 0xDC, 0xA0, 0x40, 0x00, 0x8B, 0x75, 0xFC, 0x33, 0x75, 0xF8, 0xFF, 0x15, 0xD8, 0xA0, 0x40, 0x00, 0x33, 0xF0, 0xFF, 0x15, 0x98, 0xA0, 0x40, 0x00, 0x33, 0xF0, 0xFF, 0x15, 0xD4, 0xA0, 0x40, 0x00, 0x33, 0xF0, 0x8D, 0x45, 0xF0, 0x50, 0xFF, 0x15, 0xD0, 0xA0, 0x40, 0x00, 0x8B, 0x45, 0xF4, 0x33, 0x45, 0xF0, 0x33, 0xF0,
				0x3B, 0xF7, 0x75, 0x07, 0xBE, 0x4F, 0xE6, 0x40, 0xBB, 0xEB, 0x10, 0x85, 0xF3, 0x75, 0x0C, 0x8B, 0xC6, 0x0D, 0x11, 0x47, 0x00, 0x00, 0xC1, 0xE0, 0x10, 0x0B, 0xF0, 0x89, 0x35, 0x04, 0xD0, 0x40, 0x00, 0xF7, 0xD6, 0x89, 0x35, 0x08, 0xD0, 0x40, 0x00, 0x5E, 0x5F, 0x5B, 0xC9, 0xC3, 0x83, 0x25, 0x64, 0xFA, 0x40, 0x00, 0x00, 0xC3, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0x8B, 0x54, 0x24, 0x0C, 0x8B, 0x4C, 0x24, 0x04, 0x85, 0xD2, 0x74, 0x69, 0x33, 0xC0, 0x8A, 0x44, 0x24, 0x08, 0x84, 0xC0, 0x75, 0x16, 0x81, 0xFA, 0x80, 0x00, 0x00, 0x00, 0x72, 0x0E, 0x83, 0x3D, 0x3C, 0xEA, 0x40, 0x00, 0x00, 0x74, 0x05, 0xE9, 0xE6, 0x29, 0x00, 0x00, 0x57, 0x8B, 0xF9, 0x83, 0xFA, 0x04, 0x72, 0x31, 0xF7, 0xD9, 0x83, 0xE1, 0x03, 0x74, 0x0C, 0x2B, 0xD1, 0x88, 0x07, 0x83, 0xC7, 0x01, 0x83, 0xE9, 0x01, 0x75, 0xF6, 0x8B, 0xC8, 0xC1, 0xE0, 0x08, 0x03, 0xC1, 0x8B, 0xC8, 0xC1, 0xE0, 0x10, 0x03, 0xC1, 0x8B, 0xCA, 0x83, 0xE2, 0x03, 0xC1, 0xE9, 0x02, 0x74, 0x06, 0xF3, 0xAB, 0x85, 0xD2, 0x74, 0x0A, 0x88, 0x07, 0x83, 0xC7, 0x01, 0x83, 0xEA, 0x01, 0x75, 0xF6, 0x8B, 0x44, 0x24, 0x08, 0x5F, 0xC3, 0x8B, 0x44, 0x24, 0x04, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x8B, 0x45, 0x08, 0x85, 0xC0, 0x74, 0x12, 0x83, 0xE8, 0x08, 0x81, 0x38, 0xDD, 0xDD, 0x00, 0x00, 0x75, 0x07, 0x50, 0xE8, 0x6F, 0xCB, 0xFF, 0xFF, 0x59, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x83, 0xEC, 0x10, 0xA1, 0x04, 0xD0, 0x40, 0x00, 0x33, 0xC5, 0x89, 0x45, 0xFC, 0x8B, 0x55, 0x18, 0x53, 0x33, 0xDB, 0x56, 0x57, 0x3B, 0xD3, 0x7E, 0x1F, 0x8B, 0x45, 0x14, 0x8B, 0xCA, 0x49, 0x38, 0x18, 0x74, 0x08, 0x40, 0x3B, 0xCB, 0x75, 0xF6, 0x83, 0xC9, 0xFF, 0x8B, 0xC2, 0x2B, 0xC1, 0x48, 0x3B, 0xC2, 0x7D, 0x01, 0x40, 0x89, 0x45, 0x18, 0x89, 0x5D, 0xF8, 0x39, 0x5D, 0x24, 0x75, 0x0B, 0x8B, 0x45, 0x08, 0x8B, 0x00, 0x8B, 0x40, 0x04, 0x89, 0x45, 0x24, 0x8B, 0x35, 0xE4, 0xA0, 0x40, 0x00, 0x33, 0xC0, 0x39, 0x5D, 0x28, 0x53, 0x53, 0xFF, 0x75, 0x18, 0x0F, 0x95, 0xC0, 0xFF, 0x75, 0x14, 0x8D, 0x04, 0xC5, 0x01, 0x00, 0x00, 0x00, 0x50, 0xFF, 0x75, 0x24, 0xFF, 0xD6, 0x8B, 0xF8, 0x89, 0x7D, 0xF0, 0x3B, 0xFB, 0x75, 0x07, 0x33, 0xC0, 0xE9, 0x52, 0x01, 0x00, 0x00, 0x7E, 0x43, 0x6A, 0xE0, 0x33, 0xD2, 0x58, 0xF7, 0xF7, 0x83, 0xF8, 0x02, 0x72, 0x37, 0x8D, 0x44, 0x3F, 0x08, 0x3D, 0x00, 0x04, 0x00, 0x00, 0x77, 0x13, 0xE8, 0xA5, 0x29, 0x00, 0x00, 0x8B, 0xC4, 0x3B, 0xC3, 0x74, 0x1C, 0xC7, 0x00, 0xCC, 0xCC, 0x00, 0x00, 0xEB, 0x11, 0x50, 0xE8, 0xF1, 0xCA, 0xFF, 0xFF, 0x59, 0x3B, 0xC3, 0x74, 0x09, 0xC7, 0x00, 0xDD, 0xDD, 0x00, 0x00, 0x83, 0xC0, 0x08, 0x89, 0x45, 0xF4, 0xEB, 0x03, 0x89, 0x5D, 0xF4, 0x39, 0x5D, 0xF4, 0x74, 0xAC, 0x57, 0xFF, 0x75, 0xF4, 0xFF, 0x75, 0x18, 0xFF, 0x75, 0x14, 0x6A, 0x01, 0xFF, 0x75, 0x24, 0xFF, 0xD6, 0x85, 0xC0, 0x0F, 0x84, 0xE0, 0x00, 0x00, 0x00, 0x8B, 0x35, 0xE0, 0xA0, 0x40, 0x00, 0x53, 0x53, 0x57, 0xFF, 0x75, 0xF4, 0xFF, 0x75, 0x10, 0xFF, 0x75, 0x0C, 0xFF, 0xD6, 0x89, 0x45, 0xF8, 0x3B, 0xC3, 0x0F, 0x84, 0xC1, 0x00, 0x00, 0x00, 0xB9, 0x00, 0x04, 0x00, 0x00, 0x85, 0x4D, 0x10, 0x74, 0x29, 0x8B, 0x45, 0x20, 0x3B,
				0xC3, 0x0F, 0x84, 0xAC, 0x00, 0x00, 0x00, 0x39, 0x45, 0xF8, 0x0F, 0x8F, 0xA3, 0x00, 0x00, 0x00, 0x50, 0xFF, 0x75, 0x1C, 0x57, 0xFF, 0x75, 0xF4, 0xFF, 0x75, 0x10, 0xFF, 0x75, 0x0C, 0xFF, 0xD6, 0xE9, 0x8E, 0x00, 0x00, 0x00, 0x8B, 0x7D, 0xF8, 0x3B, 0xFB, 0x7E, 0x42, 0x6A, 0xE0, 0x33, 0xD2, 0x58, 0xF7, 0xF7, 0x83, 0xF8, 0x02, 0x72, 0x36, 0x8D, 0x44, 0x3F, 0x08, 0x3B, 0xC1, 0x77, 0x16, 0xE8, 0xEB, 0x28, 0x00, 0x00, 0x8B, 0xFC, 0x3B, 0xFB, 0x74, 0x68, 0xC7, 0x07, 0xCC, 0xCC, 0x00, 0x00, 0x83, 0xC7, 0x08, 0xEB, 0x1A, 0x50, 0xE8, 0x34, 0xCA, 0xFF, 0xFF, 0x59, 0x3B, 0xC3, 0x74, 0x09, 0xC7, 0x00, 0xDD, 0xDD, 0x00, 0x00, 0x83, 0xC0, 0x08, 0x8B, 0xF8, 0xEB, 0x02, 0x33, 0xFF, 0x3B, 0xFB, 0x74, 0x3F, 0xFF, 0x75, 0xF8, 0x57, 0xFF, 0x75, 0xF0, 0xFF, 0x75, 0xF4, 0xFF, 0x75, 0x10, 0xFF, 0x75, 0x0C, 0xFF, 0xD6, 0x85, 0xC0, 0x74, 0x22, 0x53, 0x53, 0x39, 0x5D, 0x20, 0x75, 0x04, 0x53, 0x53, 0xEB, 0x06, 0xFF, 0x75, 0x20, 0xFF, 0x75, 0x1C, 0xFF, 0x75, 0xF8, 0x57, 0x53, 0xFF, 0x75, 0x24, 0xFF, 0x15, 0xB8, 0xA0, 0x40, 0x00, 0x89, 0x45, 0xF8, 0x57, 0xE8, 0x18, 0xFE, 0xFF, 0xFF, 0x59, 0xFF, 0x75, 0xF4, 0xE8, 0x0F, 0xFE, 0xFF, 0xFF, 0x8B, 0x45, 0xF8, 0x59, 0x8D, 0x65, 0xE4, 0x5F, 0x5E, 0x5B, 0x8B, 0x4D, 0xFC, 0x33, 0xCD, 0xE8, 0xD0, 0xC5, 0xFF, 0xFF, 0xC9, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x83, 0xEC, 0x10, 0xFF, 0x75, 0x08, 0x8D, 0x4D, 0xF0, 0xE8, 0xCA, 0xC5, 0xFF, 0xFF, 0xFF, 0x75, 0x28, 0x8D, 0x45, 0xF0, 0xFF, 0x75, 0x24, 0xFF, 0x75, 0x20, 0xFF, 0x75, 0x1C, 0xFF, 0x75, 0x18, 0xFF, 0x75, 0x14, 0xFF, 0x75, 0x10, 0xFF, 0x75, 0x0C, 0x50, 0xE8, 0xE5, 0xFD, 0xFF, 0xFF, 0x83, 0xC4, 0x24, 0x80, 0x7D, 0xFC, 0x00, 0x74, 0x07, 0x8B, 0x4D, 0xF8, 0x83, 0x61, 0x70, 0xFD, 0xC9, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x51, 0x51, 0xA1, 0x04, 0xD0, 0x40, 0x00, 0x33, 0xC5, 0x89, 0x45, 0xFC, 0x53, 0x33, 0xDB, 0x56, 0x57, 0x89, 0x5D, 0xF8, 0x39, 0x5D, 0x1C, 0x75, 0x0B, 0x8B, 0x45, 0x08, 0x8B, 0x00, 0x8B, 0x40, 0x04, 0x89, 0x45, 0x1C, 0x8B, 0x35, 0xE4, 0xA0, 0x40, 0x00, 0x33, 0xC0, 0x39, 0x5D, 0x20, 0x53, 0x53, 0xFF, 0x75, 0x14, 0x0F, 0x95, 0xC0, 0xFF, 0x75, 0x10, 0x8D, 0x04, 0xC5, 0x01, 0x00, 0x00, 0x00, 0x50, 0xFF, 0x75, 0x1C, 0xFF, 0xD6, 0x8B, 0xF8, 0x3B, 0xFB, 0x75, 0x04, 0x33, 0xC0, 0xEB, 0x7F, 0x7E, 0x3C, 0x81, 0xFF, 0xF0, 0xFF, 0xFF, 0x7F, 0x77, 0x34, 0x8D, 0x44, 0x3F, 0x08, 0x3D, 0x00, 0x04, 0x00, 0x00, 0x77, 0x13, 0xE8, 0xA9, 0x27, 0x00, 0x00, 0x8B, 0xC4, 0x3B, 0xC3, 0x74, 0x1C, 0xC7, 0x00, 0xCC, 0xCC, 0x00, 0x00, 0xEB, 0x11, 0x50, 0xE8, 0xF5, 0xC8, 0xFF, 0xFF, 0x59, 0x3B, 0xC3, 0x74, 0x09, 0xC7, 0x00, 0xDD, 0xDD, 0x00, 0x00, 0x83, 0xC0, 0x08, 0x8B, 0xD8, 0x85, 0xDB, 0x74, 0xBA, 0x8D, 0x04, 0x3F, 0x50, 0x6A, 0x00, 0x53, 0xE8, 0x95, 0xFC, 0xFF, 0xFF, 0x83, 0xC4, 0x0C, 0x57, 0x53, 0xFF, 0x75, 0x14, 0xFF, 0x75, 0x10, 0x6A, 0x01, 0xFF, 0x75, 0x1C, 0xFF, 0xD6, 0x85, 0xC0, 0x74, 0x11, 0xFF, 0x75, 0x18, 0x50, 0x53, 0xFF, 0x75, 0x0C, 0xFF, 0x15, 0xE8, 0xA0, 0x40, 0x00, 0x89, 0x45, 0xF8, 0x53, 0xE8, 0xE2, 0xFC, 0xFF, 0xFF, 0x8B, 0x45, 0xF8, 0x59, 0x8D, 0x65, 0xEC, 0x5F, 0x5E, 0x5B, 0x8B, 0x4D,
				0xFC, 0x33, 0xCD, 0xE8, 0xA3, 0xC4, 0xFF, 0xFF, 0xC9, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x83, 0xEC, 0x10, 0xFF, 0x75, 0x08, 0x8D, 0x4D, 0xF0, 0xE8, 0x9D, 0xC4, 0xFF, 0xFF, 0xFF, 0x75, 0x24, 0x8D, 0x45, 0xF0, 0xFF, 0x75, 0x1C, 0xFF, 0x75, 0x18, 0xFF, 0x75, 0x14, 0xFF, 0x75, 0x10, 0xFF, 0x75, 0x0C, 0x50, 0xE8, 0xEB, 0xFE, 0xFF, 0xFF, 0x83, 0xC4, 0x1C, 0x80, 0x7D, 0xFC, 0x00, 0x74, 0x07, 0x8B, 0x4D, 0xF8, 0x83, 0x61, 0x70, 0xFD, 0xC9, 0xC3, 0x8B, 0xFF, 0x56, 0x57, 0x33, 0xF6, 0xBF, 0x98, 0xE8, 0x40, 0x00, 0x83, 0x3C, 0xF5, 0x54, 0xD9, 0x40, 0x00, 0x01, 0x75, 0x1D, 0x8D, 0x04, 0xF5, 0x50, 0xD9, 0x40, 0x00, 0x89, 0x38, 0x68, 0xA0, 0x0F, 0x00, 0x00, 0xFF, 0x30, 0x83, 0xC7, 0x18, 0xFF, 0x15, 0xC4, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x74, 0x0C, 0x46, 0x83, 0xFE, 0x24, 0x7C, 0xD3, 0x33, 0xC0, 0x40, 0x5F, 0x5E, 0xC3, 0x83, 0x24, 0xF5, 0x50, 0xD9, 0x40, 0x00, 0x00, 0x33, 0xC0, 0xEB, 0xF1, 0x8B, 0xFF, 0x53, 0x8B, 0x1D, 0xCC, 0xA0, 0x40, 0x00, 0x56, 0xBE, 0x50, 0xD9, 0x40, 0x00, 0x57, 0x8B, 0x3E, 0x85, 0xFF, 0x74, 0x13, 0x83, 0x7E, 0x04, 0x01, 0x74, 0x0D, 0x57, 0xFF, 0xD3, 0x57, 0xE8, 0xA9, 0xC7, 0xFF, 0xFF, 0x83, 0x26, 0x00, 0x59, 0x83, 0xC6, 0x08, 0x81, 0xFE, 0x70, 0xDA, 0x40, 0x00, 0x7C, 0xDC, 0xBE, 0x50, 0xD9, 0x40, 0x00, 0x5F, 0x8B, 0x06, 0x85, 0xC0, 0x74, 0x09, 0x83, 0x7E, 0x04, 0x01, 0x75, 0x03, 0x50, 0xFF, 0xD3, 0x83, 0xC6, 0x08, 0x81, 0xFE, 0x70, 0xDA, 0x40, 0x00, 0x7C, 0xE6, 0x5E, 0x5B, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x8B, 0x45, 0x08, 0xFF, 0x34, 0xC5, 0x50, 0xD9, 0x40, 0x00, 0xFF, 0x15, 0xEC, 0xA0, 0x40, 0x00, 0x5D, 0xC3, 0x6A, 0x0C, 0x68, 0xD0, 0xC1, 0x40, 0x00, 0xE8, 0xAE, 0xF8, 0xFF, 0xFF, 0x33, 0xFF, 0x47, 0x89, 0x7D, 0xE4, 0x33, 0xDB, 0x39, 0x1D, 0x28, 0xE1, 0x40, 0x00, 0x75, 0x18, 0xE8, 0xC9, 0xEF, 0xFF, 0xFF, 0x6A, 0x1E, 0xE8, 0x13, 0xEE, 0xFF, 0xFF, 0x68, 0xFF, 0x00, 0x00, 0x00, 0xE8, 0x23, 0xEB, 0xFF, 0xFF, 0x59, 0x59, 0x8B, 0x75, 0x08, 0x8D, 0x34, 0xF5, 0x50, 0xD9, 0x40, 0x00, 0x39, 0x1E, 0x74, 0x04, 0x8B, 0xC7, 0xEB, 0x6D, 0x6A, 0x18, 0xE8, 0xA8, 0x00, 0x00, 0x00, 0x59, 0x8B, 0xF8, 0x3B, 0xFB, 0x75, 0x0F, 0xE8, 0xFD, 0xDB, 0xFF, 0xFF, 0xC7, 0x00, 0x0C, 0x00, 0x00, 0x00, 0x33, 0xC0, 0xEB, 0x50, 0x6A, 0x0A, 0xE8, 0x58, 0x00, 0x00, 0x00, 0x59, 0x89, 0x5D, 0xFC, 0x39, 0x1E, 0x75, 0x2B, 0x68, 0xA0, 0x0F, 0x00, 0x00, 0x57, 0xFF, 0x15, 0xC4, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x75, 0x17, 0x57, 0xE8, 0xD8, 0xC6, 0xFF, 0xFF, 0x59, 0xE8, 0xC8, 0xDB, 0xFF, 0xFF, 0xC7, 0x00, 0x0C, 0x00, 0x00, 0x00, 0x89, 0x5D, 0xE4, 0xEB, 0x0B, 0x89, 0x3E, 0xEB, 0x07, 0x57, 0xE8, 0xBD, 0xC6, 0xFF, 0xFF, 0x59, 0xC7, 0x45, 0xFC, 0xFE, 0xFF, 0xFF, 0xFF, 0xE8, 0x09, 0x00, 0x00, 0x00, 0x8B, 0x45, 0xE4, 0xE8, 0x47, 0xF8, 0xFF, 0xFF, 0xC3, 0x6A, 0x0A, 0xE8, 0x29, 0xFF, 0xFF, 0xFF, 0x59, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x8B, 0x45, 0x08, 0x56, 0x8D, 0x34, 0xC5, 0x50, 0xD9, 0x40, 0x00, 0x83, 0x3E, 0x00, 0x75, 0x13, 0x50, 0xE8, 0x23, 0xFF, 0xFF, 0xFF, 0x59, 0x85, 0xC0, 0x75, 0x08, 0x6A, 0x11, 0xE8, 0x0E, 0xED, 0xFF, 0xFF, 0x59, 0xFF, 0x36, 0xFF, 0x15, 0xF0, 0xA0, 0x40, 0x00,
				0x5E, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x56, 0x57, 0x33, 0xF6, 0xFF, 0x75, 0x08, 0xE8, 0x94, 0xC6, 0xFF, 0xFF, 0x8B, 0xF8, 0x59, 0x85, 0xFF, 0x75, 0x27, 0x39, 0x05, 0xE8, 0xE9, 0x40, 0x00, 0x76, 0x1F, 0x56, 0xFF, 0x15, 0xF4, 0xA0, 0x40, 0x00, 0x8D, 0x86, 0xE8, 0x03, 0x00, 0x00, 0x3B, 0x05, 0xE8, 0xE9, 0x40, 0x00, 0x76, 0x03, 0x83, 0xC8, 0xFF, 0x8B, 0xF0, 0x83, 0xF8, 0xFF, 0x75, 0xCA, 0x8B, 0xC7, 0x5F, 0x5E, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x56, 0x57, 0x33, 0xF6, 0x6A, 0x00, 0xFF, 0x75, 0x0C, 0xFF, 0x75, 0x08, 0xE8, 0x16, 0x25, 0x00, 0x00, 0x8B, 0xF8, 0x83, 0xC4, 0x0C, 0x85, 0xFF, 0x75, 0x27, 0x39, 0x05, 0xE8, 0xE9, 0x40, 0x00, 0x76, 0x1F, 0x56, 0xFF, 0x15, 0xF4, 0xA0, 0x40, 0x00, 0x8D, 0x86, 0xE8, 0x03, 0x00, 0x00, 0x3B, 0x05, 0xE8, 0xE9, 0x40, 0x00, 0x76, 0x03, 0x83, 0xC8, 0xFF, 0x8B, 0xF0, 0x83, 0xF8, 0xFF, 0x75, 0xC3, 0x8B, 0xC7, 0x5F, 0x5E, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x56, 0x57, 0x33, 0xF6, 0xFF, 0x75, 0x0C, 0xFF, 0x75, 0x08, 0xE8, 0x4E, 0x25, 0x00, 0x00, 0x8B, 0xF8, 0x59, 0x59, 0x85, 0xFF, 0x75, 0x2C, 0x39, 0x45, 0x0C, 0x74, 0x27, 0x39, 0x05, 0xE8, 0xE9, 0x40, 0x00, 0x76, 0x1F, 0x56, 0xFF, 0x15, 0xF4, 0xA0, 0x40, 0x00, 0x8D, 0x86, 0xE8, 0x03, 0x00, 0x00, 0x3B, 0x05, 0xE8, 0xE9, 0x40, 0x00, 0x76, 0x03, 0x83, 0xC8, 0xFF, 0x8B, 0xF0, 0x83, 0xF8, 0xFF, 0x75, 0xC1, 0x8B, 0xC7, 0x5F, 0x5E, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x56, 0x8B, 0x75, 0x08, 0x85, 0xF6, 0x0F, 0x84, 0x63, 0x03, 0x00, 0x00, 0xFF, 0x76, 0x04, 0xE8, 0x73, 0xC5, 0xFF, 0xFF, 0xFF, 0x76, 0x08, 0xE8, 0x6B, 0xC5, 0xFF, 0xFF, 0xFF, 0x76, 0x0C, 0xE8, 0x63, 0xC5, 0xFF, 0xFF, 0xFF, 0x76, 0x10, 0xE8, 0x5B, 0xC5, 0xFF, 0xFF, 0xFF, 0x76, 0x14, 0xE8, 0x53, 0xC5, 0xFF, 0xFF, 0xFF, 0x76, 0x18, 0xE8, 0x4B, 0xC5, 0xFF, 0xFF, 0xFF, 0x36, 0xE8, 0x44, 0xC5, 0xFF, 0xFF, 0xFF, 0x76, 0x20, 0xE8, 0x3C, 0xC5, 0xFF, 0xFF, 0xFF, 0x76, 0x24, 0xE8, 0x34, 0xC5, 0xFF, 0xFF, 0xFF, 0x76, 0x28, 0xE8, 0x2C, 0xC5, 0xFF, 0xFF, 0xFF, 0x76, 0x2C, 0xE8, 0x24, 0xC5, 0xFF, 0xFF, 0xFF, 0x76, 0x30, 0xE8, 0x1C, 0xC5, 0xFF, 0xFF, 0xFF, 0x76, 0x34, 0xE8, 0x14, 0xC5, 0xFF, 0xFF, 0xFF, 0x76, 0x1C, 0xE8, 0x0C, 0xC5, 0xFF, 0xFF, 0xFF, 0x76, 0x38, 0xE8, 0x04, 0xC5, 0xFF, 0xFF, 0xFF, 0x76, 0x3C, 0xE8, 0xFC, 0xC4, 0xFF, 0xFF, 0x83, 0xC4, 0x40, 0xFF, 0x76, 0x40, 0xE8, 0xF1, 0xC4, 0xFF, 0xFF, 0xFF, 0x76, 0x44, 0xE8, 0xE9, 0xC4, 0xFF, 0xFF, 0xFF, 0x76, 0x48, 0xE8, 0xE1, 0xC4, 0xFF, 0xFF, 0xFF, 0x76, 0x4C, 0xE8, 0xD9, 0xC4, 0xFF, 0xFF, 0xFF, 0x76, 0x50, 0xE8, 0xD1, 0xC4, 0xFF, 0xFF, 0xFF, 0x76, 0x54, 0xE8, 0xC9, 0xC4, 0xFF, 0xFF, 0xFF, 0x76, 0x58, 0xE8, 0xC1, 0xC4, 0xFF, 0xFF, 0xFF, 0x76, 0x5C, 0xE8, 0xB9, 0xC4, 0xFF, 0xFF, 0xFF, 0x76, 0x60, 0xE8, 0xB1, 0xC4, 0xFF, 0xFF, 0xFF, 0x76, 0x64, 0xE8, 0xA9, 0xC4, 0xFF, 0xFF, 0xFF, 0x76, 0x68, 0xE8, 0xA1, 0xC4, 0xFF, 0xFF, 0xFF, 0x76, 0x6C, 0xE8, 0x99, 0xC4, 0xFF, 0xFF, 0xFF, 0x76, 0x70, 0xE8, 0x91, 0xC4, 0xFF, 0xFF, 0xFF, 0x76, 0x74, 0xE8, 0x89, 0xC4, 0xFF, 0xFF, 0xFF, 0x76, 0x78, 0xE8, 0x81, 0xC4, 0xFF, 0xFF, 0xFF, 0x76, 0x7C, 0xE8, 0x79, 0xC4, 0xFF,
				0xFF, 0x83, 0xC4, 0x40, 0xFF, 0xB6, 0x80, 0x00, 0x00, 0x00, 0xE8, 0x6B, 0xC4, 0xFF, 0xFF, 0xFF, 0xB6, 0x84, 0x00, 0x00, 0x00, 0xE8, 0x60, 0xC4, 0xFF, 0xFF, 0xFF, 0xB6, 0x88, 0x00, 0x00, 0x00, 0xE8, 0x55, 0xC4, 0xFF, 0xFF, 0xFF, 0xB6, 0x8C, 0x00, 0x00, 0x00, 0xE8, 0x4A, 0xC4, 0xFF, 0xFF, 0xFF, 0xB6, 0x90, 0x00, 0x00, 0x00, 0xE8, 0x3F, 0xC4, 0xFF, 0xFF, 0xFF, 0xB6, 0x94, 0x00, 0x00, 0x00, 0xE8, 0x34, 0xC4, 0xFF, 0xFF, 0xFF, 0xB6, 0x98, 0x00, 0x00, 0x00, 0xE8, 0x29, 0xC4, 0xFF, 0xFF, 0xFF, 0xB6, 0x9C, 0x00, 0x00, 0x00, 0xE8, 0x1E, 0xC4, 0xFF, 0xFF, 0xFF, 0xB6, 0xA0, 0x00, 0x00, 0x00, 0xE8, 0x13, 0xC4, 0xFF, 0xFF, 0xFF, 0xB6, 0xA4, 0x00, 0x00, 0x00, 0xE8, 0x08, 0xC4, 0xFF, 0xFF, 0xFF, 0xB6, 0xA8, 0x00, 0x00, 0x00, 0xE8, 0xFD, 0xC3, 0xFF, 0xFF, 0xFF, 0xB6, 0xBC, 0x00, 0x00, 0x00, 0xE8, 0xF2, 0xC3, 0xFF, 0xFF, 0xFF, 0xB6, 0xC0, 0x00, 0x00, 0x00, 0xE8, 0xE7, 0xC3, 0xFF, 0xFF, 0xFF, 0xB6, 0xC4, 0x00, 0x00, 0x00, 0xE8, 0xDC, 0xC3, 0xFF, 0xFF, 0xFF, 0xB6, 0xC8, 0x00, 0x00, 0x00, 0xE8, 0xD1, 0xC3, 0xFF, 0xFF, 0xFF, 0xB6, 0xCC, 0x00, 0x00, 0x00, 0xE8, 0xC6, 0xC3, 0xFF, 0xFF, 0x83, 0xC4, 0x40, 0xFF, 0xB6, 0xD0, 0x00, 0x00, 0x00, 0xE8, 0xB8, 0xC3, 0xFF, 0xFF, 0xFF, 0xB6, 0xB8, 0x00, 0x00, 0x00, 0xE8, 0xAD, 0xC3, 0xFF, 0xFF, 0xFF, 0xB6, 0xD8, 0x00, 0x00, 0x00, 0xE8, 0xA2, 0xC3, 0xFF, 0xFF, 0xFF, 0xB6, 0xDC, 0x00, 0x00, 0x00, 0xE8, 0x97, 0xC3, 0xFF, 0xFF, 0xFF, 0xB6, 0xE0, 0x00, 0x00, 0x00, 0xE8, 0x8C, 0xC3, 0xFF, 0xFF, 0xFF, 0xB6, 0xE4, 0x00, 0x00, 0x00, 0xE8, 0x81, 0xC3, 0xFF, 0xFF, 0xFF, 0xB6, 0xE8, 0x00, 0x00, 0x00, 0xE8, 0x76, 0xC3, 0xFF, 0xFF, 0xFF, 0xB6, 0xEC, 0x00, 0x00, 0x00, 0xE8, 0x6B, 0xC3, 0xFF, 0xFF, 0xFF, 0xB6, 0xD4, 0x00, 0x00, 0x00, 0xE8, 0x60, 0xC3, 0xFF, 0xFF, 0xFF, 0xB6, 0xF0, 0x00, 0x00, 0x00, 0xE8, 0x55, 0xC3, 0xFF, 0xFF, 0xFF, 0xB6, 0xF4, 0x00, 0x00, 0x00, 0xE8, 0x4A, 0xC3, 0xFF, 0xFF, 0xFF, 0xB6, 0xF8, 0x00, 0x00, 0x00, 0xE8, 0x3F, 0xC3, 0xFF, 0xFF, 0xFF, 0xB6, 0xFC, 0x00, 0x00, 0x00, 0xE8, 0x34, 0xC3, 0xFF, 0xFF, 0xFF, 0xB6, 0x00, 0x01, 0x00, 0x00, 0xE8, 0x29, 0xC3, 0xFF, 0xFF, 0xFF, 0xB6, 0x04, 0x01, 0x00, 0x00, 0xE8, 0x1E, 0xC3, 0xFF, 0xFF, 0xFF, 0xB6, 0x08, 0x01, 0x00, 0x00, 0xE8, 0x13, 0xC3, 0xFF, 0xFF, 0x83, 0xC4, 0x40, 0xFF, 0xB6, 0x0C, 0x01, 0x00, 0x00, 0xE8, 0x05, 0xC3, 0xFF, 0xFF, 0xFF, 0xB6, 0x10, 0x01, 0x00, 0x00, 0xE8, 0xFA, 0xC2, 0xFF, 0xFF, 0xFF, 0xB6, 0x14, 0x01, 0x00, 0x00, 0xE8, 0xEF, 0xC2, 0xFF, 0xFF, 0xFF, 0xB6, 0x18, 0x01, 0x00, 0x00, 0xE8, 0xE4, 0xC2, 0xFF, 0xFF, 0xFF, 0xB6, 0x1C, 0x01, 0x00, 0x00, 0xE8, 0xD9, 0xC2, 0xFF, 0xFF, 0xFF, 0xB6, 0x20, 0x01, 0x00, 0x00, 0xE8, 0xCE, 0xC2, 0xFF, 0xFF, 0xFF, 0xB6, 0x24, 0x01, 0x00, 0x00, 0xE8, 0xC3, 0xC2, 0xFF, 0xFF, 0xFF, 0xB6, 0x28, 0x01, 0x00, 0x00, 0xE8, 0xB8, 0xC2, 0xFF, 0xFF, 0xFF, 0xB6, 0x2C, 0x01, 0x00, 0x00, 0xE8, 0xAD, 0xC2, 0xFF, 0xFF, 0xFF, 0xB6, 0x30, 0x01, 0x00, 0x00, 0xE8, 0xA2, 0xC2, 0xFF, 0xFF, 0xFF, 0xB6, 0x34, 0x01, 0x00, 0x00, 0xE8, 0x97, 0xC2, 0xFF, 0xFF, 0xFF, 0xB6, 0x38, 0x01, 0x00, 0x00, 0xE8, 0x8C, 0xC2, 0xFF, 0xFF, 0xFF, 0xB6, 0x3C, 0x01, 0x00, 0x00,
				0xE8, 0x81, 0xC2, 0xFF, 0xFF, 0xFF, 0xB6, 0x40, 0x01, 0x00, 0x00, 0xE8, 0x76, 0xC2, 0xFF, 0xFF, 0xFF, 0xB6, 0x44, 0x01, 0x00, 0x00, 0xE8, 0x6B, 0xC2, 0xFF, 0xFF, 0xFF, 0xB6, 0x48, 0x01, 0x00, 0x00, 0xE8, 0x60, 0xC2, 0xFF, 0xFF, 0x83, 0xC4, 0x40, 0xFF, 0xB6, 0x4C, 0x01, 0x00, 0x00, 0xE8, 0x52, 0xC2, 0xFF, 0xFF, 0xFF, 0xB6, 0x50, 0x01, 0x00, 0x00, 0xE8, 0x47, 0xC2, 0xFF, 0xFF, 0xFF, 0xB6, 0x54, 0x01, 0x00, 0x00, 0xE8, 0x3C, 0xC2, 0xFF, 0xFF, 0xFF, 0xB6, 0x58, 0x01, 0x00, 0x00, 0xE8, 0x31, 0xC2, 0xFF, 0xFF, 0xFF, 0xB6, 0x5C, 0x01, 0x00, 0x00, 0xE8, 0x26, 0xC2, 0xFF, 0xFF, 0xFF, 0xB6, 0x60, 0x01, 0x00, 0x00, 0xE8, 0x1B, 0xC2, 0xFF, 0xFF, 0x83, 0xC4, 0x18, 0x5E, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x56, 0x8B, 0x75, 0x08, 0x85, 0xF6, 0x74, 0x59, 0x8B, 0x06, 0x3B, 0x05, 0x78, 0xDA, 0x40, 0x00, 0x74, 0x07, 0x50, 0xE8, 0xF8, 0xC1, 0xFF, 0xFF, 0x59, 0x8B, 0x46, 0x04, 0x3B, 0x05, 0x7C, 0xDA, 0x40, 0x00, 0x74, 0x07, 0x50, 0xE8, 0xE6, 0xC1, 0xFF, 0xFF, 0x59, 0x8B, 0x46, 0x08, 0x3B, 0x05, 0x80, 0xDA, 0x40, 0x00, 0x74, 0x07, 0x50, 0xE8, 0xD4, 0xC1, 0xFF, 0xFF, 0x59, 0x8B, 0x46, 0x30, 0x3B, 0x05, 0xA8, 0xDA, 0x40, 0x00, 0x74, 0x07, 0x50, 0xE8, 0xC2, 0xC1, 0xFF, 0xFF, 0x59, 0x8B, 0x76, 0x34, 0x3B, 0x35, 0xAC, 0xDA, 0x40, 0x00, 0x74, 0x07, 0x56, 0xE8, 0xB0, 0xC1, 0xFF, 0xFF, 0x59, 0x5E, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x56, 0x8B, 0x75, 0x08, 0x85, 0xF6, 0x0F, 0x84, 0xEA, 0x00, 0x00, 0x00, 0x8B, 0x46, 0x0C, 0x3B, 0x05, 0x84, 0xDA, 0x40, 0x00, 0x74, 0x07, 0x50, 0xE8, 0x8A, 0xC1, 0xFF, 0xFF, 0x59, 0x8B, 0x46, 0x10, 0x3B, 0x05, 0x88, 0xDA, 0x40, 0x00, 0x74, 0x07, 0x50, 0xE8, 0x78, 0xC1, 0xFF, 0xFF, 0x59, 0x8B, 0x46, 0x14, 0x3B, 0x05, 0x8C, 0xDA, 0x40, 0x00, 0x74, 0x07, 0x50, 0xE8, 0x66, 0xC1, 0xFF, 0xFF, 0x59, 0x8B, 0x46, 0x18, 0x3B, 0x05, 0x90, 0xDA, 0x40, 0x00, 0x74, 0x07, 0x50, 0xE8, 0x54, 0xC1, 0xFF, 0xFF, 0x59, 0x8B, 0x46, 0x1C, 0x3B, 0x05, 0x94, 0xDA, 0x40, 0x00, 0x74, 0x07, 0x50, 0xE8, 0x42, 0xC1, 0xFF, 0xFF, 0x59, 0x8B, 0x46, 0x20, 0x3B, 0x05, 0x98, 0xDA, 0x40, 0x00, 0x74, 0x07, 0x50, 0xE8, 0x30, 0xC1, 0xFF, 0xFF, 0x59, 0x8B, 0x46, 0x24, 0x3B, 0x05, 0x9C, 0xDA, 0x40, 0x00, 0x74, 0x07, 0x50, 0xE8, 0x1E, 0xC1, 0xFF, 0xFF, 0x59, 0x8B, 0x46, 0x38, 0x3B, 0x05, 0xB0, 0xDA, 0x40, 0x00, 0x74, 0x07, 0x50, 0xE8, 0x0C, 0xC1, 0xFF, 0xFF, 0x59, 0x8B, 0x46, 0x3C, 0x3B, 0x05, 0xB4, 0xDA, 0x40, 0x00, 0x74, 0x07, 0x50, 0xE8, 0xFA, 0xC0, 0xFF, 0xFF, 0x59, 0x8B, 0x46, 0x40, 0x3B, 0x05, 0xB8, 0xDA, 0x40, 0x00, 0x74, 0x07, 0x50, 0xE8, 0xE8, 0xC0, 0xFF, 0xFF, 0x59, 0x8B, 0x46, 0x44, 0x3B, 0x05, 0xBC, 0xDA, 0x40, 0x00, 0x74, 0x07, 0x50, 0xE8, 0xD6, 0xC0, 0xFF, 0xFF, 0x59, 0x8B, 0x46, 0x48, 0x3B, 0x05, 0xC0, 0xDA, 0x40, 0x00, 0x74, 0x07, 0x50, 0xE8, 0xC4, 0xC0, 0xFF, 0xFF, 0x59, 0x8B, 0x76, 0x4C, 0x3B, 0x35, 0xC4, 0xDA, 0x40, 0x00, 0x74, 0x07, 0x56, 0xE8, 0xB2, 0xC0, 0xFF, 0xFF, 0x59, 0x5E, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x83, 0xEC, 0x10, 0xFF, 0x75, 0x0C, 0x8D, 0x4D, 0xF0, 0xE8, 0xF3, 0xBC, 0xFF, 0xFF, 0x0F, 0xB6, 0x45, 0x08, 0x8B, 0x4D, 0xF0, 0x8B, 0x89,
				0xC8, 0x00, 0x00, 0x00, 0x0F, 0xB7, 0x04, 0x41, 0x25, 0x00, 0x80, 0x00, 0x00, 0x80, 0x7D, 0xFC, 0x00, 0x74, 0x07, 0x8B, 0x4D, 0xF8, 0x83, 0x61, 0x70, 0xFD, 0xC9, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x6A, 0x00, 0xFF, 0x75, 0x08, 0xE8, 0xB9, 0xFF, 0xFF, 0xFF, 0x59, 0x59, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x83, 0xEC, 0x18, 0x53, 0xFF, 0x75, 0x10, 0x8D, 0x4D, 0xE8, 0xE8, 0xA7, 0xBC, 0xFF, 0xFF, 0x8B, 0x5D, 0x08, 0x8D, 0x43, 0x01, 0x3D, 0x00, 0x01, 0x00, 0x00, 0x77, 0x0F, 0x8B, 0x45, 0xE8, 0x8B, 0x80, 0xC8, 0x00, 0x00, 0x00, 0x0F, 0xB7, 0x04, 0x58, 0xEB, 0x75, 0x89, 0x5D, 0x08, 0xC1, 0x7D, 0x08, 0x08, 0x8D, 0x45, 0xE8, 0x50, 0x8B, 0x45, 0x08, 0x25, 0xFF, 0x00, 0x00, 0x00, 0x50, 0xE8, 0x6C, 0xFF, 0xFF, 0xFF, 0x59, 0x59, 0x85, 0xC0, 0x74, 0x12, 0x8A, 0x45, 0x08, 0x6A, 0x02, 0x88, 0x45, 0xF8, 0x88, 0x5D, 0xF9, 0xC6, 0x45, 0xFA, 0x00, 0x59, 0xEB, 0x0A, 0x33, 0xC9, 0x88, 0x5D, 0xF8, 0xC6, 0x45, 0xF9, 0x00, 0x41, 0x8B, 0x45, 0xE8, 0x6A, 0x01, 0xFF, 0x70, 0x14, 0xFF, 0x70, 0x04, 0x8D, 0x45, 0xFC, 0x50, 0x51, 0x8D, 0x45, 0xF8, 0x50, 0x8D, 0x45, 0xE8, 0x6A, 0x01, 0x50, 0xE8, 0x81, 0xF7, 0xFF, 0xFF, 0x83, 0xC4, 0x20, 0x85, 0xC0, 0x75, 0x10, 0x38, 0x45, 0xF4, 0x74, 0x07, 0x8B, 0x45, 0xF0, 0x83, 0x60, 0x70, 0xFD, 0x33, 0xC0, 0xEB, 0x14, 0x0F, 0xB7, 0x45, 0xFC, 0x23, 0x45, 0x0C, 0x80, 0x7D, 0xF4, 0x00, 0x74, 0x07, 0x8B, 0x4D, 0xF0, 0x83, 0x61, 0x70, 0xFD, 0x5B, 0xC9, 0xC3, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0x55, 0x8B, 0xEC, 0x56, 0x33, 0xC0, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x8B, 0x55, 0x0C, 0x8D, 0x49, 0x00, 0x8A, 0x02, 0x0A, 0xC0, 0x74, 0x09, 0x83, 0xC2, 0x01, 0x0F, 0xAB, 0x04, 0x24, 0xEB, 0xF1, 0x8B, 0x75, 0x08, 0x83, 0xC9, 0xFF, 0x8D, 0x49, 0x00, 0x83, 0xC1, 0x01, 0x8A, 0x06, 0x0A, 0xC0, 0x74, 0x09, 0x83, 0xC6, 0x01, 0x0F, 0xA3, 0x04, 0x24, 0x73, 0xEE, 0x8B, 0xC1, 0x83, 0xC4, 0x20, 0x5E, 0xC9, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x8B, 0x55, 0x08, 0x56, 0x57, 0x85, 0xD2, 0x74, 0x07, 0x8B, 0x7D, 0x0C, 0x85, 0xFF, 0x75, 0x13, 0xE8, 0x3C, 0xD4, 0xFF, 0xFF, 0x6A, 0x16, 0x5E, 0x89, 0x30, 0xE8, 0xE0, 0xD3, 0xFF, 0xFF, 0x8B, 0xC6, 0xEB, 0x33, 0x8B, 0x45, 0x10, 0x85, 0xC0, 0x75, 0x04, 0x88, 0x02, 0xEB, 0xE2, 0x8B, 0xF2, 0x2B, 0xF0, 0x8A, 0x08, 0x88, 0x0C, 0x06, 0x40, 0x84, 0xC9, 0x74, 0x03, 0x4F, 0x75, 0xF3, 0x85, 0xFF, 0x75, 0x11, 0xC6, 0x02, 0x00, 0xE8, 0x06, 0xD4, 0xFF, 0xFF, 0x6A, 0x22, 0x59, 0x89, 0x08, 0x8B, 0xF1, 0xEB, 0xC6, 0x33, 0xC0, 0x5F, 0x5E, 0x5D, 0xC3, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0x8B, 0x54, 0x24, 0x04, 0x8B, 0x4C, 0x24, 0x08, 0xF7, 0xC2, 0x03, 0x00, 0x00, 0x00, 0x75, 0x3C, 0x8B, 0x02, 0x3A, 0x01, 0x75, 0x2E, 0x0A, 0xC0, 0x74, 0x26, 0x3A, 0x61, 0x01, 0x75, 0x25, 0x0A, 0xE4, 0x74, 0x1D, 0xC1, 0xE8, 0x10, 0x3A, 0x41, 0x02, 0x75, 0x19, 0x0A, 0xC0, 0x74, 0x11, 0x3A, 0x61, 0x03, 0x75, 0x10, 0x83, 0xC1, 0x04, 0x83, 0xC2, 0x04, 0x0A, 0xE4, 0x75, 0xD2, 0x8B, 0xFF, 0x33, 0xC0, 0xC3, 0x90, 0x1B, 0xC0, 0xD1, 0xE0, 0x83, 0xC0, 0x01, 0xC3, 0xF7, 0xC2, 0x01, 0x00, 0x00, 0x00, 0x74, 0x18, 0x8A, 0x02, 0x83, 0xC2,
				0x01, 0x3A, 0x01, 0x75, 0xE7, 0x83, 0xC1, 0x01, 0x0A, 0xC0, 0x74, 0xDC, 0xF7, 0xC2, 0x02, 0x00, 0x00, 0x00, 0x74, 0xA4, 0x66, 0x8B, 0x02, 0x83, 0xC2, 0x02, 0x3A, 0x01, 0x75, 0xCE, 0x0A, 0xC0, 0x74, 0xC6, 0x3A, 0x61, 0x01, 0x75, 0xC5, 0x0A, 0xE4, 0x74, 0xBD, 0x83, 0xC1, 0x02, 0xEB, 0x88, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0x55, 0x8B, 0xEC, 0x57, 0x56, 0x8B, 0x75, 0x0C, 0x8B, 0x4D, 0x10, 0x8B, 0x7D, 0x08, 0x8B, 0xC1, 0x8B, 0xD1, 0x03, 0xC6, 0x3B, 0xFE, 0x76, 0x08, 0x3B, 0xF8, 0x0F, 0x82, 0xA0, 0x01, 0x00, 0x00, 0x81, 0xF9, 0x80, 0x00, 0x00, 0x00, 0x72, 0x1C, 0x83, 0x3D, 0x3C, 0xEA, 0x40, 0x00, 0x00, 0x74, 0x13, 0x57, 0x56, 0x83, 0xE7, 0x0F, 0x83, 0xE6, 0x0F, 0x3B, 0xFE, 0x5E, 0x5F, 0x75, 0x05, 0xE9, 0x57, 0x1E, 0x00, 0x00, 0xF7, 0xC7, 0x03, 0x00, 0x00, 0x00, 0x75, 0x14, 0xC1, 0xE9, 0x02, 0x83, 0xE2, 0x03, 0x83, 0xF9, 0x08, 0x72, 0x29, 0xF3, 0xA5, 0xFF, 0x24, 0x95, 0x80, 0x67, 0x40, 0x00, 0x8B, 0xC7, 0xBA, 0x03, 0x00, 0x00, 0x00, 0x83, 0xE9, 0x04, 0x72, 0x0C, 0x83, 0xE0, 0x03, 0x03, 0xC8, 0xFF, 0x24, 0x85, 0x94, 0x66, 0x40, 0x00, 0xFF, 0x24, 0x8D, 0x90, 0x67, 0x40, 0x00, 0x90, 0xFF, 0x24, 0x8D, 0x14, 0x67, 0x40, 0x00, 0x90, 0xA4, 0x66, 0x40, 0x00, 0xD0, 0x66, 0x40, 0x00, 0xF4, 0x66, 0x40, 0x00, 0x23, 0xD1, 0x8A, 0x06, 0x88, 0x07, 0x8A, 0x46, 0x01, 0x88, 0x47, 0x01, 0x8A, 0x46, 0x02, 0xC1, 0xE9, 0x02, 0x88, 0x47, 0x02, 0x83, 0xC6, 0x03, 0x83, 0xC7, 0x03, 0x83, 0xF9, 0x08, 0x72, 0xCC, 0xF3, 0xA5, 0xFF, 0x24, 0x95, 0x80, 0x67, 0x40, 0x00, 0x8D, 0x49, 0x00, 0x23, 0xD1, 0x8A, 0x06, 0x88, 0x07, 0x8A, 0x46, 0x01, 0xC1, 0xE9, 0x02, 0x88, 0x47, 0x01, 0x83, 0xC6, 0x02, 0x83, 0xC7, 0x02, 0x83, 0xF9, 0x08, 0x72, 0xA6, 0xF3, 0xA5, 0xFF, 0x24, 0x95, 0x80, 0x67, 0x40, 0x00, 0x90, 0x23, 0xD1, 0x8A, 0x06, 0x88, 0x07, 0x83, 0xC6, 0x01, 0xC1, 0xE9, 0x02, 0x83, 0xC7, 0x01, 0x83, 0xF9, 0x08, 0x72, 0x88, 0xF3, 0xA5, 0xFF, 0x24, 0x95, 0x80, 0x67, 0x40, 0x00, 0x8D, 0x49, 0x00, 0x77, 0x67, 0x40, 0x00, 0x64, 0x67, 0x40, 0x00, 0x5C, 0x67, 0x40, 0x00, 0x54, 0x67, 0x40, 0x00, 0x4C, 0x67, 0x40, 0x00, 0x44, 0x67, 0x40, 0x00, 0x3C, 0x67, 0x40, 0x00, 0x34, 0x67, 0x40, 0x00, 0x8B, 0x44, 0x8E, 0xE4, 0x89, 0x44, 0x8F, 0xE4, 0x8B, 0x44, 0x8E, 0xE8, 0x89, 0x44, 0x8F, 0xE8, 0x8B, 0x44, 0x8E, 0xEC, 0x89, 0x44, 0x8F, 0xEC, 0x8B, 0x44, 0x8E, 0xF0, 0x89, 0x44, 0x8F, 0xF0, 0x8B, 0x44, 0x8E, 0xF4, 0x89, 0x44, 0x8F, 0xF4, 0x8B, 0x44, 0x8E, 0xF8, 0x89, 0x44, 0x8F, 0xF8, 0x8B, 0x44, 0x8E, 0xFC, 0x89, 0x44, 0x8F, 0xFC, 0x8D, 0x04, 0x8D, 0x00, 0x00, 0x00, 0x00, 0x03, 0xF0, 0x03, 0xF8, 0xFF, 0x24, 0x95, 0x80, 0x67, 0x40, 0x00, 0x8B, 0xFF, 0x90, 0x67, 0x40, 0x00, 0x98, 0x67, 0x40, 0x00, 0xA4, 0x67, 0x40, 0x00, 0xB8, 0x67, 0x40, 0x00, 0x8B, 0x45, 0x08, 0x5E, 0x5F, 0xC9, 0xC3, 0x90, 0x8A, 0x06, 0x88, 0x07, 0x8B, 0x45, 0x08, 0x5E, 0x5F, 0xC9, 0xC3, 0x90, 0x8A, 0x06, 0x88, 0x07, 0x8A, 0x46, 0x01, 0x88, 0x47, 0x01, 0x8B, 0x45, 0x08, 0x5E, 0x5F, 0xC9, 0xC3, 0x8D, 0x49, 0x00, 0x8A, 0x06, 0x88, 0x07, 0x8A, 0x46, 0x01, 0x88, 0x47, 0x01, 0x8A, 0x46, 0x02, 0x88, 0x47, 0x02, 0x8B, 0x45, 0x08, 0x5E,
				0x5F, 0xC9, 0xC3, 0x90, 0x8D, 0x74, 0x31, 0xFC, 0x8D, 0x7C, 0x39, 0xFC, 0xF7, 0xC7, 0x03, 0x00, 0x00, 0x00, 0x75, 0x24, 0xC1, 0xE9, 0x02, 0x83, 0xE2, 0x03, 0x83, 0xF9, 0x08, 0x72, 0x0D, 0xFD, 0xF3, 0xA5, 0xFC, 0xFF, 0x24, 0x95, 0x1C, 0x69, 0x40, 0x00, 0x8B, 0xFF, 0xF7, 0xD9, 0xFF, 0x24, 0x8D, 0xCC, 0x68, 0x40, 0x00, 0x8D, 0x49, 0x00, 0x8B, 0xC7, 0xBA, 0x03, 0x00, 0x00, 0x00, 0x83, 0xF9, 0x04, 0x72, 0x0C, 0x83, 0xE0, 0x03, 0x2B, 0xC8, 0xFF, 0x24, 0x85, 0x20, 0x68, 0x40, 0x00, 0xFF, 0x24, 0x8D, 0x1C, 0x69, 0x40, 0x00, 0x90, 0x30, 0x68, 0x40, 0x00, 0x54, 0x68, 0x40, 0x00, 0x7C, 0x68, 0x40, 0x00, 0x8A, 0x46, 0x03, 0x23, 0xD1, 0x88, 0x47, 0x03, 0x83, 0xEE, 0x01, 0xC1, 0xE9, 0x02, 0x83, 0xEF, 0x01, 0x83, 0xF9, 0x08, 0x72, 0xB2, 0xFD, 0xF3, 0xA5, 0xFC, 0xFF, 0x24, 0x95, 0x1C, 0x69, 0x40, 0x00, 0x8D, 0x49, 0x00, 0x8A, 0x46, 0x03, 0x23, 0xD1, 0x88, 0x47, 0x03, 0x8A, 0x46, 0x02, 0xC1, 0xE9, 0x02, 0x88, 0x47, 0x02, 0x83, 0xEE, 0x02, 0x83, 0xEF, 0x02, 0x83, 0xF9, 0x08, 0x72, 0x88, 0xFD, 0xF3, 0xA5, 0xFC, 0xFF, 0x24, 0x95, 0x1C, 0x69, 0x40, 0x00, 0x90, 0x8A, 0x46, 0x03, 0x23, 0xD1, 0x88, 0x47, 0x03, 0x8A, 0x46, 0x02, 0x88, 0x47, 0x02, 0x8A, 0x46, 0x01, 0xC1, 0xE9, 0x02, 0x88, 0x47, 0x01, 0x83, 0xEE, 0x03, 0x83, 0xEF, 0x03, 0x83, 0xF9, 0x08, 0x0F, 0x82, 0x56, 0xFF, 0xFF, 0xFF, 0xFD, 0xF3, 0xA5, 0xFC, 0xFF, 0x24, 0x95, 0x1C, 0x69, 0x40, 0x00, 0x8D, 0x49, 0x00, 0xD0, 0x68, 0x40, 0x00, 0xD8, 0x68, 0x40, 0x00, 0xE0, 0x68, 0x40, 0x00, 0xE8, 0x68, 0x40, 0x00, 0xF0, 0x68, 0x40, 0x00, 0xF8, 0x68, 0x40, 0x00, 0x00, 0x69, 0x40, 0x00, 0x13, 0x69, 0x40, 0x00, 0x8B, 0x44, 0x8E, 0x1C, 0x89, 0x44, 0x8F, 0x1C, 0x8B, 0x44, 0x8E, 0x18, 0x89, 0x44, 0x8F, 0x18, 0x8B, 0x44, 0x8E, 0x14, 0x89, 0x44, 0x8F, 0x14, 0x8B, 0x44, 0x8E, 0x10, 0x89, 0x44, 0x8F, 0x10, 0x8B, 0x44, 0x8E, 0x0C, 0x89, 0x44, 0x8F, 0x0C, 0x8B, 0x44, 0x8E, 0x08, 0x89, 0x44, 0x8F, 0x08, 0x8B, 0x44, 0x8E, 0x04, 0x89, 0x44, 0x8F, 0x04, 0x8D, 0x04, 0x8D, 0x00, 0x00, 0x00, 0x00, 0x03, 0xF0, 0x03, 0xF8, 0xFF, 0x24, 0x95, 0x1C, 0x69, 0x40, 0x00, 0x8B, 0xFF, 0x2C, 0x69, 0x40, 0x00, 0x34, 0x69, 0x40, 0x00, 0x44, 0x69, 0x40, 0x00, 0x58, 0x69, 0x40, 0x00, 0x8B, 0x45, 0x08, 0x5E, 0x5F, 0xC9, 0xC3, 0x90, 0x8A, 0x46, 0x03, 0x88, 0x47, 0x03, 0x8B, 0x45, 0x08, 0x5E, 0x5F, 0xC9, 0xC3, 0x8D, 0x49, 0x00, 0x8A, 0x46, 0x03, 0x88, 0x47, 0x03, 0x8A, 0x46, 0x02, 0x88, 0x47, 0x02, 0x8B, 0x45, 0x08, 0x5E, 0x5F, 0xC9, 0xC3, 0x90, 0x8A, 0x46, 0x03, 0x88, 0x47, 0x03, 0x8A, 0x46, 0x02, 0x88, 0x47, 0x02, 0x8A, 0x46, 0x01, 0x88, 0x47, 0x01, 0x8B, 0x45, 0x08, 0x5E, 0x5F, 0xC9, 0xC3, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0x8B, 0x4C, 0x24, 0x04, 0xF7, 0xC1, 0x03, 0x00, 0x00, 0x00, 0x74, 0x24, 0x8A, 0x01, 0x83, 0xC1, 0x01, 0x84, 0xC0, 0x74, 0x4E, 0xF7, 0xC1, 0x03, 0x00, 0x00, 0x00, 0x75, 0xEF, 0x05, 0x00, 0x00, 0x00, 0x00, 0x8D, 0xA4, 0x24, 0x00, 0x00, 0x00, 0x00, 0x8D, 0xA4, 0x24, 0x00, 0x00, 0x00, 0x00, 0x8B, 0x01, 0xBA, 0xFF, 0xFE, 0xFE, 0x7E, 0x03, 0xD0, 0x83, 0xF0, 0xFF, 0x33, 0xC2, 0x83, 0xC1,
				0x04, 0xA9, 0x00, 0x01, 0x01, 0x81, 0x74, 0xE8, 0x8B, 0x41, 0xFC, 0x84, 0xC0, 0x74, 0x32, 0x84, 0xE4, 0x74, 0x24, 0xA9, 0x00, 0x00, 0xFF, 0x00, 0x74, 0x13, 0xA9, 0x00, 0x00, 0x00, 0xFF, 0x74, 0x02, 0xEB, 0xCD, 0x8D, 0x41, 0xFF, 0x8B, 0x4C, 0x24, 0x04, 0x2B, 0xC1, 0xC3, 0x8D, 0x41, 0xFE, 0x8B, 0x4C, 0x24, 0x04, 0x2B, 0xC1, 0xC3, 0x8D, 0x41, 0xFD, 0x8B, 0x4C, 0x24, 0x04, 0x2B, 0xC1, 0xC3, 0x8D, 0x41, 0xFC, 0x8B, 0x4C, 0x24, 0x04, 0x2B, 0xC1, 0xC3, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0x55, 0x8B, 0xEC, 0x56, 0x33, 0xC0, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x8B, 0x55, 0x0C, 0x8D, 0x49, 0x00, 0x8A, 0x02, 0x0A, 0xC0, 0x74, 0x09, 0x83, 0xC2, 0x01, 0x0F, 0xAB, 0x04, 0x24, 0xEB, 0xF1, 0x8B, 0x75, 0x08, 0x8B, 0xFF, 0x8A, 0x06, 0x0A, 0xC0, 0x74, 0x0C, 0x83, 0xC6, 0x01, 0x0F, 0xA3, 0x04, 0x24, 0x73, 0xF1, 0x8D, 0x46, 0xFF, 0x83, 0xC4, 0x20, 0x5E, 0xC9, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x51, 0x51, 0x8B, 0x45, 0x0C, 0x56, 0x8B, 0x75, 0x08, 0x89, 0x45, 0xF8, 0x8B, 0x45, 0x10, 0x57, 0x56, 0x89, 0x45, 0xFC, 0xE8, 0xC6, 0x1B, 0x00, 0x00, 0x83, 0xCF, 0xFF, 0x59, 0x3B, 0xC7, 0x75, 0x11, 0xE8, 0xF1, 0xCE, 0xFF, 0xFF, 0xC7, 0x00, 0x09, 0x00, 0x00, 0x00, 0x8B, 0xC7, 0x8B, 0xD7, 0xEB, 0x4A, 0xFF, 0x75, 0x14, 0x8D, 0x4D, 0xFC, 0x51, 0xFF, 0x75, 0xF8, 0x50, 0xFF, 0x15, 0xF8, 0xA0, 0x40, 0x00, 0x89, 0x45, 0xF8, 0x3B, 0xC7, 0x75, 0x13, 0xFF, 0x15, 0x34, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x74, 0x09, 0x50, 0xE8, 0xE3, 0xCE, 0xFF, 0xFF, 0x59, 0xEB, 0xCF, 0x8B, 0xC6, 0xC1, 0xF8, 0x05, 0x8B, 0x04, 0x85, 0x80, 0xFA, 0x40, 0x00, 0x83, 0xE6, 0x1F, 0xC1, 0xE6, 0x06, 0x8D, 0x44, 0x30, 0x04, 0x80, 0x20, 0xFD, 0x8B, 0x45, 0xF8, 0x8B, 0x55, 0xFC, 0x5F, 0x5E, 0xC9, 0xC3, 0x6A, 0x14, 0x68, 0xF0, 0xC1, 0x40, 0x00, 0xE8, 0xEF, 0xEA, 0xFF, 0xFF, 0x83, 0xCB, 0xFF, 0x89, 0x5D, 0xDC, 0x89, 0x5D, 0xE0, 0x8B, 0x45, 0x08, 0x83, 0xF8, 0xFE, 0x75, 0x1C, 0xE8, 0x88, 0xCE, 0xFF, 0xFF, 0x83, 0x20, 0x00, 0xE8, 0x6D, 0xCE, 0xFF, 0xFF, 0xC7, 0x00, 0x09, 0x00, 0x00, 0x00, 0x8B, 0xC3, 0x8B, 0xD3, 0xE9, 0xA1, 0x00, 0x00, 0x00, 0x85, 0xC0, 0x78, 0x08, 0x3B, 0x05, 0x6C, 0xFA, 0x40, 0x00, 0x72, 0x1A, 0xE8, 0x60, 0xCE, 0xFF, 0xFF, 0x83, 0x20, 0x00, 0xE8, 0x45, 0xCE, 0xFF, 0xFF, 0xC7, 0x00, 0x09, 0x00, 0x00, 0x00, 0xE8, 0xE8, 0xCD, 0xFF, 0xFF, 0xEB, 0xD1, 0x8B, 0xC8, 0xC1, 0xF9, 0x05, 0x8D, 0x3C, 0x8D, 0x80, 0xFA, 0x40, 0x00, 0x8B, 0xF0, 0x83, 0xE6, 0x1F, 0xC1, 0xE6, 0x06, 0x8B, 0x0F, 0x0F, 0xBE, 0x4C, 0x31, 0x04, 0x83, 0xE1, 0x01, 0x74, 0xC6, 0x50, 0xE8, 0x43, 0x1B, 0x00, 0x00, 0x59, 0x83, 0x65, 0xFC, 0x00, 0x8B, 0x07, 0xF6, 0x44, 0x30, 0x04, 0x01, 0x74, 0x1C, 0xFF, 0x75, 0x14, 0xFF, 0x75, 0x10, 0xFF, 0x75, 0x0C, 0xFF, 0x75, 0x08, 0xE8, 0xD7, 0xFE, 0xFF, 0xFF, 0x83, 0xC4, 0x10, 0x89, 0x45, 0xDC, 0x89, 0x55, 0xE0, 0xEB, 0x19, 0xE8, 0xE3, 0xCD, 0xFF, 0xFF, 0xC7, 0x00, 0x09, 0x00, 0x00, 0x00, 0xE8, 0xEB, 0xCD, 0xFF, 0xFF, 0x83, 0x20, 0x00, 0x89, 0x5D, 0xDC, 0x89, 0x5D, 0xE0, 0xC7, 0x45, 0xFC, 0xFE, 0xFF, 0xFF, 0xFF, 0xE8, 0x0C, 0x00, 0x00, 0x00, 0x8B, 0x45, 0xDC, 0x8B, 0x55, 0xE0, 0xE8, 0x61, 0xEA, 0xFF, 0xFF,
				0xC3, 0xFF, 0x75, 0x08, 0xE8, 0x7F, 0x1B, 0x00, 0x00, 0x59, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0xB8, 0xE4, 0x1A, 0x00, 0x00, 0xE8, 0x12, 0x1D, 0x00, 0x00, 0xA1, 0x04, 0xD0, 0x40, 0x00, 0x33, 0xC5, 0x89, 0x45, 0xFC, 0x8B, 0x45, 0x0C, 0x56, 0x8B, 0x75, 0x08, 0x57, 0x33, 0xFF, 0x89, 0x85, 0x34, 0xE5, 0xFF, 0xFF, 0x89, 0xBD, 0x38, 0xE5, 0xFF, 0xFF, 0x89, 0xBD, 0x30, 0xE5, 0xFF, 0xFF, 0x39, 0x7D, 0x10, 0x75, 0x07, 0x33, 0xC0, 0xE9, 0xAE, 0x06, 0x00, 0x00, 0x3B, 0xC7, 0x75, 0x1F, 0xE8, 0x76, 0xCD, 0xFF, 0xFF, 0x89, 0x38, 0xE8, 0x5C, 0xCD, 0xFF, 0xFF, 0xC7, 0x00, 0x16, 0x00, 0x00, 0x00, 0xE8, 0xFF, 0xCC, 0xFF, 0xFF, 0x83, 0xC8, 0xFF, 0xE9, 0x8B, 0x06, 0x00, 0x00, 0x8B, 0xC6, 0xC1, 0xF8, 0x05, 0x8B, 0xFE, 0x53, 0x8D, 0x1C, 0x85, 0x80, 0xFA, 0x40, 0x00, 0x8B, 0x03, 0x83, 0xE7, 0x1F, 0xC1, 0xE7, 0x06, 0x8A, 0x4C, 0x38, 0x24, 0x02, 0xC9, 0xD0, 0xF9, 0x89, 0x9D, 0x24, 0xE5, 0xFF, 0xFF, 0x88, 0x8D, 0x3F, 0xE5, 0xFF, 0xFF, 0x80, 0xF9, 0x02, 0x74, 0x05, 0x80, 0xF9, 0x01, 0x75, 0x27, 0x8B, 0x4D, 0x10, 0xF7, 0xD1, 0xF6, 0xC1, 0x01, 0x75, 0x1D, 0xE8, 0x18, 0xCD, 0xFF, 0xFF, 0x83, 0x20, 0x00, 0xE8, 0xFD, 0xCC, 0xFF, 0xFF, 0xC7, 0x00, 0x16, 0x00, 0x00, 0x00, 0xE8, 0xA0, 0xCC, 0xFF, 0xFF, 0xE9, 0x1D, 0x06, 0x00, 0x00, 0xF6, 0x44, 0x38, 0x04, 0x20, 0x74, 0x0F, 0x6A, 0x02, 0x6A, 0x00, 0x6A, 0x00, 0x56, 0xE8, 0xBE, 0xFD, 0xFF, 0xFF, 0x83, 0xC4, 0x10, 0x56, 0xE8, 0x3E, 0x07, 0x00, 0x00, 0x59, 0x85, 0xC0, 0x0F, 0x84, 0x99, 0x02, 0x00, 0x00, 0x8B, 0x03, 0xF6, 0x44, 0x07, 0x04, 0x80, 0x0F, 0x84, 0x8C, 0x02, 0x00, 0x00, 0xE8, 0x01, 0xC7, 0xFF, 0xFF, 0x8B, 0x40, 0x6C, 0x33, 0xC9, 0x39, 0x48, 0x14, 0x8D, 0x85, 0x20, 0xE5, 0xFF, 0xFF, 0x0F, 0x94, 0xC1, 0x50, 0x8B, 0x03, 0xFF, 0x34, 0x07, 0x8B, 0xF1, 0xFF, 0x15, 0x00, 0xA1, 0x40, 0x00, 0x33, 0xC9, 0x3B, 0xC1, 0x0F, 0x84, 0x60, 0x02, 0x00, 0x00, 0x3B, 0xF1, 0x74, 0x0C, 0x38, 0x8D, 0x3F, 0xE5, 0xFF, 0xFF, 0x0F, 0x84, 0x50, 0x02, 0x00, 0x00, 0xFF, 0x15, 0xFC, 0xA0, 0x40, 0x00, 0x8B, 0x9D, 0x34, 0xE5, 0xFF, 0xFF, 0x89, 0x85, 0x20, 0xE5, 0xFF, 0xFF, 0x33, 0xC0, 0x89, 0x85, 0x2C, 0xE5, 0xFF, 0xFF, 0x39, 0x45, 0x10, 0x0F, 0x86, 0x23, 0x05, 0x00, 0x00, 0x89, 0x85, 0x40, 0xE5, 0xFF, 0xFF, 0x8A, 0x85, 0x3F, 0xE5, 0xFF, 0xFF, 0x84, 0xC0, 0x0F, 0x85, 0x67, 0x01, 0x00, 0x00, 0x8A, 0x0B, 0x8B, 0xB5, 0x24, 0xE5, 0xFF, 0xFF, 0x33, 0xC0, 0x80, 0xF9, 0x0A, 0x0F, 0x94, 0xC0, 0x89, 0x85, 0x1C, 0xE5, 0xFF, 0xFF, 0x8B, 0x06, 0x03, 0xC7, 0x83, 0x78, 0x38, 0x00, 0x74, 0x15, 0x8A, 0x50, 0x34, 0x88, 0x55, 0xF4, 0x88, 0x4D, 0xF5, 0x83, 0x60, 0x38, 0x00, 0x6A, 0x02, 0x8D, 0x45, 0xF4, 0x50, 0xEB, 0x4B, 0x0F, 0xBE, 0xC1, 0x50, 0xE8, 0x9C, 0xF6, 0xFF, 0xFF, 0x59, 0x85, 0xC0, 0x74, 0x3A, 0x8B, 0x8D, 0x34, 0xE5, 0xFF, 0xFF, 0x2B, 0xCB, 0x03, 0x4D, 0x10, 0x33, 0xC0, 0x40, 0x3B, 0xC8, 0x0F, 0x86, 0xA5, 0x01, 0x00, 0x00, 0x6A, 0x02, 0x8D, 0x85, 0x44, 0xE5, 0xFF, 0xFF, 0x53, 0x50, 0xE8, 0x2D, 0x1B, 0x00, 0x00, 0x83, 0xC4, 0x0C, 0x83, 0xF8, 0xFF, 0x0F, 0x84, 0x92, 0x04, 0x00, 0x00, 0x43, 0xFF, 0x85, 0x40, 0xE5, 0xFF, 0xFF, 0xEB, 0x1B, 0x6A, 0x01, 0x53, 0x8D, 0x85,
				0x44, 0xE5, 0xFF, 0xFF, 0x50, 0xE8, 0x09, 0x1B, 0x00, 0x00, 0x83, 0xC4, 0x0C, 0x83, 0xF8, 0xFF, 0x0F, 0x84, 0x6E, 0x04, 0x00, 0x00, 0x33, 0xC0, 0x50, 0x50, 0x6A, 0x05, 0x8D, 0x4D, 0xF4, 0x51, 0x6A, 0x01, 0x8D, 0x8D, 0x44, 0xE5, 0xFF, 0xFF, 0x51, 0x50, 0xFF, 0xB5, 0x20, 0xE5, 0xFF, 0xFF, 0x43, 0xFF, 0x85, 0x40, 0xE5, 0xFF, 0xFF, 0xFF, 0x15, 0xB8, 0xA0, 0x40, 0x00, 0x8B, 0xF0, 0x85, 0xF6, 0x0F, 0x84, 0x3D, 0x04, 0x00, 0x00, 0x6A, 0x00, 0x8D, 0x85, 0x2C, 0xE5, 0xFF, 0xFF, 0x50, 0x56, 0x8D, 0x45, 0xF4, 0x50, 0x8B, 0x85, 0x24, 0xE5, 0xFF, 0xFF, 0x8B, 0x00, 0xFF, 0x34, 0x07, 0xFF, 0x15, 0xA4, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x0F, 0x84, 0x0A, 0x04, 0x00, 0x00, 0x8B, 0x85, 0x40, 0xE5, 0xFF, 0xFF, 0x8B, 0x8D, 0x30, 0xE5, 0xFF, 0xFF, 0x03, 0xC1, 0x89, 0x85, 0x38, 0xE5, 0xFF, 0xFF, 0x39, 0xB5, 0x2C, 0xE5, 0xFF, 0xFF, 0x0F, 0x8C, 0xF6, 0x03, 0x00, 0x00, 0x83, 0xBD, 0x1C, 0xE5, 0xFF, 0xFF, 0x00, 0x0F, 0x84, 0xCD, 0x00, 0x00, 0x00, 0x6A, 0x00, 0x8D, 0x85, 0x2C, 0xE5, 0xFF, 0xFF, 0x50, 0x6A, 0x01, 0x8D, 0x45, 0xF4, 0x50, 0x8B, 0x85, 0x24, 0xE5, 0xFF, 0xFF, 0x8B, 0x00, 0xC6, 0x45, 0xF4, 0x0D, 0xFF, 0x34, 0x07, 0xFF, 0x15, 0xA4, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x0F, 0x84, 0xB1, 0x03, 0x00, 0x00, 0x83, 0xBD, 0x2C, 0xE5, 0xFF, 0xFF, 0x01, 0x0F, 0x8C, 0xB0, 0x03, 0x00, 0x00, 0xFF, 0x85, 0x30, 0xE5, 0xFF, 0xFF, 0xFF, 0x85, 0x38, 0xE5, 0xFF, 0xFF, 0xE9, 0x83, 0x00, 0x00, 0x00, 0x3C, 0x01, 0x74, 0x04, 0x3C, 0x02, 0x75, 0x21, 0x0F, 0xB7, 0x33, 0x33, 0xC9, 0x83, 0xFE, 0x0A, 0x0F, 0x94, 0xC1, 0x83, 0xC3, 0x02, 0x83, 0x85, 0x40, 0xE5, 0xFF, 0xFF, 0x02, 0x89, 0xB5, 0x44, 0xE5, 0xFF, 0xFF, 0x89, 0x8D, 0x1C, 0xE5, 0xFF, 0xFF, 0x3C, 0x01, 0x74, 0x04, 0x3C, 0x02, 0x75, 0x52, 0xFF, 0xB5, 0x44, 0xE5, 0xFF, 0xFF, 0xE8, 0x9A, 0x18, 0x00, 0x00, 0x59, 0x66, 0x3B, 0x85, 0x44, 0xE5, 0xFF, 0xFF, 0x0F, 0x85, 0x49, 0x03, 0x00, 0x00, 0x83, 0x85, 0x38, 0xE5, 0xFF, 0xFF, 0x02, 0x83, 0xBD, 0x1C, 0xE5, 0xFF, 0xFF, 0x00, 0x74, 0x29, 0x6A, 0x0D, 0x58, 0x50, 0x89, 0x85, 0x44, 0xE5, 0xFF, 0xFF, 0xE8, 0x6D, 0x18, 0x00, 0x00, 0x59, 0x66, 0x3B, 0x85, 0x44, 0xE5, 0xFF, 0xFF, 0x0F, 0x85, 0x1C, 0x03, 0x00, 0x00, 0xFF, 0x85, 0x38, 0xE5, 0xFF, 0xFF, 0xFF, 0x85, 0x30, 0xE5, 0xFF, 0xFF, 0x8B, 0x45, 0x10, 0x39, 0x85, 0x40, 0xE5, 0xFF, 0xFF, 0x0F, 0x82, 0xF9, 0xFD, 0xFF, 0xFF, 0xE9, 0x08, 0x03, 0x00, 0x00, 0x8B, 0x0E, 0x8A, 0x13, 0xFF, 0x85, 0x38, 0xE5, 0xFF, 0xFF, 0x88, 0x54, 0x0F, 0x34, 0x8B, 0x0E, 0x89, 0x44, 0x0F, 0x38, 0xE9, 0xEF, 0x02, 0x00, 0x00, 0x33, 0xC9, 0x8B, 0x03, 0xF6, 0x44, 0x38, 0x04, 0x80, 0x0F, 0x84, 0xA1, 0x02, 0x00, 0x00, 0x80, 0xBD, 0x3F, 0xE5, 0xFF, 0xFF, 0x00, 0x89, 0x8D, 0x44, 0xE5, 0xFF, 0xFF, 0x0F, 0x85, 0xA8, 0x00, 0x00, 0x00, 0x8B, 0x9D, 0x34, 0xE5, 0xFF, 0xFF, 0x39, 0x4D, 0x10, 0x0F, 0x86, 0xFD, 0x02, 0x00, 0x00, 0x8B, 0xCB, 0x33, 0xF6, 0x2B, 0x8D, 0x34, 0xE5, 0xFF, 0xFF, 0x8D, 0x85, 0x48, 0xE5, 0xFF, 0xFF, 0x3B, 0x4D, 0x10, 0x73, 0x26, 0x8A, 0x13, 0x43, 0x41, 0x89, 0x9D, 0x20, 0xE5, 0xFF, 0xFF, 0x80, 0xFA, 0x0A, 0x75, 0x0B, 0xFF, 0x85, 0x30, 0xE5, 0xFF, 0xFF, 0xC6, 0x00, 0x0D, 0x40,
				0x46, 0x88, 0x10, 0x40, 0x46, 0x81, 0xFE, 0xFF, 0x13, 0x00, 0x00, 0x72, 0xD5, 0x8B, 0xF0, 0x8D, 0x85, 0x48, 0xE5, 0xFF, 0xFF, 0x2B, 0xF0, 0x6A, 0x00, 0x8D, 0x85, 0x28, 0xE5, 0xFF, 0xFF, 0x50, 0x56, 0x8D, 0x85, 0x48, 0xE5, 0xFF, 0xFF, 0x50, 0x8B, 0x85, 0x24, 0xE5, 0xFF, 0xFF, 0x8B, 0x00, 0xFF, 0x34, 0x07, 0xFF, 0x15, 0xA4, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x0F, 0x84, 0x43, 0x02, 0x00, 0x00, 0x8B, 0x85, 0x28, 0xE5, 0xFF, 0xFF, 0x01, 0x85, 0x38, 0xE5, 0xFF, 0xFF, 0x3B, 0xC6, 0x0F, 0x8C, 0x3B, 0x02, 0x00, 0x00, 0x8B, 0xC3, 0x2B, 0x85, 0x34, 0xE5, 0xFF, 0xFF, 0x3B, 0x45, 0x10, 0x0F, 0x82, 0x6C, 0xFF, 0xFF, 0xFF, 0xE9, 0x25, 0x02, 0x00, 0x00, 0x80, 0xBD, 0x3F, 0xE5, 0xFF, 0xFF, 0x02, 0x0F, 0x85, 0xCD, 0x00, 0x00, 0x00, 0x8B, 0x9D, 0x34, 0xE5, 0xFF, 0xFF, 0x39, 0x4D, 0x10, 0x0F, 0x86, 0x48, 0x02, 0x00, 0x00, 0x83, 0xA5, 0x40, 0xE5, 0xFF, 0xFF, 0x00, 0x8B, 0xCB, 0x2B, 0x8D, 0x34, 0xE5, 0xFF, 0xFF, 0x6A, 0x02, 0x8D, 0x85, 0x48, 0xE5, 0xFF, 0xFF, 0x5E, 0x3B, 0x4D, 0x10, 0x73, 0x43, 0x0F, 0xB7, 0x13, 0x03, 0xDE, 0x03, 0xCE, 0x89, 0x9D, 0x20, 0xE5, 0xFF, 0xFF, 0x83, 0xFA, 0x0A, 0x75, 0x1A, 0x01, 0xB5, 0x30, 0xE5, 0xFF, 0xFF, 0x6A, 0x0D, 0x5B, 0x66, 0x89, 0x18, 0x8B, 0x9D, 0x20, 0xE5, 0xFF, 0xFF, 0x03, 0xC6, 0x01, 0xB5, 0x40, 0xE5, 0xFF, 0xFF, 0x01, 0xB5, 0x40, 0xE5, 0xFF, 0xFF, 0x66, 0x89, 0x10, 0x03, 0xC6, 0x81, 0xBD, 0x40, 0xE5, 0xFF, 0xFF, 0xFE, 0x13, 0x00, 0x00, 0x72, 0xB8, 0x8B, 0xF0, 0x8D, 0x85, 0x48, 0xE5, 0xFF, 0xFF, 0x2B, 0xF0, 0x6A, 0x00, 0x8D, 0x85, 0x28, 0xE5, 0xFF, 0xFF, 0x50, 0x56, 0x8D, 0x85, 0x48, 0xE5, 0xFF, 0xFF, 0x50, 0x8B, 0x85, 0x24, 0xE5, 0xFF, 0xFF, 0x8B, 0x00, 0xFF, 0x34, 0x07, 0xFF, 0x15, 0xA4, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x0F, 0x84, 0x69, 0x01, 0x00, 0x00, 0x8B, 0x85, 0x28, 0xE5, 0xFF, 0xFF, 0x01, 0x85, 0x38, 0xE5, 0xFF, 0xFF, 0x3B, 0xC6, 0x0F, 0x8C, 0x61, 0x01, 0x00, 0x00, 0x8B, 0xC3, 0x2B, 0x85, 0x34, 0xE5, 0xFF, 0xFF, 0x3B, 0x45, 0x10, 0x0F, 0x82, 0x47, 0xFF, 0xFF, 0xFF, 0xE9, 0x4B, 0x01, 0x00, 0x00, 0x8B, 0x85, 0x34, 0xE5, 0xFF, 0xFF, 0x89, 0x85, 0x2C, 0xE5, 0xFF, 0xFF, 0x39, 0x4D, 0x10, 0x0F, 0x86, 0x75, 0x01, 0x00, 0x00, 0x8B, 0x8D, 0x2C, 0xE5, 0xFF, 0xFF, 0x83, 0xA5, 0x40, 0xE5, 0xFF, 0xFF, 0x00, 0x2B, 0x8D, 0x34, 0xE5, 0xFF, 0xFF, 0x6A, 0x02, 0x8D, 0x85, 0x48, 0xF9, 0xFF, 0xFF, 0x5E, 0x3B, 0x4D, 0x10, 0x73, 0x3B, 0x8B, 0x95, 0x2C, 0xE5, 0xFF, 0xFF, 0x0F, 0xB7, 0x12, 0x01, 0xB5, 0x2C, 0xE5, 0xFF, 0xFF, 0x03, 0xCE, 0x83, 0xFA, 0x0A, 0x75, 0x0E, 0x6A, 0x0D, 0x5B, 0x66, 0x89, 0x18, 0x03, 0xC6, 0x01, 0xB5, 0x40, 0xE5, 0xFF, 0xFF, 0x01, 0xB5, 0x40, 0xE5, 0xFF, 0xFF, 0x66, 0x89, 0x10, 0x03, 0xC6, 0x81, 0xBD, 0x40, 0xE5, 0xFF, 0xFF, 0xA8, 0x06, 0x00, 0x00, 0x72, 0xC0, 0x33, 0xF6, 0x56, 0x56, 0x68, 0x55, 0x0D, 0x00, 0x00, 0x8D, 0x8D, 0xF0, 0xEB, 0xFF, 0xFF, 0x51, 0x8D, 0x8D, 0x48, 0xF9, 0xFF, 0xFF, 0x2B, 0xC1, 0x99, 0x2B, 0xC2, 0xD1, 0xF8, 0x50, 0x8B, 0xC1, 0x50, 0x56, 0x68, 0xE9, 0xFD, 0x00, 0x00, 0xFF, 0x15, 0xB8, 0xA0, 0x40, 0x00, 0x8B, 0xD8, 0x3B, 0xDE, 0x0F, 0x84, 0x97, 0x00, 0x00, 0x00, 0x6A, 0x00, 0x8D, 0x85, 0x28, 0xE5, 0xFF,
				0xFF, 0x50, 0x8B, 0xC3, 0x2B, 0xC6, 0x50, 0x8D, 0x84, 0x35, 0xF0, 0xEB, 0xFF, 0xFF, 0x50, 0x8B, 0x85, 0x24, 0xE5, 0xFF, 0xFF, 0x8B, 0x00, 0xFF, 0x34, 0x07, 0xFF, 0x15, 0xA4, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x74, 0x0C, 0x03, 0xB5, 0x28, 0xE5, 0xFF, 0xFF, 0x3B, 0xDE, 0x7F, 0xCB, 0xEB, 0x0C, 0xFF, 0x15, 0x34, 0xA0, 0x40, 0x00, 0x89, 0x85, 0x44, 0xE5, 0xFF, 0xFF, 0x3B, 0xDE, 0x7F, 0x5C, 0x8B, 0x85, 0x2C, 0xE5, 0xFF, 0xFF, 0x2B, 0x85, 0x34, 0xE5, 0xFF, 0xFF, 0x89, 0x85, 0x38, 0xE5, 0xFF, 0xFF, 0x3B, 0x45, 0x10, 0x0F, 0x82, 0x0B, 0xFF, 0xFF, 0xFF, 0xEB, 0x3F, 0x51, 0x8D, 0x8D, 0x28, 0xE5, 0xFF, 0xFF, 0x51, 0xFF, 0x75, 0x10, 0xFF, 0xB5, 0x34, 0xE5, 0xFF, 0xFF, 0xFF, 0x34, 0x38, 0xFF, 0x15, 0xA4, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x74, 0x15, 0x8B, 0x85, 0x28, 0xE5, 0xFF, 0xFF, 0x83, 0xA5, 0x44, 0xE5, 0xFF, 0xFF, 0x00, 0x89, 0x85, 0x38, 0xE5, 0xFF, 0xFF, 0xEB, 0x0C, 0xFF, 0x15, 0x34, 0xA0, 0x40, 0x00, 0x89, 0x85, 0x44, 0xE5, 0xFF, 0xFF, 0x83, 0xBD, 0x38, 0xE5, 0xFF, 0xFF, 0x00, 0x75, 0x6C, 0x83, 0xBD, 0x44, 0xE5, 0xFF, 0xFF, 0x00, 0x74, 0x2D, 0x6A, 0x05, 0x5E, 0x39, 0xB5, 0x44, 0xE5, 0xFF, 0xFF, 0x75, 0x14, 0xE8, 0x1E, 0xC7, 0xFF, 0xFF, 0xC7, 0x00, 0x09, 0x00, 0x00, 0x00, 0xE8, 0x26, 0xC7, 0xFF, 0xFF, 0x89, 0x30, 0xEB, 0x3F, 0xFF, 0xB5, 0x44, 0xE5, 0xFF, 0xFF, 0xE8, 0x2A, 0xC7, 0xFF, 0xFF, 0x59, 0xEB, 0x31, 0x8B, 0x85, 0x24, 0xE5, 0xFF, 0xFF, 0x8B, 0x00, 0xF6, 0x44, 0x07, 0x04, 0x40, 0x74, 0x0F, 0x8B, 0x85, 0x34, 0xE5, 0xFF, 0xFF, 0x80, 0x38, 0x1A, 0x75, 0x04, 0x33, 0xC0, 0xEB, 0x24, 0xE8, 0xDE, 0xC6, 0xFF, 0xFF, 0xC7, 0x00, 0x1C, 0x00, 0x00, 0x00, 0xE8, 0xE6, 0xC6, 0xFF, 0xFF, 0x83, 0x20, 0x00, 0x83, 0xC8, 0xFF, 0xEB, 0x0C, 0x8B, 0x85, 0x38, 0xE5, 0xFF, 0xFF, 0x2B, 0x85, 0x30, 0xE5, 0xFF, 0xFF, 0x5B, 0x8B, 0x4D, 0xFC, 0x5F, 0x33, 0xCD, 0x5E, 0xE8, 0x05, 0xAE, 0xFF, 0xFF, 0xC9, 0xC3, 0x6A, 0x10, 0x68, 0x10, 0xC2, 0x40, 0x00, 0xE8, 0x08, 0xE3, 0xFF, 0xFF, 0x8B, 0x5D, 0x08, 0x83, 0xFB, 0xFE, 0x75, 0x1B, 0xE8, 0xAA, 0xC6, 0xFF, 0xFF, 0x83, 0x20, 0x00, 0xE8, 0x8F, 0xC6, 0xFF, 0xFF, 0xC7, 0x00, 0x09, 0x00, 0x00, 0x00, 0x83, 0xC8, 0xFF, 0xE9, 0x94, 0x00, 0x00, 0x00, 0x85, 0xDB, 0x78, 0x08, 0x3B, 0x1D, 0x6C, 0xFA, 0x40, 0x00, 0x72, 0x1A, 0xE8, 0x83, 0xC6, 0xFF, 0xFF, 0x83, 0x20, 0x00, 0xE8, 0x68, 0xC6, 0xFF, 0xFF, 0xC7, 0x00, 0x09, 0x00, 0x00, 0x00, 0xE8, 0x0B, 0xC6, 0xFF, 0xFF, 0xEB, 0xD2, 0x8B, 0xC3, 0xC1, 0xF8, 0x05, 0x8D, 0x3C, 0x85, 0x80, 0xFA, 0x40, 0x00, 0x8B, 0xF3, 0x83, 0xE6, 0x1F, 0xC1, 0xE6, 0x06, 0x8B, 0x07, 0x0F, 0xBE, 0x44, 0x30, 0x04, 0x83, 0xE0, 0x01, 0x74, 0xC6, 0x53, 0xE8, 0x66, 0x13, 0x00, 0x00, 0x59, 0x83, 0x65, 0xFC, 0x00, 0x8B, 0x07, 0xF6, 0x44, 0x30, 0x04, 0x01, 0x74, 0x14, 0xFF, 0x75, 0x10, 0xFF, 0x75, 0x0C, 0x53, 0xE8, 0x6E, 0xF8, 0xFF, 0xFF, 0x83, 0xC4, 0x0C, 0x89, 0x45, 0xE4, 0xEB, 0x17, 0xE8, 0x0E, 0xC6, 0xFF, 0xFF, 0xC7, 0x00, 0x09, 0x00, 0x00, 0x00, 0xE8, 0x16, 0xC6, 0xFF, 0xFF, 0x83, 0x20, 0x00, 0x83, 0x4D, 0xE4, 0xFF, 0xC7, 0x45, 0xFC, 0xFE, 0xFF, 0xFF, 0xFF, 0xE8, 0x0C, 0x00, 0x00, 0x00, 0x8B, 0x45, 0xE4, 0xE8, 0x91, 0xE2, 0xFF, 0xFF,
				0xC3, 0x8B, 0x5D, 0x08, 0x53, 0xE8, 0xAE, 0x13, 0x00, 0x00, 0x59, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0xFF, 0x05, 0xF4, 0xE9, 0x40, 0x00, 0x68, 0x00, 0x10, 0x00, 0x00, 0xE8, 0x66, 0xEA, 0xFF, 0xFF, 0x59, 0x8B, 0x4D, 0x08, 0x89, 0x41, 0x08, 0x85, 0xC0, 0x74, 0x0D, 0x83, 0x49, 0x0C, 0x08, 0xC7, 0x41, 0x18, 0x00, 0x10, 0x00, 0x00, 0xEB, 0x11, 0x83, 0x49, 0x0C, 0x04, 0x8D, 0x41, 0x14, 0x89, 0x41, 0x08, 0xC7, 0x41, 0x18, 0x02, 0x00, 0x00, 0x00, 0x8B, 0x41, 0x08, 0x83, 0x61, 0x04, 0x00, 0x89, 0x01, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x8B, 0x45, 0x08, 0x83, 0xF8, 0xFE, 0x75, 0x0F, 0xE8, 0x81, 0xC5, 0xFF, 0xFF, 0xC7, 0x00, 0x09, 0x00, 0x00, 0x00, 0x33, 0xC0, 0x5D, 0xC3, 0x85, 0xC0, 0x78, 0x08, 0x3B, 0x05, 0x6C, 0xFA, 0x40, 0x00, 0x72, 0x12, 0xE8, 0x66, 0xC5, 0xFF, 0xFF, 0xC7, 0x00, 0x09, 0x00, 0x00, 0x00, 0xE8, 0x09, 0xC5, 0xFF, 0xFF, 0xEB, 0xDE, 0x8B, 0xC8, 0x83, 0xE0, 0x1F, 0xC1, 0xF9, 0x05, 0x8B, 0x0C, 0x8D, 0x80, 0xFA, 0x40, 0x00, 0xC1, 0xE0, 0x06, 0x0F, 0xBE, 0x44, 0x01, 0x04, 0x83, 0xE0, 0x40, 0x5D, 0xC3, 0xB8, 0xE0, 0xDA, 0x40, 0x00, 0xC3, 0xA1, 0x60, 0xFA, 0x40, 0x00, 0x56, 0x6A, 0x14, 0x5E, 0x85, 0xC0, 0x75, 0x07, 0xB8, 0x00, 0x02, 0x00, 0x00, 0xEB, 0x06, 0x3B, 0xC6, 0x7D, 0x07, 0x8B, 0xC6, 0xA3, 0x60, 0xFA, 0x40, 0x00, 0x6A, 0x04, 0x50, 0xE8, 0xF4, 0xE9, 0xFF, 0xFF, 0x59, 0x59, 0xA3, 0x40, 0xEA, 0x40, 0x00, 0x85, 0xC0, 0x75, 0x1E, 0x6A, 0x04, 0x56, 0x89, 0x35, 0x60, 0xFA, 0x40, 0x00, 0xE8, 0xDB, 0xE9, 0xFF, 0xFF, 0x59, 0x59, 0xA3, 0x40, 0xEA, 0x40, 0x00, 0x85, 0xC0, 0x75, 0x05, 0x6A, 0x1A, 0x58, 0x5E, 0xC3, 0x33, 0xD2, 0xB9, 0xE0, 0xDA, 0x40, 0x00, 0xEB, 0x05, 0xA1, 0x40, 0xEA, 0x40, 0x00, 0x89, 0x0C, 0x02, 0x83, 0xC1, 0x20, 0x83, 0xC2, 0x04, 0x81, 0xF9, 0x60, 0xDD, 0x40, 0x00, 0x7C, 0xEA, 0x6A, 0xFE, 0x5E, 0x33, 0xD2, 0xB9, 0xF0, 0xDA, 0x40, 0x00, 0x57, 0x8B, 0xC2, 0xC1, 0xF8, 0x05, 0x8B, 0x04, 0x85, 0x80, 0xFA, 0x40, 0x00, 0x8B, 0xFA, 0x83, 0xE7, 0x1F, 0xC1, 0xE7, 0x06, 0x8B, 0x04, 0x07, 0x83, 0xF8, 0xFF, 0x74, 0x08, 0x3B, 0xC6, 0x74, 0x04, 0x85, 0xC0, 0x75, 0x02, 0x89, 0x31, 0x83, 0xC1, 0x20, 0x42, 0x81, 0xF9, 0x50, 0xDB, 0x40, 0x00, 0x7C, 0xCE, 0x5F, 0x33, 0xC0, 0x5E, 0xC3, 0xE8, 0x46, 0x16, 0x00, 0x00, 0x80, 0x3D, 0x54, 0xE1, 0x40, 0x00, 0x00, 0x74, 0x05, 0xE8, 0x12, 0x14, 0x00, 0x00, 0xFF, 0x35, 0x40, 0xEA, 0x40, 0x00, 0xE8, 0x72, 0xAF, 0xFF, 0xFF, 0x59, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x56, 0x8B, 0x75, 0x08, 0xB8, 0xE0, 0xDA, 0x40, 0x00, 0x3B, 0xF0, 0x72, 0x22, 0x81, 0xFE, 0x40, 0xDD, 0x40, 0x00, 0x77, 0x1A, 0x8B, 0xCE, 0x2B, 0xC8, 0xC1, 0xF9, 0x05, 0x83, 0xC1, 0x10, 0x51, 0xE8, 0xA8, 0xE8, 0xFF, 0xFF, 0x81, 0x4E, 0x0C, 0x00, 0x80, 0x00, 0x00, 0x59, 0xEB, 0x0A, 0x83, 0xC6, 0x20, 0x56, 0xFF, 0x15, 0xF0, 0xA0, 0x40, 0x00, 0x5E, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x8B, 0x45, 0x08, 0x83, 0xF8, 0x14, 0x7D, 0x16, 0x83, 0xC0, 0x10, 0x50, 0xE8, 0x7B, 0xE8, 0xFF, 0xFF, 0x8B, 0x45, 0x0C, 0x81, 0x48, 0x0C, 0x00, 0x80, 0x00, 0x00, 0x59, 0x5D, 0xC3, 0x8B, 0x45, 0x0C, 0x83, 0xC0, 0x20, 0x50, 0xFF, 0x15, 0xF0, 0xA0, 0x40, 0x00, 0x5D,
				0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x8B, 0x45, 0x08, 0xB9, 0xE0, 0xDA, 0x40, 0x00, 0x3B, 0xC1, 0x72, 0x1F, 0x3D, 0x40, 0xDD, 0x40, 0x00, 0x77, 0x18, 0x81, 0x60, 0x0C, 0xFF, 0x7F, 0xFF, 0xFF, 0x2B, 0xC1, 0xC1, 0xF8, 0x05, 0x83, 0xC0, 0x10, 0x50, 0xE8, 0x59, 0xE7, 0xFF, 0xFF, 0x59, 0x5D, 0xC3, 0x83, 0xC0, 0x20, 0x50, 0xFF, 0x15, 0xEC, 0xA0, 0x40, 0x00, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x8B, 0x4D, 0x08, 0x8B, 0x45, 0x0C, 0x83, 0xF9, 0x14, 0x7D, 0x13, 0x81, 0x60, 0x0C, 0xFF, 0x7F, 0xFF, 0xFF, 0x83, 0xC1, 0x10, 0x51, 0xE8, 0x2A, 0xE7, 0xFF, 0xFF, 0x59, 0x5D, 0xC3, 0x83, 0xC0, 0x20, 0x50, 0xFF, 0x15, 0xEC, 0xA0, 0x40, 0x00, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x8B, 0x45, 0x08, 0x85, 0xC0, 0x75, 0x15, 0xE8, 0x77, 0xC3, 0xFF, 0xFF, 0xC7, 0x00, 0x16, 0x00, 0x00, 0x00, 0xE8, 0x1A, 0xC3, 0xFF, 0xFF, 0x83, 0xC8, 0xFF, 0x5D, 0xC3, 0x8B, 0x40, 0x10, 0x5D, 0xC3, 0x8B, 0xFF, 0x56, 0x57, 0x33, 0xFF, 0xFF, 0xB7, 0x60, 0xDD, 0x40, 0x00, 0xFF, 0x15, 0x78, 0xA0, 0x40, 0x00, 0x89, 0x87, 0x60, 0xDD, 0x40, 0x00, 0x83, 0xC7, 0x04, 0x83, 0xFF, 0x28, 0x72, 0xE6, 0x5F, 0x5E, 0xC3, 0xA1, 0x04, 0xD0, 0x40, 0x00, 0x83, 0xC8, 0x01, 0x33, 0xC9, 0x39, 0x05, 0xF8, 0xE9, 0x40, 0x00, 0x0F, 0x94, 0xC1, 0x8B, 0xC1, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x83, 0xEC, 0x10, 0x53, 0x56, 0x8B, 0x75, 0x0C, 0x33, 0xDB, 0x57, 0x8B, 0x7D, 0x10, 0x3B, 0xF3, 0x75, 0x11, 0x3B, 0xFB, 0x76, 0x0D, 0x8B, 0x45, 0x08, 0x3B, 0xC3, 0x74, 0x02, 0x89, 0x18, 0x33, 0xC0, 0xEB, 0x7B, 0x8B, 0x45, 0x08, 0x3B, 0xC3, 0x74, 0x03, 0x83, 0x08, 0xFF, 0x81, 0xFF, 0xFF, 0xFF, 0xFF, 0x7F, 0x76, 0x13, 0xE8, 0xEA, 0xC2, 0xFF, 0xFF, 0x6A, 0x16, 0x5E, 0x89, 0x30, 0xE8, 0x8E, 0xC2, 0xFF, 0xFF, 0x8B, 0xC6, 0xEB, 0x56, 0xFF, 0x75, 0x18, 0x8D, 0x4D, 0xF0, 0xE8, 0x33, 0xAA, 0xFF, 0xFF, 0x8B, 0x45, 0xF0, 0x39, 0x58, 0x14, 0x0F, 0x85, 0x90, 0x00, 0x00, 0x00, 0x66, 0x8B, 0x45, 0x14, 0xB9, 0xFF, 0x00, 0x00, 0x00, 0x66, 0x3B, 0xC1, 0x76, 0x36, 0x3B, 0xF3, 0x74, 0x0F, 0x3B, 0xFB, 0x76, 0x0B, 0x57, 0x53, 0x56, 0xE8, 0xAB, 0xE1, 0xFF, 0xFF, 0x83, 0xC4, 0x0C, 0xE8, 0x9F, 0xC2, 0xFF, 0xFF, 0xC7, 0x00, 0x2A, 0x00, 0x00, 0x00, 0xE8, 0x94, 0xC2, 0xFF, 0xFF, 0x8B, 0x00, 0x38, 0x5D, 0xFC, 0x74, 0x07, 0x8B, 0x4D, 0xF8, 0x83, 0x61, 0x70, 0xFD, 0x5F, 0x5E, 0x5B, 0xC9, 0xC3, 0x3B, 0xF3, 0x74, 0x26, 0x3B, 0xFB, 0x77, 0x20, 0xE8, 0x74, 0xC2, 0xFF, 0xFF, 0x6A, 0x22, 0x5E, 0x89, 0x30, 0xE8, 0x18, 0xC2, 0xFF, 0xFF, 0x38, 0x5D, 0xFC, 0x74, 0x85, 0x8B, 0x45, 0xF8, 0x83, 0x60, 0x70, 0xFD, 0xE9, 0x79, 0xFF, 0xFF, 0xFF, 0x88, 0x06, 0x8B, 0x45, 0x08, 0x3B, 0xC3, 0x74, 0x06, 0xC7, 0x00, 0x01, 0x00, 0x00, 0x00, 0x38, 0x5D, 0xFC, 0x0F, 0x84, 0x3C, 0xFF, 0xFF, 0xFF, 0x8B, 0x45, 0xF8, 0x83, 0x60, 0x70, 0xFD, 0xE9, 0x30, 0xFF, 0xFF, 0xFF, 0x8D, 0x4D, 0x0C, 0x51, 0x53, 0x57, 0x56, 0x6A, 0x01, 0x8D, 0x4D, 0x14, 0x51, 0x53, 0x89, 0x5D, 0x0C, 0xFF, 0x70, 0x04, 0xFF, 0x15, 0xB8, 0xA0, 0x40, 0x00, 0x3B, 0xC3, 0x74, 0x14, 0x39, 0x5D, 0x0C, 0x0F, 0x85, 0x6A, 0xFF, 0xFF, 0xFF, 0x8B, 0x4D, 0x08, 0x3B, 0xCB, 0x74, 0xBD, 0x89, 0x01, 0xEB, 0xB9, 0xFF, 0x15, 0x34,
				0xA0, 0x40, 0x00, 0x83, 0xF8, 0x7A, 0x0F, 0x85, 0x50, 0xFF, 0xFF, 0xFF, 0x3B, 0xF3, 0x0F, 0x84, 0x73, 0xFF, 0xFF, 0xFF, 0x3B, 0xFB, 0x0F, 0x86, 0x6B, 0xFF, 0xFF, 0xFF, 0x57, 0x53, 0x56, 0xE8, 0xE0, 0xE0, 0xFF, 0xFF, 0x83, 0xC4, 0x0C, 0xE9, 0x5B, 0xFF, 0xFF, 0xFF, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x6A, 0x00, 0xFF, 0x75, 0x14, 0xFF, 0x75, 0x10, 0xFF, 0x75, 0x0C, 0xFF, 0x75, 0x08, 0xE8, 0x93, 0xFE, 0xFF, 0xFF, 0x83, 0xC4, 0x14, 0x5D, 0xC3, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0x56, 0x8B, 0x44, 0x24, 0x14, 0x0B, 0xC0, 0x75, 0x28, 0x8B, 0x4C, 0x24, 0x10, 0x8B, 0x44, 0x24, 0x0C, 0x33, 0xD2, 0xF7, 0xF1, 0x8B, 0xD8, 0x8B, 0x44, 0x24, 0x08, 0xF7, 0xF1, 0x8B, 0xF0, 0x8B, 0xC3, 0xF7, 0x64, 0x24, 0x10, 0x8B, 0xC8, 0x8B, 0xC6, 0xF7, 0x64, 0x24, 0x10, 0x03, 0xD1, 0xEB, 0x47, 0x8B, 0xC8, 0x8B, 0x5C, 0x24, 0x10, 0x8B, 0x54, 0x24, 0x0C, 0x8B, 0x44, 0x24, 0x08, 0xD1, 0xE9, 0xD1, 0xDB, 0xD1, 0xEA, 0xD1, 0xD8, 0x0B, 0xC9, 0x75, 0xF4, 0xF7, 0xF3, 0x8B, 0xF0, 0xF7, 0x64, 0x24, 0x14, 0x8B, 0xC8, 0x8B, 0x44, 0x24, 0x10, 0xF7, 0xE6, 0x03, 0xD1, 0x72, 0x0E, 0x3B, 0x54, 0x24, 0x0C, 0x77, 0x08, 0x72, 0x0F, 0x3B, 0x44, 0x24, 0x08, 0x76, 0x09, 0x4E, 0x2B, 0x44, 0x24, 0x10, 0x1B, 0x54, 0x24, 0x14, 0x33, 0xDB, 0x2B, 0x44, 0x24, 0x08, 0x1B, 0x54, 0x24, 0x0C, 0xF7, 0xDA, 0xF7, 0xD8, 0x83, 0xDA, 0x00, 0x8B, 0xCA, 0x8B, 0xD3, 0x8B, 0xD9, 0x8B, 0xC8, 0x8B, 0xC6, 0x5E, 0xC2, 0x10, 0x00, 0x6A, 0x08, 0x68, 0x30, 0xC2, 0x40, 0x00, 0xE8, 0x6F, 0xDD, 0xFF, 0xFF, 0xE8, 0x51, 0xBB, 0xFF, 0xFF, 0x8B, 0x40, 0x78, 0x85, 0xC0, 0x74, 0x16, 0x83, 0x65, 0xFC, 0x00, 0xFF, 0xD0, 0xEB, 0x07, 0x33, 0xC0, 0x40, 0xC3, 0x8B, 0x65, 0xE8, 0xC7, 0x45, 0xFC, 0xFE, 0xFF, 0xFF, 0xFF, 0xE8, 0xBB, 0x12, 0x00, 0x00, 0xE8, 0x88, 0xDD, 0xFF, 0xFF, 0xC3, 0x68, 0x55, 0x78, 0x40, 0x00, 0xFF, 0x15, 0x78, 0xA0, 0x40, 0x00, 0xA3, 0xFC, 0xE9, 0x40, 0x00, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x8B, 0x45, 0x08, 0xA3, 0x00, 0xEA, 0x40, 0x00, 0xA3, 0x04, 0xEA, 0x40, 0x00, 0xA3, 0x08, 0xEA, 0x40, 0x00, 0xA3, 0x0C, 0xEA, 0x40, 0x00, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x8B, 0x45, 0x08, 0x8B, 0x0D, 0xBC, 0xB0, 0x40, 0x00, 0x56, 0x39, 0x50, 0x04, 0x74, 0x0F, 0x8B, 0xF1, 0x6B, 0xF6, 0x0C, 0x03, 0x75, 0x08, 0x83, 0xC0, 0x0C, 0x3B, 0xC6, 0x72, 0xEC, 0x6B, 0xC9, 0x0C, 0x03, 0x4D, 0x08, 0x5E, 0x3B, 0xC1, 0x73, 0x05, 0x39, 0x50, 0x04, 0x74, 0x02, 0x33, 0xC0, 0x5D, 0xC3, 0xFF, 0x35, 0x08, 0xEA, 0x40, 0x00, 0xFF, 0x15, 0x88, 0xA0, 0x40, 0x00, 0xC3, 0x6A, 0x20, 0x68, 0x50, 0xC2, 0x40, 0x00, 0xE8, 0xC3, 0xDC, 0xFF, 0xFF, 0x33, 0xFF, 0x89, 0x7D, 0xE4, 0x89, 0x7D, 0xD8, 0x8B, 0x5D, 0x08, 0x83, 0xFB, 0x0B, 0x7F, 0x4B, 0x74, 0x15, 0x8B, 0xC3, 0x6A, 0x02, 0x59, 0x2B, 0xC1, 0x74, 0x22, 0x2B, 0xC1, 0x74, 0x08, 0x2B, 0xC1, 0x74, 0x59, 0x2B, 0xC1, 0x75, 0x43, 0xE8, 0x05, 0xBA, 0xFF, 0xFF, 0x8B, 0xF8, 0x89, 0x7D, 0xD8, 0x85, 0xFF, 0x75, 0x14, 0x83, 0xC8, 0xFF, 0xE9, 0x54, 0x01, 0x00, 0x00, 0xBE, 0x00, 0xEA, 0x40, 0x00, 0xA1, 0x00, 0xEA, 0x40, 0x00, 0xEB, 0x55, 0xFF, 0x77, 0x5C, 0x8B, 0xD3, 0xE8, 0x5D, 0xFF, 0xFF, 0xFF,
				0x59, 0x8D, 0x70, 0x08, 0x8B, 0x06, 0xEB, 0x51, 0x8B, 0xC3, 0x83, 0xE8, 0x0F, 0x74, 0x32, 0x83, 0xE8, 0x06, 0x74, 0x21, 0x48, 0x74, 0x12, 0xE8, 0xF0, 0xBF, 0xFF, 0xFF, 0xC7, 0x00, 0x16, 0x00, 0x00, 0x00, 0xE8, 0x93, 0xBF, 0xFF, 0xFF, 0xEB, 0xB9, 0xBE, 0x08, 0xEA, 0x40, 0x00, 0xA1, 0x08, 0xEA, 0x40, 0x00, 0xEB, 0x16, 0xBE, 0x04, 0xEA, 0x40, 0x00, 0xA1, 0x04, 0xEA, 0x40, 0x00, 0xEB, 0x0A, 0xBE, 0x0C, 0xEA, 0x40, 0x00, 0xA1, 0x0C, 0xEA, 0x40, 0x00, 0xC7, 0x45, 0xE4, 0x01, 0x00, 0x00, 0x00, 0x50, 0xFF, 0x15, 0x88, 0xA0, 0x40, 0x00, 0x89, 0x45, 0xE0, 0x33, 0xC0, 0x83, 0x7D, 0xE0, 0x01, 0x0F, 0x84, 0xD6, 0x00, 0x00, 0x00, 0x39, 0x45, 0xE0, 0x75, 0x07, 0x6A, 0x03, 0xE8, 0x05, 0xD1, 0xFF, 0xFF, 0x39, 0x45, 0xE4, 0x74, 0x07, 0x50, 0xE8, 0xF9, 0xE3, 0xFF, 0xFF, 0x59, 0x33, 0xC0, 0x89, 0x45, 0xFC, 0x83, 0xFB, 0x08, 0x74, 0x0A, 0x83, 0xFB, 0x0B, 0x74, 0x05, 0x83, 0xFB, 0x04, 0x75, 0x1B, 0x8B, 0x4F, 0x60, 0x89, 0x4D, 0xD4, 0x89, 0x47, 0x60, 0x83, 0xFB, 0x08, 0x75, 0x3E, 0x8B, 0x4F, 0x64, 0x89, 0x4D, 0xD0, 0xC7, 0x47, 0x64, 0x8C, 0x00, 0x00, 0x00, 0x83, 0xFB, 0x08, 0x75, 0x2C, 0x8B, 0x0D, 0xB0, 0xB0, 0x40, 0x00, 0x89, 0x4D, 0xDC, 0x8B, 0x0D, 0xB4, 0xB0, 0x40, 0x00, 0x03, 0x0D, 0xB0, 0xB0, 0x40, 0x00, 0x39, 0x4D, 0xDC, 0x7D, 0x19, 0x8B, 0x4D, 0xDC, 0x6B, 0xC9, 0x0C, 0x8B, 0x57, 0x5C, 0x89, 0x44, 0x11, 0x08, 0xFF, 0x45, 0xDC, 0xEB, 0xDD, 0xE8, 0xC2, 0xB7, 0xFF, 0xFF, 0x89, 0x06, 0xC7, 0x45, 0xFC, 0xFE, 0xFF, 0xFF, 0xFF, 0xE8, 0x15, 0x00, 0x00, 0x00, 0x83, 0xFB, 0x08, 0x75, 0x1F, 0xFF, 0x77, 0x64, 0x53, 0xFF, 0x55, 0xE0, 0x59, 0xEB, 0x19, 0x8B, 0x5D, 0x08, 0x8B, 0x7D, 0xD8, 0x83, 0x7D, 0xE4, 0x00, 0x74, 0x08, 0x6A, 0x00, 0xE8, 0x8A, 0xE2, 0xFF, 0xFF, 0x59, 0xC3, 0x53, 0xFF, 0x55, 0xE0, 0x59, 0x83, 0xFB, 0x08, 0x74, 0x0A, 0x83, 0xFB, 0x0B, 0x74, 0x05, 0x83, 0xFB, 0x04, 0x75, 0x11, 0x8B, 0x45, 0xD4, 0x89, 0x47, 0x60, 0x83, 0xFB, 0x08, 0x75, 0x06, 0x8B, 0x45, 0xD0, 0x89, 0x47, 0x64, 0x33, 0xC0, 0xE8, 0x72, 0xDB, 0xFF, 0xFF, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x8B, 0x45, 0x08, 0xA3, 0x14, 0xEA, 0x40, 0x00, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x8B, 0x45, 0x08, 0xA3, 0x18, 0xEA, 0x40, 0x00, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x51, 0x53, 0x56, 0x8B, 0x35, 0x88, 0xA0, 0x40, 0x00, 0x57, 0xFF, 0x35, 0x88, 0xFB, 0x40, 0x00, 0xFF, 0xD6, 0xFF, 0x35, 0x84, 0xFB, 0x40, 0x00, 0x8B, 0xD8, 0x89, 0x5D, 0xFC, 0xFF, 0xD6, 0x8B, 0xF0, 0x3B, 0xF3, 0x0F, 0x82, 0x81, 0x00, 0x00, 0x00, 0x8B, 0xFE, 0x2B, 0xFB, 0x8D, 0x47, 0x04, 0x83, 0xF8, 0x04, 0x72, 0x75, 0x53, 0xE8, 0x74, 0x10, 0x00, 0x00, 0x8B, 0xD8, 0x8D, 0x47, 0x04, 0x59, 0x3B, 0xD8, 0x73, 0x48, 0xB8, 0x00, 0x08, 0x00, 0x00, 0x3B, 0xD8, 0x73, 0x02, 0x8B, 0xC3, 0x03, 0xC3, 0x3B, 0xC3, 0x72, 0x0F, 0x50, 0xFF, 0x75, 0xFC, 0xE8, 0x76, 0xE3, 0xFF, 0xFF, 0x59, 0x59, 0x85, 0xC0, 0x75, 0x16, 0x8D, 0x43, 0x10, 0x3B, 0xC3, 0x72, 0x3E, 0x50, 0xFF, 0x75, 0xFC, 0xE8, 0x60, 0xE3, 0xFF, 0xFF, 0x59, 0x59, 0x85, 0xC0, 0x74, 0x2F, 0xC1, 0xFF, 0x02, 0x50, 0x8D, 0x34, 0xB8, 0xFF, 0x15, 0x78, 0xA0, 0x40, 0x00, 0xA3, 0x88, 0xFB, 0x40, 0x00,
				0xFF, 0x75, 0x08, 0x8B, 0x3D, 0x78, 0xA0, 0x40, 0x00, 0xFF, 0xD7, 0x89, 0x06, 0x83, 0xC6, 0x04, 0x56, 0xFF, 0xD7, 0xA3, 0x84, 0xFB, 0x40, 0x00, 0x8B, 0x45, 0x08, 0xEB, 0x02, 0x33, 0xC0, 0x5F, 0x5E, 0x5B, 0xC9, 0xC3, 0x8B, 0xFF, 0x56, 0x6A, 0x04, 0x6A, 0x20, 0xE8, 0xCC, 0xE2, 0xFF, 0xFF, 0x59, 0x59, 0x8B, 0xF0, 0x56, 0xFF, 0x15, 0x78, 0xA0, 0x40, 0x00, 0xA3, 0x88, 0xFB, 0x40, 0x00, 0xA3, 0x84, 0xFB, 0x40, 0x00, 0x85, 0xF6, 0x75, 0x05, 0x6A, 0x18, 0x58, 0x5E, 0xC3, 0x83, 0x26, 0x00, 0x33, 0xC0, 0x5E, 0xC3, 0x6A, 0x0C, 0x68, 0x70, 0xC2, 0x40, 0x00, 0xE8, 0x1B, 0xDA, 0xFF, 0xFF, 0xE8, 0xC9, 0xCC, 0xFF, 0xFF, 0x83, 0x65, 0xFC, 0x00, 0xFF, 0x75, 0x08, 0xE8, 0xFC, 0xFE, 0xFF, 0xFF, 0x59, 0x89, 0x45, 0xE4, 0xC7, 0x45, 0xFC, 0xFE, 0xFF, 0xFF, 0xFF, 0xE8, 0x09, 0x00, 0x00, 0x00, 0x8B, 0x45, 0xE4, 0xE8, 0x37, 0xDA, 0xFF, 0xFF, 0xC3, 0xE8, 0xA8, 0xCC, 0xFF, 0xFF, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0xFF, 0x75, 0x08, 0xE8, 0xB7, 0xFF, 0xFF, 0xFF, 0xF7, 0xD8, 0x1B, 0xC0, 0xF7, 0xD8, 0x59, 0x48, 0x5D, 0xC3, 0xCC, 0xCC, 0xCC, 0xCC, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x8B, 0x4D, 0x08, 0xB8, 0x4D, 0x5A, 0x00, 0x00, 0x66, 0x39, 0x01, 0x74, 0x04, 0x33, 0xC0, 0x5D, 0xC3, 0x8B, 0x41, 0x3C, 0x03, 0xC1, 0x81, 0x38, 0x50, 0x45, 0x00, 0x00, 0x75, 0xEF, 0x33, 0xD2, 0xB9, 0x0B, 0x01, 0x00, 0x00, 0x66, 0x39, 0x48, 0x18, 0x0F, 0x94, 0xC2, 0x8B, 0xC2, 0x5D, 0xC3, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x8B, 0x45, 0x08, 0x8B, 0x48, 0x3C, 0x03, 0xC8, 0x0F, 0xB7, 0x41, 0x14, 0x53, 0x56, 0x0F, 0xB7, 0x71, 0x06, 0x33, 0xD2, 0x57, 0x8D, 0x44, 0x08, 0x18, 0x85, 0xF6, 0x74, 0x1B, 0x8B, 0x7D, 0x0C, 0x8B, 0x48, 0x0C, 0x3B, 0xF9, 0x72, 0x09, 0x8B, 0x58, 0x08, 0x03, 0xD9, 0x3B, 0xFB, 0x72, 0x0A, 0x42, 0x83, 0xC0, 0x28, 0x3B, 0xD6, 0x72, 0xE8, 0x33, 0xC0, 0x5F, 0x5E, 0x5B, 0x5D, 0xC3, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x6A, 0xFE, 0x68, 0x90, 0xC2, 0x40, 0x00, 0x68, 0x30, 0x56, 0x40, 0x00, 0x64, 0xA1, 0x00, 0x00, 0x00, 0x00, 0x50, 0x83, 0xEC, 0x08, 0x53, 0x56, 0x57, 0xA1, 0x04, 0xD0, 0x40, 0x00, 0x31, 0x45, 0xF8, 0x33, 0xC5, 0x50, 0x8D, 0x45, 0xF0, 0x64, 0xA3, 0x00, 0x00, 0x00, 0x00, 0x89, 0x65, 0xE8, 0xC7, 0x45, 0xFC, 0x00, 0x00, 0x00, 0x00, 0x68, 0x00, 0x00, 0x40, 0x00, 0xE8, 0x2A, 0xFF, 0xFF, 0xFF, 0x83, 0xC4, 0x04, 0x85, 0xC0, 0x74, 0x54, 0x8B, 0x45, 0x08, 0x2D, 0x00, 0x00, 0x40, 0x00, 0x50, 0x68, 0x00, 0x00, 0x40, 0x00, 0xE8, 0x50, 0xFF, 0xFF, 0xFF, 0x83, 0xC4, 0x08, 0x85, 0xC0, 0x74, 0x3A, 0x8B, 0x40, 0x24, 0xC1, 0xE8, 0x1F, 0xF7, 0xD0, 0x83, 0xE0, 0x01, 0xC7, 0x45, 0xFC, 0xFE, 0xFF, 0xFF, 0xFF, 0x8B, 0x4D, 0xF0, 0x64, 0x89, 0x0D, 0x00, 0x00, 0x00, 0x00, 0x59, 0x5F, 0x5E, 0x5B, 0x8B, 0xE5, 0x5D, 0xC3, 0x8B, 0x45, 0xEC, 0x8B, 0x08, 0x33, 0xD2, 0x81, 0x39, 0x05, 0x00, 0x00, 0xC0, 0x0F, 0x94, 0xC2, 0x8B, 0xC2, 0xC3, 0x8B, 0x65, 0xE8, 0xC7, 0x45, 0xFC, 0xFE, 0xFF, 0xFF, 0xFF, 0x33, 0xC0, 0x8B, 0x4D, 0xF0, 0x64, 0x89, 0x0D, 0x00, 0x00, 0x00, 0x00, 0x59, 0x5F, 0x5E, 0x5B,
				0x8B, 0xE5, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x83, 0xEC, 0x24, 0xA1, 0x04, 0xD0, 0x40, 0x00, 0x33, 0xC5, 0x89, 0x45, 0xFC, 0x8B, 0x45, 0x08, 0x53, 0x89, 0x45, 0xE0, 0x8B, 0x45, 0x0C, 0x56, 0x57, 0x89, 0x45, 0xE4, 0xE8, 0x95, 0xB4, 0xFF, 0xFF, 0x83, 0x65, 0xEC, 0x00, 0x83, 0x3D, 0x1C, 0xEA, 0x40, 0x00, 0x00, 0x89, 0x45, 0xE8, 0x75, 0x7D, 0x68, 0x2C, 0xB9, 0x40, 0x00, 0xFF, 0x15, 0x04, 0xA1, 0x40, 0x00, 0x8B, 0xD8, 0x85, 0xDB, 0x0F, 0x84, 0x10, 0x01, 0x00, 0x00, 0x8B, 0x3D, 0x9C, 0xA0, 0x40, 0x00, 0x68, 0x20, 0xB9, 0x40, 0x00, 0x53, 0xFF, 0xD7, 0x85, 0xC0, 0x0F, 0x84, 0xFA, 0x00, 0x00, 0x00, 0x8B, 0x35, 0x78, 0xA0, 0x40, 0x00, 0x50, 0xFF, 0xD6, 0x68, 0x10, 0xB9, 0x40, 0x00, 0x53, 0xA3, 0x1C, 0xEA, 0x40, 0x00, 0xFF, 0xD7, 0x50, 0xFF, 0xD6, 0x68, 0xFC, 0xB8, 0x40, 0x00, 0x53, 0xA3, 0x20, 0xEA, 0x40, 0x00, 0xFF, 0xD7, 0x50, 0xFF, 0xD6, 0x68, 0xE0, 0xB8, 0x40, 0x00, 0x53, 0xA3, 0x24, 0xEA, 0x40, 0x00, 0xFF, 0xD7, 0x50, 0xFF, 0xD6, 0xA3, 0x2C, 0xEA, 0x40, 0x00, 0x85, 0xC0, 0x74, 0x10, 0x68, 0xC8, 0xB8, 0x40, 0x00, 0x53, 0xFF, 0xD7, 0x50, 0xFF, 0xD6, 0xA3, 0x28, 0xEA, 0x40, 0x00, 0xA1, 0x28, 0xEA, 0x40, 0x00, 0x8B, 0x4D, 0xE8, 0x8B, 0x35, 0x88, 0xA0, 0x40, 0x00, 0x3B, 0xC1, 0x74, 0x47, 0x39, 0x0D, 0x2C, 0xEA, 0x40, 0x00, 0x74, 0x3F, 0x50, 0xFF, 0xD6, 0xFF, 0x35, 0x2C, 0xEA, 0x40, 0x00, 0x8B, 0xF8, 0xFF, 0xD6, 0x8B, 0xD8, 0x85, 0xFF, 0x74, 0x2C, 0x85, 0xDB, 0x74, 0x28, 0xFF, 0xD7, 0x85, 0xC0, 0x74, 0x19, 0x8D, 0x4D, 0xDC, 0x51, 0x6A, 0x0C, 0x8D, 0x4D, 0xF0, 0x51, 0x6A, 0x01, 0x50, 0xFF, 0xD3, 0x85, 0xC0, 0x74, 0x06, 0xF6, 0x45, 0xF8, 0x01, 0x75, 0x09, 0x81, 0x4D, 0x10, 0x00, 0x00, 0x20, 0x00, 0xEB, 0x33, 0xA1, 0x20, 0xEA, 0x40, 0x00, 0x3B, 0x45, 0xE8, 0x74, 0x29, 0x50, 0xFF, 0xD6, 0x85, 0xC0, 0x74, 0x22, 0xFF, 0xD0, 0x89, 0x45, 0xEC, 0x85, 0xC0, 0x74, 0x19, 0xA1, 0x24, 0xEA, 0x40, 0x00, 0x3B, 0x45, 0xE8, 0x74, 0x0F, 0x50, 0xFF, 0xD6, 0x85, 0xC0, 0x74, 0x08, 0xFF, 0x75, 0xEC, 0xFF, 0xD0, 0x89, 0x45, 0xEC, 0xFF, 0x35, 0x1C, 0xEA, 0x40, 0x00, 0xFF, 0xD6, 0x85, 0xC0, 0x74, 0x10, 0xFF, 0x75, 0x10, 0xFF, 0x75, 0xE4, 0xFF, 0x75, 0xE0, 0xFF, 0x75, 0xEC, 0xFF, 0xD0, 0xEB, 0x02, 0x33, 0xC0, 0x8B, 0x4D, 0xFC, 0x5F, 0x5E, 0x33, 0xCD, 0x5B, 0xE8, 0x09, 0xA2, 0xFF, 0xFF, 0xC9, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x56, 0x8B, 0x75, 0x08, 0x57, 0x85, 0xF6, 0x74, 0x07, 0x8B, 0x7D, 0x0C, 0x85, 0xFF, 0x75, 0x15, 0xE8, 0x9A, 0xBA, 0xFF, 0xFF, 0x6A, 0x16, 0x5E, 0x89, 0x30, 0xE8, 0x3E, 0xBA, 0xFF, 0xFF, 0x8B, 0xC6, 0x5F, 0x5E, 0x5D, 0xC3, 0x8B, 0x4D, 0x10, 0x85, 0xC9, 0x75, 0x07, 0x33, 0xC0, 0x66, 0x89, 0x06, 0xEB, 0xDD, 0x8B, 0xD6, 0x66, 0x83, 0x3A, 0x00, 0x74, 0x06, 0x83, 0xC2, 0x02, 0x4F, 0x75, 0xF4, 0x85, 0xFF, 0x74, 0xE7, 0x2B, 0xD1, 0x0F, 0xB7, 0x01, 0x66, 0x89, 0x04, 0x0A, 0x83, 0xC1, 0x02, 0x66, 0x85, 0xC0, 0x74, 0x03, 0x4F, 0x75, 0xEE, 0x33, 0xC0, 0x85, 0xFF, 0x75, 0xC2, 0x66, 0x89, 0x06, 0xE8, 0x48, 0xBA, 0xFF, 0xFF, 0x6A, 0x22, 0x59, 0x89, 0x08, 0x8B, 0xF1, 0xEB, 0xAA, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x8B, 0x55, 0x08, 0x53, 0x8B, 0x5D, 0x14, 0x56, 0x57, 0x85,
				0xDB, 0x75, 0x10, 0x85, 0xD2, 0x75, 0x10, 0x39, 0x55, 0x0C, 0x75, 0x12, 0x33, 0xC0, 0x5F, 0x5E, 0x5B, 0x5D, 0xC3, 0x85, 0xD2, 0x74, 0x07, 0x8B, 0x7D, 0x0C, 0x85, 0xFF, 0x75, 0x13, 0xE8, 0x0D, 0xBA, 0xFF, 0xFF, 0x6A, 0x16, 0x5E, 0x89, 0x30, 0xE8, 0xB1, 0xB9, 0xFF, 0xFF, 0x8B, 0xC6, 0xEB, 0xDD, 0x85, 0xDB, 0x75, 0x07, 0x33, 0xC0, 0x66, 0x89, 0x02, 0xEB, 0xD0, 0x8B, 0x4D, 0x10, 0x85, 0xC9, 0x75, 0x07, 0x33, 0xC0, 0x66, 0x89, 0x02, 0xEB, 0xD4, 0x8B, 0xC2, 0x83, 0xFB, 0xFF, 0x75, 0x18, 0x8B, 0xF2, 0x2B, 0xF1, 0x0F, 0xB7, 0x01, 0x66, 0x89, 0x04, 0x0E, 0x83, 0xC1, 0x02, 0x66, 0x85, 0xC0, 0x74, 0x27, 0x4F, 0x75, 0xEE, 0xEB, 0x22, 0x8B, 0xF1, 0x2B, 0xF2, 0x0F, 0xB7, 0x0C, 0x06, 0x66, 0x89, 0x08, 0x83, 0xC0, 0x02, 0x66, 0x85, 0xC9, 0x74, 0x06, 0x4F, 0x74, 0x03, 0x4B, 0x75, 0xEB, 0x85, 0xDB, 0x75, 0x05, 0x33, 0xC9, 0x66, 0x89, 0x08, 0x85, 0xFF, 0x0F, 0x85, 0x79, 0xFF, 0xFF, 0xFF, 0x33, 0xC0, 0x83, 0xFB, 0xFF, 0x75, 0x10, 0x8B, 0x4D, 0x0C, 0x6A, 0x50, 0x66, 0x89, 0x44, 0x4A, 0xFE, 0x58, 0xE9, 0x64, 0xFF, 0xFF, 0xFF, 0x66, 0x89, 0x02, 0xE8, 0x7E, 0xB9, 0xFF, 0xFF, 0x6A, 0x22, 0x59, 0x89, 0x08, 0x8B, 0xF1, 0xE9, 0x6A, 0xFF, 0xFF, 0xFF, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x8B, 0x45, 0x08, 0x66, 0x8B, 0x08, 0x83, 0xC0, 0x02, 0x66, 0x85, 0xC9, 0x75, 0xF5, 0x2B, 0x45, 0x08, 0xD1, 0xF8, 0x48, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x56, 0x8B, 0x75, 0x08, 0x57, 0x85, 0xF6, 0x74, 0x07, 0x8B, 0x7D, 0x0C, 0x85, 0xFF, 0x75, 0x15, 0xE8, 0x3D, 0xB9, 0xFF, 0xFF, 0x6A, 0x16, 0x5E, 0x89, 0x30, 0xE8, 0xE1, 0xB8, 0xFF, 0xFF, 0x8B, 0xC6, 0x5F, 0x5E, 0x5D, 0xC3, 0x8B, 0x45, 0x10, 0x85, 0xC0, 0x75, 0x05, 0x66, 0x89, 0x06, 0xEB, 0xDF, 0x8B, 0xD6, 0x2B, 0xD0, 0x0F, 0xB7, 0x08, 0x66, 0x89, 0x0C, 0x02, 0x83, 0xC0, 0x02, 0x66, 0x85, 0xC9, 0x74, 0x03, 0x4F, 0x75, 0xEE, 0x33, 0xC0, 0x85, 0xFF, 0x75, 0xD4, 0x66, 0x89, 0x06, 0xE8, 0xFD, 0xB8, 0xFF, 0xFF, 0x6A, 0x22, 0x59, 0x89, 0x08, 0x8B, 0xF1, 0xEB, 0xBC, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x8B, 0x4D, 0x08, 0x85, 0xC9, 0x78, 0x1E, 0x83, 0xF9, 0x02, 0x7E, 0x0C, 0x83, 0xF9, 0x03, 0x75, 0x14, 0xA1, 0xC8, 0xDD, 0x40, 0x00, 0x5D, 0xC3, 0xA1, 0xC8, 0xDD, 0x40, 0x00, 0x89, 0x0D, 0xC8, 0xDD, 0x40, 0x00, 0x5D, 0xC3, 0xE8, 0xC5, 0xB8, 0xFF, 0xFF, 0xC7, 0x00, 0x16, 0x00, 0x00, 0x00, 0xE8, 0x68, 0xB8, 0xFF, 0xFF, 0x83, 0xC8, 0xFF, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x83, 0xEC, 0x10, 0xFF, 0x75, 0x08, 0x8D, 0x4D, 0xF0, 0xE8, 0x04, 0xA0, 0xFF, 0xFF, 0x0F, 0xB6, 0x45, 0x0C, 0x8B, 0x4D, 0xF4, 0x8A, 0x55, 0x14, 0x84, 0x54, 0x01, 0x1D, 0x75, 0x1E, 0x83, 0x7D, 0x10, 0x00, 0x74, 0x12, 0x8B, 0x4D, 0xF0, 0x8B, 0x89, 0xC8, 0x00, 0x00, 0x00, 0x0F, 0xB7, 0x04, 0x41, 0x23, 0x45, 0x10, 0xEB, 0x02, 0x33, 0xC0, 0x85, 0xC0, 0x74, 0x03, 0x33, 0xC0, 0x40, 0x80, 0x7D, 0xFC, 0x00, 0x74, 0x07, 0x8B, 0x4D, 0xF8, 0x83, 0x61, 0x70, 0xFD, 0xC9, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x6A, 0x04, 0x6A, 0x00, 0xFF, 0x75, 0x08, 0x6A, 0x00, 0xE8, 0x9A, 0xFF, 0xFF, 0xFF, 0x83, 0xC4, 0x10, 0x5D, 0xC3, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC,
				0x53, 0x56, 0x57, 0x8B, 0x54, 0x24, 0x10, 0x8B, 0x44, 0x24, 0x14, 0x8B, 0x4C, 0x24, 0x18, 0x55, 0x52, 0x50, 0x51, 0x51, 0x68, 0xC0, 0x81, 0x40, 0x00, 0x64, 0xFF, 0x35, 0x00, 0x00, 0x00, 0x00, 0xA1, 0x04, 0xD0, 0x40, 0x00, 0x33, 0xC4, 0x89, 0x44, 0x24, 0x08, 0x64, 0x89, 0x25, 0x00, 0x00, 0x00, 0x00, 0x8B, 0x44, 0x24, 0x30, 0x8B, 0x58, 0x08, 0x8B, 0x4C, 0x24, 0x2C, 0x33, 0x19, 0x8B, 0x70, 0x0C, 0x83, 0xFE, 0xFE, 0x74, 0x3B, 0x8B, 0x54, 0x24, 0x34, 0x83, 0xFA, 0xFE, 0x74, 0x04, 0x3B, 0xF2, 0x76, 0x2E, 0x8D, 0x34, 0x76, 0x8D, 0x5C, 0xB3, 0x10, 0x8B, 0x0B, 0x89, 0x48, 0x0C, 0x83, 0x7B, 0x04, 0x00, 0x75, 0xCC, 0x68, 0x01, 0x01, 0x00, 0x00, 0x8B, 0x43, 0x08, 0xE8, 0x22, 0x0B, 0x00, 0x00, 0xB9, 0x01, 0x00, 0x00, 0x00, 0x8B, 0x43, 0x08, 0xE8, 0x34, 0x0B, 0x00, 0x00, 0xEB, 0xB0, 0x64, 0x8F, 0x05, 0x00, 0x00, 0x00, 0x00, 0x83, 0xC4, 0x18, 0x5F, 0x5E, 0x5B, 0xC3, 0x8B, 0x4C, 0x24, 0x04, 0xF7, 0x41, 0x04, 0x06, 0x00, 0x00, 0x00, 0xB8, 0x01, 0x00, 0x00, 0x00, 0x74, 0x33, 0x8B, 0x44, 0x24, 0x08, 0x8B, 0x48, 0x08, 0x33, 0xC8, 0xE8, 0xDF, 0x9E, 0xFF, 0xFF, 0x55, 0x8B, 0x68, 0x18, 0xFF, 0x70, 0x0C, 0xFF, 0x70, 0x10, 0xFF, 0x70, 0x14, 0xE8, 0x3E, 0xFF, 0xFF, 0xFF, 0x83, 0xC4, 0x0C, 0x5D, 0x8B, 0x44, 0x24, 0x08, 0x8B, 0x54, 0x24, 0x10, 0x89, 0x02, 0xB8, 0x03, 0x00, 0x00, 0x00, 0xC3, 0x55, 0x8B, 0x4C, 0x24, 0x08, 0x8B, 0x29, 0xFF, 0x71, 0x1C, 0xFF, 0x71, 0x18, 0xFF, 0x71, 0x28, 0xE8, 0x15, 0xFF, 0xFF, 0xFF, 0x83, 0xC4, 0x0C, 0x5D, 0xC2, 0x04, 0x00, 0x55, 0x56, 0x57, 0x53, 0x8B, 0xEA, 0x33, 0xC0, 0x33, 0xDB, 0x33, 0xD2, 0x33, 0xF6, 0x33, 0xFF, 0xFF, 0xD1, 0x5B, 0x5F, 0x5E, 0x5D, 0xC3, 0x8B, 0xEA, 0x8B, 0xF1, 0x8B, 0xC1, 0x6A, 0x01, 0xE8, 0x7F, 0x0A, 0x00, 0x00, 0x33, 0xC0, 0x33, 0xDB, 0x33, 0xC9, 0x33, 0xD2, 0x33, 0xFF, 0xFF, 0xE6, 0x55, 0x8B, 0xEC, 0x53, 0x56, 0x57, 0x6A, 0x00, 0x52, 0x68, 0x66, 0x82, 0x40, 0x00, 0x51, 0xE8, 0x40, 0x0E, 0x00, 0x00, 0x5F, 0x5E, 0x5B, 0x5D, 0xC3, 0x55, 0x8B, 0x6C, 0x24, 0x08, 0x52, 0x51, 0xFF, 0x74, 0x24, 0x14, 0xE8, 0xB5, 0xFE, 0xFF, 0xFF, 0x83, 0xC4, 0x0C, 0x5D, 0xC2, 0x08, 0x00, 0x66, 0x0F, 0xEF, 0xC0, 0x51, 0x53, 0x8B, 0xC1, 0x83, 0xE0, 0x0F, 0x85, 0xC0, 0x75, 0x7F, 0x8B, 0xC2, 0x83, 0xE2, 0x7F, 0xC1, 0xE8, 0x07, 0x74, 0x37, 0x8D, 0xA4, 0x24, 0x00, 0x00, 0x00, 0x00, 0x66, 0x0F, 0x7F, 0x01, 0x66, 0x0F, 0x7F, 0x41, 0x10, 0x66, 0x0F, 0x7F, 0x41, 0x20, 0x66, 0x0F, 0x7F, 0x41, 0x30, 0x66, 0x0F, 0x7F, 0x41, 0x40, 0x66, 0x0F, 0x7F, 0x41, 0x50, 0x66, 0x0F, 0x7F, 0x41, 0x60, 0x66, 0x0F, 0x7F, 0x41, 0x70, 0x8D, 0x89, 0x80, 0x00, 0x00, 0x00, 0x48, 0x75, 0xD0, 0x85, 0xD2, 0x74, 0x37, 0x8B, 0xC2, 0xC1, 0xE8, 0x04, 0x74, 0x0F, 0xEB, 0x03, 0x8D, 0x49, 0x00, 0x66, 0x0F, 0x7F, 0x01, 0x8D, 0x49, 0x10, 0x48, 0x75, 0xF6, 0x83, 0xE2, 0x0F, 0x74, 0x1C, 0x8B, 0xC2, 0x33, 0xDB, 0xC1, 0xEA, 0x02, 0x74, 0x08, 0x89, 0x19, 0x8D, 0x49, 0x04, 0x4A, 0x75, 0xF8, 0x83, 0xE0, 0x03, 0x74, 0x06, 0x88, 0x19, 0x41, 0x48, 0x75, 0xFA, 0x5B, 0x58, 0xC3, 0x8B, 0xD8, 0xF7, 0xDB, 0x83, 0xC3, 0x10, 0x2B, 0xD3, 0x33, 0xC0, 0x52, 0x8B, 0xD3, 0x83, 0xE2, 0x03, 0x74, 0x06, 0x88,
				0x01, 0x41, 0x4A, 0x75, 0xFA, 0xC1, 0xEB, 0x02, 0x74, 0x08, 0x89, 0x01, 0x8D, 0x49, 0x04, 0x4B, 0x75, 0xF8, 0x5A, 0xE9, 0x55, 0xFF, 0xFF, 0xFF, 0x6A, 0x0A, 0xFF, 0x15, 0x0C, 0xA1, 0x40, 0x00, 0xA3, 0x3C, 0xEA, 0x40, 0x00, 0x33, 0xC0, 0xC3, 0xCC, 0xCC, 0xCC, 0xCC, 0x51, 0x8D, 0x4C, 0x24, 0x08, 0x2B, 0xC8, 0x83, 0xE1, 0x0F, 0x03, 0xC1, 0x1B, 0xC9, 0x0B, 0xC1, 0x59, 0xE9, 0x7A, 0x05, 0x00, 0x00, 0x51, 0x8D, 0x4C, 0x24, 0x08, 0x2B, 0xC8, 0x83, 0xE1, 0x07, 0x03, 0xC1, 0x1B, 0xC9, 0x0B, 0xC1, 0x59, 0xE9, 0x64, 0x05, 0x00, 0x00, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x8B, 0x4D, 0x08, 0x85, 0xC9, 0x74, 0x1B, 0x6A, 0xE0, 0x33, 0xD2, 0x58, 0xF7, 0xF1, 0x3B, 0x45, 0x0C, 0x73, 0x0F, 0xE8, 0xD3, 0xB5, 0xFF, 0xFF, 0xC7, 0x00, 0x0C, 0x00, 0x00, 0x00, 0x33, 0xC0, 0x5D, 0xC3, 0x0F, 0xAF, 0x4D, 0x0C, 0x56, 0x8B, 0xF1, 0x85, 0xF6, 0x75, 0x01, 0x46, 0x33, 0xC0, 0x83, 0xFE, 0xE0, 0x77, 0x13, 0x56, 0x6A, 0x08, 0xFF, 0x35, 0x28, 0xE1, 0x40, 0x00, 0xFF, 0x15, 0x3C, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x75, 0x32, 0x83, 0x3D, 0x8C, 0xE7, 0x40, 0x00, 0x00, 0x74, 0x1C, 0x56, 0xE8, 0x70, 0xC9, 0xFF, 0xFF, 0x59, 0x85, 0xC0, 0x75, 0xD2, 0x8B, 0x45, 0x10, 0x85, 0xC0, 0x74, 0x06, 0xC7, 0x00, 0x0C, 0x00, 0x00, 0x00, 0x33, 0xC0, 0xEB, 0x0D, 0x8B, 0x4D, 0x10, 0x85, 0xC9, 0x74, 0x06, 0xC7, 0x01, 0x0C, 0x00, 0x00, 0x00, 0x5E, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x83, 0x7D, 0x08, 0x00, 0x75, 0x0B, 0xFF, 0x75, 0x0C, 0xE8, 0x9F, 0xA0, 0xFF, 0xFF, 0x59, 0x5D, 0xC3, 0x56, 0x8B, 0x75, 0x0C, 0x85, 0xF6, 0x75, 0x0D, 0xFF, 0x75, 0x08, 0xE8, 0x52, 0xA0, 0xFF, 0xFF, 0x59, 0x33, 0xC0, 0xEB, 0x4D, 0x57, 0xEB, 0x30, 0x85, 0xF6, 0x75, 0x01, 0x46, 0x56, 0xFF, 0x75, 0x08, 0x6A, 0x00, 0xFF, 0x35, 0x28, 0xE1, 0x40, 0x00, 0xFF, 0x15, 0x10, 0xA1, 0x40, 0x00, 0x8B, 0xF8, 0x85, 0xFF, 0x75, 0x5E, 0x39, 0x05, 0x8C, 0xE7, 0x40, 0x00, 0x74, 0x40, 0x56, 0xE8, 0xF1, 0xC8, 0xFF, 0xFF, 0x59, 0x85, 0xC0, 0x74, 0x1D, 0x83, 0xFE, 0xE0, 0x76, 0xCB, 0x56, 0xE8, 0xE1, 0xC8, 0xFF, 0xFF, 0x59, 0xE8, 0xFF, 0xB4, 0xFF, 0xFF, 0xC7, 0x00, 0x0C, 0x00, 0x00, 0x00, 0x33, 0xC0, 0x5F, 0x5E, 0x5D, 0xC3, 0xE8, 0xEE, 0xB4, 0xFF, 0xFF, 0x8B, 0xF0, 0xFF, 0x15, 0x34, 0xA0, 0x40, 0x00, 0x50, 0xE8, 0x9E, 0xB4, 0xFF, 0xFF, 0x59, 0x89, 0x06, 0xEB, 0xE2, 0xE8, 0xD6, 0xB4, 0xFF, 0xFF, 0x8B, 0xF0, 0xFF, 0x15, 0x34, 0xA0, 0x40, 0x00, 0x50, 0xE8, 0x86, 0xB4, 0xFF, 0xFF, 0x59, 0x89, 0x06, 0x8B, 0xC7, 0xEB, 0xCA, 0x57, 0x8B, 0xC6, 0x83, 0xE0, 0x0F, 0x85, 0xC0, 0x0F, 0x85, 0xC1, 0x00, 0x00, 0x00, 0x8B, 0xD1, 0x83, 0xE1, 0x7F, 0xC1, 0xEA, 0x07, 0x74, 0x65, 0xEB, 0x06, 0x8D, 0x9B, 0x00, 0x00, 0x00, 0x00, 0x66, 0x0F, 0x6F, 0x06, 0x66, 0x0F, 0x6F, 0x4E, 0x10, 0x66, 0x0F, 0x6F, 0x56, 0x20, 0x66, 0x0F, 0x6F, 0x5E, 0x30, 0x66, 0x0F, 0x7F, 0x07, 0x66, 0x0F, 0x7F, 0x4F, 0x10, 0x66, 0x0F, 0x7F, 0x57, 0x20, 0x66, 0x0F, 0x7F, 0x5F, 0x30, 0x66, 0x0F, 0x6F, 0x66, 0x40, 0x66, 0x0F, 0x6F, 0x6E, 0x50, 0x66, 0x0F, 0x6F, 0x76, 0x60, 0x66, 0x0F, 0x6F, 0x7E, 0x70, 0x66, 0x0F, 0x7F, 0x67, 0x40, 0x66, 0x0F, 0x7F, 0x6F, 0x50, 0x66, 0x0F, 0x7F, 0x77, 0x60, 0x66, 0x0F, 0x7F, 0x7F,
				0x70, 0x8D, 0xB6, 0x80, 0x00, 0x00, 0x00, 0x8D, 0xBF, 0x80, 0x00, 0x00, 0x00, 0x4A, 0x75, 0xA3, 0x85, 0xC9, 0x74, 0x49, 0x8B, 0xD1, 0xC1, 0xEA, 0x04, 0x85, 0xD2, 0x74, 0x17, 0x8D, 0x9B, 0x00, 0x00, 0x00, 0x00, 0x66, 0x0F, 0x6F, 0x06, 0x66, 0x0F, 0x7F, 0x07, 0x8D, 0x76, 0x10, 0x8D, 0x7F, 0x10, 0x4A, 0x75, 0xEF, 0x83, 0xE1, 0x0F, 0x74, 0x24, 0x8B, 0xC1, 0xC1, 0xE9, 0x02, 0x74, 0x0D, 0x8B, 0x16, 0x89, 0x17, 0x8D, 0x76, 0x04, 0x8D, 0x7F, 0x04, 0x49, 0x75, 0xF3, 0x8B, 0xC8, 0x83, 0xE1, 0x03, 0x74, 0x09, 0x8A, 0x06, 0x88, 0x07, 0x46, 0x47, 0x49, 0x75, 0xF7, 0x58, 0x5E, 0x5F, 0x5D, 0xC3, 0xBA, 0x10, 0x00, 0x00, 0x00, 0x2B, 0xD0, 0x2B, 0xCA, 0x51, 0x8B, 0xC2, 0x8B, 0xC8, 0x83, 0xE1, 0x03, 0x74, 0x09, 0x8A, 0x16, 0x88, 0x17, 0x46, 0x47, 0x49, 0x75, 0xF7, 0xC1, 0xE8, 0x02, 0x74, 0x0D, 0x8B, 0x16, 0x89, 0x17, 0x8D, 0x76, 0x04, 0x8D, 0x7F, 0x04, 0x48, 0x75, 0xF3, 0x59, 0xE9, 0x0B, 0xFF, 0xFF, 0xFF, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x8B, 0x4D, 0x08, 0x53, 0x33, 0xDB, 0x56, 0x57, 0x3B, 0xCB, 0x7C, 0x5B, 0x3B, 0x0D, 0x6C, 0xFA, 0x40, 0x00, 0x73, 0x53, 0x8B, 0xC1, 0xC1, 0xF8, 0x05, 0x8B, 0xF1, 0x83, 0xE6, 0x1F, 0x8D, 0x3C, 0x85, 0x80, 0xFA, 0x40, 0x00, 0x8B, 0x07, 0xC1, 0xE6, 0x06, 0xF6, 0x44, 0x30, 0x04, 0x01, 0x74, 0x36, 0x83, 0x3C, 0x30, 0xFF, 0x74, 0x30, 0x83, 0x3D, 0x00, 0xD0, 0x40, 0x00, 0x01, 0x75, 0x1D, 0x2B, 0xCB, 0x74, 0x10, 0x49, 0x74, 0x08, 0x49, 0x75, 0x13, 0x53, 0x6A, 0xF4, 0xEB, 0x08, 0x53, 0x6A, 0xF5, 0xEB, 0x03, 0x53, 0x6A, 0xF6, 0xFF, 0x15, 0x14, 0xA1, 0x40, 0x00, 0x8B, 0x07, 0x83, 0x0C, 0x06, 0xFF, 0x33, 0xC0, 0xEB, 0x15, 0xE8, 0x4D, 0xB3, 0xFF, 0xFF, 0xC7, 0x00, 0x09, 0x00, 0x00, 0x00, 0xE8, 0x55, 0xB3, 0xFF, 0xFF, 0x89, 0x18, 0x83, 0xC8, 0xFF, 0x5F, 0x5E, 0x5B, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x8B, 0x45, 0x08, 0x83, 0xF8, 0xFE, 0x75, 0x18, 0xE8, 0x39, 0xB3, 0xFF, 0xFF, 0x83, 0x20, 0x00, 0xE8, 0x1E, 0xB3, 0xFF, 0xFF, 0xC7, 0x00, 0x09, 0x00, 0x00, 0x00, 0x83, 0xC8, 0xFF, 0x5D, 0xC3, 0x85, 0xC0, 0x78, 0x08, 0x3B, 0x05, 0x6C, 0xFA, 0x40, 0x00, 0x72, 0x1A, 0xE8, 0x15, 0xB3, 0xFF, 0xFF, 0x83, 0x20, 0x00, 0xE8, 0xFA, 0xB2, 0xFF, 0xFF, 0xC7, 0x00, 0x09, 0x00, 0x00, 0x00, 0xE8, 0x9D, 0xB2, 0xFF, 0xFF, 0xEB, 0xD5, 0x8B, 0xC8, 0xC1, 0xF9, 0x05, 0x8B, 0x0C, 0x8D, 0x80, 0xFA, 0x40, 0x00, 0x83, 0xE0, 0x1F, 0xC1, 0xE0, 0x06, 0xF6, 0x44, 0x08, 0x04, 0x01, 0x74, 0xCD, 0x8B, 0x04, 0x08, 0x5D, 0xC3, 0x6A, 0x0C, 0x68, 0xB0, 0xC2, 0x40, 0x00, 0xE8, 0x27, 0xCF, 0xFF, 0xFF, 0x8B, 0x7D, 0x08, 0x8B, 0xC7, 0xC1, 0xF8, 0x05, 0x8B, 0xF7, 0x83, 0xE6, 0x1F, 0xC1, 0xE6, 0x06, 0x03, 0x34, 0x85, 0x80, 0xFA, 0x40, 0x00, 0xC7, 0x45, 0xE4, 0x01, 0x00, 0x00, 0x00, 0x33, 0xDB, 0x39, 0x5E, 0x08, 0x75, 0x35, 0x6A, 0x0A, 0xE8, 0x03, 0xD7, 0xFF, 0xFF, 0x59, 0x89, 0x5D, 0xFC, 0x39, 0x5E, 0x08, 0x75, 0x19, 0x68, 0xA0, 0x0F, 0x00, 0x00, 0x8D, 0x46, 0x0C, 0x50, 0xFF, 0x15, 0xC4, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x75, 0x03, 0x89, 0x5D, 0xE4, 0xFF, 0x46, 0x08, 0xC7, 0x45, 0xFC, 0xFE, 0xFF, 0xFF, 0xFF, 0xE8, 0x30, 0x00, 0x00, 0x00, 0x39, 0x5D, 0xE4, 0x74, 0x1D, 0x8B, 0xC7, 0xC1, 0xF8,
				0x05, 0x83, 0xE7, 0x1F, 0xC1, 0xE7, 0x06, 0x8B, 0x04, 0x85, 0x80, 0xFA, 0x40, 0x00, 0x8D, 0x44, 0x38, 0x0C, 0x50, 0xFF, 0x15, 0xF0, 0xA0, 0x40, 0x00, 0x8B, 0x45, 0xE4, 0xE8, 0xE8, 0xCE, 0xFF, 0xFF, 0xC3, 0x33, 0xDB, 0x8B, 0x7D, 0x08, 0x6A, 0x0A, 0xE8, 0xC5, 0xD5, 0xFF, 0xFF, 0x59, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x8B, 0x45, 0x08, 0x8B, 0xC8, 0x83, 0xE0, 0x1F, 0xC1, 0xF9, 0x05, 0x8B, 0x0C, 0x8D, 0x80, 0xFA, 0x40, 0x00, 0xC1, 0xE0, 0x06, 0x8D, 0x44, 0x01, 0x0C, 0x50, 0xFF, 0x15, 0xEC, 0xA0, 0x40, 0x00, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x51, 0x83, 0x3D, 0xB0, 0xDD, 0x40, 0x00, 0xFE, 0x75, 0x05, 0xE8, 0x70, 0x05, 0x00, 0x00, 0xA1, 0xB0, 0xDD, 0x40, 0x00, 0x83, 0xF8, 0xFF, 0x75, 0x07, 0xB8, 0xFF, 0xFF, 0x00, 0x00, 0xC9, 0xC3, 0x6A, 0x00, 0x8D, 0x4D, 0xFC, 0x51, 0x6A, 0x01, 0x8D, 0x4D, 0x08, 0x51, 0x50, 0xFF, 0x15, 0x18, 0xA1, 0x40, 0x00, 0x85, 0xC0, 0x74, 0xE2, 0x66, 0x8B, 0x45, 0x08, 0xC9, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x83, 0xEC, 0x10, 0x53, 0x56, 0x8B, 0x75, 0x0C, 0x33, 0xDB, 0x3B, 0xF3, 0x74, 0x15, 0x39, 0x5D, 0x10, 0x74, 0x10, 0x38, 0x1E, 0x75, 0x12, 0x8B, 0x45, 0x08, 0x3B, 0xC3, 0x74, 0x05, 0x33, 0xC9, 0x66, 0x89, 0x08, 0x33, 0xC0, 0x5E, 0x5B, 0xC9, 0xC3, 0xFF, 0x75, 0x14, 0x8D, 0x4D, 0xF0, 0xE8, 0xF0, 0x98, 0xFF, 0xFF, 0x8B, 0x45, 0xF0, 0x39, 0x58, 0x14, 0x75, 0x1E, 0x8B, 0x45, 0x08, 0x3B, 0xC3, 0x74, 0x06, 0x0F, 0xB6, 0x0E, 0x66, 0x89, 0x08, 0x38, 0x5D, 0xFC, 0x74, 0x07, 0x8B, 0x45, 0xF8, 0x83, 0x60, 0x70, 0xFD, 0x33, 0xC0, 0x40, 0xEB, 0xCB, 0x8D, 0x45, 0xF0, 0x50, 0x0F, 0xB6, 0x06, 0x50, 0xE8, 0xB7, 0xDB, 0xFF, 0xFF, 0x59, 0x59, 0x85, 0xC0, 0x74, 0x7D, 0x8B, 0x45, 0xF0, 0x8B, 0x88, 0xAC, 0x00, 0x00, 0x00, 0x83, 0xF9, 0x01, 0x7E, 0x25, 0x39, 0x4D, 0x10, 0x7C, 0x20, 0x33, 0xD2, 0x39, 0x5D, 0x08, 0x0F, 0x95, 0xC2, 0x52, 0xFF, 0x75, 0x08, 0x51, 0x56, 0x6A, 0x09, 0xFF, 0x70, 0x04, 0xFF, 0x15, 0xE4, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x8B, 0x45, 0xF0, 0x75, 0x10, 0x8B, 0x4D, 0x10, 0x3B, 0x88, 0xAC, 0x00, 0x00, 0x00, 0x72, 0x20, 0x38, 0x5E, 0x01, 0x74, 0x1B, 0x8B, 0x80, 0xAC, 0x00, 0x00, 0x00, 0x38, 0x5D, 0xFC, 0x0F, 0x84, 0x66, 0xFF, 0xFF, 0xFF, 0x8B, 0x4D, 0xF8, 0x83, 0x61, 0x70, 0xFD, 0xE9, 0x5A, 0xFF, 0xFF, 0xFF, 0xE8, 0xF2, 0xB0, 0xFF, 0xFF, 0xC7, 0x00, 0x2A, 0x00, 0x00, 0x00, 0x38, 0x5D, 0xFC, 0x74, 0x07, 0x8B, 0x45, 0xF8, 0x83, 0x60, 0x70, 0xFD, 0x83, 0xC8, 0xFF, 0xE9, 0x3B, 0xFF, 0xFF, 0xFF, 0x33, 0xC0, 0x39, 0x5D, 0x08, 0x0F, 0x95, 0xC0, 0x50, 0xFF, 0x75, 0x08, 0x8B, 0x45, 0xF0, 0x6A, 0x01, 0x56, 0x6A, 0x09, 0xFF, 0x70, 0x04, 0xFF, 0x15, 0xE4, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x0F, 0x85, 0x3A, 0xFF, 0xFF, 0xFF, 0xEB, 0xBA, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x6A, 0x00, 0xFF, 0x75, 0x10, 0xFF, 0x75, 0x0C, 0xFF, 0x75, 0x08, 0xE8, 0xD5, 0xFE, 0xFF, 0xFF, 0x83, 0xC4, 0x10, 0x5D, 0xC3, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0x51, 0x8D, 0x4C, 0x24, 0x04, 0x2B, 0xC8, 0x1B, 0xC0, 0xF7, 0xD0, 0x23, 0xC8, 0x8B, 0xC4, 0x25, 0x00, 0xF0, 0xFF, 0xFF, 0x3B, 0xC8, 0x72, 0x0A, 0x8B, 0xC1, 0x59, 0x94, 0x8B, 0x00, 0x89, 0x04,
				0x24, 0xC3, 0x2D, 0x00, 0x10, 0x00, 0x00, 0x85, 0x00, 0xEB, 0xE9, 0x6A, 0x10, 0x68, 0xD0, 0xC2, 0x40, 0x00, 0xE8, 0xB9, 0xCC, 0xFF, 0xFF, 0x33, 0xDB, 0x89, 0x5D, 0xE4, 0x6A, 0x01, 0xE8, 0xB5, 0xD4, 0xFF, 0xFF, 0x59, 0x89, 0x5D, 0xFC, 0x6A, 0x03, 0x5F, 0x89, 0x7D, 0xE0, 0x3B, 0x3D, 0x60, 0xFA, 0x40, 0x00, 0x7D, 0x54, 0x8B, 0xF7, 0xA1, 0x40, 0xEA, 0x40, 0x00, 0x39, 0x1C, 0xB0, 0x74, 0x45, 0x8B, 0x04, 0xB0, 0xF6, 0x40, 0x0C, 0x83, 0x74, 0x0F, 0x50, 0xE8, 0x3A, 0x04, 0x00, 0x00, 0x59, 0x83, 0xF8, 0xFF, 0x74, 0x03, 0xFF, 0x45, 0xE4, 0x83, 0xFF, 0x14, 0x7C, 0x28, 0xA1, 0x40, 0xEA, 0x40, 0x00, 0x8B, 0x04, 0xB0, 0x83, 0xC0, 0x20, 0x50, 0xFF, 0x15, 0xCC, 0xA0, 0x40, 0x00, 0xA1, 0x40, 0xEA, 0x40, 0x00, 0xFF, 0x34, 0xB0, 0xE8, 0xF9, 0x9A, 0xFF, 0xFF, 0x59, 0xA1, 0x40, 0xEA, 0x40, 0x00, 0x89, 0x1C, 0xB0, 0x47, 0xEB, 0xA1, 0xC7, 0x45, 0xFC, 0xFE, 0xFF, 0xFF, 0xFF, 0xE8, 0x09, 0x00, 0x00, 0x00, 0x8B, 0x45, 0xE4, 0xE8, 0x78, 0xCC, 0xFF, 0xFF, 0xC3, 0x6A, 0x01, 0xE8, 0x5A, 0xD3, 0xFF, 0xFF, 0x59, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x53, 0x56, 0x8B, 0x75, 0x08, 0x8B, 0x46, 0x0C, 0x8B, 0xC8, 0x80, 0xE1, 0x03, 0x33, 0xDB, 0x80, 0xF9, 0x02, 0x75, 0x40, 0xA9, 0x08, 0x01, 0x00, 0x00, 0x74, 0x39, 0x8B, 0x46, 0x08, 0x57, 0x8B, 0x3E, 0x2B, 0xF8, 0x85, 0xFF, 0x7E, 0x2C, 0x57, 0x50, 0x56, 0xE8, 0x09, 0xEC, 0xFF, 0xFF, 0x59, 0x50, 0xE8, 0xDA, 0xE8, 0xFF, 0xFF, 0x83, 0xC4, 0x0C, 0x3B, 0xC7, 0x75, 0x0F, 0x8B, 0x46, 0x0C, 0x84, 0xC0, 0x79, 0x0F, 0x83, 0xE0, 0xFD, 0x89, 0x46, 0x0C, 0xEB, 0x07, 0x83, 0x4E, 0x0C, 0x20, 0x83, 0xCB, 0xFF, 0x5F, 0x8B, 0x46, 0x08, 0x83, 0x66, 0x04, 0x00, 0x89, 0x06, 0x5E, 0x8B, 0xC3, 0x5B, 0x5D, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x56, 0x8B, 0x75, 0x08, 0x85, 0xF6, 0x75, 0x09, 0x56, 0xE8, 0x35, 0x00, 0x00, 0x00, 0x59, 0xEB, 0x2F, 0x56, 0xE8, 0x7C, 0xFF, 0xFF, 0xFF, 0x59, 0x85, 0xC0, 0x74, 0x05, 0x83, 0xC8, 0xFF, 0xEB, 0x1F, 0xF7, 0x46, 0x0C, 0x00, 0x40, 0x00, 0x00, 0x74, 0x14, 0x56, 0xE8, 0xA0, 0xEB, 0xFF, 0xFF, 0x50, 0xE8, 0xB4, 0x03, 0x00, 0x00, 0x59, 0xF7, 0xD8, 0x59, 0x1B, 0xC0, 0xEB, 0x02, 0x33, 0xC0, 0x5E, 0x5D, 0xC3, 0x6A, 0x14, 0x68, 0xF0, 0xC2, 0x40, 0x00, 0xE8, 0x6D, 0xCB, 0xFF, 0xFF, 0x33, 0xFF, 0x89, 0x7D, 0xE4, 0x89, 0x7D, 0xDC, 0x6A, 0x01, 0xE8, 0x66, 0xD3, 0xFF, 0xFF, 0x59, 0x89, 0x7D, 0xFC, 0x33, 0xF6, 0x89, 0x75, 0xE0, 0x3B, 0x35, 0x60, 0xFA, 0x40, 0x00, 0x0F, 0x8D, 0x83, 0x00, 0x00, 0x00, 0xA1, 0x40, 0xEA, 0x40, 0x00, 0x8D, 0x04, 0xB0, 0x39, 0x38, 0x74, 0x5E, 0x8B, 0x00, 0xF6, 0x40, 0x0C, 0x83, 0x74, 0x56, 0x50, 0x56, 0xE8, 0xA5, 0xEA, 0xFF, 0xFF, 0x59, 0x59, 0x33, 0xD2, 0x42, 0x89, 0x55, 0xFC, 0xA1, 0x40, 0xEA, 0x40, 0x00, 0x8B, 0x04, 0xB0, 0x8B, 0x48, 0x0C, 0xF6, 0xC1, 0x83, 0x74, 0x2F, 0x39, 0x55, 0x08, 0x75, 0x11, 0x50, 0xE8, 0x4A, 0xFF, 0xFF, 0xFF, 0x59, 0x83, 0xF8, 0xFF, 0x74, 0x1E, 0xFF, 0x45, 0xE4, 0xEB, 0x19, 0x39, 0x7D, 0x08, 0x75, 0x14, 0xF6, 0xC1, 0x02, 0x74, 0x0F, 0x50, 0xE8, 0x2F, 0xFF, 0xFF, 0xFF, 0x59, 0x83, 0xF8, 0xFF, 0x75, 0x03, 0x09, 0x45, 0xDC, 0x89, 0x7D, 0xFC, 0xE8, 0x08, 0x00, 0x00, 0x00, 0x46, 0xEB, 0x84,
				0x33, 0xFF, 0x8B, 0x75, 0xE0, 0xA1, 0x40, 0xEA, 0x40, 0x00, 0xFF, 0x34, 0xB0, 0x56, 0xE8, 0xAE, 0xEA, 0xFF, 0xFF, 0x59, 0x59, 0xC3, 0xC7, 0x45, 0xFC, 0xFE, 0xFF, 0xFF, 0xFF, 0xE8, 0x12, 0x00, 0x00, 0x00, 0x83, 0x7D, 0x08, 0x01, 0x8B, 0x45, 0xE4, 0x74, 0x03, 0x8B, 0x45, 0xDC, 0xE8, 0xEE, 0xCA, 0xFF, 0xFF, 0xC3, 0x6A, 0x01, 0xE8, 0xD0, 0xD1, 0xFF, 0xFF, 0x59, 0xC3, 0x6A, 0x01, 0xE8, 0x1F, 0xFF, 0xFF, 0xFF, 0x59, 0xC3, 0x6A, 0x02, 0xE8, 0xCC, 0xBF, 0xFF, 0xFF, 0x59, 0xC3, 0xE8, 0xAC, 0xED, 0xFF, 0xFF, 0x85, 0xC0, 0x74, 0x08, 0x6A, 0x16, 0xE8, 0xAE, 0xED, 0xFF, 0xFF, 0x59, 0xF6, 0x05, 0x90, 0xDD, 0x40, 0x00, 0x02, 0x74, 0x11, 0x6A, 0x01, 0x68, 0x15, 0x00, 0x00, 0x40, 0x6A, 0x03, 0xE8, 0x34, 0xAC, 0xFF, 0xFF, 0x83, 0xC4, 0x0C, 0x6A, 0x03, 0xE8, 0x64, 0xBF, 0xFF, 0xFF, 0xCC, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x83, 0x7D, 0x08, 0x00, 0x75, 0x15, 0xE8, 0xE6, 0xAD, 0xFF, 0xFF, 0xC7, 0x00, 0x16, 0x00, 0x00, 0x00, 0xE8, 0x89, 0xAD, 0xFF, 0xFF, 0x83, 0xC8, 0xFF, 0x5D, 0xC3, 0xFF, 0x75, 0x08, 0x6A, 0x00, 0xFF, 0x35, 0x28, 0xE1, 0x40, 0x00, 0xFF, 0x15, 0x1C, 0xA1, 0x40, 0x00, 0x5D, 0xC3, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0x55, 0x8B, 0xEC, 0x53, 0x56, 0x57, 0x55, 0x6A, 0x00, 0x6A, 0x00, 0x68, 0xC8, 0x8B, 0x40, 0x00, 0xFF, 0x75, 0x08, 0xE8, 0xDE, 0x04, 0x00, 0x00, 0x5D, 0x5F, 0x5E, 0x5B, 0x8B, 0xE5, 0x5D, 0xC3, 0x8B, 0x4C, 0x24, 0x04, 0xF7, 0x41, 0x04, 0x06, 0x00, 0x00, 0x00, 0xB8, 0x01, 0x00, 0x00, 0x00, 0x74, 0x32, 0x8B, 0x44, 0x24, 0x14, 0x8B, 0x48, 0xFC, 0x33, 0xC8, 0xE8, 0xCF, 0x94, 0xFF, 0xFF, 0x55, 0x8B, 0x68, 0x10, 0x8B, 0x50, 0x28, 0x52, 0x8B, 0x50, 0x24, 0x52, 0xE8, 0x14, 0x00, 0x00, 0x00, 0x83, 0xC4, 0x08, 0x5D, 0x8B, 0x44, 0x24, 0x08, 0x8B, 0x54, 0x24, 0x10, 0x89, 0x02, 0xB8, 0x03, 0x00, 0x00, 0x00, 0xC3, 0x53, 0x56, 0x57, 0x8B, 0x44, 0x24, 0x10, 0x55, 0x50, 0x6A, 0xFE, 0x68, 0xD0, 0x8B, 0x40, 0x00, 0x64, 0xFF, 0x35, 0x00, 0x00, 0x00, 0x00, 0xA1, 0x04, 0xD0, 0x40, 0x00, 0x33, 0xC4, 0x50, 0x8D, 0x44, 0x24, 0x04, 0x64, 0xA3, 0x00, 0x00, 0x00, 0x00, 0x8B, 0x44, 0x24, 0x28, 0x8B, 0x58, 0x08, 0x8B, 0x70, 0x0C, 0x83, 0xFE, 0xFF, 0x74, 0x3A, 0x83, 0x7C, 0x24, 0x2C, 0xFF, 0x74, 0x06, 0x3B, 0x74, 0x24, 0x2C, 0x76, 0x2D, 0x8D, 0x34, 0x76, 0x8B, 0x0C, 0xB3, 0x89, 0x4C, 0x24, 0x0C, 0x89, 0x48, 0x0C, 0x83, 0x7C, 0xB3, 0x04, 0x00, 0x75, 0x17, 0x68, 0x01, 0x01, 0x00, 0x00, 0x8B, 0x44, 0xB3, 0x08, 0xE8, 0x49, 0x00, 0x00, 0x00, 0x8B, 0x44, 0xB3, 0x08, 0xE8, 0x5F, 0x00, 0x00, 0x00, 0xEB, 0xB7, 0x8B, 0x4C, 0x24, 0x04, 0x64, 0x89, 0x0D, 0x00, 0x00, 0x00, 0x00, 0x83, 0xC4, 0x18, 0x5F, 0x5E, 0x5B, 0xC3, 0x33, 0xC0, 0x64, 0x8B, 0x0D, 0x00, 0x00, 0x00, 0x00, 0x81, 0x79, 0x04, 0xD0, 0x8B, 0x40, 0x00, 0x75, 0x10, 0x8B, 0x51, 0x0C, 0x8B, 0x52, 0x0C, 0x39, 0x51, 0x08, 0x75, 0x05, 0xB8, 0x01, 0x00, 0x00, 0x00, 0xC3, 0x53, 0x51, 0xBB, 0xA0, 0xDD, 0x40, 0x00, 0xEB, 0x0B, 0x53, 0x51, 0xBB, 0xA0, 0xDD, 0x40, 0x00, 0x8B, 0x4C, 0x24, 0x0C, 0x89, 0x4B, 0x08, 0x89, 0x43, 0x04, 0x89, 0x6B, 0x0C, 0x55, 0x51, 0x50, 0x58, 0x59, 0x5D, 0x59, 0x5B, 0xC2, 0x04, 0x00, 0xFF, 0xD0, 0xC3, 0x33,
				0xC0, 0x50, 0x50, 0x6A, 0x03, 0x50, 0x6A, 0x03, 0x68, 0x00, 0x00, 0x00, 0x40, 0x68, 0x44, 0xB9, 0x40, 0x00, 0xFF, 0x15, 0x30, 0xA0, 0x40, 0x00, 0xA3, 0xB0, 0xDD, 0x40, 0x00, 0xC3, 0xA1, 0xB0, 0xDD, 0x40, 0x00, 0x83, 0xF8, 0xFF, 0x74, 0x0C, 0x83, 0xF8, 0xFE, 0x74, 0x07, 0x50, 0xFF, 0x15, 0x24, 0xA0, 0x40, 0x00, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x56, 0x8B, 0x75, 0x08, 0x57, 0x83, 0xCF, 0xFF, 0x85, 0xF6, 0x75, 0x14, 0xE8, 0x39, 0xAC, 0xFF, 0xFF, 0xC7, 0x00, 0x16, 0x00, 0x00, 0x00, 0xE8, 0xDC, 0xAB, 0xFF, 0xFF, 0x0B, 0xC7, 0xEB, 0x44, 0xF6, 0x46, 0x0C, 0x83, 0x74, 0x38, 0x56, 0xE8, 0x59, 0xFC, 0xFF, 0xFF, 0x56, 0x8B, 0xF8, 0xE8, 0x1E, 0x03, 0x00, 0x00, 0x56, 0xE8, 0x88, 0xE8, 0xFF, 0xFF, 0x50, 0xE8, 0x4E, 0x02, 0x00, 0x00, 0x83, 0xC4, 0x10, 0x85, 0xC0, 0x79, 0x05, 0x83, 0xCF, 0xFF, 0xEB, 0x12, 0x8B, 0x46, 0x1C, 0x85, 0xC0, 0x74, 0x0B, 0x50, 0xE8, 0xFB, 0x96, 0xFF, 0xFF, 0x83, 0x66, 0x1C, 0x00, 0x59, 0x83, 0x66, 0x0C, 0x00, 0x8B, 0xC7, 0x5F, 0x5E, 0x5D, 0xC3, 0x6A, 0x0C, 0x68, 0x18, 0xC3, 0x40, 0x00, 0xE8, 0x3A, 0xC8, 0xFF, 0xFF, 0x83, 0x4D, 0xE4, 0xFF, 0x33, 0xC0, 0x8B, 0x75, 0x08, 0x85, 0xF6, 0x0F, 0x95, 0xC0, 0x85, 0xC0, 0x75, 0x15, 0xE8, 0xBF, 0xAB, 0xFF, 0xFF, 0xC7, 0x00, 0x16, 0x00, 0x00, 0x00, 0xE8, 0x62, 0xAB, 0xFF, 0xFF, 0x83, 0xC8, 0xFF, 0xEB, 0x0D, 0xF6, 0x46, 0x0C, 0x40, 0x74, 0x0D, 0x83, 0x66, 0x0C, 0x00, 0x8B, 0x45, 0xE4, 0xE8, 0x46, 0xC8, 0xFF, 0xFF, 0xC3, 0x56, 0xE8, 0x30, 0xE7, 0xFF, 0xFF, 0x59, 0x83, 0x65, 0xFC, 0x00, 0x56, 0xE8, 0x3C, 0xFF, 0xFF, 0xFF, 0x59, 0x89, 0x45, 0xE4, 0xC7, 0x45, 0xFC, 0xFE, 0xFF, 0xFF, 0xFF, 0xE8, 0x05, 0x00, 0x00, 0x00, 0xEB, 0xD4, 0x8B, 0x75, 0x08, 0x56, 0xE8, 0x7D, 0xE7, 0xFF, 0xFF, 0x59, 0xC3, 0x6A, 0x10, 0x68, 0x38, 0xC3, 0x40, 0x00, 0xE8, 0xC6, 0xC7, 0xFF, 0xFF, 0x8B, 0x5D, 0x08, 0x83, 0xFB, 0xFE, 0x75, 0x13, 0xE8, 0x55, 0xAB, 0xFF, 0xFF, 0xC7, 0x00, 0x09, 0x00, 0x00, 0x00, 0x83, 0xC8, 0xFF, 0xE9, 0xA1, 0x00, 0x00, 0x00, 0x85, 0xDB, 0x78, 0x08, 0x3B, 0x1D, 0x6C, 0xFA, 0x40, 0x00, 0x72, 0x12, 0xE8, 0x36, 0xAB, 0xFF, 0xFF, 0xC7, 0x00, 0x09, 0x00, 0x00, 0x00, 0xE8, 0xD9, 0xAA, 0xFF, 0xFF, 0xEB, 0xDA, 0x8B, 0xC3, 0xC1, 0xF8, 0x05, 0x8D, 0x3C, 0x85, 0x80, 0xFA, 0x40, 0x00, 0x8B, 0xF3, 0x83, 0xE6, 0x1F, 0xC1, 0xE6, 0x06, 0x8B, 0x07, 0x0F, 0xBE, 0x44, 0x06, 0x04, 0x83, 0xE0, 0x01, 0x74, 0xCE, 0x53, 0xE8, 0x34, 0xF8, 0xFF, 0xFF, 0x59, 0x83, 0x65, 0xFC, 0x00, 0x8B, 0x07, 0xF6, 0x44, 0x06, 0x04, 0x01, 0x74, 0x31, 0x53, 0xE8, 0xB7, 0xF7, 0xFF, 0xFF, 0x59, 0x50, 0xFF, 0x15, 0x28, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x75, 0x0B, 0xFF, 0x15, 0x34, 0xA0, 0x40, 0x00, 0x89, 0x45, 0xE4, 0xEB, 0x04, 0x83, 0x65, 0xE4, 0x00, 0x83, 0x7D, 0xE4, 0x00, 0x74, 0x19, 0xE8, 0xDC, 0xAA, 0xFF, 0xFF, 0x8B, 0x4D, 0xE4, 0x89, 0x08, 0xE8, 0xBF, 0xAA, 0xFF, 0xFF, 0xC7, 0x00, 0x09, 0x00, 0x00, 0x00, 0x83, 0x4D, 0xE4, 0xFF, 0xC7, 0x45, 0xFC, 0xFE, 0xFF, 0xFF, 0xFF, 0xE8, 0x0C, 0x00, 0x00, 0x00, 0x8B, 0x45, 0xE4, 0xE8, 0x4A, 0xC7, 0xFF, 0xFF, 0xC3, 0x8B, 0x5D, 0x08, 0x53, 0xE8, 0x67, 0xF8, 0xFF, 0xFF, 0x59, 0xC3, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC,
				0xCC, 0xCC, 0xCC, 0xCC, 0x8B, 0x44, 0x24, 0x08, 0x8B, 0x4C, 0x24, 0x10, 0x0B, 0xC8, 0x8B, 0x4C, 0x24, 0x0C, 0x75, 0x09, 0x8B, 0x44, 0x24, 0x04, 0xF7, 0xE1, 0xC2, 0x10, 0x00, 0x53, 0xF7, 0xE1, 0x8B, 0xD8, 0x8B, 0x44, 0x24, 0x08, 0xF7, 0x64, 0x24, 0x14, 0x03, 0xD8, 0x8B, 0x44, 0x24, 0x08, 0xF7, 0xE1, 0x03, 0xD3, 0x5B, 0xC2, 0x10, 0x00, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x56, 0x8B, 0x75, 0x08, 0x57, 0x56, 0xE8, 0x10, 0xF7, 0xFF, 0xFF, 0x59, 0x83, 0xF8, 0xFF, 0x74, 0x50, 0xA1, 0x80, 0xFA, 0x40, 0x00, 0x83, 0xFE, 0x01, 0x75, 0x09, 0xF6, 0x80, 0x84, 0x00, 0x00, 0x00, 0x01, 0x75, 0x0B, 0x83, 0xFE, 0x02, 0x75, 0x1C, 0xF6, 0x40, 0x44, 0x01, 0x74, 0x16, 0x6A, 0x02, 0xE8, 0xE5, 0xF6, 0xFF, 0xFF, 0x6A, 0x01, 0x8B, 0xF8, 0xE8, 0xDC, 0xF6, 0xFF, 0xFF, 0x59, 0x59, 0x3B, 0xC7, 0x74, 0x1C, 0x56, 0xE8, 0xD0, 0xF6, 0xFF, 0xFF, 0x59, 0x50, 0xFF, 0x15, 0x24, 0xA0, 0x40, 0x00, 0x85, 0xC0, 0x75, 0x0A, 0xFF, 0x15, 0x34, 0xA0, 0x40, 0x00, 0x8B, 0xF8, 0xEB, 0x02, 0x33, 0xFF, 0x56, 0xE8, 0x2C, 0xF6, 0xFF, 0xFF, 0x8B, 0xC6, 0xC1, 0xF8, 0x05, 0x8B, 0x04, 0x85, 0x80, 0xFA, 0x40, 0x00, 0x83, 0xE6, 0x1F, 0xC1, 0xE6, 0x06, 0x59, 0xC6, 0x44, 0x30, 0x04, 0x00, 0x85, 0xFF, 0x74, 0x0C, 0x57, 0xE8, 0xEE, 0xA9, 0xFF, 0xFF, 0x59, 0x83, 0xC8, 0xFF, 0xEB, 0x02, 0x33, 0xC0, 0x5F, 0x5E, 0x5D, 0xC3, 0x6A, 0x10, 0x68, 0x58, 0xC3, 0x40, 0x00, 0xE8, 0x14, 0xC6, 0xFF, 0xFF, 0x8B, 0x5D, 0x08, 0x83, 0xFB, 0xFE, 0x75, 0x1B, 0xE8, 0xB6, 0xA9, 0xFF, 0xFF, 0x83, 0x20, 0x00, 0xE8, 0x9B, 0xA9, 0xFF, 0xFF, 0xC7, 0x00, 0x09, 0x00, 0x00, 0x00, 0x83, 0xC8, 0xFF, 0xE9, 0x84, 0x00, 0x00, 0x00, 0x85, 0xDB, 0x78, 0x08, 0x3B, 0x1D, 0x6C, 0xFA, 0x40, 0x00, 0x72, 0x1A, 0xE8, 0x8F, 0xA9, 0xFF, 0xFF, 0x83, 0x20, 0x00, 0xE8, 0x74, 0xA9, 0xFF, 0xFF, 0xC7, 0x00, 0x09, 0x00, 0x00, 0x00, 0xE8, 0x17, 0xA9, 0xFF, 0xFF, 0xEB, 0xD2, 0x8B, 0xC3, 0xC1, 0xF8, 0x05, 0x8D, 0x3C, 0x85, 0x80, 0xFA, 0x40, 0x00, 0x8B, 0xF3, 0x83, 0xE6, 0x1F, 0xC1, 0xE6, 0x06, 0x8B, 0x07, 0x0F, 0xBE, 0x44, 0x30, 0x04, 0x83, 0xE0, 0x01, 0x74, 0xC6, 0x53, 0xE8, 0x72, 0xF6, 0xFF, 0xFF, 0x59, 0x83, 0x65, 0xFC, 0x00, 0x8B, 0x07, 0xF6, 0x44, 0x30, 0x04, 0x01, 0x74, 0x0C, 0x53, 0xE8, 0xD5, 0xFE, 0xFF, 0xFF, 0x59, 0x89, 0x45, 0xE4, 0xEB, 0x0F, 0xE8, 0x22, 0xA9, 0xFF, 0xFF, 0xC7, 0x00, 0x09, 0x00, 0x00, 0x00, 0x83, 0x4D, 0xE4, 0xFF, 0xC7, 0x45, 0xFC, 0xFE, 0xFF, 0xFF, 0xFF, 0xE8, 0x0C, 0x00, 0x00, 0x00, 0x8B, 0x45, 0xE4, 0xE8, 0xAD, 0xC5, 0xFF, 0xFF, 0xC3, 0x8B, 0x5D, 0x08, 0x53, 0xE8, 0xCA, 0xF6, 0xFF, 0xFF, 0x59, 0xC3, 0x8B, 0xFF, 0x55, 0x8B, 0xEC, 0x56, 0x8B, 0x75, 0x08, 0x8B, 0x46, 0x0C, 0xA8, 0x83, 0x74, 0x1E, 0xA8, 0x08, 0x74, 0x1A, 0xFF, 0x76, 0x08, 0xE8, 0xE6, 0x93, 0xFF, 0xFF, 0x81, 0x66, 0x0C, 0xF7, 0xFB, 0xFF, 0xFF, 0x33, 0xC0, 0x59, 0x89, 0x06, 0x89, 0x46, 0x08, 0x89, 0x46, 0x04, 0x5E, 0x5D, 0xC3, 0xCC, 0xFF, 0x25, 0x08, 0xA1, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
				0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x68, 0xC5, 0x00, 0x00, 0xAC, 0xC5, 0x00, 0x00, 0x30, 0xC5, 0x00, 0x00, 0x42, 0xC5, 0x00, 0x00, 0x54, 0xC5, 0x00, 0x00, 0x78, 0xC5, 0x00, 0x00, 0x8A, 0xC5, 0x00, 0x00, 0x9A, 0xC5, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x2E, 0xCA, 0x00, 0x00, 0x3C, 0xCA, 0x00, 0x00, 0xB0, 0xC7, 0x00, 0x00, 0x20, 0xCA, 0x00, 0x00, 0xC8, 0xC5, 0x00, 0x00, 0xD8, 0xC5, 0x00, 0x00, 0xE4, 0xC5, 0x00, 0x00, 0xF0, 0xC5, 0x00, 0x00, 0x02, 0xC6, 0x00, 0x00, 0x18, 0xC6, 0x00, 0x00, 0x2A, 0xC6, 0x00, 0x00, 0x3E, 0xC6, 0x00, 0x00, 0x52, 0xC6, 0x00, 0x00, 0x6E, 0xC6, 0x00, 0x00, 0x8C, 0xC6, 0x00, 0x00, 0xA0, 0xC6, 0x00, 0x00, 0xAC, 0xC6, 0x00, 0x00, 0xC4, 0xC6, 0x00, 0x00, 0xDC, 0xC6, 0x00, 0x00, 0xE6, 0xC6, 0x00, 0x00, 0xF2, 0xC6, 0x00, 0x00, 0x04, 0xC7, 0x00, 0x00, 0x14, 0xC7, 0x00, 0x00, 0x20, 0xC7, 0x00, 0x00, 0x2E, 0xC7, 0x00, 0x00, 0x3C, 0xC7, 0x00, 0x00, 0x4C, 0xC7, 0x00, 0x00, 0x56, 0xC7, 0x00, 0x00, 0x6A, 0xC7, 0x00, 0x00, 0x7A, 0xC7, 0x00, 0x00, 0x90, 0xC7, 0x00, 0x00, 0xA2, 0xC7, 0x00, 0x00, 0xBE, 0xC7, 0x00, 0x00, 0xCA, 0xC7, 0x00, 0x00, 0xDA, 0xC7, 0x00, 0x00, 0xF0, 0xC7, 0x00, 0x00, 0x06, 0xC8, 0x00, 0x00, 0x20, 0xC8, 0x00, 0x00, 0x36, 0xC8, 0x00, 0x00, 0x50, 0xC8, 0x00, 0x00,
				0x62, 0xC8, 0x00, 0x00, 0x8A, 0xC8, 0x00, 0x00, 0x98, 0xC8, 0x00, 0x00, 0xB0, 0xC8, 0x00, 0x00, 0xCA, 0xC8, 0x00, 0x00, 0xDA, 0xC8, 0x00, 0x00, 0xF0, 0xC8, 0x00, 0x00, 0x0A, 0xC9, 0x00, 0x00, 0x1A, 0xC9, 0x00, 0x00, 0x30, 0xC9, 0x00, 0x00, 0x42, 0xC9, 0x00, 0x00, 0x5A, 0xC9, 0x00, 0x00, 0x72, 0xC9, 0x00, 0x00, 0x7A, 0xC9, 0x00, 0x00, 0x8C, 0xC9, 0x00, 0x00, 0x9C, 0xC9, 0x00, 0x00, 0xAE, 0xC9, 0x00, 0x00, 0xBE, 0xC9, 0x00, 0x00, 0xCA, 0xC9, 0x00, 0x00, 0xE6, 0xC9, 0x00, 0x00, 0xF4, 0xC9, 0x00, 0x00, 0x04, 0xCA, 0x00, 0x00, 0x14, 0xCA, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x16, 0xC5, 0x00, 0x00, 0x0A, 0xC5, 0x00, 0x00, 0xF8, 0xC4, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xB0, 0x2E, 0x40, 0x00, 0x35, 0x74, 0x40, 0x00, 0x78, 0x7B, 0x40, 0x00, 0x3C, 0x83, 0x40, 0x00, 0xB2, 0x4D, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x06, 0x8D, 0x40, 0x00, 0xE6, 0x74, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xD0, 0xDD, 0x40, 0x00, 0x28, 0xDE, 0x40, 0x00, 0x48, 0x00, 0x48, 0x00, 0x3A, 0x00, 0x6D, 0x00, 0x6D, 0x00, 0x3A, 0x00, 0x73, 0x00, 0x73, 0x00, 0x00, 0x00, 0x00, 0x00, 0x64, 0x00, 0x64, 0x00, 0x64, 0x00, 0x64, 0x00, 0x2C, 0x00, 0x20, 0x00, 0x4D, 0x00, 0x4D, 0x00, 0x4D, 0x00, 0x4D, 0x00, 0x20, 0x00, 0x64, 0x00, 0x64, 0x00, 0x2C, 0x00, 0x20, 0x00, 0x79, 0x00, 0x79, 0x00, 0x79, 0x00, 0x79, 0x00, 0x00, 0x00, 0x4D, 0x00, 0x4D, 0x00, 0x2F, 0x00, 0x64, 0x00, 0x64, 0x00, 0x2F, 0x00, 0x79, 0x00, 0x79, 0x00, 0x00, 0x00, 0x00, 0x00, 0x50, 0x00, 0x4D, 0x00, 0x00, 0x00, 0x00, 0x00, 0x41, 0x00, 0x4D, 0x00, 0x00, 0x00, 0x00, 0x00, 0x44, 0x00, 0x65, 0x00, 0x63, 0x00, 0x65, 0x00, 0x6D, 0x00, 0x62, 0x00, 0x65, 0x00, 0x72, 0x00, 0x00, 0x00, 0x00, 0x00, 0x4E, 0x00, 0x6F, 0x00, 0x76, 0x00, 0x65, 0x00, 0x6D, 0x00, 0x62, 0x00, 0x65, 0x00, 0x72, 0x00, 0x00, 0x00, 0x00, 0x00, 0x4F, 0x00, 0x63, 0x00, 0x74, 0x00, 0x6F, 0x00, 0x62, 0x00, 0x65, 0x00, 0x72, 0x00, 0x00, 0x00, 0x53, 0x00, 0x65, 0x00, 0x70, 0x00, 0x74, 0x00, 0x65, 0x00, 0x6D, 0x00, 0x62, 0x00, 0x65, 0x00, 0x72, 0x00, 0x00, 0x00, 0x41, 0x00, 0x75, 0x00, 0x67, 0x00, 0x75, 0x00, 0x73, 0x00, 0x74, 0x00, 0x00, 0x00, 0x00, 0x00, 0x4A, 0x00, 0x75, 0x00, 0x6C, 0x00, 0x79, 0x00, 0x00, 0x00, 0x00, 0x00, 0x4A, 0x00, 0x75, 0x00, 0x6E, 0x00, 0x65, 0x00, 0x00, 0x00, 0x00, 0x00, 0x41, 0x00, 0x70, 0x00, 0x72, 0x00, 0x69, 0x00, 0x6C, 0x00, 0x00, 0x00, 0x4D, 0x00, 0x61, 0x00, 0x72, 0x00, 0x63, 0x00, 0x68, 0x00, 0x00, 0x00, 0x46, 0x00, 0x65, 0x00, 0x62, 0x00, 0x72, 0x00, 0x75, 0x00, 0x61, 0x00, 0x72, 0x00, 0x79, 0x00, 0x00, 0x00, 0x00, 0x00, 0x4A, 0x00, 0x61, 0x00, 0x6E, 0x00, 0x75, 0x00, 0x61, 0x00, 0x72, 0x00, 0x79, 0x00, 0x00, 0x00, 0x44, 0x00, 0x65, 0x00, 0x63, 0x00, 0x00, 0x00, 0x4E, 0x00, 0x6F, 0x00, 0x76, 0x00, 0x00, 0x00, 0x4F, 0x00, 0x63, 0x00, 0x74, 0x00, 0x00, 0x00, 0x53, 0x00, 0x65, 0x00, 0x70, 0x00, 0x00, 0x00, 0x41, 0x00, 0x75, 0x00, 0x67, 0x00, 0x00, 0x00, 0x4A, 0x00, 0x75, 0x00, 0x6C, 0x00, 0x00, 0x00,
				0x4A, 0x00, 0x75, 0x00, 0x6E, 0x00, 0x00, 0x00, 0x4D, 0x00, 0x61, 0x00, 0x79, 0x00, 0x00, 0x00, 0x41, 0x00, 0x70, 0x00, 0x72, 0x00, 0x00, 0x00, 0x4D, 0x00, 0x61, 0x00, 0x72, 0x00, 0x00, 0x00, 0x46, 0x00, 0x65, 0x00, 0x62, 0x00, 0x00, 0x00, 0x4A, 0x00, 0x61, 0x00, 0x6E, 0x00, 0x00, 0x00, 0x53, 0x00, 0x61, 0x00, 0x74, 0x00, 0x75, 0x00, 0x72, 0x00, 0x64, 0x00, 0x61, 0x00, 0x79, 0x00, 0x00, 0x00, 0x00, 0x00, 0x46, 0x00, 0x72, 0x00, 0x69, 0x00, 0x64, 0x00, 0x61, 0x00, 0x79, 0x00, 0x00, 0x00, 0x00, 0x00, 0x54, 0x00, 0x68, 0x00, 0x75, 0x00, 0x72, 0x00, 0x73, 0x00, 0x64, 0x00, 0x61, 0x00, 0x79, 0x00, 0x00, 0x00, 0x00, 0x00, 0x57, 0x00, 0x65, 0x00, 0x64, 0x00, 0x6E, 0x00, 0x65, 0x00, 0x73, 0x00, 0x64, 0x00, 0x61, 0x00, 0x79, 0x00, 0x00, 0x00, 0x54, 0x00, 0x75, 0x00, 0x65, 0x00, 0x73, 0x00, 0x64, 0x00, 0x61, 0x00, 0x79, 0x00, 0x00, 0x00, 0x4D, 0x00, 0x6F, 0x00, 0x6E, 0x00, 0x64, 0x00, 0x61, 0x00, 0x79, 0x00, 0x00, 0x00, 0x00, 0x00, 0x53, 0x00, 0x75, 0x00, 0x6E, 0x00, 0x64, 0x00, 0x61, 0x00, 0x79, 0x00, 0x00, 0x00, 0x00, 0x00, 0x53, 0x00, 0x61, 0x00, 0x74, 0x00, 0x00, 0x00, 0x46, 0x00, 0x72, 0x00, 0x69, 0x00, 0x00, 0x00, 0x54, 0x00, 0x68, 0x00, 0x75, 0x00, 0x00, 0x00, 0x57, 0x00, 0x65, 0x00, 0x64, 0x00, 0x00, 0x00, 0x54, 0x00, 0x75, 0x00, 0x65, 0x00, 0x00, 0x00, 0x4D, 0x00, 0x6F, 0x00, 0x6E, 0x00, 0x00, 0x00, 0x53, 0x00, 0x75, 0x00, 0x6E, 0x00, 0x00, 0x00, 0x48, 0x48, 0x3A, 0x6D, 0x6D, 0x3A, 0x73, 0x73, 0x00, 0x00, 0x00, 0x00, 0x64, 0x64, 0x64, 0x64, 0x2C, 0x20, 0x4D, 0x4D, 0x4D, 0x4D, 0x20, 0x64, 0x64, 0x2C, 0x20, 0x79, 0x79, 0x79, 0x79, 0x00, 0x4D, 0x4D, 0x2F, 0x64, 0x64, 0x2F, 0x79, 0x79, 0x00, 0x00, 0x00, 0x00, 0x50, 0x4D, 0x00, 0x00, 0x41, 0x4D, 0x00, 0x00, 0x44, 0x65, 0x63, 0x65, 0x6D, 0x62, 0x65, 0x72, 0x00, 0x00, 0x00, 0x00, 0x4E, 0x6F, 0x76, 0x65, 0x6D, 0x62, 0x65, 0x72, 0x00, 0x00, 0x00, 0x00, 0x4F, 0x63, 0x74, 0x6F, 0x62, 0x65, 0x72, 0x00, 0x53, 0x65, 0x70, 0x74, 0x65, 0x6D, 0x62, 0x65, 0x72, 0x00, 0x00, 0x00, 0x41, 0x75, 0x67, 0x75, 0x73, 0x74, 0x00, 0x00, 0x4A, 0x75, 0x6C, 0x79, 0x00, 0x00, 0x00, 0x00, 0x4A, 0x75, 0x6E, 0x65, 0x00, 0x00, 0x00, 0x00, 0x41, 0x70, 0x72, 0x69, 0x6C, 0x00, 0x00, 0x00, 0x4D, 0x61, 0x72, 0x63, 0x68, 0x00, 0x00, 0x00, 0x46, 0x65, 0x62, 0x72, 0x75, 0x61, 0x72, 0x79, 0x00, 0x00, 0x00, 0x00, 0x4A, 0x61, 0x6E, 0x75, 0x61, 0x72, 0x79, 0x00, 0x44, 0x65, 0x63, 0x00, 0x4E, 0x6F, 0x76, 0x00, 0x4F, 0x63, 0x74, 0x00, 0x53, 0x65, 0x70, 0x00, 0x41, 0x75, 0x67, 0x00, 0x4A, 0x75, 0x6C, 0x00, 0x4A, 0x75, 0x6E, 0x00, 0x4D, 0x61, 0x79, 0x00, 0x41, 0x70, 0x72, 0x00, 0x4D, 0x61, 0x72, 0x00, 0x46, 0x65, 0x62, 0x00, 0x4A, 0x61, 0x6E, 0x00, 0x53, 0x61, 0x74, 0x75, 0x72, 0x64, 0x61, 0x79, 0x00, 0x00, 0x00, 0x00, 0x46, 0x72, 0x69, 0x64, 0x61, 0x79, 0x00, 0x00, 0x54, 0x68, 0x75, 0x72, 0x73, 0x64, 0x61, 0x79, 0x00, 0x00, 0x00, 0x00, 0x57, 0x65, 0x64, 0x6E, 0x65, 0x73, 0x64, 0x61, 0x79, 0x00, 0x00, 0x00, 0x54, 0x75, 0x65, 0x73, 0x64, 0x61, 0x79, 0x00, 0x4D, 0x6F, 0x6E, 0x64, 0x61, 0x79, 0x00, 0x00, 0x53, 0x75, 0x6E, 0x64, 0x61, 0x79, 0x00, 0x00,
				0x53, 0x61, 0x74, 0x00, 0x46, 0x72, 0x69, 0x00, 0x54, 0x68, 0x75, 0x00, 0x57, 0x65, 0x64, 0x00, 0x54, 0x75, 0x65, 0x00, 0x4D, 0x6F, 0x6E, 0x00, 0x53, 0x75, 0x6E, 0x00, 0x4B, 0x00, 0x45, 0x00, 0x52, 0x00, 0x4E, 0x00, 0x45, 0x00, 0x4C, 0x00, 0x33, 0x00, 0x32, 0x00, 0x2E, 0x00, 0x44, 0x00, 0x4C, 0x00, 0x4C, 0x00, 0x00, 0x00, 0x00, 0x00, 0x46, 0x6C, 0x73, 0x46, 0x72, 0x65, 0x65, 0x00, 0x46, 0x6C, 0x73, 0x53, 0x65, 0x74, 0x56, 0x61, 0x6C, 0x75, 0x65, 0x00, 0x46, 0x6C, 0x73, 0x47, 0x65, 0x74, 0x56, 0x61, 0x6C, 0x75, 0x65, 0x00, 0x46, 0x6C, 0x73, 0x41, 0x6C, 0x6C, 0x6F, 0x63, 0x00, 0x00, 0x00, 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1A, 0x1B, 0x1C, 0x1D, 0x1E, 0x1F, 0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2A, 0x2B, 0x2C, 0x2D, 0x2E, 0x2F, 0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x3E, 0x3F, 0x40, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4A, 0x4B, 0x4C, 0x4D, 0x4E, 0x4F, 0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5A, 0x5B, 0x5C, 0x5D, 0x5E, 0x5F, 0x60, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6A, 0x6B, 0x6C, 0x6D, 0x6E, 0x6F, 0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7A, 0x7B, 0x7C, 0x7D, 0x7E, 0x7F, 0x00, 0x28, 0x00, 0x6E, 0x00, 0x75, 0x00, 0x6C, 0x00, 0x6C, 0x00, 0x29, 0x00, 0x00, 0x00, 0x00, 0x00, 0x28, 0x6E, 0x75, 0x6C, 0x6C, 0x29, 0x00, 0x00, 0x06, 0x00, 0x00, 0x06, 0x00, 0x01, 0x00, 0x00, 0x10, 0x00, 0x03, 0x06, 0x00, 0x06, 0x02, 0x10, 0x04, 0x45, 0x45, 0x45, 0x05, 0x05, 0x05, 0x05, 0x05, 0x35, 0x30, 0x00, 0x50, 0x00, 0x00, 0x00, 0x00, 0x28, 0x20, 0x38, 0x50, 0x58, 0x07, 0x08, 0x00, 0x37, 0x30, 0x30, 0x57, 0x50, 0x07, 0x00, 0x00, 0x20, 0x20, 0x08, 0x00, 0x00, 0x00, 0x00, 0x08, 0x60, 0x68, 0x60, 0x60, 0x60, 0x60, 0x00, 0x00, 0x78, 0x70, 0x78, 0x78, 0x78, 0x78, 0x08, 0x07, 0x08, 0x00, 0x00, 0x07, 0x00, 0x08, 0x08, 0x08, 0x00, 0x00, 0x08, 0x00, 0x08, 0x00, 0x07, 0x08, 0x00, 0x00, 0x00, 0x43, 0x6F, 0x72, 0x45, 0x78, 0x69, 0x74, 0x50, 0x72, 0x6F, 0x63, 0x65, 0x73, 0x73, 0x00, 0x00, 0x6D, 0x00, 0x73, 0x00, 0x63, 0x00, 0x6F, 0x00, 0x72, 0x00, 0x65, 0x00, 0x65, 0x00, 0x2E, 0x00, 0x64, 0x00, 0x6C, 0x00, 0x6C, 0x00, 0x00, 0x00, 0x72, 0x00, 0x75, 0x00, 0x6E, 0x00, 0x74, 0x00, 0x69, 0x00, 0x6D, 0x00, 0x65, 0x00, 0x20, 0x00, 0x65, 0x00, 0x72, 0x00, 0x72, 0x00, 0x6F, 0x00, 0x72, 0x00, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x54, 0x00, 0x4C, 0x00, 0x4F, 0x00, 0x53, 0x00, 0x53, 0x00, 0x20, 0x00, 0x65, 0x00, 0x72, 0x00, 0x72, 0x00, 0x6F, 0x00, 0x72, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x53, 0x00, 0x49, 0x00, 0x4E, 0x00, 0x47, 0x00, 0x20, 0x00, 0x65, 0x00, 0x72, 0x00, 0x72, 0x00, 0x6F, 0x00, 0x72, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x44, 0x00, 0x4F, 0x00, 0x4D, 0x00, 0x41, 0x00, 0x49, 0x00, 0x4E, 0x00, 0x20, 0x00, 0x65, 0x00, 0x72, 0x00, 0x72, 0x00,
				0x6F, 0x00, 0x72, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x52, 0x00, 0x36, 0x00, 0x30, 0x00, 0x33, 0x00, 0x33, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x2D, 0x00, 0x20, 0x00, 0x41, 0x00, 0x74, 0x00, 0x74, 0x00, 0x65, 0x00, 0x6D, 0x00, 0x70, 0x00, 0x74, 0x00, 0x20, 0x00, 0x74, 0x00, 0x6F, 0x00, 0x20, 0x00, 0x75, 0x00, 0x73, 0x00, 0x65, 0x00, 0x20, 0x00, 0x4D, 0x00, 0x53, 0x00, 0x49, 0x00, 0x4C, 0x00, 0x20, 0x00, 0x63, 0x00, 0x6F, 0x00, 0x64, 0x00, 0x65, 0x00, 0x20, 0x00, 0x66, 0x00, 0x72, 0x00, 0x6F, 0x00, 0x6D, 0x00, 0x20, 0x00, 0x74, 0x00, 0x68, 0x00, 0x69, 0x00, 0x73, 0x00, 0x20, 0x00, 0x61, 0x00, 0x73, 0x00, 0x73, 0x00, 0x65, 0x00, 0x6D, 0x00, 0x62, 0x00, 0x6C, 0x00, 0x79, 0x00, 0x20, 0x00, 0x64, 0x00, 0x75, 0x00, 0x72, 0x00, 0x69, 0x00, 0x6E, 0x00, 0x67, 0x00, 0x20, 0x00, 0x6E, 0x00, 0x61, 0x00, 0x74, 0x00, 0x69, 0x00, 0x76, 0x00, 0x65, 0x00, 0x20, 0x00, 0x63, 0x00, 0x6F, 0x00, 0x64, 0x00, 0x65, 0x00, 0x20, 0x00, 0x69, 0x00, 0x6E, 0x00, 0x69, 0x00, 0x74, 0x00, 0x69, 0x00, 0x61, 0x00, 0x6C, 0x00, 0x69, 0x00, 0x7A, 0x00, 0x61, 0x00, 0x74, 0x00, 0x69, 0x00, 0x6F, 0x00, 0x6E, 0x00, 0x0A, 0x00, 0x54, 0x00, 0x68, 0x00, 0x69, 0x00, 0x73, 0x00, 0x20, 0x00, 0x69, 0x00, 0x6E, 0x00, 0x64, 0x00, 0x69, 0x00, 0x63, 0x00, 0x61, 0x00, 0x74, 0x00, 0x65, 0x00, 0x73, 0x00, 0x20, 0x00, 0x61, 0x00, 0x20, 0x00, 0x62, 0x00, 0x75, 0x00, 0x67, 0x00, 0x20, 0x00, 0x69, 0x00, 0x6E, 0x00, 0x20, 0x00, 0x79, 0x00, 0x6F, 0x00, 0x75, 0x00, 0x72, 0x00, 0x20, 0x00, 0x61, 0x00, 0x70, 0x00, 0x70, 0x00, 0x6C, 0x00, 0x69, 0x00, 0x63, 0x00, 0x61, 0x00, 0x74, 0x00, 0x69, 0x00, 0x6F, 0x00, 0x6E, 0x00, 0x2E, 0x00, 0x20, 0x00, 0x49, 0x00, 0x74, 0x00, 0x20, 0x00, 0x69, 0x00, 0x73, 0x00, 0x20, 0x00, 0x6D, 0x00, 0x6F, 0x00, 0x73, 0x00, 0x74, 0x00, 0x20, 0x00, 0x6C, 0x00, 0x69, 0x00, 0x6B, 0x00, 0x65, 0x00, 0x6C, 0x00, 0x79, 0x00, 0x20, 0x00, 0x74, 0x00, 0x68, 0x00, 0x65, 0x00, 0x20, 0x00, 0x72, 0x00, 0x65, 0x00, 0x73, 0x00, 0x75, 0x00, 0x6C, 0x00, 0x74, 0x00, 0x20, 0x00, 0x6F, 0x00, 0x66, 0x00, 0x20, 0x00, 0x63, 0x00, 0x61, 0x00, 0x6C, 0x00, 0x6C, 0x00, 0x69, 0x00, 0x6E, 0x00, 0x67, 0x00, 0x20, 0x00, 0x61, 0x00, 0x6E, 0x00, 0x20, 0x00, 0x4D, 0x00, 0x53, 0x00, 0x49, 0x00, 0x4C, 0x00, 0x2D, 0x00, 0x63, 0x00, 0x6F, 0x00, 0x6D, 0x00, 0x70, 0x00, 0x69, 0x00, 0x6C, 0x00, 0x65, 0x00, 0x64, 0x00, 0x20, 0x00, 0x28, 0x00, 0x2F, 0x00, 0x63, 0x00, 0x6C, 0x00, 0x72, 0x00, 0x29, 0x00, 0x20, 0x00, 0x66, 0x00, 0x75, 0x00, 0x6E, 0x00, 0x63, 0x00, 0x74, 0x00, 0x69, 0x00, 0x6F, 0x00, 0x6E, 0x00, 0x20, 0x00, 0x66, 0x00, 0x72, 0x00, 0x6F, 0x00, 0x6D, 0x00, 0x20, 0x00, 0x61, 0x00, 0x20, 0x00, 0x6E, 0x00, 0x61, 0x00, 0x74, 0x00, 0x69, 0x00, 0x76, 0x00, 0x65, 0x00, 0x20, 0x00, 0x63, 0x00, 0x6F, 0x00, 0x6E, 0x00, 0x73, 0x00, 0x74, 0x00, 0x72, 0x00, 0x75, 0x00, 0x63, 0x00, 0x74, 0x00, 0x6F, 0x00, 0x72, 0x00, 0x20, 0x00, 0x6F, 0x00, 0x72, 0x00, 0x20, 0x00, 0x66, 0x00, 0x72, 0x00, 0x6F, 0x00, 0x6D, 0x00, 0x20, 0x00, 0x44, 0x00, 0x6C, 0x00, 0x6C, 0x00, 0x4D, 0x00, 0x61, 0x00, 0x69, 0x00,
				0x6E, 0x00, 0x2E, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x52, 0x00, 0x36, 0x00, 0x30, 0x00, 0x33, 0x00, 0x32, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x2D, 0x00, 0x20, 0x00, 0x6E, 0x00, 0x6F, 0x00, 0x74, 0x00, 0x20, 0x00, 0x65, 0x00, 0x6E, 0x00, 0x6F, 0x00, 0x75, 0x00, 0x67, 0x00, 0x68, 0x00, 0x20, 0x00, 0x73, 0x00, 0x70, 0x00, 0x61, 0x00, 0x63, 0x00, 0x65, 0x00, 0x20, 0x00, 0x66, 0x00, 0x6F, 0x00, 0x72, 0x00, 0x20, 0x00, 0x6C, 0x00, 0x6F, 0x00, 0x63, 0x00, 0x61, 0x00, 0x6C, 0x00, 0x65, 0x00, 0x20, 0x00, 0x69, 0x00, 0x6E, 0x00, 0x66, 0x00, 0x6F, 0x00, 0x72, 0x00, 0x6D, 0x00, 0x61, 0x00, 0x74, 0x00, 0x69, 0x00, 0x6F, 0x00, 0x6E, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x52, 0x00, 0x36, 0x00, 0x30, 0x00, 0x33, 0x00, 0x31, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x2D, 0x00, 0x20, 0x00, 0x41, 0x00, 0x74, 0x00, 0x74, 0x00, 0x65, 0x00, 0x6D, 0x00, 0x70, 0x00, 0x74, 0x00, 0x20, 0x00, 0x74, 0x00, 0x6F, 0x00, 0x20, 0x00, 0x69, 0x00, 0x6E, 0x00, 0x69, 0x00, 0x74, 0x00, 0x69, 0x00, 0x61, 0x00, 0x6C, 0x00, 0x69, 0x00, 0x7A, 0x00, 0x65, 0x00, 0x20, 0x00, 0x74, 0x00, 0x68, 0x00, 0x65, 0x00, 0x20, 0x00, 0x43, 0x00, 0x52, 0x00, 0x54, 0x00, 0x20, 0x00, 0x6D, 0x00, 0x6F, 0x00, 0x72, 0x00, 0x65, 0x00, 0x20, 0x00, 0x74, 0x00, 0x68, 0x00, 0x61, 0x00, 0x6E, 0x00, 0x20, 0x00, 0x6F, 0x00, 0x6E, 0x00, 0x63, 0x00, 0x65, 0x00, 0x2E, 0x00, 0x0A, 0x00, 0x54, 0x00, 0x68, 0x00, 0x69, 0x00, 0x73, 0x00, 0x20, 0x00, 0x69, 0x00, 0x6E, 0x00, 0x64, 0x00, 0x69, 0x00, 0x63, 0x00, 0x61, 0x00, 0x74, 0x00, 0x65, 0x00, 0x73, 0x00, 0x20, 0x00, 0x61, 0x00, 0x20, 0x00, 0x62, 0x00, 0x75, 0x00, 0x67, 0x00, 0x20, 0x00, 0x69, 0x00, 0x6E, 0x00, 0x20, 0x00, 0x79, 0x00, 0x6F, 0x00, 0x75, 0x00, 0x72, 0x00, 0x20, 0x00, 0x61, 0x00, 0x70, 0x00, 0x70, 0x00, 0x6C, 0x00, 0x69, 0x00, 0x63, 0x00, 0x61, 0x00, 0x74, 0x00, 0x69, 0x00, 0x6F, 0x00, 0x6E, 0x00, 0x2E, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x52, 0x00, 0x36, 0x00, 0x30, 0x00, 0x33, 0x00, 0x30, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x2D, 0x00, 0x20, 0x00, 0x43, 0x00, 0x52, 0x00, 0x54, 0x00, 0x20, 0x00, 0x6E, 0x00, 0x6F, 0x00, 0x74, 0x00, 0x20, 0x00, 0x69, 0x00, 0x6E, 0x00, 0x69, 0x00, 0x74, 0x00, 0x69, 0x00, 0x61, 0x00, 0x6C, 0x00, 0x69, 0x00, 0x7A, 0x00, 0x65, 0x00, 0x64, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x52, 0x00, 0x36, 0x00, 0x30, 0x00, 0x32, 0x00, 0x38, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x2D, 0x00, 0x20, 0x00, 0x75, 0x00, 0x6E, 0x00, 0x61, 0x00, 0x62, 0x00, 0x6C, 0x00, 0x65, 0x00, 0x20, 0x00, 0x74, 0x00, 0x6F, 0x00, 0x20, 0x00, 0x69, 0x00, 0x6E, 0x00, 0x69, 0x00, 0x74, 0x00, 0x69, 0x00, 0x61, 0x00, 0x6C, 0x00, 0x69, 0x00, 0x7A, 0x00, 0x65, 0x00, 0x20, 0x00, 0x68, 0x00, 0x65, 0x00, 0x61, 0x00, 0x70, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x52, 0x00, 0x36, 0x00, 0x30, 0x00, 0x32, 0x00, 0x37, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x2D, 0x00, 0x20, 0x00, 0x6E, 0x00, 0x6F, 0x00, 0x74, 0x00, 0x20, 0x00, 0x65, 0x00, 0x6E, 0x00, 0x6F, 0x00, 0x75, 0x00, 0x67, 0x00, 0x68, 0x00, 0x20, 0x00,
				0x73, 0x00, 0x70, 0x00, 0x61, 0x00, 0x63, 0x00, 0x65, 0x00, 0x20, 0x00, 0x66, 0x00, 0x6F, 0x00, 0x72, 0x00, 0x20, 0x00, 0x6C, 0x00, 0x6F, 0x00, 0x77, 0x00, 0x69, 0x00, 0x6F, 0x00, 0x20, 0x00, 0x69, 0x00, 0x6E, 0x00, 0x69, 0x00, 0x74, 0x00, 0x69, 0x00, 0x61, 0x00, 0x6C, 0x00, 0x69, 0x00, 0x7A, 0x00, 0x61, 0x00, 0x74, 0x00, 0x69, 0x00, 0x6F, 0x00, 0x6E, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x52, 0x00, 0x36, 0x00, 0x30, 0x00, 0x32, 0x00, 0x36, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x2D, 0x00, 0x20, 0x00, 0x6E, 0x00, 0x6F, 0x00, 0x74, 0x00, 0x20, 0x00, 0x65, 0x00, 0x6E, 0x00, 0x6F, 0x00, 0x75, 0x00, 0x67, 0x00, 0x68, 0x00, 0x20, 0x00, 0x73, 0x00, 0x70, 0x00, 0x61, 0x00, 0x63, 0x00, 0x65, 0x00, 0x20, 0x00, 0x66, 0x00, 0x6F, 0x00, 0x72, 0x00, 0x20, 0x00, 0x73, 0x00, 0x74, 0x00, 0x64, 0x00, 0x69, 0x00, 0x6F, 0x00, 0x20, 0x00, 0x69, 0x00, 0x6E, 0x00, 0x69, 0x00, 0x74, 0x00, 0x69, 0x00, 0x61, 0x00, 0x6C, 0x00, 0x69, 0x00, 0x7A, 0x00, 0x61, 0x00, 0x74, 0x00, 0x69, 0x00, 0x6F, 0x00, 0x6E, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x52, 0x00, 0x36, 0x00, 0x30, 0x00, 0x32, 0x00, 0x35, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x2D, 0x00, 0x20, 0x00, 0x70, 0x00, 0x75, 0x00, 0x72, 0x00, 0x65, 0x00, 0x20, 0x00, 0x76, 0x00, 0x69, 0x00, 0x72, 0x00, 0x74, 0x00, 0x75, 0x00, 0x61, 0x00, 0x6C, 0x00, 0x20, 0x00, 0x66, 0x00, 0x75, 0x00, 0x6E, 0x00, 0x63, 0x00, 0x74, 0x00, 0x69, 0x00, 0x6F, 0x00, 0x6E, 0x00, 0x20, 0x00, 0x63, 0x00, 0x61, 0x00, 0x6C, 0x00, 0x6C, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x52, 0x00, 0x36, 0x00, 0x30, 0x00, 0x32, 0x00, 0x34, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x2D, 0x00, 0x20, 0x00, 0x6E, 0x00, 0x6F, 0x00, 0x74, 0x00, 0x20, 0x00, 0x65, 0x00, 0x6E, 0x00, 0x6F, 0x00, 0x75, 0x00, 0x67, 0x00, 0x68, 0x00, 0x20, 0x00, 0x73, 0x00, 0x70, 0x00, 0x61, 0x00, 0x63, 0x00, 0x65, 0x00, 0x20, 0x00, 0x66, 0x00, 0x6F, 0x00, 0x72, 0x00, 0x20, 0x00, 0x5F, 0x00, 0x6F, 0x00, 0x6E, 0x00, 0x65, 0x00, 0x78, 0x00, 0x69, 0x00, 0x74, 0x00, 0x2F, 0x00, 0x61, 0x00, 0x74, 0x00, 0x65, 0x00, 0x78, 0x00, 0x69, 0x00, 0x74, 0x00, 0x20, 0x00, 0x74, 0x00, 0x61, 0x00, 0x62, 0x00, 0x6C, 0x00, 0x65, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x52, 0x00, 0x36, 0x00, 0x30, 0x00, 0x31, 0x00, 0x39, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x2D, 0x00, 0x20, 0x00, 0x75, 0x00, 0x6E, 0x00, 0x61, 0x00, 0x62, 0x00, 0x6C, 0x00, 0x65, 0x00, 0x20, 0x00, 0x74, 0x00, 0x6F, 0x00, 0x20, 0x00, 0x6F, 0x00, 0x70, 0x00, 0x65, 0x00, 0x6E, 0x00, 0x20, 0x00, 0x63, 0x00, 0x6F, 0x00, 0x6E, 0x00, 0x73, 0x00, 0x6F, 0x00, 0x6C, 0x00, 0x65, 0x00, 0x20, 0x00, 0x64, 0x00, 0x65, 0x00, 0x76, 0x00, 0x69, 0x00, 0x63, 0x00, 0x65, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x52, 0x00, 0x36, 0x00, 0x30, 0x00, 0x31, 0x00, 0x38, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x2D, 0x00, 0x20, 0x00, 0x75, 0x00, 0x6E, 0x00, 0x65, 0x00, 0x78, 0x00, 0x70, 0x00, 0x65, 0x00, 0x63, 0x00, 0x74, 0x00, 0x65, 0x00,
				0x64, 0x00, 0x20, 0x00, 0x68, 0x00, 0x65, 0x00, 0x61, 0x00, 0x70, 0x00, 0x20, 0x00, 0x65, 0x00, 0x72, 0x00, 0x72, 0x00, 0x6F, 0x00, 0x72, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x52, 0x00, 0x36, 0x00, 0x30, 0x00, 0x31, 0x00, 0x37, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x2D, 0x00, 0x20, 0x00, 0x75, 0x00, 0x6E, 0x00, 0x65, 0x00, 0x78, 0x00, 0x70, 0x00, 0x65, 0x00, 0x63, 0x00, 0x74, 0x00, 0x65, 0x00, 0x64, 0x00, 0x20, 0x00, 0x6D, 0x00, 0x75, 0x00, 0x6C, 0x00, 0x74, 0x00, 0x69, 0x00, 0x74, 0x00, 0x68, 0x00, 0x72, 0x00, 0x65, 0x00, 0x61, 0x00, 0x64, 0x00, 0x20, 0x00, 0x6C, 0x00, 0x6F, 0x00, 0x63, 0x00, 0x6B, 0x00, 0x20, 0x00, 0x65, 0x00, 0x72, 0x00, 0x72, 0x00, 0x6F, 0x00, 0x72, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x52, 0x00, 0x36, 0x00, 0x30, 0x00, 0x31, 0x00, 0x36, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x2D, 0x00, 0x20, 0x00, 0x6E, 0x00, 0x6F, 0x00, 0x74, 0x00, 0x20, 0x00, 0x65, 0x00, 0x6E, 0x00, 0x6F, 0x00, 0x75, 0x00, 0x67, 0x00, 0x68, 0x00, 0x20, 0x00, 0x73, 0x00, 0x70, 0x00, 0x61, 0x00, 0x63, 0x00, 0x65, 0x00, 0x20, 0x00, 0x66, 0x00, 0x6F, 0x00, 0x72, 0x00, 0x20, 0x00, 0x74, 0x00, 0x68, 0x00, 0x72, 0x00, 0x65, 0x00, 0x61, 0x00, 0x64, 0x00, 0x20, 0x00, 0x64, 0x00, 0x61, 0x00, 0x74, 0x00, 0x61, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x52, 0x00, 0x36, 0x00, 0x30, 0x00, 0x31, 0x00, 0x30, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x2D, 0x00, 0x20, 0x00, 0x61, 0x00, 0x62, 0x00, 0x6F, 0x00, 0x72, 0x00, 0x74, 0x00, 0x28, 0x00, 0x29, 0x00, 0x20, 0x00, 0x68, 0x00, 0x61, 0x00, 0x73, 0x00, 0x20, 0x00, 0x62, 0x00, 0x65, 0x00, 0x65, 0x00, 0x6E, 0x00, 0x20, 0x00, 0x63, 0x00, 0x61, 0x00, 0x6C, 0x00, 0x6C, 0x00, 0x65, 0x00, 0x64, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x52, 0x00, 0x36, 0x00, 0x30, 0x00, 0x30, 0x00, 0x39, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x2D, 0x00, 0x20, 0x00, 0x6E, 0x00, 0x6F, 0x00, 0x74, 0x00, 0x20, 0x00, 0x65, 0x00, 0x6E, 0x00, 0x6F, 0x00, 0x75, 0x00, 0x67, 0x00, 0x68, 0x00, 0x20, 0x00, 0x73, 0x00, 0x70, 0x00, 0x61, 0x00, 0x63, 0x00, 0x65, 0x00, 0x20, 0x00, 0x66, 0x00, 0x6F, 0x00, 0x72, 0x00, 0x20, 0x00, 0x65, 0x00, 0x6E, 0x00, 0x76, 0x00, 0x69, 0x00, 0x72, 0x00, 0x6F, 0x00, 0x6E, 0x00, 0x6D, 0x00, 0x65, 0x00, 0x6E, 0x00, 0x74, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x52, 0x00, 0x36, 0x00, 0x30, 0x00, 0x30, 0x00, 0x38, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x2D, 0x00, 0x20, 0x00, 0x6E, 0x00, 0x6F, 0x00, 0x74, 0x00, 0x20, 0x00, 0x65, 0x00, 0x6E, 0x00, 0x6F, 0x00, 0x75, 0x00, 0x67, 0x00, 0x68, 0x00, 0x20, 0x00, 0x73, 0x00, 0x70, 0x00, 0x61, 0x00, 0x63, 0x00, 0x65, 0x00, 0x20, 0x00, 0x66, 0x00, 0x6F, 0x00, 0x72, 0x00, 0x20, 0x00, 0x61, 0x00, 0x72, 0x00, 0x67, 0x00, 0x75, 0x00, 0x6D, 0x00, 0x65, 0x00, 0x6E, 0x00, 0x74, 0x00, 0x73, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x52, 0x00, 0x36, 0x00, 0x30, 0x00, 0x30, 0x00, 0x32, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x2D, 0x00, 0x20, 0x00, 0x66, 0x00, 0x6C, 0x00, 0x6F, 0x00, 0x61, 0x00, 0x74, 0x00, 0x69, 0x00, 0x6E, 0x00,
				0x67, 0x00, 0x20, 0x00, 0x70, 0x00, 0x6F, 0x00, 0x69, 0x00, 0x6E, 0x00, 0x74, 0x00, 0x20, 0x00, 0x73, 0x00, 0x75, 0x00, 0x70, 0x00, 0x70, 0x00, 0x6F, 0x00, 0x72, 0x00, 0x74, 0x00, 0x20, 0x00, 0x6E, 0x00, 0x6F, 0x00, 0x74, 0x00, 0x20, 0x00, 0x6C, 0x00, 0x6F, 0x00, 0x61, 0x00, 0x64, 0x00, 0x65, 0x00, 0x64, 0x00, 0x0D, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x50, 0xAE, 0x40, 0x00, 0x08, 0x00, 0x00, 0x00, 0xF8, 0xAD, 0x40, 0x00, 0x09, 0x00, 0x00, 0x00, 0xA0, 0xAD, 0x40, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x58, 0xAD, 0x40, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0xAD, 0x40, 0x00, 0x11, 0x00, 0x00, 0x00, 0xA0, 0xAC, 0x40, 0x00, 0x12, 0x00, 0x00, 0x00, 0x58, 0xAC, 0x40, 0x00, 0x13, 0x00, 0x00, 0x00, 0x00, 0xAC, 0x40, 0x00, 0x18, 0x00, 0x00, 0x00, 0x90, 0xAB, 0x40, 0x00, 0x19, 0x00, 0x00, 0x00, 0x40, 0xAB, 0x40, 0x00, 0x1A, 0x00, 0x00, 0x00, 0xD0, 0xAA, 0x40, 0x00, 0x1B, 0x00, 0x00, 0x00, 0x60, 0xAA, 0x40, 0x00, 0x1C, 0x00, 0x00, 0x00, 0x10, 0xAA, 0x40, 0x00, 0x1E, 0x00, 0x00, 0x00, 0xD0, 0xA9, 0x40, 0x00, 0x1F, 0x00, 0x00, 0x00, 0x08, 0xA9, 0x40, 0x00, 0x20, 0x00, 0x00, 0x00, 0xA0, 0xA8, 0x40, 0x00, 0x21, 0x00, 0x00, 0x00, 0xB0, 0xA6, 0x40, 0x00, 0x78, 0x00, 0x00, 0x00, 0x8C, 0xA6, 0x40, 0x00, 0x79, 0x00, 0x00, 0x00, 0x70, 0xA6, 0x40, 0x00, 0x7A, 0x00, 0x00, 0x00, 0x54, 0xA6, 0x40, 0x00, 0xFC, 0x00, 0x00, 0x00, 0x4C, 0xA6, 0x40, 0x00, 0xFF, 0x00, 0x00, 0x00, 0x2C, 0xA6, 0x40, 0x00, 0x4D, 0x00, 0x69, 0x00, 0x63, 0x00, 0x72, 0x00, 0x6F, 0x00, 0x73, 0x00, 0x6F, 0x00, 0x66, 0x00, 0x74, 0x00, 0x20, 0x00, 0x56, 0x00, 0x69, 0x00, 0x73, 0x00, 0x75, 0x00, 0x61, 0x00, 0x6C, 0x00, 0x20, 0x00, 0x43, 0x00, 0x2B, 0x00, 0x2B, 0x00, 0x20, 0x00, 0x52, 0x00, 0x75, 0x00, 0x6E, 0x00, 0x74, 0x00, 0x69, 0x00, 0x6D, 0x00, 0x65, 0x00, 0x20, 0x00, 0x4C, 0x00, 0x69, 0x00, 0x62, 0x00, 0x72, 0x00, 0x61, 0x00, 0x72, 0x00, 0x79, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0A, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x2E, 0x00, 0x2E, 0x00, 0x2E, 0x00, 0x00, 0x00, 0x3C, 0x00, 0x70, 0x00, 0x72, 0x00, 0x6F, 0x00, 0x67, 0x00, 0x72, 0x00, 0x61, 0x00, 0x6D, 0x00, 0x20, 0x00, 0x6E, 0x00, 0x61, 0x00, 0x6D, 0x00, 0x65, 0x00, 0x20, 0x00, 0x75, 0x00, 0x6E, 0x00, 0x6B, 0x00, 0x6E, 0x00, 0x6F, 0x00, 0x77, 0x00, 0x6E, 0x00, 0x3E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x52, 0x00, 0x75, 0x00, 0x6E, 0x00, 0x74, 0x00, 0x69, 0x00, 0x6D, 0x00, 0x65, 0x00, 0x20, 0x00, 0x45, 0x00, 0x72, 0x00, 0x72, 0x00, 0x6F, 0x00, 0x72, 0x00, 0x21, 0x00, 0x0A, 0x00, 0x0A, 0x00, 0x50, 0x00, 0x72, 0x00, 0x6F, 0x00, 0x67, 0x00, 0x72, 0x00, 0x61, 0x00, 0x6D, 0x00, 0x3A, 0x00, 0x20, 0x00, 0x00, 0x00, 0x05, 0x00, 0x00, 0xC0, 0x0B, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1D, 0x00, 0x00, 0xC0, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x96, 0x00, 0x00, 0xC0, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x8D, 0x00, 0x00, 0xC0, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x8E, 0x00, 0x00, 0xC0, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x8F, 0x00, 0x00, 0xC0, 0x08, 0x00, 0x00, 0x00,
				0x00, 0x00, 0x00, 0x00, 0x90, 0x00, 0x00, 0xC0, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x91, 0x00, 0x00, 0xC0, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x92, 0x00, 0x00, 0xC0, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x93, 0x00, 0x00, 0xC0, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xB4, 0x02, 0x00, 0xC0, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xB5, 0x02, 0x00, 0xC0, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x00, 0x00, 0x00, 0x09, 0x00, 0x00, 0x00, 0x90, 0x00, 0x00, 0x00, 0x0C, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x28, 0x00, 0x28, 0x00, 0x28, 0x00, 0x28, 0x00, 0x28, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x48, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x84, 0x00, 0x84, 0x00, 0x84, 0x00, 0x84, 0x00, 0x84, 0x00, 0x84, 0x00, 0x84, 0x00, 0x84, 0x00, 0x84, 0x00, 0x84, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x81, 0x00, 0x81, 0x00, 0x81, 0x00, 0x81, 0x00, 0x81, 0x00, 0x81, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00,
				0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x82, 0x00, 0x82, 0x00, 0x82, 0x00, 0x82, 0x00, 0x82, 0x00, 0x82, 0x00, 0x02, 0x00, 0x02, 0x00, 0x02, 0x00, 0x02, 0x00, 0x02, 0x00, 0x02, 0x00, 0x02, 0x00, 0x02, 0x00, 0x02, 0x00, 0x02, 0x00, 0x02, 0x00, 0x02, 0x00, 0x02, 0x00, 0x02, 0x00, 0x02, 0x00, 0x02, 0x00, 0x02, 0x00, 0x02, 0x00, 0x02, 0x00, 0x02, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x68, 0x00, 0x28, 0x00, 0x28, 0x00, 0x28, 0x00, 0x28, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x48, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x84, 0x00, 0x84, 0x00, 0x84, 0x00, 0x84, 0x00, 0x84, 0x00, 0x84, 0x00, 0x84, 0x00, 0x84, 0x00, 0x84, 0x00, 0x84, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x81, 0x01, 0x81, 0x01, 0x81, 0x01, 0x81, 0x01,
				0x81, 0x01, 0x81, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x82, 0x01, 0x82, 0x01, 0x82, 0x01, 0x82, 0x01, 0x82, 0x01, 0x82, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x20, 0x00, 0x48, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x14, 0x00, 0x14, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x14, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x10, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x10, 0x00, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x80, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 0x89, 0x8A, 0x8B, 0x8C, 0x8D, 0x8E, 0x8F, 0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98, 0x99, 0x9A, 0x9B, 0x9C, 0x9D, 0x9E, 0x9F, 0xA0, 0xA1, 0xA2, 0xA3, 0xA4, 0xA5, 0xA6, 0xA7, 0xA8, 0xA9, 0xAA, 0xAB, 0xAC, 0xAD, 0xAE, 0xAF, 0xB0, 0xB1, 0xB2, 0xB3, 0xB4, 0xB5, 0xB6, 0xB7, 0xB8, 0xB9, 0xBA, 0xBB, 0xBC, 0xBD, 0xBE, 0xBF, 0xC0, 0xC1, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xCB, 0xCC, 0xCD, 0xCE, 0xCF, 0xD0, 0xD1, 0xD2, 0xD3, 0xD4, 0xD5, 0xD6, 0xD7, 0xD8, 0xD9, 0xDA, 0xDB, 0xDC, 0xDD, 0xDE, 0xDF, 0xE0, 0xE1, 0xE2, 0xE3, 0xE4, 0xE5, 0xE6, 0xE7, 0xE8, 0xE9, 0xEA, 0xEB, 0xEC, 0xED, 0xEE, 0xEF, 0xF0, 0xF1, 0xF2, 0xF3, 0xF4, 0xF5, 0xF6, 0xF7,
				0xF8, 0xF9, 0xFA, 0xFB, 0xFC, 0xFD, 0xFE, 0xFF, 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1A, 0x1B, 0x1C, 0x1D, 0x1E, 0x1F, 0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2A, 0x2B, 0x2C, 0x2D, 0x2E, 0x2F, 0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x3E, 0x3F, 0x40, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6A, 0x6B, 0x6C, 0x6D, 0x6E, 0x6F, 0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7A, 0x5B, 0x5C, 0x5D, 0x5E, 0x5F, 0x60, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6A, 0x6B, 0x6C, 0x6D, 0x6E, 0x6F, 0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7A, 0x7B, 0x7C, 0x7D, 0x7E, 0x7F, 0x80, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 0x89, 0x8A, 0x8B, 0x8C, 0x8D, 0x8E, 0x8F, 0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98, 0x99, 0x9A, 0x9B, 0x9C, 0x9D, 0x9E, 0x9F, 0xA0, 0xA1, 0xA2, 0xA3, 0xA4, 0xA5, 0xA6, 0xA7, 0xA8, 0xA9, 0xAA, 0xAB, 0xAC, 0xAD, 0xAE, 0xAF, 0xB0, 0xB1, 0xB2, 0xB3, 0xB4, 0xB5, 0xB6, 0xB7, 0xB8, 0xB9, 0xBA, 0xBB, 0xBC, 0xBD, 0xBE, 0xBF, 0xC0, 0xC1, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xCB, 0xCC, 0xCD, 0xCE, 0xCF, 0xD0, 0xD1, 0xD2, 0xD3, 0xD4, 0xD5, 0xD6, 0xD7, 0xD8, 0xD9, 0xDA, 0xDB, 0xDC, 0xDD, 0xDE, 0xDF, 0xE0, 0xE1, 0xE2, 0xE3, 0xE4, 0xE5, 0xE6, 0xE7, 0xE8, 0xE9, 0xEA, 0xEB, 0xEC, 0xED, 0xEE, 0xEF, 0xF0, 0xF1, 0xF2, 0xF3, 0xF4, 0xF5, 0xF6, 0xF7, 0xF8, 0xF9, 0xFA, 0xFB, 0xFC, 0xFD, 0xFE, 0xFF, 0x80, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 0x89, 0x8A, 0x8B, 0x8C, 0x8D, 0x8E, 0x8F, 0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98, 0x99, 0x9A, 0x9B, 0x9C, 0x9D, 0x9E, 0x9F, 0xA0, 0xA1, 0xA2, 0xA3, 0xA4, 0xA5, 0xA6, 0xA7, 0xA8, 0xA9, 0xAA, 0xAB, 0xAC, 0xAD, 0xAE, 0xAF, 0xB0, 0xB1, 0xB2, 0xB3, 0xB4, 0xB5, 0xB6, 0xB7, 0xB8, 0xB9, 0xBA, 0xBB, 0xBC, 0xBD, 0xBE, 0xBF, 0xC0, 0xC1, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xCB, 0xCC, 0xCD, 0xCE, 0xCF, 0xD0, 0xD1, 0xD2, 0xD3, 0xD4, 0xD5, 0xD6, 0xD7, 0xD8, 0xD9, 0xDA, 0xDB, 0xDC, 0xDD, 0xDE, 0xDF, 0xE0, 0xE1, 0xE2, 0xE3, 0xE4, 0xE5, 0xE6, 0xE7, 0xE8, 0xE9, 0xEA, 0xEB, 0xEC, 0xED, 0xEE, 0xEF, 0xF0, 0xF1, 0xF2, 0xF3, 0xF4, 0xF5, 0xF6, 0xF7, 0xF8, 0xF9, 0xFA, 0xFB, 0xFC, 0xFD, 0xFE, 0xFF, 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1A, 0x1B, 0x1C, 0x1D, 0x1E, 0x1F, 0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2A, 0x2B, 0x2C, 0x2D, 0x2E, 0x2F, 0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x3E, 0x3F, 0x40, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4A, 0x4B, 0x4C, 0x4D, 0x4E, 0x4F, 0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5A, 0x5B, 0x5C, 0x5D, 0x5E, 0x5F, 0x60, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4A, 0x4B,
				0x4C, 0x4D, 0x4E, 0x4F, 0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5A, 0x7B, 0x7C, 0x7D, 0x7E, 0x7F, 0x80, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 0x89, 0x8A, 0x8B, 0x8C, 0x8D, 0x8E, 0x8F, 0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98, 0x99, 0x9A, 0x9B, 0x9C, 0x9D, 0x9E, 0x9F, 0xA0, 0xA1, 0xA2, 0xA3, 0xA4, 0xA5, 0xA6, 0xA7, 0xA8, 0xA9, 0xAA, 0xAB, 0xAC, 0xAD, 0xAE, 0xAF, 0xB0, 0xB1, 0xB2, 0xB3, 0xB4, 0xB5, 0xB6, 0xB7, 0xB8, 0xB9, 0xBA, 0xBB, 0xBC, 0xBD, 0xBE, 0xBF, 0xC0, 0xC1, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xCB, 0xCC, 0xCD, 0xCE, 0xCF, 0xD0, 0xD1, 0xD2, 0xD3, 0xD4, 0xD5, 0xD6, 0xD7, 0xD8, 0xD9, 0xDA, 0xDB, 0xDC, 0xDD, 0xDE, 0xDF, 0xE0, 0xE1, 0xE2, 0xE3, 0xE4, 0xE5, 0xE6, 0xE7, 0xE8, 0xE9, 0xEA, 0xEB, 0xEC, 0xED, 0xEE, 0xEF, 0xF0, 0xF1, 0xF2, 0xF3, 0xF4, 0xF5, 0xF6, 0xF7, 0xF8, 0xF9, 0xFA, 0xFB, 0xFC, 0xFD, 0xFE, 0xFF, 0x47, 0x65, 0x74, 0x50, 0x72, 0x6F, 0x63, 0x65, 0x73, 0x73, 0x57, 0x69, 0x6E, 0x64, 0x6F, 0x77, 0x53, 0x74, 0x61, 0x74, 0x69, 0x6F, 0x6E, 0x00, 0x47, 0x65, 0x74, 0x55, 0x73, 0x65, 0x72, 0x4F, 0x62, 0x6A, 0x65, 0x63, 0x74, 0x49, 0x6E, 0x66, 0x6F, 0x72, 0x6D, 0x61, 0x74, 0x69, 0x6F, 0x6E, 0x57, 0x00, 0x00, 0x00, 0x47, 0x65, 0x74, 0x4C, 0x61, 0x73, 0x74, 0x41, 0x63, 0x74, 0x69, 0x76, 0x65, 0x50, 0x6F, 0x70, 0x75, 0x70, 0x00, 0x00, 0x47, 0x65, 0x74, 0x41, 0x63, 0x74, 0x69, 0x76, 0x65, 0x57, 0x69, 0x6E, 0x64, 0x6F, 0x77, 0x00, 0x4D, 0x65, 0x73, 0x73, 0x61, 0x67, 0x65, 0x42, 0x6F, 0x78, 0x57, 0x00, 0x55, 0x00, 0x53, 0x00, 0x45, 0x00, 0x52, 0x00, 0x33, 0x00, 0x32, 0x00, 0x2E, 0x00, 0x44, 0x00, 0x4C, 0x00, 0x4C, 0x00, 0x00, 0x00, 0x00, 0x00, 0x43, 0x00, 0x4F, 0x00, 0x4E, 0x00, 0x4F, 0x00, 0x55, 0x00, 0x54, 0x00, 0x24, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x06, 0x80, 0x80, 0x86, 0x80, 0x81, 0x80, 0x00, 0x00, 0x10, 0x03, 0x86, 0x80, 0x86, 0x82, 0x80, 0x14, 0x05, 0x05, 0x45, 0x45, 0x45, 0x85, 0x85, 0x85, 0x05, 0x00, 0x00, 0x30, 0x30, 0x80, 0x50, 0x80, 0x88, 0x00, 0x08, 0x00, 0x28, 0x27, 0x38, 0x50, 0x57, 0x80, 0x00, 0x07, 0x00, 0x37, 0x30, 0x30, 0x50, 0x50, 0x88, 0x00, 0x00, 0x00, 0x20, 0x28, 0x80, 0x88, 0x80, 0x80, 0x00, 0x00, 0x00, 0x60, 0x68, 0x60, 0x68, 0x68, 0x68, 0x08, 0x08, 0x07, 0x78, 0x70, 0x70, 0x77, 0x70, 0x70, 0x08, 0x08, 0x00, 0x00, 0x08, 0x00, 0x08, 0x00, 0x07, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x43, 0x52, 0x55, 0x31, 0x32, 0x33, 0x34, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0E, 0x55, 0x34, 0x12, 0x00, 0x00, 0x00, 0x00, 0x71, 0x42, 0x12, 0x83, 0x54, 0x24, 0x95, 0x66, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x0E, 0x55, 0x34, 0x12, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x02, 0x3A, 0x80, 0x18, 0x71, 0x38, 0x2D, 0x40, 0x58, 0x2C, 0x45, 0x00, 0xE0, 0x0E, 0x11, 0x00, 0x00, 0x1E, 0x00, 0x00, 0x00, 0xFC, 0x00, 0x54, 0x6F, 0x61,
				0x73, 0x74, 0x79, 0x21, 0x0A, 0x20, 0x20, 0x20, 0x20, 0x20, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x0D, 0xAF, 0x23, 0x17, 0x00, 0x00, 0x00, 0x00, 0x02, 0x15, 0x01, 0x04, 0x95, 0x26, 0x15, 0x78, 0x02, 0xD1, 0xF5, 0x93, 0x5D, 0x59, 0x90, 0x26, 0x1D, 0x50, 0x54, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x1D, 0x36, 0x80, 0xA0, 0x70, 0x38, 0x1E, 0x40, 0x2E, 0x1E, 0x24, 0x00, 0x7E, 0xD7,
				0x10, 0x00, 0x00, 0x18, 0x00, 0x00, 0x00, 0x05, 0x00, 0x74, 0x8B, 0x80, 0x50, 0x70, 0x38, 0x97, 0x41, 0x08, 0x40, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFE, 0x00, 0x43, 0x4D, 0x49, 0x0A, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x00, 0x00, 0x00, 0xFE, 0x00, 0x4E, 0x31, 0x37, 0x33, 0x48, 0x48, 0x46, 0x2D, 0x45, 0x32, 0x31, 0x20, 0x20, 0x00, 0x39, 0x53, 0x59, 0x53, 0x54, 0x45, 0x4D, 0x5C, 0x43, 0x75, 0x72, 0x72, 0x65, 0x6E, 0x74, 0x43, 0x6F, 0x6E, 0x74, 0x72, 0x6F, 0x6C, 0x53, 0x65, 0x74, 0x5C, 0x45, 0x6E, 0x75, 0x6D, 0x5C, 0x44, 0x49, 0x53, 0x50, 0x4C, 0x41, 0x59, 0x00, 0x00, 0x00, 0x25, 0x73, 0x5C, 0x25, 0x68, 0x73, 0x5C, 0x25, 0x73, 0x5C, 0x44, 0x65, 0x76, 0x69, 0x63, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x6D, 0x65, 0x74, 0x65, 0x72, 0x73, 0x5C, 0x45, 0x44, 0x49, 0x44, 0x5F, 0x4F, 0x56, 0x45, 0x52, 0x52, 0x49, 0x44, 0x45, 0x00, 0x00, 0x00, 0x25, 0x64, 0x00, 0x00, 0x25, 0x73, 0x5C, 0x25, 0x68, 0x73, 0x5C, 0x25, 0x73, 0x5C, 0x44, 0x65, 0x76, 0x69, 0x63, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x6D, 0x65, 0x74, 0x65, 0x72, 0x73, 0x00, 0x45, 0x44, 0x49, 0x44, 0x5F, 0x4F, 0x56, 0x45, 0x52, 0x52, 0x49, 0x44, 0x45, 0x00, 0x00, 0x00, 0x45, 0x44, 0x49, 0x44, 0x00, 0x00, 0x00, 0x00, 0x45, 0x44, 0x49, 0x44, 0x5F, 0x52, 0x45, 0x43, 0x4F, 0x56, 0x45, 0x52, 0x59, 0x00, 0x00, 0x00, 0x25, 0x73, 0x5C, 0x25, 0x68, 0x73, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x53, 0x59, 0x53, 0x54, 0x45, 0x4D, 0x5C, 0x43, 0x75, 0x72, 0x72, 0x65, 0x6E, 0x74, 0x43, 0x6F, 0x6E, 0x74, 0x72, 0x6F, 0x6C, 0x53, 0x65, 0x74, 0x5C, 0x43, 0x6F, 0x6E, 0x74, 0x72, 0x6F, 0x6C, 0x5C, 0x43, 0x6C, 0x61, 0x73, 0x73, 0x5C, 0x7B, 0x34, 0x64, 0x33, 0x36, 0x65, 0x39, 0x36, 0x38, 0x2D, 0x65, 0x33, 0x32, 0x35, 0x2D, 0x31, 0x31, 0x63, 0x65, 0x2D, 0x62, 0x66, 0x63, 0x31, 0x2D, 0x30, 0x38, 0x30, 0x30, 0x32, 0x62, 0x65, 0x31, 0x30, 0x33, 0x31, 0x38, 0x7D, 0x00, 0x00, 0x00, 0x25, 0x73, 0x5C, 0x25, 0x73, 0x00, 0x00, 0x00, 0x50, 0x72, 0x6F, 0x76, 0x69, 0x64, 0x65, 0x72, 0x4E, 0x61, 0x6D, 0x65, 0x00, 0x00, 0x00, 0x00, 0x49, 0x6E, 0x74, 0x65, 0x6C, 0x00, 0x00, 0x00, 0x46, 0x61, 0x6B, 0x65, 0x45, 0x44, 0x49, 0x44, 0x5F, 0x25, 0x64, 0x5F, 0x25, 0x64, 0x5F, 0x25, 0x78, 0x5F, 0x25, 0x78, 0x00, 0x00, 0x00, 0x00, 0x52, 0x65, 0x61, 0x64, 0x45, 0x44, 0x49, 0x44, 0x46, 0x72, 0x6F, 0x6D, 0x52, 0x65, 0x67, 0x69, 0x73, 0x74, 0x72, 0x79, 0x00, 0x00, 0x00, 0x00, 0x46, 0x61, 0x6B, 0x65, 0x45, 0x44, 0x49, 0x44, 0x5F, 0x31, 0x34, 0x5F, 0x30, 0x5F, 0x61, 0x66, 0x30, 0x64, 0x5F, 0x31, 0x37, 0x32, 0x33, 0x00, 0x44, 0x65, 0x66, 0x61, 0x75, 0x6C, 0x74, 0x5F, 0x4D, 0x6F, 0x6E, 0x69, 0x74, 0x6F, 0x72, 0x00, 0x45, 0x44, 0x49, 0x44, 0x20, 0x49, 0x6E, 0x73, 0x74, 0x61, 0x6C, 0x6C, 0x65, 0x72, 0x00, 0x00, 0x4E, 0x6F, 0x20, 0x6D, 0x61, 0x74, 0x63, 0x68, 0x69, 0x6E, 0x67, 0x20, 0x64, 0x69, 0x73, 0x70, 0x6C, 0x61, 0x79, 0x73, 0x20, 0x66, 0x6F, 0x75, 0x6E, 0x64, 0x2E, 0x00, 0x46, 0x61, 0x69, 0x6C, 0x65, 0x64, 0x20, 0x74, 0x6F, 0x20, 0x6F, 0x70, 0x65, 0x6E, 0x20, 0x74, 0x68, 0x65, 0x20, 0x72, 0x65, 0x67, 0x69, 0x73, 0x74, 0x72, 0x79, 0x2E,
				0x0A, 0x54, 0x72, 0x79, 0x20, 0x72, 0x75, 0x6E, 0x6E, 0x69, 0x6E, 0x67, 0x20, 0x61, 0x73, 0x20, 0x61, 0x64, 0x6D, 0x69, 0x6E, 0x69, 0x73, 0x74, 0x72, 0x61, 0x74, 0x6F, 0x72, 0x2E, 0x00, 0x00, 0x55, 0x6E, 0x6B, 0x6E, 0x6F, 0x77, 0x6E, 0x20, 0x65, 0x72, 0x72, 0x6F, 0x72, 0x2E, 0x00, 0x00, 0x49, 0x6E, 0x73, 0x74, 0x61, 0x6C, 0x6C, 0x20, 0x45, 0x44, 0x49, 0x44, 0x00, 0x00, 0x00, 0x00, 0x45, 0x44, 0x49, 0x44, 0x20, 0x69, 0x6E, 0x73, 0x74, 0x61, 0x6C, 0x6C, 0x65, 0x64, 0x20, 0x73, 0x75, 0x63, 0x63, 0x65, 0x73, 0x73, 0x66, 0x75, 0x6C, 0x6C, 0x79, 0x2E, 0x0A, 0x52, 0x75, 0x6E, 0x20, 0x72, 0x65, 0x73, 0x74, 0x61, 0x72, 0x74, 0x2E, 0x65, 0x78, 0x65, 0x20, 0x6F, 0x72, 0x20, 0x72, 0x65, 0x62, 0x6F, 0x6F, 0x74, 0x20, 0x74, 0x6F, 0x20, 0x72, 0x65, 0x64, 0x65, 0x74, 0x65, 0x63, 0x74, 0x20, 0x64, 0x69, 0x73, 0x70, 0x6C, 0x61, 0x79, 0x73, 0x2E, 0x00, 0x00, 0x00, 0x00, 0x46, 0x61, 0x69, 0x6C, 0x65, 0x64, 0x20, 0x74, 0x6F, 0x20, 0x69, 0x6E, 0x73, 0x74, 0x61, 0x6C, 0x6C, 0x20, 0x45, 0x44, 0x49, 0x44, 0x20, 0x6F, 0x6E, 0x20, 0x73, 0x6F, 0x6D, 0x65, 0x20, 0x64, 0x69, 0x73, 0x70, 0x6C, 0x61, 0x79, 0x73, 0x2E, 0x0A, 0x52, 0x75, 0x6E, 0x20, 0x72, 0x65, 0x73, 0x74, 0x61, 0x72, 0x74, 0x2E, 0x65, 0x78, 0x65, 0x20, 0x6F, 0x72, 0x20, 0x72, 0x65, 0x62, 0x6F, 0x6F, 0x74, 0x20, 0x74, 0x6F, 0x20, 0x72, 0x65, 0x64, 0x65, 0x74, 0x65, 0x63, 0x74, 0x20, 0x64, 0x69, 0x73, 0x70, 0x6C, 0x61, 0x79, 0x73, 0x2E, 0x00, 0x00, 0x00, 0x00, 0x46, 0x61, 0x69, 0x6C, 0x65, 0x64, 0x20, 0x74, 0x6F, 0x20, 0x69, 0x6E, 0x73, 0x74, 0x61, 0x6C, 0x6C, 0x20, 0x45, 0x44, 0x49, 0x44, 0x2E, 0x0A, 0x54, 0x72, 0x79, 0x20, 0x72, 0x75, 0x6E, 0x6E, 0x69, 0x6E, 0x67, 0x20, 0x61, 0x73, 0x20, 0x61, 0x64, 0x6D, 0x69, 0x6E, 0x69, 0x73, 0x74, 0x72, 0x61, 0x74, 0x6F, 0x72, 0x2E, 0x00, 0x00, 0x00, 0x52, 0x65, 0x73, 0x65, 0x74, 0x20, 0x45, 0x44, 0x49, 0x44, 0x00, 0x00, 0x45, 0x44, 0x49, 0x44, 0x20, 0x72, 0x65, 0x73, 0x65, 0x74, 0x20, 0x73, 0x75, 0x63, 0x63, 0x65, 0x73, 0x73, 0x66, 0x75, 0x6C, 0x6C, 0x79, 0x2E, 0x0A, 0x52, 0x75, 0x6E, 0x20, 0x72, 0x65, 0x73, 0x74, 0x61, 0x72, 0x74, 0x2E, 0x65, 0x78, 0x65, 0x20, 0x6F, 0x72, 0x20, 0x72, 0x65, 0x62, 0x6F, 0x6F, 0x74, 0x20, 0x74, 0x6F, 0x20, 0x72, 0x65, 0x64, 0x65, 0x74, 0x65, 0x63, 0x74, 0x20, 0x64, 0x69, 0x73, 0x70, 0x6C, 0x61, 0x79, 0x73, 0x2E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x45, 0x44, 0x49, 0x44, 0x20, 0x68, 0x61, 0x73, 0x20, 0x61, 0x6C, 0x72, 0x65, 0x61, 0x64, 0x79, 0x20, 0x62, 0x65, 0x65, 0x6E, 0x20, 0x72, 0x65, 0x73, 0x65, 0x74, 0x2E, 0x0A, 0x52, 0x75, 0x6E, 0x20, 0x72, 0x65, 0x73, 0x74, 0x61, 0x72, 0x74, 0x2E, 0x65, 0x78, 0x65, 0x20, 0x6F, 0x72, 0x20, 0x72, 0x65, 0x62, 0x6F, 0x6F, 0x74, 0x20, 0x74, 0x6F, 0x20, 0x72, 0x65, 0x64, 0x65, 0x74, 0x65, 0x63, 0x74, 0x20, 0x64, 0x69, 0x73, 0x70, 0x6C, 0x61, 0x79, 0x73, 0x2E, 0x00, 0x00, 0x00, 0x00, 0x46, 0x61, 0x69, 0x6C, 0x65, 0x64, 0x20, 0x74, 0x6F, 0x20, 0x72, 0x65, 0x73, 0x65, 0x74, 0x20, 0x45, 0x44, 0x49, 0x44, 0x20, 0x6F, 0x6E, 0x20, 0x73, 0x6F, 0x6D, 0x65, 0x20, 0x64, 0x69, 0x73, 0x70, 0x6C, 0x61, 0x79,
				0x73, 0x2E, 0x0A, 0x52, 0x75, 0x6E, 0x20, 0x72, 0x65, 0x73, 0x74, 0x61, 0x72, 0x74, 0x2E, 0x65, 0x78, 0x65, 0x20, 0x6F, 0x72, 0x20, 0x72, 0x65, 0x62, 0x6F, 0x6F, 0x74, 0x20, 0x74, 0x6F, 0x20, 0x72, 0x65, 0x64, 0x65, 0x74, 0x65, 0x63, 0x74, 0x20, 0x64, 0x69, 0x73, 0x70, 0x6C, 0x61, 0x79, 0x73, 0x2E, 0x00, 0x00, 0x46, 0x61, 0x69, 0x6C, 0x65, 0x64, 0x20, 0x74, 0x6F, 0x20, 0x72, 0x65, 0x73, 0x65, 0x74, 0x20, 0x45, 0x44, 0x49, 0x44, 0x2E, 0x0A, 0x54, 0x72, 0x79, 0x20, 0x72, 0x75, 0x6E, 0x6E, 0x69, 0x6E, 0x67, 0x20, 0x61, 0x73, 0x20, 0x61, 0x64, 0x6D, 0x69, 0x6E, 0x69, 0x73, 0x74, 0x72, 0x61, 0x74, 0x6F, 0x72, 0x2E, 0x00, 0x49, 0x44, 0x44, 0x5F, 0x44, 0x49, 0x41, 0x4C, 0x4F, 0x47, 0x00, 0x00, 0x48, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0xD0, 0x40, 0x00, 0xC0, 0xC0, 0x40, 0x00, 0x03, 0x00, 0x00, 0x00, 0x30, 0x56, 0x00, 0x00, 0xC0, 0x81, 0x00, 0x00, 0xD0, 0x8B, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x88, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x9C, 0x26, 0x40, 0x00, 0xB0, 0x26, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xD4, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xA5, 0x2A, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xCC, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x77, 0x2E, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xD4, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFB, 0x31, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xD8, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x26, 0x33, 0x40, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x35, 0x33, 0x40, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xD8, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xE8, 0x34, 0x40, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xF4, 0x34, 0x40, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xC0, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xAE, 0x4A, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xD4, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xCF, 0x5D, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00,
				0xCC, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xB5, 0x6B, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xD0, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x85, 0x73, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xD8, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x75, 0x78, 0x40, 0x00, 0x79, 0x78, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xC0, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x62, 0x7A, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xD4, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xDF, 0x7B, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xD8, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x1B, 0x7D, 0x40, 0x00, 0x2E, 0x7D, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xD4, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x2E, 0x87, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xD0, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x9E, 0x89, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xCC, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x28, 0x8B, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xF4, 0x8A, 0x40, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xD4, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xF3, 0x8D, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xD0, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xCC, 0x8E, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xD0, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x69, 0x90, 0x40, 0x00, 0xE8, 0xC4, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x24, 0xC5, 0x00, 0x00, 0x24, 0xA1, 0x00, 0x00, 0xC4, 0xC3, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xBA, 0xC5, 0x00, 0x00, 0x00, 0xA0, 0x00, 0x00, 0xE8, 0xC3, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x50, 0xCA, 0x00, 0x00, 0x24, 0xA0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x68, 0xC5, 0x00, 0x00, 0xAC, 0xC5, 0x00, 0x00, 0x30, 0xC5, 0x00, 0x00, 0x42, 0xC5, 0x00, 0x00, 0x54, 0xC5, 0x00, 0x00, 0x78, 0xC5, 0x00, 0x00, 0x8A, 0xC5, 0x00, 0x00, 0x9A, 0xC5, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x2E, 0xCA, 0x00, 0x00,
				0x3C, 0xCA, 0x00, 0x00, 0xB0, 0xC7, 0x00, 0x00, 0x20, 0xCA, 0x00, 0x00, 0xC8, 0xC5, 0x00, 0x00, 0xD8, 0xC5, 0x00, 0x00, 0xE4, 0xC5, 0x00, 0x00, 0xF0, 0xC5, 0x00, 0x00, 0x02, 0xC6, 0x00, 0x00, 0x18, 0xC6, 0x00, 0x00, 0x2A, 0xC6, 0x00, 0x00, 0x3E, 0xC6, 0x00, 0x00, 0x52, 0xC6, 0x00, 0x00, 0x6E, 0xC6, 0x00, 0x00, 0x8C, 0xC6, 0x00, 0x00, 0xA0, 0xC6, 0x00, 0x00, 0xAC, 0xC6, 0x00, 0x00, 0xC4, 0xC6, 0x00, 0x00, 0xDC, 0xC6, 0x00, 0x00, 0xE6, 0xC6, 0x00, 0x00, 0xF2, 0xC6, 0x00, 0x00, 0x04, 0xC7, 0x00, 0x00, 0x14, 0xC7, 0x00, 0x00, 0x20, 0xC7, 0x00, 0x00, 0x2E, 0xC7, 0x00, 0x00, 0x3C, 0xC7, 0x00, 0x00, 0x4C, 0xC7, 0x00, 0x00, 0x56, 0xC7, 0x00, 0x00, 0x6A, 0xC7, 0x00, 0x00, 0x7A, 0xC7, 0x00, 0x00, 0x90, 0xC7, 0x00, 0x00, 0xA2, 0xC7, 0x00, 0x00, 0xBE, 0xC7, 0x00, 0x00, 0xCA, 0xC7, 0x00, 0x00, 0xDA, 0xC7, 0x00, 0x00, 0xF0, 0xC7, 0x00, 0x00, 0x06, 0xC8, 0x00, 0x00, 0x20, 0xC8, 0x00, 0x00, 0x36, 0xC8, 0x00, 0x00, 0x50, 0xC8, 0x00, 0x00, 0x62, 0xC8, 0x00, 0x00, 0x8A, 0xC8, 0x00, 0x00, 0x98, 0xC8, 0x00, 0x00, 0xB0, 0xC8, 0x00, 0x00, 0xCA, 0xC8, 0x00, 0x00, 0xDA, 0xC8, 0x00, 0x00, 0xF0, 0xC8, 0x00, 0x00, 0x0A, 0xC9, 0x00, 0x00, 0x1A, 0xC9, 0x00, 0x00, 0x30, 0xC9, 0x00, 0x00, 0x42, 0xC9, 0x00, 0x00, 0x5A, 0xC9, 0x00, 0x00, 0x72, 0xC9, 0x00, 0x00, 0x7A, 0xC9, 0x00, 0x00, 0x8C, 0xC9, 0x00, 0x00, 0x9C, 0xC9, 0x00, 0x00, 0xAE, 0xC9, 0x00, 0x00, 0xBE, 0xC9, 0x00, 0x00, 0xCA, 0xC9, 0x00, 0x00, 0xE6, 0xC9, 0x00, 0x00, 0xF4, 0xC9, 0x00, 0x00, 0x04, 0xCA, 0x00, 0x00, 0x14, 0xCA, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x16, 0xC5, 0x00, 0x00, 0x0A, 0xC5, 0x00, 0x00, 0xF8, 0xC4, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xAB, 0x00, 0x44, 0x69, 0x61, 0x6C, 0x6F, 0x67, 0x42, 0x6F, 0x78, 0x50, 0x61, 0x72, 0x61, 0x6D, 0x41, 0x00, 0xDA, 0x00, 0x45, 0x6E, 0x64, 0x44, 0x69, 0x61, 0x6C, 0x6F, 0x67, 0x00, 0x0E, 0x02, 0x4D, 0x65, 0x73, 0x73, 0x61, 0x67, 0x65, 0x42, 0x6F, 0x78, 0x41, 0x00, 0x55, 0x53, 0x45, 0x52, 0x33, 0x32, 0x2E, 0x64, 0x6C, 0x6C, 0x00, 0x00, 0x45, 0x02, 0x52, 0x65, 0x67, 0x44, 0x65, 0x6C, 0x65, 0x74, 0x65, 0x54, 0x72, 0x65, 0x65, 0x41, 0x00, 0x00, 0x7D, 0x02, 0x52, 0x65, 0x67, 0x53, 0x65, 0x74, 0x56, 0x61, 0x6C, 0x75, 0x65, 0x45, 0x78, 0x41, 0x00, 0x00, 0x6D, 0x02, 0x52, 0x65, 0x67, 0x51, 0x75, 0x65, 0x72, 0x79, 0x56, 0x61, 0x6C, 0x75, 0x65, 0x45, 0x78, 0x41, 0x00, 0x00, 0x4E, 0x02, 0x52, 0x65, 0x67, 0x45, 0x6E, 0x75, 0x6D, 0x4B, 0x65, 0x79, 0x45, 0x78, 0x41, 0x00, 0x38, 0x02, 0x52, 0x65, 0x67, 0x43, 0x72, 0x65, 0x61, 0x74, 0x65, 0x4B, 0x65, 0x79, 0x45, 0x78, 0x41, 0x00, 0x60, 0x02, 0x52, 0x65, 0x67, 0x4F, 0x70, 0x65, 0x6E, 0x4B, 0x65, 0x79, 0x45, 0x78, 0x41, 0x00, 0x47, 0x02, 0x52, 0x65, 0x67, 0x44, 0x65, 0x6C, 0x65, 0x74, 0x65, 0x56, 0x61, 0x6C, 0x75, 0x65, 0x41, 0x00, 0x30, 0x02, 0x52, 0x65, 0x67, 0x43, 0x6C, 0x6F, 0x73, 0x65, 0x4B, 0x65, 0x79, 0x00, 0x41, 0x44, 0x56, 0x41, 0x50, 0x49, 0x33, 0x32, 0x2E, 0x64, 0x6C, 0x6C, 0x00, 0x00, 0x02, 0x02, 0x47, 0x65, 0x74, 0x4C, 0x61, 0x73, 0x74, 0x45, 0x72, 0x72, 0x6F, 0x72, 0x00, 0x00, 0xCF, 0x02, 0x48, 0x65, 0x61, 0x70, 0x46, 0x72,
				0x65, 0x65, 0x00, 0x00, 0xCB, 0x02, 0x48, 0x65, 0x61, 0x70, 0x41, 0x6C, 0x6C, 0x6F, 0x63, 0x00, 0x86, 0x01, 0x47, 0x65, 0x74, 0x43, 0x6F, 0x6D, 0x6D, 0x61, 0x6E, 0x64, 0x4C, 0x69, 0x6E, 0x65, 0x41, 0x00, 0xD3, 0x02, 0x48, 0x65, 0x61, 0x70, 0x53, 0x65, 0x74, 0x49, 0x6E, 0x66, 0x6F, 0x72, 0x6D, 0x61, 0x74, 0x69, 0x6F, 0x6E, 0x00, 0x00, 0x63, 0x02, 0x47, 0x65, 0x74, 0x53, 0x74, 0x61, 0x72, 0x74, 0x75, 0x70, 0x49, 0x6E, 0x66, 0x6F, 0x57, 0x00, 0xC0, 0x04, 0x54, 0x65, 0x72, 0x6D, 0x69, 0x6E, 0x61, 0x74, 0x65, 0x50, 0x72, 0x6F, 0x63, 0x65, 0x73, 0x73, 0x00, 0x00, 0xC0, 0x01, 0x47, 0x65, 0x74, 0x43, 0x75, 0x72, 0x72, 0x65, 0x6E, 0x74, 0x50, 0x72, 0x6F, 0x63, 0x65, 0x73, 0x73, 0x00, 0xD3, 0x04, 0x55, 0x6E, 0x68, 0x61, 0x6E, 0x64, 0x6C, 0x65, 0x64, 0x45, 0x78, 0x63, 0x65, 0x70, 0x74, 0x69, 0x6F, 0x6E, 0x46, 0x69, 0x6C, 0x74, 0x65, 0x72, 0x00, 0x00, 0xA5, 0x04, 0x53, 0x65, 0x74, 0x55, 0x6E, 0x68, 0x61, 0x6E, 0x64, 0x6C, 0x65, 0x64, 0x45, 0x78, 0x63, 0x65, 0x70, 0x74, 0x69, 0x6F, 0x6E, 0x46, 0x69, 0x6C, 0x74, 0x65, 0x72, 0x00, 0x00, 0x03, 0x49, 0x73, 0x44, 0x65, 0x62, 0x75, 0x67, 0x67, 0x65, 0x72, 0x50, 0x72, 0x65, 0x73, 0x65, 0x6E, 0x74, 0x00, 0x72, 0x01, 0x47, 0x65, 0x74, 0x43, 0x50, 0x49, 0x6E, 0x66, 0x6F, 0x00, 0xEF, 0x02, 0x49, 0x6E, 0x74, 0x65, 0x72, 0x6C, 0x6F, 0x63, 0x6B, 0x65, 0x64, 0x49, 0x6E, 0x63, 0x72, 0x65, 0x6D, 0x65, 0x6E, 0x74, 0x00, 0x00, 0xEB, 0x02, 0x49, 0x6E, 0x74, 0x65, 0x72, 0x6C, 0x6F, 0x63, 0x6B, 0x65, 0x64, 0x44, 0x65, 0x63, 0x72, 0x65, 0x6D, 0x65, 0x6E, 0x74, 0x00, 0x00, 0x68, 0x01, 0x47, 0x65, 0x74, 0x41, 0x43, 0x50, 0x00, 0x00, 0x37, 0x02, 0x47, 0x65, 0x74, 0x4F, 0x45, 0x4D, 0x43, 0x50, 0x00, 0x00, 0x0A, 0x03, 0x49, 0x73, 0x56, 0x61, 0x6C, 0x69, 0x64, 0x43, 0x6F, 0x64, 0x65, 0x50, 0x61, 0x67, 0x65, 0x00, 0xEA, 0x00, 0x45, 0x6E, 0x63, 0x6F, 0x64, 0x65, 0x50, 0x6F, 0x69, 0x6E, 0x74, 0x65, 0x72, 0x00, 0xC5, 0x04, 0x54, 0x6C, 0x73, 0x41, 0x6C, 0x6C, 0x6F, 0x63, 0x00, 0x00, 0xC7, 0x04, 0x54, 0x6C, 0x73, 0x47, 0x65, 0x74, 0x56, 0x61, 0x6C, 0x75, 0x65, 0x00, 0xC8, 0x04, 0x54, 0x6C, 0x73, 0x53, 0x65, 0x74, 0x56, 0x61, 0x6C, 0x75, 0x65, 0x00, 0xCA, 0x00, 0x44, 0x65, 0x63, 0x6F, 0x64, 0x65, 0x50, 0x6F, 0x69, 0x6E, 0x74, 0x65, 0x72, 0x00, 0xC6, 0x04, 0x54, 0x6C, 0x73, 0x46, 0x72, 0x65, 0x65, 0x00, 0x18, 0x02, 0x47, 0x65, 0x74, 0x4D, 0x6F, 0x64, 0x75, 0x6C, 0x65, 0x48, 0x61, 0x6E, 0x64, 0x6C, 0x65, 0x57, 0x00, 0x00, 0x73, 0x04, 0x53, 0x65, 0x74, 0x4C, 0x61, 0x73, 0x74, 0x45, 0x72, 0x72, 0x6F, 0x72, 0x00, 0x00, 0xC5, 0x01, 0x47, 0x65, 0x74, 0x43, 0x75, 0x72, 0x72, 0x65, 0x6E, 0x74, 0x54, 0x68, 0x72, 0x65, 0x61, 0x64, 0x49, 0x64, 0x00, 0x00, 0x45, 0x02, 0x47, 0x65, 0x74, 0x50, 0x72, 0x6F, 0x63, 0x41, 0x64, 0x64, 0x72, 0x65, 0x73, 0x73, 0x00, 0x00, 0xCD, 0x02, 0x48, 0x65, 0x61, 0x70, 0x43, 0x72, 0x65, 0x61, 0x74, 0x65, 0x00, 0x00, 0x19, 0x01, 0x45, 0x78, 0x69, 0x74, 0x50, 0x72, 0x6F, 0x63, 0x65, 0x73, 0x73, 0x00, 0x25, 0x05, 0x57, 0x72, 0x69, 0x74, 0x65, 0x46, 0x69, 0x6C, 0x65, 0x00, 0x64, 0x02, 0x47, 0x65, 0x74, 0x53, 0x74, 0x64, 0x48, 0x61,
				0x6E, 0x64, 0x6C, 0x65, 0x00, 0x00, 0x14, 0x02, 0x47, 0x65, 0x74, 0x4D, 0x6F, 0x64, 0x75, 0x6C, 0x65, 0x46, 0x69, 0x6C, 0x65, 0x4E, 0x61, 0x6D, 0x65, 0x57, 0x00, 0x00, 0x13, 0x02, 0x47, 0x65, 0x74, 0x4D, 0x6F, 0x64, 0x75, 0x6C, 0x65, 0x46, 0x69, 0x6C, 0x65, 0x4E, 0x61, 0x6D, 0x65, 0x41, 0x00, 0x00, 0x61, 0x01, 0x46, 0x72, 0x65, 0x65, 0x45, 0x6E, 0x76, 0x69, 0x72, 0x6F, 0x6E, 0x6D, 0x65, 0x6E, 0x74, 0x53, 0x74, 0x72, 0x69, 0x6E, 0x67, 0x73, 0x57, 0x00, 0x11, 0x05, 0x57, 0x69, 0x64, 0x65, 0x43, 0x68, 0x61, 0x72, 0x54, 0x6F, 0x4D, 0x75, 0x6C, 0x74, 0x69, 0x42, 0x79, 0x74, 0x65, 0x00, 0xDA, 0x01, 0x47, 0x65, 0x74, 0x45, 0x6E, 0x76, 0x69, 0x72, 0x6F, 0x6E, 0x6D, 0x65, 0x6E, 0x74, 0x53, 0x74, 0x72, 0x69, 0x6E, 0x67, 0x73, 0x57, 0x00, 0x00, 0x6F, 0x04, 0x53, 0x65, 0x74, 0x48, 0x61, 0x6E, 0x64, 0x6C, 0x65, 0x43, 0x6F, 0x75, 0x6E, 0x74, 0x00, 0x00, 0xE3, 0x02, 0x49, 0x6E, 0x69, 0x74, 0x69, 0x61, 0x6C, 0x69, 0x7A, 0x65, 0x43, 0x72, 0x69, 0x74, 0x69, 0x63, 0x61, 0x6C, 0x53, 0x65, 0x63, 0x74, 0x69, 0x6F, 0x6E, 0x41, 0x6E, 0x64, 0x53, 0x70, 0x69, 0x6E, 0x43, 0x6F, 0x75, 0x6E, 0x74, 0x00, 0xF3, 0x01, 0x47, 0x65, 0x74, 0x46, 0x69, 0x6C, 0x65, 0x54, 0x79, 0x70, 0x65, 0x00, 0xD1, 0x00, 0x44, 0x65, 0x6C, 0x65, 0x74, 0x65, 0x43, 0x72, 0x69, 0x74, 0x69, 0x63, 0x61, 0x6C, 0x53, 0x65, 0x63, 0x74, 0x69, 0x6F, 0x6E, 0x00, 0xA7, 0x03, 0x51, 0x75, 0x65, 0x72, 0x79, 0x50, 0x65, 0x72, 0x66, 0x6F, 0x72, 0x6D, 0x61, 0x6E, 0x63, 0x65, 0x43, 0x6F, 0x75, 0x6E, 0x74, 0x65, 0x72, 0x00, 0x93, 0x02, 0x47, 0x65, 0x74, 0x54, 0x69, 0x63, 0x6B, 0x43, 0x6F, 0x75, 0x6E, 0x74, 0x00, 0x00, 0xC1, 0x01, 0x47, 0x65, 0x74, 0x43, 0x75, 0x72, 0x72, 0x65, 0x6E, 0x74, 0x50, 0x72, 0x6F, 0x63, 0x65, 0x73, 0x73, 0x49, 0x64, 0x00, 0x79, 0x02, 0x47, 0x65, 0x74, 0x53, 0x79, 0x73, 0x74, 0x65, 0x6D, 0x54, 0x69, 0x6D, 0x65, 0x41, 0x73, 0x46, 0x69, 0x6C, 0x65, 0x54, 0x69, 0x6D, 0x65, 0x00, 0x2D, 0x03, 0x4C, 0x43, 0x4D, 0x61, 0x70, 0x53, 0x74, 0x72, 0x69, 0x6E, 0x67, 0x57, 0x00, 0x00, 0x67, 0x03, 0x4D, 0x75, 0x6C, 0x74, 0x69, 0x42, 0x79, 0x74, 0x65, 0x54, 0x6F, 0x57, 0x69, 0x64, 0x65, 0x43, 0x68, 0x61, 0x72, 0x00, 0x69, 0x02, 0x47, 0x65, 0x74, 0x53, 0x74, 0x72, 0x69, 0x6E, 0x67, 0x54, 0x79, 0x70, 0x65, 0x57, 0x00, 0x00, 0x39, 0x03, 0x4C, 0x65, 0x61, 0x76, 0x65, 0x43, 0x72, 0x69, 0x74, 0x69, 0x63, 0x61, 0x6C, 0x53, 0x65, 0x63, 0x74, 0x69, 0x6F, 0x6E, 0x00, 0x00, 0xEE, 0x00, 0x45, 0x6E, 0x74, 0x65, 0x72, 0x43, 0x72, 0x69, 0x74, 0x69, 0x63, 0x61, 0x6C, 0x53, 0x65, 0x63, 0x74, 0x69, 0x6F, 0x6E, 0x00, 0x00, 0xB2, 0x04, 0x53, 0x6C, 0x65, 0x65, 0x70, 0x00, 0x66, 0x04, 0x53, 0x65, 0x74, 0x46, 0x69, 0x6C, 0x65, 0x50, 0x6F, 0x69, 0x6E, 0x74, 0x65, 0x72, 0x00, 0x00, 0x9A, 0x01, 0x47, 0x65, 0x74, 0x43, 0x6F, 0x6E, 0x73, 0x6F, 0x6C, 0x65, 0x43, 0x50, 0x00, 0x00, 0xAC, 0x01, 0x47, 0x65, 0x74, 0x43, 0x6F, 0x6E, 0x73, 0x6F, 0x6C, 0x65, 0x4D, 0x6F, 0x64, 0x65, 0x00, 0x00, 0x3F, 0x03, 0x4C, 0x6F, 0x61, 0x64, 0x4C, 0x69, 0x62, 0x72, 0x61, 0x72, 0x79, 0x57, 0x00, 0x00, 0x18, 0x04, 0x52, 0x74, 0x6C, 0x55, 0x6E, 0x77, 0x69, 0x6E,
				0x64, 0x00, 0x04, 0x03, 0x49, 0x73, 0x50, 0x72, 0x6F, 0x63, 0x65, 0x73, 0x73, 0x6F, 0x72, 0x46, 0x65, 0x61, 0x74, 0x75, 0x72, 0x65, 0x50, 0x72, 0x65, 0x73, 0x65, 0x6E, 0x74, 0x00, 0xD2, 0x02, 0x48, 0x65, 0x61, 0x70, 0x52, 0x65, 0x41, 0x6C, 0x6C, 0x6F, 0x63, 0x00, 0x87, 0x04, 0x53, 0x65, 0x74, 0x53, 0x74, 0x64, 0x48, 0x61, 0x6E, 0x64, 0x6C, 0x65, 0x00, 0x00, 0x24, 0x05, 0x57, 0x72, 0x69, 0x74, 0x65, 0x43, 0x6F, 0x6E, 0x73, 0x6F, 0x6C, 0x65, 0x57, 0x00, 0xD4, 0x02, 0x48, 0x65, 0x61, 0x70, 0x53, 0x69, 0x7A, 0x65, 0x00, 0x00, 0x8F, 0x00, 0x43, 0x72, 0x65, 0x61, 0x74, 0x65, 0x46, 0x69, 0x6C, 0x65, 0x57, 0x00, 0x52, 0x00, 0x43, 0x6C, 0x6F, 0x73, 0x65, 0x48, 0x61, 0x6E, 0x64, 0x6C, 0x65, 0x00, 0x57, 0x01, 0x46, 0x6C, 0x75, 0x73, 0x68, 0x46, 0x69, 0x6C, 0x65, 0x42, 0x75, 0x66, 0x66, 0x65, 0x72, 0x73, 0x00, 0x00, 0x4B, 0x45, 0x52, 0x4E, 0x45, 0x4C, 0x33, 0x32, 0x2E, 0x64, 0x6C, 0x6C, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
				0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x4E, 0xE6, 0x40, 0xBB, 0xB1, 0x19, 0xBF, 0x44, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6A, 0x6B, 0x6C, 0x6D, 0x6E, 0x6F, 0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4A, 0x4B, 0x4C, 0x4D, 0x4E, 0x4F, 0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
				0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6A, 0x6B, 0x6C, 0x6D, 0x6E, 0x6F, 0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4A, 0x4B,
				0x4C, 0x4D, 0x4E, 0x4F, 0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0xD0, 0x40, 0x00, 0x01, 0x02, 0x04, 0x08, 0xA4, 0x03, 0x00, 0x00, 0x60, 0x82, 0x79, 0x82, 0x21, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xA6, 0xDF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xA1, 0xA5, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x81, 0x9F, 0xE0, 0xFC, 0x00, 0x00, 0x00, 0x00, 0x40, 0x7E, 0x80, 0xFC, 0x00, 0x00, 0x00, 0x00, 0xA8, 0x03, 0x00, 0x00, 0xC1, 0xA3, 0xDA, 0xA3, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x81, 0xFE, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0xFE, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xB5, 0x03, 0x00, 0x00, 0xC1, 0xA3, 0xDA, 0xA3, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x81, 0xFE, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x41, 0xFE, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xB6, 0x03, 0x00, 0x00, 0xCF, 0xA2, 0xE4, 0xA2, 0x1A, 0x00, 0xE5, 0xA2, 0xE8, 0xA2, 0x5B, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x81, 0xFE, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x7E, 0xA1, 0xFE, 0x00, 0x00, 0x00, 0x00, 0x51, 0x05, 0x00, 0x00, 0x51, 0xDA, 0x5E, 0xDA, 0x20, 0x00, 0x5F, 0xDA, 0x6A, 0xDA, 0x32, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x81, 0xD3, 0xD8, 0xDE, 0xE0, 0xF9, 0x00, 0x00, 0x31, 0x7E, 0x81, 0xFE, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x43, 0x00, 0x00, 0x00, 0xC4, 0xA4, 0x40, 0x00, 0xC0, 0xA4, 0x40, 0x00, 0xBC, 0xA4, 0x40, 0x00, 0xB8, 0xA4, 0x40, 0x00, 0xB4, 0xA4, 0x40, 0x00, 0xB0, 0xA4, 0x40, 0x00, 0xAC, 0xA4, 0x40, 0x00, 0xA4, 0xA4, 0x40, 0x00, 0x9C, 0xA4, 0x40, 0x00, 0x94, 0xA4, 0x40, 0x00, 0x88, 0xA4, 0x40, 0x00, 0x7C, 0xA4, 0x40, 0x00, 0x74, 0xA4, 0x40, 0x00, 0x68, 0xA4, 0x40, 0x00, 0x64, 0xA4, 0x40, 0x00, 0x60, 0xA4, 0x40, 0x00, 0x5C, 0xA4, 0x40, 0x00, 0x58, 0xA4, 0x40, 0x00, 0x54, 0xA4, 0x40, 0x00, 0x50, 0xA4, 0x40, 0x00, 0x4C, 0xA4, 0x40, 0x00, 0x48, 0xA4, 0x40, 0x00, 0x44, 0xA4, 0x40, 0x00, 0x40, 0xA4, 0x40, 0x00,
				0x3C, 0xA4, 0x40, 0x00, 0x38, 0xA4, 0x40, 0x00, 0x30, 0xA4, 0x40, 0x00, 0x24, 0xA4, 0x40, 0x00, 0x1C, 0xA4, 0x40, 0x00, 0x14, 0xA4, 0x40, 0x00, 0x54, 0xA4, 0x40, 0x00, 0x0C, 0xA4, 0x40, 0x00, 0x04, 0xA4, 0x40, 0x00, 0xFC, 0xA3, 0x40, 0x00, 0xF0, 0xA3, 0x40, 0x00, 0xE8, 0xA3, 0x40, 0x00, 0xDC, 0xA3, 0x40, 0x00, 0xD0, 0xA3, 0x40, 0x00, 0xCC, 0xA3, 0x40, 0x00, 0xC8, 0xA3, 0x40, 0x00, 0xBC, 0xA3, 0x40, 0x00, 0xA8, 0xA3, 0x40, 0x00, 0x9C, 0xA3, 0x40, 0x00, 0x09, 0x04, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x94, 0xA3, 0x40, 0x00, 0x8C, 0xA3, 0x40, 0x00, 0x84, 0xA3, 0x40, 0x00, 0x7C, 0xA3, 0x40, 0x00, 0x74, 0xA3, 0x40, 0x00, 0x6C, 0xA3, 0x40, 0x00, 0x64, 0xA3, 0x40, 0x00, 0x54, 0xA3, 0x40, 0x00, 0x44, 0xA3, 0x40, 0x00, 0x34, 0xA3, 0x40, 0x00, 0x20, 0xA3, 0x40, 0x00, 0x0C, 0xA3, 0x40, 0x00, 0xFC, 0xA2, 0x40, 0x00, 0xE8, 0xA2, 0x40, 0x00, 0xE0, 0xA2, 0x40, 0x00, 0xD8, 0xA2, 0x40, 0x00, 0xD0, 0xA2, 0x40, 0x00, 0xC8, 0xA2, 0x40, 0x00, 0xC0, 0xA2, 0x40, 0x00, 0xB8, 0xA2, 0x40, 0x00, 0xB0, 0xA2, 0x40, 0x00, 0xA8, 0xA2, 0x40, 0x00, 0xA0, 0xA2, 0x40, 0x00, 0x98, 0xA2, 0x40, 0x00, 0x90, 0xA2, 0x40, 0x00, 0x88, 0xA2, 0x40, 0x00, 0x78, 0xA2, 0x40, 0x00, 0x64, 0xA2, 0x40, 0x00, 0x58, 0xA2, 0x40, 0x00, 0x4C, 0xA2, 0x40, 0x00, 0xC0, 0xA2, 0x40, 0x00, 0x40, 0xA2, 0x40, 0x00, 0x34, 0xA2, 0x40, 0x00, 0x24, 0xA2, 0x40, 0x00, 0x10, 0xA2, 0x40, 0x00, 0x00, 0xA2, 0x40, 0x00, 0xEC, 0xA1, 0x40, 0x00, 0xD8, 0xA1, 0x40, 0x00, 0xD0, 0xA1, 0x40, 0x00, 0xC8, 0xA1, 0x40, 0x00, 0xB4, 0xA1, 0x40, 0x00, 0x8C, 0xA1, 0x40, 0x00, 0x78, 0xA1, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x34, 0xD5, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x34, 0xD5, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x34, 0xD5, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x34, 0xD5, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x34, 0xD5, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x78, 0xDA, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xC0, 0xB1, 0x40, 0x00, 0x48, 0xB6, 0x40, 0x00, 0xC8, 0xB7, 0x40, 0x00, 0x38, 0xD5, 0x40, 0x00, 0xA0, 0xD6, 0x40, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00,
				0x16, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x03, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0x18, 0x00, 0x00, 0x00, 0x05, 0x00, 0x00, 0x00, 0x0D, 0x00, 0x00, 0x00, 0x06, 0x00, 0x00, 0x00, 0x09, 0x00, 0x00, 0x00, 0x07, 0x00, 0x00, 0x00, 0x0C, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x0C, 0x00, 0x00, 0x00, 0x09, 0x00, 0x00, 0x00, 0x0C, 0x00, 0x00, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x07, 0x00, 0x00, 0x00, 0x0B, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x0C, 0x00, 0x00, 0x00, 0x16, 0x00, 0x00, 0x00, 0x0D, 0x00, 0x00, 0x00, 0x16, 0x00, 0x00, 0x00, 0x0F, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x0D, 0x00, 0x00, 0x00, 0x11, 0x00, 0x00, 0x00, 0x12, 0x00, 0x00, 0x00, 0x12, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x21, 0x00, 0x00, 0x00, 0x0D, 0x00, 0x00, 0x00, 0x35, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x41, 0x00, 0x00, 0x00, 0x0D, 0x00, 0x00, 0x00, 0x43, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x50, 0x00, 0x00, 0x00, 0x11, 0x00, 0x00, 0x00, 0x52, 0x00, 0x00, 0x00, 0x0D, 0x00, 0x00, 0x00, 0x53, 0x00, 0x00, 0x00, 0x0D, 0x00, 0x00, 0x00, 0x57, 0x00, 0x00, 0x00, 0x16, 0x00, 0x00, 0x00, 0x59, 0x00, 0x00, 0x00, 0x0B, 0x00, 0x00, 0x00, 0x6C, 0x00, 0x00, 0x00, 0x0D, 0x00, 0x00, 0x00, 0x6D, 0x00, 0x00, 0x00, 0x20, 0x00, 0x00, 0x00, 0x70, 0x00, 0x00, 0x00, 0x1C, 0x00, 0x00, 0x00, 0x72, 0x00, 0x00, 0x00, 0x09, 0x00, 0x00, 0x00, 0x06, 0x00, 0x00, 0x00, 0x16, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x81, 0x00, 0x00, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x82, 0x00, 0x00, 0x00, 0x09, 0x00, 0x00, 0x00, 0x83, 0x00, 0x00, 0x00, 0x16, 0x00, 0x00, 0x00, 0x84, 0x00, 0x00, 0x00, 0x0D, 0x00, 0x00, 0x00, 0x91, 0x00, 0x00, 0x00, 0x29, 0x00, 0x00, 0x00, 0x9E, 0x00, 0x00, 0x00, 0x0D, 0x00, 0x00, 0x00, 0xA1, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0xA4, 0x00, 0x00, 0x00, 0x0B, 0x00, 0x00, 0x00, 0xA7, 0x00, 0x00, 0x00, 0x0D, 0x00, 0x00, 0x00, 0xB7, 0x00, 0x00, 0x00, 0x11, 0x00, 0x00, 0x00, 0xCE, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0xD7, 0x00, 0x00, 0x00, 0x0B, 0x00, 0x00, 0x00, 0x18, 0x07, 0x00, 0x00, 0x0C, 0x00, 0x00, 0x00, 0x0C, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0xC4, 0xB3, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0xA0, 0xA5, 0x40, 0x00, 0x90, 0xA5, 0x40, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0x80, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
				0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x2E, 0x00, 0x00, 0x00, 0x2E, 0x00, 0x00, 0x00, 0x70, 0xDA, 0x40, 0x00, 0xEC, 0xE9, 0x40, 0x00, 0xEC, 0xE9, 0x40, 0x00, 0xEC, 0xE9, 0x40, 0x00, 0xEC, 0xE9, 0x40, 0x00, 0xEC, 0xE9, 0x40, 0x00, 0xEC, 0xE9, 0x40, 0x00, 0xEC, 0xE9, 0x40, 0x00, 0xEC, 0xE9, 0x40, 0x00, 0xEC, 0xE9, 0x40, 0x00, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x74, 0xDA, 0x40, 0x00, 0xF0, 0xE9, 0x40, 0x00, 0xF0, 0xE9, 0x40, 0x00, 0xF0, 0xE9, 0x40, 0x00, 0xF0, 0xE9, 0x40, 0x00, 0xF0, 0xE9, 0x40, 0x00, 0xF0, 0xE9, 0x40, 0x00, 0xF0, 0xE9, 0x40, 0x00, 0x78, 0xDA, 0x40, 0x00, 0xC0, 0xB1, 0x40, 0x00, 0xC2, 0xB3, 0x40, 0x00, 0x01, 0x00, 0x00, 0x00, 0x2E, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x60, 0xEA, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x60, 0xEA, 0x40, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
				0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3A, 0x8B, 0x40, 0x00, 0x3A, 0x8B, 0x40, 0x00,
				0x3A, 0x8B, 0x40, 0x00, 0x3A, 0x8B, 0x40, 0x00, 0x3A, 0x8B, 0x40, 0x00, 0x3A, 0x8B, 0x40, 0x00, 0x3A, 0x8B, 0x40, 0x00, 0x3A, 0x8B, 0x40, 0x00, 0x3A, 0x8B, 0x40, 0x00, 0x3A, 0x8B, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x05, 0x93, 0x19, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x00, 0x05, 0x00, 0x00, 0x00, 0x20, 0x00, 0x00, 0x80, 0x18, 0x00, 0x00, 0x00, 0x38, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0xA0, 0x00, 0x00, 0x80, 0x50, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x68, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x09, 0x04, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x09, 0x04, 0x00, 0x00, 0x90, 0x00, 0x00, 0x00, 0xB8, 0x00, 0x01, 0x00, 0xCE, 0x00, 0x00, 0x00, 0xE4, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x88, 0x01, 0x01, 0x00, 0x53, 0x03, 0x00, 0x00, 0xE4, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0A, 0x00, 0x49, 0x00, 0x44, 0x00, 0x44, 0x00, 0x5F, 0x00, 0x44, 0x00, 0x49, 0x00, 0x41, 0x00, 0x4C, 0x00, 0x4F, 0x00, 0x47, 0x00, 0x00, 0x00, 0xC0, 0x08, 0xC8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x58, 0x00, 0x42, 0x00, 0x00, 0x00, 0x00, 0x00, 0x45, 0x00, 0x44, 0x00, 0x49, 0x00, 0x44, 0x00, 0x20, 0x00, 0x49, 0x00, 0x6E, 0x00, 0x73, 0x00, 0x74, 0x00, 0x61, 0x00, 0x6C, 0x00, 0x6C, 0x00, 0x65, 0x00, 0x72, 0x00, 0x00, 0x00, 0x09, 0x00, 0x53, 0x00, 0x65, 0x00, 0x67, 0x00, 0x6F, 0x00, 0x65, 0x00, 0x20, 0x00, 0x55, 0x00, 0x49, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x50, 0x00, 0x00, 0x00, 0x00, 0x0B, 0x00, 0x0A, 0x00, 0x42, 0x00, 0x0E, 0x00, 0x64, 0x00, 0xFF, 0xFF, 0x80, 0x00, 0x49, 0x00, 0x6E, 0x00, 0x73, 0x00, 0x74, 0x00, 0x61, 0x00, 0x6C, 0x00, 0x6C, 0x00, 0x20, 0x00, 0x45, 0x00, 0x44, 0x00, 0x49, 0x00, 0x44, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x50, 0x00, 0x00, 0x00, 0x00, 0x0B, 0x00, 0x1A, 0x00, 0x42, 0x00, 0x0E, 0x00, 0x65, 0x00, 0xFF, 0xFF, 0x80, 0x00, 0x52, 0x00, 0x65, 0x00, 0x73, 0x00, 0x65, 0x00, 0x74, 0x00, 0x20, 0x00, 0x45, 0x00, 0x44, 0x00, 0x49, 0x00,
				0x44, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x50, 0x00, 0x00, 0x00, 0x00, 0x0B, 0x00, 0x2A, 0x00, 0x42, 0x00, 0x0E, 0x00, 0x02, 0x00, 0xFF, 0xFF, 0x80, 0x00, 0x45, 0x00, 0x78, 0x00, 0x69, 0x00, 0x74, 0x00, 0x00, 0x00, 0x00, 0x00, 0x50, 0x41, 0x3C, 0x61, 0x73, 0x73, 0x65, 0x6D, 0x62, 0x6C, 0x79, 0x20, 0x78, 0x6D, 0x6C, 0x6E, 0x73, 0x3D, 0x22, 0x75, 0x72, 0x6E, 0x3A, 0x73, 0x63, 0x68, 0x65, 0x6D, 0x61, 0x73, 0x2D, 0x6D, 0x69, 0x63, 0x72, 0x6F, 0x73, 0x6F, 0x66, 0x74, 0x2D, 0x63, 0x6F, 0x6D, 0x3A, 0x61, 0x73, 0x6D, 0x2E, 0x76, 0x31, 0x22, 0x20, 0x6D, 0x61, 0x6E, 0x69, 0x66, 0x65, 0x73, 0x74, 0x56, 0x65, 0x72, 0x73, 0x69, 0x6F, 0x6E, 0x3D, 0x22, 0x31, 0x2E, 0x30, 0x22, 0x3E, 0x3C, 0x64, 0x65, 0x70, 0x65, 0x6E, 0x64, 0x65, 0x6E, 0x63, 0x79, 0x3E, 0x3C, 0x64, 0x65, 0x70, 0x65, 0x6E, 0x64, 0x65, 0x6E, 0x74, 0x41, 0x73, 0x73, 0x65, 0x6D, 0x62, 0x6C, 0x79, 0x3E, 0x3C, 0x61, 0x73, 0x73, 0x65, 0x6D, 0x62, 0x6C, 0x79, 0x49, 0x64, 0x65, 0x6E, 0x74, 0x69, 0x74, 0x79, 0x20, 0x6E, 0x61, 0x6D, 0x65, 0x3D, 0x22, 0x4D, 0x69, 0x63, 0x72, 0x6F, 0x73, 0x6F, 0x66, 0x74, 0x2E, 0x57, 0x69, 0x6E, 0x64, 0x6F, 0x77, 0x73, 0x2E, 0x43, 0x6F, 0x6D, 0x6D, 0x6F, 0x6E, 0x2D, 0x43, 0x6F, 0x6E, 0x74, 0x72, 0x6F, 0x6C, 0x73, 0x22, 0x20, 0x70, 0x72, 0x6F, 0x63, 0x65, 0x73, 0x73, 0x6F, 0x72, 0x41, 0x72, 0x63, 0x68, 0x69, 0x74, 0x65, 0x63, 0x74, 0x75, 0x72, 0x65, 0x3D, 0x22, 0x2A, 0x22, 0x20, 0x70, 0x75, 0x62, 0x6C, 0x69, 0x63, 0x4B, 0x65, 0x79, 0x54, 0x6F, 0x6B, 0x65, 0x6E, 0x3D, 0x22, 0x36, 0x35, 0x39, 0x35, 0x62, 0x36, 0x34, 0x31, 0x34, 0x34, 0x63, 0x63, 0x66, 0x31, 0x64, 0x66, 0x22, 0x20, 0x74, 0x79, 0x70, 0x65, 0x3D, 0x22, 0x77, 0x69, 0x6E, 0x33, 0x32, 0x22, 0x20, 0x76, 0x65, 0x72, 0x73, 0x69, 0x6F, 0x6E, 0x3D, 0x22, 0x36, 0x2E, 0x30, 0x2E, 0x30, 0x2E, 0x30, 0x22, 0x3E, 0x3C, 0x2F, 0x61, 0x73, 0x73, 0x65, 0x6D, 0x62, 0x6C, 0x79, 0x49, 0x64, 0x65, 0x6E, 0x74, 0x69, 0x74, 0x79, 0x3E, 0x3C, 0x2F, 0x64, 0x65, 0x70, 0x65, 0x6E, 0x64, 0x65, 0x6E, 0x74, 0x41, 0x73, 0x73, 0x65, 0x6D, 0x62, 0x6C, 0x79, 0x3E, 0x3C, 0x2F, 0x64, 0x65, 0x70, 0x65, 0x6E, 0x64, 0x65, 0x6E, 0x63, 0x79, 0x3E, 0x3C, 0x74, 0x72, 0x75, 0x73, 0x74, 0x49, 0x6E, 0x66, 0x6F, 0x20, 0x78, 0x6D, 0x6C, 0x6E, 0x73, 0x3D, 0x22, 0x75, 0x72, 0x6E, 0x3A, 0x73, 0x63, 0x68, 0x65, 0x6D, 0x61, 0x73, 0x2D, 0x6D, 0x69, 0x63, 0x72, 0x6F, 0x73, 0x6F, 0x66, 0x74, 0x2D, 0x63, 0x6F, 0x6D, 0x3A, 0x61, 0x73, 0x6D, 0x2E, 0x76, 0x33, 0x22, 0x3E, 0x3C, 0x73, 0x65, 0x63, 0x75, 0x72, 0x69, 0x74, 0x79, 0x3E, 0x3C, 0x72, 0x65, 0x71, 0x75, 0x65, 0x73, 0x74, 0x65, 0x64, 0x50, 0x72, 0x69, 0x76, 0x69, 0x6C, 0x65, 0x67, 0x65, 0x73, 0x3E, 0x3C, 0x72, 0x65, 0x71, 0x75, 0x65, 0x73, 0x74, 0x65, 0x64, 0x45, 0x78, 0x65, 0x63, 0x75, 0x74, 0x69, 0x6F, 0x6E, 0x4C, 0x65, 0x76, 0x65, 0x6C, 0x20, 0x6C, 0x65, 0x76, 0x65, 0x6C, 0x3D, 0x22, 0x72, 0x65, 0x71, 0x75, 0x69, 0x72, 0x65, 0x41, 0x64, 0x6D, 0x69, 0x6E, 0x69, 0x73, 0x74, 0x72, 0x61, 0x74, 0x6F, 0x72, 0x22, 0x20, 0x75, 0x69, 0x41, 0x63, 0x63, 0x65, 0x73, 0x73, 0x3D, 0x22, 0x66, 0x61, 0x6C,
				0x73, 0x65, 0x22, 0x3E, 0x3C, 0x2F, 0x72, 0x65, 0x71, 0x75, 0x65, 0x73, 0x74, 0x65, 0x64, 0x45, 0x78, 0x65, 0x63, 0x75, 0x74, 0x69, 0x6F, 0x6E, 0x4C, 0x65, 0x76, 0x65, 0x6C, 0x3E, 0x3C, 0x2F, 0x72, 0x65, 0x71, 0x75, 0x65, 0x73, 0x74, 0x65, 0x64, 0x50, 0x72, 0x69, 0x76, 0x69, 0x6C, 0x65, 0x67, 0x65, 0x73, 0x3E, 0x3C, 0x2F, 0x73, 0x65, 0x63, 0x75, 0x72, 0x69, 0x74, 0x79, 0x3E, 0x3C, 0x2F, 0x74, 0x72, 0x75, 0x73, 0x74, 0x49, 0x6E, 0x66, 0x6F, 0x3E, 0x3C, 0x61, 0x70, 0x70, 0x6C, 0x69, 0x63, 0x61, 0x74, 0x69, 0x6F, 0x6E, 0x20, 0x78, 0x6D, 0x6C, 0x6E, 0x73, 0x3D, 0x22, 0x75, 0x72, 0x6E, 0x3A, 0x73, 0x63, 0x68, 0x65, 0x6D, 0x61, 0x73, 0x2D, 0x6D, 0x69, 0x63, 0x72, 0x6F, 0x73, 0x6F, 0x66, 0x74, 0x2D, 0x63, 0x6F, 0x6D, 0x3A, 0x61, 0x73, 0x6D, 0x2E, 0x76, 0x33, 0x22, 0x3E, 0x3C, 0x77, 0x69, 0x6E, 0x64, 0x6F, 0x77, 0x73, 0x53, 0x65, 0x74, 0x74, 0x69, 0x6E, 0x67, 0x73, 0x3E, 0x3C, 0x6D, 0x73, 0x5F, 0x77, 0x69, 0x6E, 0x64, 0x6F, 0x77, 0x73, 0x53, 0x65, 0x74, 0x74, 0x69, 0x6E, 0x67, 0x73, 0x3A, 0x64, 0x70, 0x69, 0x41, 0x77, 0x61, 0x72, 0x65, 0x20, 0x78, 0x6D, 0x6C, 0x6E, 0x73, 0x3A, 0x6D, 0x73, 0x5F, 0x77, 0x69, 0x6E, 0x64, 0x6F, 0x77, 0x73, 0x53, 0x65, 0x74, 0x74, 0x69, 0x6E, 0x67, 0x73, 0x3D, 0x22, 0x68, 0x74, 0x74, 0x70, 0x3A, 0x2F, 0x2F, 0x73, 0x63, 0x68, 0x65, 0x6D, 0x61, 0x73, 0x2E, 0x6D, 0x69, 0x63, 0x72, 0x6F, 0x73, 0x6F, 0x66, 0x74, 0x2E, 0x63, 0x6F, 0x6D, 0x2F, 0x53, 0x4D, 0x49, 0x2F, 0x32, 0x30, 0x30, 0x35, 0x2F, 0x57, 0x69, 0x6E, 0x64, 0x6F, 0x77, 0x73, 0x53, 0x65, 0x74, 0x74, 0x69, 0x6E, 0x67, 0x73, 0x22, 0x20, 0x78, 0x6D, 0x6C, 0x6E, 0x73, 0x3D, 0x22, 0x68, 0x74, 0x74, 0x70, 0x3A, 0x2F, 0x2F, 0x73, 0x63, 0x68, 0x65, 0x6D, 0x61, 0x73, 0x2E, 0x6D, 0x69, 0x63, 0x72, 0x6F, 0x73, 0x6F, 0x66, 0x74, 0x2E, 0x63, 0x6F, 0x6D, 0x2F, 0x53, 0x4D, 0x49, 0x2F, 0x32, 0x30, 0x30, 0x35, 0x2F, 0x57, 0x69, 0x6E, 0x64, 0x6F, 0x77, 0x73, 0x53, 0x65, 0x74, 0x74, 0x69, 0x6E, 0x67, 0x73, 0x22, 0x3E, 0x74, 0x72, 0x75, 0x65, 0x3C, 0x2F, 0x6D, 0x73, 0x5F, 0x77, 0x69, 0x6E, 0x64, 0x6F, 0x77, 0x73, 0x53, 0x65, 0x74, 0x74, 0x69, 0x6E, 0x67, 0x73, 0x3A, 0x64, 0x70, 0x69, 0x41, 0x77, 0x61, 0x72, 0x65, 0x3E, 0x3C, 0x2F, 0x77, 0x69, 0x6E, 0x64, 0x6F, 0x77, 0x73, 0x53, 0x65, 0x74, 0x74, 0x69, 0x6E, 0x67, 0x73, 0x3E, 0x3C, 0x2F, 0x61, 0x70, 0x70, 0x6C, 0x69, 0x63, 0x61, 0x74, 0x69, 0x6F, 0x6E, 0x3E, 0x3C, 0x2F, 0x61, 0x73, 0x73, 0x65, 0x6D, 0x62, 0x6C, 0x79, 0x3E, 0x50, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x58, 0x58, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x58, 0x58, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x58, 0x58, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x58, 0x58, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x58, 0x58, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x58, 0x58, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x58,
				0x58, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x58, 0x58, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x58, 0x58, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x58, 0x58, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x58, 0x58, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x58, 0x58, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x58, 0x58, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x58, 0x58, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x58, 0x58, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x58, 0x58, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x58, 0x58, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x58, 0x58, 0x50, 0x41, 0x44, 0x44, 0x49, 0x4E, 0x47, 0x50, 0x41, 0x44, 0x44, 0x00, 0x10, 0x00, 0x00, 0xCC, 0x01, 0x00, 0x00, 0x0A, 0x30, 0x18, 0x30, 0x1D, 0x30, 0x22, 0x30, 0x42, 0x30, 0x4A, 0x30, 0x7D, 0x30, 0x87, 0x30, 0x8F, 0x30, 0x98, 0x30, 0xA2, 0x30, 0xDE, 0x30, 0xFB, 0x30, 0x02, 0x31, 0x0C, 0x31, 0x32, 0x31, 0x3D, 0x31, 0x50, 0x31, 0x5A, 0x31, 0x63, 0x31, 0x82, 0x31, 0x89, 0x31, 0x93, 0x31, 0xA6, 0x31, 0xAD, 0x31, 0xB7, 0x31, 0xCA, 0x31, 0xD8, 0x31, 0xDD, 0x31, 0xE2, 0x31, 0x02, 0x32, 0x0A, 0x32, 0x35, 0x32, 0x47, 0x32, 0x4D, 0x32, 0x68, 0x32, 0x6D, 0x32, 0x8D, 0x32, 0x94, 0x32, 0x9E, 0x32, 0xA3, 0x32, 0xB8, 0x32, 0xC4, 0x32, 0xCB, 0x32, 0xE8, 0x32, 0xEF, 0x32, 0xF9, 0x32, 0xFE, 0x32, 0x13, 0x33, 0x1F, 0x33, 0x34, 0x33, 0x3D, 0x33, 0x6A, 0x33, 0x75, 0x33, 0x7A, 0x33, 0x7F, 0x33, 0x9D, 0x33, 0xA5, 0x33, 0xD1, 0x33, 0x04, 0x34, 0x4E, 0x34, 0x99, 0x34, 0xBE, 0x34, 0xCB, 0x34, 0xD7, 0x34, 0xE0, 0x34, 0xFA, 0x34, 0x20, 0x35, 0x25, 0x35, 0x45, 0x35, 0x4D, 0x35, 0x79, 0x35, 0x8B, 0x35, 0x91, 0x35, 0xBD, 0x35, 0xCE, 0x35, 0xE5, 0x35, 0xF2, 0x35, 0xFC, 0x35, 0x01, 0x36, 0x2A, 0x36, 0x65, 0x36, 0x6B, 0x36, 0x72, 0x36, 0x7C, 0x36, 0x90, 0x36, 0xB6, 0x36, 0xED, 0x36, 0x05, 0x37, 0x30, 0x37, 0x37, 0x37, 0x41, 0x37, 0x46, 0x37, 0x63, 0x37, 0x73, 0x37, 0x79, 0x37, 0x80, 0x37, 0x8A, 0x37, 0x9E, 0x37, 0xDA, 0x37, 0x06, 0x38, 0x10, 0x38, 0x1B, 0x38, 0x46, 0x38, 0x4D, 0x38, 0x65, 0x38, 0x8D, 0x38, 0x99, 0x38, 0xC3, 0x38, 0xD4, 0x38, 0xDA, 0x38, 0xFA, 0x38, 0x04, 0x39, 0x09, 0x39, 0x49, 0x39, 0x54, 0x39, 0x6A, 0x39, 0x70, 0x39, 0x86, 0x39, 0xE1, 0x39, 0xEE, 0x39, 0xF8, 0x39, 0x02, 0x3A, 0x24, 0x3A, 0x32, 0x3A, 0x4A, 0x3A, 0x4F, 0x3A, 0x57, 0x3A, 0x67, 0x3A, 0x6C, 0x3A, 0x74, 0x3A, 0x7E, 0x3A, 0x83, 0x3A, 0x8B, 0x3A, 0xA7, 0x3A, 0xBC, 0x3A, 0xCE, 0x3A, 0xDB, 0x3A, 0xE0, 0x3A, 0xE8, 0x3A, 0xF3, 0x3A, 0xF8, 0x3A, 0x00, 0x3B, 0x0E, 0x3B, 0x13, 0x3B, 0x1B, 0x3B, 0x29, 0x3B, 0x2E, 0x3B, 0x36, 0x3B, 0x44, 0x3B, 0x49, 0x3B, 0x51, 0x3B,
				0x5C, 0x3B, 0x60, 0x3B, 0x64, 0x3B, 0x68, 0x3B, 0x74, 0x3B, 0x89, 0x3B, 0x9B, 0x3B, 0xA6, 0x3B, 0xAB, 0x3B, 0xB3, 0x3B, 0xC1, 0x3B, 0xC6, 0x3B, 0xCE, 0x3B, 0xD8, 0x3B, 0xDD, 0x3B, 0xE5, 0x3B, 0xF2, 0x3B, 0xF7, 0x3B, 0xFF, 0x3B, 0x0C, 0x3C, 0x11, 0x3C, 0x19, 0x3C, 0x24, 0x3C, 0x28, 0x3C, 0x2C, 0x3C, 0x30, 0x3C, 0x5C, 0x3C, 0xA2, 0x3C, 0xA9, 0x3C, 0xCF, 0x3C, 0xD6, 0x3C, 0xDD, 0x3C, 0x09, 0x3D, 0x1E, 0x3D, 0x30, 0x3D, 0x3D, 0x3D, 0x42, 0x3D, 0x4A, 0x3D, 0x5A, 0x3D, 0x5F, 0x3D, 0x67, 0x3D, 0x77, 0x3D, 0x7C, 0x3D, 0x84, 0x3D, 0xA0, 0x3D, 0xA4, 0x3D, 0xA8, 0x3D, 0xD8, 0x3D, 0xDC, 0x3D, 0xE0, 0x3D, 0xE4, 0x3D, 0x0F, 0x3E, 0x25, 0x3E, 0x43, 0x3E, 0x4A, 0x3E, 0x54, 0x3E, 0x6F, 0x3E, 0x7D, 0x3E, 0x87, 0x3E, 0x9C, 0x3E, 0xDC, 0x3E, 0xE6, 0x3E, 0xF8, 0x3E, 0xFF, 0x3E, 0x11, 0x3F, 0x72, 0x3F, 0x7B, 0x3F, 0x82, 0x3F, 0x96, 0x3F, 0xBC, 0x3F, 0xC8, 0x3F, 0xCC, 0x3F, 0xD0, 0x3F, 0x00, 0x00, 0x00, 0x20, 0x00, 0x00, 0xE0, 0x00, 0x00, 0x00, 0xC1, 0x30, 0xF8, 0x30, 0x00, 0x31, 0x15, 0x31, 0x20, 0x31, 0x57, 0x32, 0x80, 0x33, 0x88, 0x34, 0x8E, 0x34, 0xA0, 0x34, 0xC2, 0x34, 0xF1, 0x34, 0xF7, 0x34, 0x06, 0x35, 0x4B, 0x35, 0x70, 0x35, 0x7F, 0x35, 0x87, 0x35, 0x94, 0x35, 0xA0, 0x35, 0xAC, 0x35, 0xB2, 0x35, 0xC4, 0x35, 0xCC, 0x35, 0xD7, 0x35, 0x1E, 0x36, 0x23, 0x36, 0x2D, 0x36, 0x7E, 0x36, 0xF0, 0x36, 0xF6, 0x36, 0xFC, 0x36, 0x02, 0x37, 0x08, 0x37, 0x0E, 0x37, 0x15, 0x37, 0x1C, 0x37, 0x23, 0x37, 0x2A, 0x37, 0x31, 0x37, 0x38, 0x37, 0x3F, 0x37, 0x47, 0x37, 0x4F, 0x37, 0x57, 0x37, 0x63, 0x37, 0x6C, 0x37, 0x71, 0x37, 0x77, 0x37, 0x81, 0x37, 0x8A, 0x37, 0x95, 0x37, 0xA1, 0x37, 0xA6, 0x37, 0xB6, 0x37, 0xBB, 0x37, 0xC1, 0x37, 0xC7, 0x37, 0xDD, 0x37, 0xE4, 0x37, 0x4C, 0x38, 0x89, 0x38, 0xA0, 0x38, 0x10, 0x3A, 0x21, 0x3A, 0x5B, 0x3A, 0x68, 0x3A, 0x72, 0x3A, 0x80, 0x3A, 0x89, 0x3A, 0x93, 0x3A, 0xC7, 0x3A, 0xD2, 0x3A, 0xDC, 0x3A, 0xF5, 0x3A, 0xFF, 0x3A, 0x12, 0x3B, 0x36, 0x3B, 0x6D, 0x3B, 0xA2, 0x3B, 0xB5, 0x3B, 0x25, 0x3C, 0x42, 0x3C, 0x8B, 0x3C, 0xFA, 0x3C, 0x19, 0x3D, 0x8E, 0x3D, 0x9A, 0x3D, 0xAD, 0x3D, 0xBF, 0x3D, 0xDA, 0x3D, 0xE2, 0x3D, 0xEA, 0x3D, 0x01, 0x3E, 0x1A, 0x3E, 0x36, 0x3E, 0x3F, 0x3E, 0x45, 0x3E, 0x4E, 0x3E, 0x53, 0x3E, 0x62, 0x3E, 0x89, 0x3E, 0xB2, 0x3E, 0xC3, 0x3E, 0xD7, 0x3E, 0x23, 0x3F, 0x72, 0x3F, 0xBA, 0x3F, 0x00, 0x30, 0x00, 0x00, 0xDC, 0x00, 0x00, 0x00, 0x0E, 0x30, 0xD1, 0x30, 0xFF, 0x30, 0x77, 0x31, 0x91, 0x31, 0xA2, 0x31, 0xDB, 0x31, 0x0B, 0x32, 0x12, 0x32, 0x1E, 0x32, 0x24, 0x32, 0x30, 0x32, 0x36, 0x32, 0x3F, 0x32, 0x45, 0x32, 0x4E, 0x32, 0x5A, 0x32, 0x60, 0x32, 0x68, 0x32, 0x6E, 0x32, 0x7A, 0x32, 0x80, 0x32, 0x8D, 0x32, 0x97, 0x32, 0x9D, 0x32, 0xA7, 0x32, 0xC9, 0x32, 0xDE, 0x32, 0x04, 0x33, 0x44, 0x33, 0x4A, 0x33, 0x74, 0x33, 0x7A, 0x33, 0x80, 0x33, 0x96, 0x33, 0xAE, 0x33, 0xD4, 0x33, 0x4E, 0x34, 0x71, 0x34, 0x7B, 0x34, 0xB3, 0x34, 0xBB, 0x34, 0x04, 0x35, 0x0A, 0x35, 0x20, 0x35, 0x25, 0x35, 0x2D, 0x35, 0x33, 0x35, 0x3A, 0x35, 0x40, 0x35, 0x47, 0x35, 0x4D, 0x35, 0x55, 0x35, 0x5C, 0x35, 0x61, 0x35, 0x69, 0x35, 0x72, 0x35, 0x7E, 0x35, 0x83, 0x35, 0x88, 0x35,
				0x8E, 0x35, 0x92, 0x35, 0x98, 0x35, 0x9D, 0x35, 0xA3, 0x35, 0xA8, 0x35, 0xB7, 0x35, 0xCD, 0x35, 0xD3, 0x35, 0xDB, 0x35, 0xE0, 0x35, 0xE8, 0x35, 0xED, 0x35, 0xF5, 0x35, 0xFA, 0x35, 0x01, 0x36, 0x10, 0x36, 0x15, 0x36, 0x1B, 0x36, 0x24, 0x36, 0x44, 0x36, 0x4A, 0x36, 0x62, 0x36, 0x99, 0x37, 0xAB, 0x37, 0x8B, 0x38, 0x95, 0x38, 0xA2, 0x38, 0xE0, 0x38, 0xE7, 0x38, 0xF4, 0x38, 0xFA, 0x38, 0x37, 0x39, 0x53, 0x39, 0x76, 0x39, 0x89, 0x39, 0x25, 0x3B, 0x2C, 0x3B, 0x26, 0x3C, 0xC9, 0x3C, 0xE7, 0x3C, 0x0D, 0x3D, 0x6D, 0x3D, 0x7C, 0x3D, 0x97, 0x3D, 0x00, 0x00, 0x00, 0x40, 0x00, 0x00, 0xA0, 0x00, 0x00, 0x00, 0xAC, 0x30, 0xA2, 0x31, 0x36, 0x33, 0x79, 0x33, 0xA5, 0x33, 0xC6, 0x33, 0xAA, 0x35, 0x02, 0x38, 0x06, 0x38, 0x0A, 0x38, 0x0E, 0x38, 0x12, 0x38, 0x16, 0x38, 0x1A, 0x38, 0x1E, 0x38, 0x2D, 0x38, 0x39, 0x38, 0x46, 0x38, 0x4C, 0x38, 0x55, 0x38, 0x5C, 0x38, 0x7E, 0x38, 0xF3, 0x38, 0xFB, 0x38, 0x0E, 0x39, 0x19, 0x39, 0x1E, 0x39, 0x30, 0x39, 0x3A, 0x39, 0x3F, 0x39, 0x5B, 0x39, 0x65, 0x39, 0x7B, 0x39, 0x86, 0x39, 0xA0, 0x39, 0xAB, 0x39, 0xB3, 0x39, 0xC3, 0x39, 0xC9, 0x39, 0xDA, 0x39, 0x13, 0x3A, 0x1D, 0x3A, 0x43, 0x3A, 0x4A, 0x3A, 0x64, 0x3A, 0x6B, 0x3A, 0x96, 0x3A, 0x38, 0x3B, 0x4B, 0x3B, 0x5D, 0x3B, 0xA4, 0x3B, 0xBC, 0x3B, 0xC6, 0x3B, 0xE1, 0x3B, 0xE9, 0x3B, 0xEF, 0x3B, 0xFD, 0x3B, 0x31, 0x3C, 0x3E, 0x3C, 0x53, 0x3C, 0x84, 0x3C, 0xA1, 0x3C, 0xED, 0x3C, 0x1B, 0x3D, 0x42, 0x3D, 0x4F, 0x3D, 0x55, 0x3D, 0xB3, 0x3D, 0xB9, 0x3D, 0x12, 0x3F, 0x1F, 0x3F, 0x28, 0x3F, 0x6E, 0x3F, 0x7D, 0x3F, 0xB5, 0x3F, 0xBF, 0x3F, 0x00, 0x50, 0x00, 0x00, 0xC0, 0x00, 0x00, 0x00, 0x00, 0x30, 0x0B, 0x30, 0x15, 0x30, 0x26, 0x30, 0x31, 0x30, 0xF1, 0x31, 0x02, 0x32, 0x0A, 0x32, 0x10, 0x32, 0x15, 0x32, 0x1B, 0x32, 0x87, 0x32, 0x8D, 0x32, 0xA9, 0x32, 0xD1, 0x32, 0x1D, 0x33, 0x29, 0x33, 0x43, 0x33, 0x69, 0x33, 0x6F, 0x33, 0x99, 0x33, 0xDE, 0x33, 0xE5, 0x33, 0xFA, 0x33, 0x41, 0x34, 0x4B, 0x34, 0x76, 0x34, 0x8E, 0x34, 0xAC, 0x34, 0xD0, 0x34, 0x00, 0x35, 0x12, 0x35, 0x40, 0x35, 0x63, 0x35, 0x69, 0x35, 0x7D, 0x35, 0x82, 0x35, 0xA3, 0x35, 0xA8, 0x35, 0xD1, 0x35, 0xEE, 0x35, 0x42, 0x36, 0x1C, 0x37, 0x24, 0x37, 0x3C, 0x37, 0x57, 0x37, 0xAE, 0x37, 0xC8, 0x37, 0xEB, 0x37, 0xF8, 0x37, 0x04, 0x38, 0x0C, 0x38, 0x14, 0x38, 0x20, 0x38, 0x49, 0x38, 0x51, 0x38, 0x5C, 0x38, 0x90, 0x38, 0x13, 0x39, 0x5C, 0x39, 0xF5, 0x39, 0xC5, 0x3A, 0x3F, 0x3B, 0x62, 0x3B, 0xFB, 0x3B, 0x65, 0x3C, 0x6C, 0x3C, 0x76, 0x3C, 0x88, 0x3C, 0x9F, 0x3C, 0xAD, 0x3C, 0xB3, 0x3C, 0xD6, 0x3C, 0xDD, 0x3C, 0xF6, 0x3C, 0x0A, 0x3D, 0x10, 0x3D, 0x19, 0x3D, 0x2C, 0x3D, 0x50, 0x3D, 0x90, 0x3D, 0xE4, 0x3D, 0x04, 0x3E, 0x25, 0x3E, 0x2E, 0x3E, 0x3A, 0x3E, 0x71, 0x3E, 0x7A, 0x3E, 0x86, 0x3E, 0xBF, 0x3E, 0xC8, 0x3E, 0xD4, 0x3E, 0x00, 0x60, 0x00, 0x00, 0xA8, 0x00, 0x00, 0x00, 0x72, 0x32, 0x84, 0x32, 0x96, 0x32, 0xA8, 0x32, 0xBA, 0x32, 0xE0, 0x32, 0xF2, 0x32, 0x04, 0x33, 0x16, 0x33, 0x28, 0x33, 0x3A, 0x33, 0x4C, 0x33, 0x5E, 0x33, 0x70, 0x33, 0x82, 0x33, 0x94, 0x33, 0xA6, 0x33, 0xB8, 0x33, 0x3A, 0x36, 0x6C, 0x36, 0x84, 0x36, 0x8B, 0x36, 0x93, 0x36, 0x98, 0x36,
				0x9C, 0x36, 0xA0, 0x36, 0xC9, 0x36, 0xEF, 0x36, 0x0D, 0x37, 0x14, 0x37, 0x18, 0x37, 0x1C, 0x37, 0x20, 0x37, 0x24, 0x37, 0x28, 0x37, 0x2C, 0x37, 0x30, 0x37, 0x7A, 0x37, 0x80, 0x37, 0x84, 0x37, 0x88, 0x37, 0x8C, 0x37, 0xF2, 0x37, 0xFD, 0x37, 0x18, 0x38, 0x1F, 0x38, 0x24, 0x38, 0x28, 0x38, 0x2C, 0x38, 0x4D, 0x38, 0x77, 0x38, 0xA9, 0x38, 0xB0, 0x38, 0xB4, 0x38, 0xB8, 0x38, 0xBC, 0x38, 0xC0, 0x38, 0xC4, 0x38, 0xC8, 0x38, 0xCC, 0x38, 0x16, 0x39, 0x1C, 0x39, 0x20, 0x39, 0x24, 0x39, 0x28, 0x39, 0x94, 0x3A, 0xA1, 0x3A, 0xBA, 0x3A, 0xD8, 0x3A, 0x14, 0x3B, 0x3C, 0x3B, 0xCF, 0x3B, 0x2E, 0x3C, 0xD1, 0x3C, 0xF1, 0x3C, 0xE1, 0x3D, 0x0A, 0x3E, 0x63, 0x3E, 0xD1, 0x3F, 0x00, 0x00, 0x00, 0x70, 0x00, 0x00, 0xE0, 0x00, 0x00, 0x00, 0xAB, 0x30, 0x7B, 0x31, 0xAC, 0x31, 0xC2, 0x31, 0x03, 0x32, 0x22, 0x32, 0xBF, 0x32, 0xF1, 0x32, 0x19, 0x33, 0x97, 0x33, 0xFB, 0x33, 0x1E, 0x34, 0x30, 0x34, 0x36, 0x34, 0x50, 0x34, 0x5F, 0x34, 0x6C, 0x34, 0x78, 0x34, 0x88, 0x34, 0x8F, 0x34, 0x9E, 0x34, 0xAA, 0x34, 0xB7, 0x34, 0xDB, 0x34, 0xED, 0x34, 0xFB, 0x34, 0x10, 0x35, 0x1A, 0x35, 0x40, 0x35, 0x73, 0x35, 0x82, 0x35, 0x8B, 0x35, 0xAF, 0x35, 0xDE, 0x35, 0x12, 0x36, 0x18, 0x36, 0x1E, 0x36, 0x2E, 0x36, 0x39, 0x36, 0x4D, 0x37, 0x6B, 0x37, 0x58, 0x38, 0x8F, 0x38, 0x95, 0x38, 0x9A, 0x38, 0xA8, 0x38, 0xAD, 0x38, 0xB2, 0x38, 0xB7, 0x38, 0xC7, 0x38, 0xF6, 0x38, 0xFC, 0x38, 0x04, 0x39, 0x4B, 0x39, 0x50, 0x39, 0x8A, 0x39, 0x8F, 0x39, 0x96, 0x39, 0x9B, 0x39, 0xA2, 0x39, 0xA7, 0x39, 0xB5, 0x39, 0x16, 0x3A, 0x1F, 0x3A, 0x25, 0x3A, 0xAD, 0x3A, 0xBC, 0x3A, 0xCC, 0x3A, 0xD3, 0x3A, 0xDB, 0x3A, 0x4B, 0x3B, 0x50, 0x3B, 0x59, 0x3B, 0x68, 0x3B, 0x8B, 0x3B, 0x90, 0x3B, 0x95, 0x3B, 0xAC, 0x3B, 0x98, 0x3C, 0x9D, 0x3C, 0xAF, 0x3C, 0xCD, 0x3C, 0xE1, 0x3C, 0xE7, 0x3C, 0x55, 0x3D, 0x78, 0x3D, 0x83, 0x3D, 0x89, 0x3D, 0x99, 0x3D, 0x9E, 0x3D, 0xAF, 0x3D, 0xB7, 0x3D, 0xBD, 0x3D, 0xC7, 0x3D, 0xCD, 0x3D, 0xD7, 0x3D, 0xDD, 0x3D, 0xE7, 0x3D, 0xF0, 0x3D, 0xFB, 0x3D, 0x00, 0x3E, 0x09, 0x3E, 0x13, 0x3E, 0x1E, 0x3E, 0x59, 0x3E, 0x73, 0x3E, 0x8D, 0x3E, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x98, 0x00, 0x00, 0x00, 0x8F, 0x30, 0x96, 0x30, 0x9C, 0x30, 0x45, 0x31, 0x51, 0x31, 0x5C, 0x32, 0x40, 0x33, 0x45, 0x33, 0xBB, 0x33, 0xC1, 0x33, 0xCB, 0x33, 0x39, 0x34, 0x3F, 0x34, 0x4B, 0x34, 0x82, 0x34, 0x9A, 0x34, 0xC1, 0x35, 0xD4, 0x35, 0xEC, 0x35, 0x0C, 0x36, 0x5F, 0x36, 0x87, 0x36, 0xA0, 0x36, 0xBC, 0x36, 0xE9, 0x36, 0x16, 0x37, 0x21, 0x37, 0x4F, 0x37, 0x5D, 0x37, 0x6B, 0x37, 0x78, 0x37, 0x97, 0x37, 0x3F, 0x38, 0xAD, 0x38, 0x0E, 0x39, 0x2F, 0x39, 0x38, 0x39, 0x5F, 0x39, 0x6C, 0x39, 0x71, 0x39, 0x7F, 0x39, 0x5A, 0x3A, 0x7D, 0x3A, 0x88, 0x3A, 0xAB, 0x3A, 0xFA, 0x3A, 0x56, 0x3B, 0x9D, 0x3B, 0xA3, 0x3B, 0xBC, 0x3B, 0x21, 0x3C, 0x2D, 0x3C, 0xA5, 0x3C, 0xBF, 0x3C, 0xC8, 0x3C, 0xF6, 0x3C, 0xFC, 0x3C, 0x01, 0x3D, 0x07, 0x3D, 0x18, 0x3D, 0x8D, 0x3D, 0x01, 0x3E, 0x2B, 0x3E, 0x4B, 0x3E, 0x81, 0x3E, 0x8B, 0x3E, 0x2B, 0x3F, 0x68, 0x3F, 0x72, 0x3F, 0x8A, 0x3F, 0xB3, 0x3F, 0xE5, 0x3F, 0x00, 0x90, 0x00, 0x00, 0x0C, 0x00, 0x00, 0x00, 0x0D, 0x30, 0xA8, 0x30,
				0x00, 0xA0, 0x00, 0x00, 0x48, 0x00, 0x00, 0x00, 0x40, 0x31, 0x44, 0x31, 0x48, 0x31, 0x4C, 0x31, 0x50, 0x31, 0x5C, 0x31, 0x60, 0x31, 0x70, 0x31, 0x74, 0x31, 0xB4, 0x3E, 0xBC, 0x3E, 0xC4, 0x3E, 0xCC, 0x3E, 0xD4, 0x3E, 0xDC, 0x3E, 0xE4, 0x3E, 0xEC, 0x3E, 0xF4, 0x3E, 0xFC, 0x3E, 0x04, 0x3F, 0x0C, 0x3F, 0x14, 0x3F, 0x1C, 0x3F, 0x24, 0x3F, 0x2C, 0x3F, 0x34, 0x3F, 0x3C, 0x3F, 0x44, 0x3F, 0x4C, 0x3F, 0x54, 0x3F, 0x5C, 0x3F, 0x00, 0x00, 0x00, 0xC0, 0x00, 0x00, 0x40, 0x00, 0x00, 0x00, 0xB4, 0x30, 0xB8, 0x30, 0xF4, 0x30, 0xF8, 0x30, 0x18, 0x31, 0x38, 0x31, 0x58, 0x31, 0x78, 0x31, 0x84, 0x31, 0xA0, 0x31, 0xAC, 0x31, 0xC8, 0x31, 0xE8, 0x31, 0x08, 0x32, 0x28, 0x32, 0x44, 0x32, 0x48, 0x32, 0x68, 0x32, 0x88, 0x32, 0xA4, 0x32, 0xA8, 0x32, 0xC8, 0x32, 0xE8, 0x32, 0x08, 0x33, 0x14, 0x33, 0x30, 0x33, 0x50, 0x33, 0x70, 0x33, 0x00, 0xD0, 0x00, 0x00, 0x14, 0x01, 0x00, 0x00, 0x38, 0x34, 0x38, 0x35, 0x3C, 0x35, 0x40, 0x35, 0x44, 0x35, 0x48, 0x35, 0x4C, 0x35, 0x50, 0x35, 0x54, 0x35, 0x58, 0x35, 0x5C, 0x35, 0x60, 0x35, 0x64, 0x35, 0x68, 0x35, 0x6C, 0x35, 0x70, 0x35, 0x74, 0x35, 0x78, 0x35, 0x7C, 0x35, 0x80, 0x35, 0x84, 0x35, 0x88, 0x35, 0x8C, 0x35, 0x90, 0x35, 0x94, 0x35, 0x98, 0x35, 0x9C, 0x35, 0xA0, 0x35, 0xA4, 0x35, 0xA8, 0x35, 0xAC, 0x35, 0xB0, 0x35, 0xB4, 0x35, 0xB8, 0x35, 0xBC, 0x35, 0xC0, 0x35, 0xC4, 0x35, 0xC8, 0x35, 0xCC, 0x35, 0xD0, 0x35, 0xD4, 0x35, 0xD8, 0x35, 0xDC, 0x35, 0xE0, 0x35, 0xF0, 0x35, 0xF4, 0x35, 0xF8, 0x35, 0xFC, 0x35, 0x00, 0x36, 0x04, 0x36, 0x08, 0x36, 0x0C, 0x36, 0x10, 0x36, 0x14, 0x36, 0x18, 0x36, 0x1C, 0x36, 0x20, 0x36, 0x24, 0x36, 0x28, 0x36, 0x2C, 0x36, 0x30, 0x36, 0x34, 0x36, 0x38, 0x36, 0x3C, 0x36, 0x40, 0x36, 0x44, 0x36, 0x48, 0x36, 0x4C, 0x36, 0x50, 0x36, 0x54, 0x36, 0x58, 0x36, 0x5C, 0x36, 0x60, 0x36, 0x64, 0x36, 0x68, 0x36, 0x6C, 0x36, 0x70, 0x36, 0x74, 0x36, 0x78, 0x36, 0x7C, 0x36, 0x80, 0x36, 0x84, 0x36, 0x88, 0x36, 0x8C, 0x36, 0x90, 0x36, 0x94, 0x36, 0x98, 0x36, 0xF8, 0x36, 0x08, 0x37, 0x18, 0x37, 0x28, 0x37, 0x38, 0x37, 0x5C, 0x37, 0x68, 0x37, 0x6C, 0x37, 0x70, 0x37, 0x74, 0x37, 0x78, 0x37, 0xF8, 0x38, 0x00, 0x39, 0x04, 0x39, 0x78, 0x3A, 0x7C, 0x3A, 0x80, 0x3A, 0x84, 0x3A, 0x88, 0x3A, 0x8C, 0x3A, 0x90, 0x3A, 0x94, 0x3A, 0x98, 0x3A, 0x9C, 0x3A, 0xA8, 0x3A, 0xAC, 0x3A, 0xB0, 0x3A, 0xB4, 0x3A, 0xB8, 0x3A, 0xBC, 0x3A, 0xC0, 0x3A, 0xC4, 0x3A, 0xC8, 0x3A, 0xCC, 0x3A, 0xD0, 0x3A, 0xE0, 0x3A, 0xE8, 0x3A, 0x60, 0x3D, 0x64, 0x3D, 0x68, 0x3D, 0x6C, 0x3D, 0x70, 0x3D, 0x74, 0x3D, 0x78, 0x3D, 0x7C, 0x3D, 0x80, 0x3D, 0x84, 0x3D, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
				0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
				0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
				0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
			};

			if (ExportData == OverrideData && ExportData[126] == 0 && OverrideProperties.GetExtensions())
			{
				Count++;
				ExportData[126]++;
				ExportData[127]--;
				ExtensionBlockList->WriteBlankExtension(&ExportData[128], 128);
			}

			for (Index = 0; Index < sizeof FileData; Index++)
			{
				if (std::strcmp((const char *)&FileData[Index], "CRU1234") == 0)
				{
					std::strcpy((char *)&FileData[Index], DeviceID);
					break;
				}
			}

			for (Index++; Index < sizeof FileData; Index++)
			{
				if (std::memcmp(&FileData[Index], "\x0E\x55\x34\x12", 4) == 0)
				{
					OverrideProperties.WriteResetID(&FileData[Index]);
					break;
				}
			}

			for (Index++; Index < sizeof FileData; Index++)
			{
				if (std::memcmp(&FileData[Index], "\x00\xFF\xFF\xFF\xFF\xFF\xFF\x00", 8) == 0)
				{
					std::memcpy(&FileData[Index], ExportData, Count * 128);
					break;
				}
			}

			if (std::fwrite(FileData, 1, sizeof FileData, File) != sizeof FileData)
				return false;

			return true;
	}

	return false;
}
//---------------------------------------------------------------------------
int DisplayClass::Save()
{
	if (Deleted)
		return DeleteData();

	DisplayWrite();
	OverrideProperties.Write(OverrideData);
	CalculateChecksums();

	if (std::memcmp(OverrideData, ResetData, MAX_EDID_BLOCKS * 128) == 0 && OverrideProperties == ResetProperties)
		return 0;

	return SaveOverrideData();
}
//---------------------------------------------------------------------------
int DisplayClass::SaveActiveData()
{
	HKEY Key;
	char Path[TEXTSIZE];
	int Count;

	if (!Active || Deleted)
		return 0;

	if (std::snprintf(Path, TEXTSIZE, "SYSTEM\\CurrentControlSet\\Enum\\DISPLAY\\%s\\%s\\Device Parameters", DeviceID, InstanceID) >= TEXTSIZE)
		return -2;

	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, Path, 0, KEY_SET_VALUE, &Key) != ERROR_SUCCESS)
		return -2;

	for (Count = MAX_EDID_EXTENSION_BLOCKS; Count > 0; Count--)
		if (ActiveData[Count * 128] != 0)
			break;

	Count++;

	if (RegSetValueEx(Key, "EDID", 0, REG_BINARY, ActiveData, Count * 128) != ERROR_SUCCESS)
	{
		RegCloseKey(Key);
		return -2;
	}

	RegCloseKey(Key);
	return 1;
}
//---------------------------------------------------------------------------
int DisplayClass::SaveOverrideData()
{
	HKEY Key;
	char Path[TEXTSIZE];
	int Block;

	if (std::snprintf(Path, TEXTSIZE, "SYSTEM\\CurrentControlSet\\Enum\\DISPLAY\\%s\\%s\\Device Parameters\\EDID_OVERRIDE", DeviceID, InstanceID) >= TEXTSIZE)
		return -2;

	if (RegCreateKeyEx(HKEY_LOCAL_MACHINE, Path, 0, NULL, 0, KEY_SET_VALUE, NULL, &Key, NULL) != ERROR_SUCCESS)
		return -2;

	if (OverrideData[126] == 0 && OverrideProperties.GetExtensions())
	{
		OverrideData[126]++;
		OverrideData[127]--;
		ExtensionBlockList->WriteBlankExtension(&OverrideData[128], 128);
	}

	if (RegSetValueEx(Key, "0", 0, REG_BINARY, OverrideData, 128) != ERROR_SUCCESS)
	{
		RegCloseKey(Key);
		return -2;
	}

	OverrideProperties.SaveBackupData(Key, DetailedResolutions()->GetSlotsLeft());

	for (Block = 1; Block < MAX_EDID_BLOCKS; Block++)
	{
		if (std::snprintf(Path, TEXTSIZE, "%d", Block) >= TEXTSIZE)
		{
			RegCloseKey(Key);
			return -2;
		}

		if (OverrideData[Block * 128] != 0)
		{
			if (RegSetValueEx(Key, Path, 0, REG_BINARY, &OverrideData[Block * 128], 128) != ERROR_SUCCESS)
			{
				RegCloseKey(Key);
				return -2;
			}
		}
		else
		{
			switch (RegDeleteValue(Key, Path))
			{
				case ERROR_SUCCESS:
				case ERROR_FILE_NOT_FOUND:
				case ERROR_PATH_NOT_FOUND:
					break;

				default:
					RegCloseKey(Key);
					return -2;
			}
		}
	}

	RegCloseKey(Key);
	return 1;
}
//---------------------------------------------------------------------------
int DisplayClass::DeleteData()
{
	HKEY Key;
	char Path[TEXTSIZE];

	if (std::snprintf(Path, TEXTSIZE, "SYSTEM\\CurrentControlSet\\Enum\\DISPLAY\\%s\\%s\\Device Parameters", DeviceID, InstanceID) >= TEXTSIZE)
		return -2;

	switch (RegOpenKeyEx(HKEY_LOCAL_MACHINE, Path, 0, KEY_SET_VALUE, &Key))
	{
		case ERROR_SUCCESS:
			break;

		case ERROR_FILE_NOT_FOUND:
		case ERROR_PATH_NOT_FOUND:
			return 1;

		default:
			return -2;
	}

	switch (RegDeleteKey(Key, "EDID_OVERRIDE"))
	{
		case ERROR_SUCCESS:
		case ERROR_FILE_NOT_FOUND:
		case ERROR_PATH_NOT_FOUND:
			break;

		default:
			RegCloseKey(Key);
			return -2;
	}

	switch (RegDeleteValue(Key, "EDID"))
	{
		case ERROR_SUCCESS:
		case ERROR_FILE_NOT_FOUND:
		case ERROR_PATH_NOT_FOUND:
			break;

		default:
			RegCloseKey(Key);
			return -2;
	}

	switch (RegDeleteKey(Key, "EDID_RECOVERY"))
	{
		case ERROR_SUCCESS:
		case ERROR_FILE_NOT_FOUND:
		case ERROR_PATH_NOT_FOUND:
			break;

		default:
			RegCloseKey(Key);
			return -2;
	}

	RegCloseKey(Key);
	return 1;
}
//---------------------------------------------------------------------------
bool DisplayClass::GetProperties(PropertiesClass &Properties)
{
	Properties = OverrideProperties;
	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::SetProperties(const PropertiesClass &Properties)
{
	OverrideProperties = Properties;
	return true;
}
//---------------------------------------------------------------------------
PropertiesClass *DisplayClass::Properties()
{
	return &OverrideProperties;
}
//---------------------------------------------------------------------------
bool DisplayClass::DisplayInit()
{
	DetailedInit();
	StandardInit();
	EstablishedInit();
	ExtensionInit();
	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::DisplayRead()
{
	DetailedRead();
	StandardRead();
	EstablishedRead();
	ExtensionRead();
	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::DisplayWrite()
{
	DetailedWrite();
	StandardWrite();
	EstablishedWrite();
	ExtensionWrite();
	ClearChecksums();
	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::DisplayCopyPossible()
{
	if (!CopyData)
		return true;

	DisplayWrite();

	if (std::memcmp(CopyData, &OverrideData[35], MAX_EDID_BLOCKS * 128 - 35) != 0)
		return true;

	if (OverrideProperties.CopyPossible(CopyProperties))
		return true;

	return false;
}
//---------------------------------------------------------------------------
bool DisplayClass::DisplayCopy()
{
	if (!CopyData)
		CopyData = new unsigned char[MAX_EDID_BLOCKS * 128 - 35];

	DisplayWrite();
	std::memcpy(CopyData, &OverrideData[35], MAX_EDID_BLOCKS * 128 - 35);
	CopyProperties = OverrideProperties;
	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::DisplayPastePossible()
{
	if (!CopyData)
		return false;

	DisplayWrite();

	if (std::memcmp(&OverrideData[35], CopyData, MAX_EDID_BLOCKS * 128 - 35) != 0)
		return true;

	if (OverrideProperties.PastePossible(CopyProperties))
		return true;

	return false;
}
//---------------------------------------------------------------------------
bool DisplayClass::DisplayPaste()
{
	if (!CopyData)
		return false;

	std::memcpy(&OverrideData[35], CopyData, MAX_EDID_BLOCKS * 128 - 35);
	OverrideProperties.Paste(CopyProperties);
	DisplayRead();
	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::DisplayDeletePossible()
{
	return !Deleted;
}
//---------------------------------------------------------------------------
bool DisplayClass::DisplayDelete()
{
	Deleted = true;
	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::DisplayRestore()
{
	Deleted = false;
	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::IsDigital()
{
	return OverrideData[20] & 128;
}
//---------------------------------------------------------------------------
bool DisplayClass::GetNative(DetailedResolutionClass &DetailedResolution)
{
	if (!DetailedResolutionList->GetNative(DetailedResolution))
		return false;

	if (!DetailedResolution.CalculateNative(IsDigital()))
		return false;

	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::DetailedInit()
{
	DetailedRead();
	DetailedResolutionList->UpdateUndo();
	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::DetailedRead()
{
	if (!DetailedResolutionList->Read(&OverrideData[54], 72))
		return false;

	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::DetailedWrite()
{
	int Slot;

	for (Slot = 0; Slot < 4; Slot++)
	{
		std::memset(&OverrideData[Slot * 18 + 54], 0, 18);
		OverrideData[Slot * 18 + 57] = '\x10';
	}

	if (!DetailedResolutionList->Write(&OverrideData[54], 72))
		return false;

	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::DetailedGetStandardText(int Slot, char *Text, int TextSize)
{
	return DetailedResolutionList->GetStandardText(Slot, Text, TextSize, StandardResolutionList->GetSlotsLeft());
}
//---------------------------------------------------------------------------
DetailedResolutionListClass *DisplayClass::DetailedResolutions()
{
	DetailedResolutionList->SetMaxCount(4 - (StandardResolutionList->GetCount() - 3) / 6);
	return DetailedResolutionList;
}
//---------------------------------------------------------------------------
bool DisplayClass::StandardInit()
{
	StandardRead();
	StandardResolutionList->UpdateUndo();
	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::StandardRead()
{
	if (!StandardResolutionList->Read(&OverrideData[38], 64))
		return false;

	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::StandardWrite()
{
	if (!StandardResolutionList->Write(&OverrideData[38], 64))
		return false;

	return true;
}
//---------------------------------------------------------------------------
StandardResolutionListClass *DisplayClass::StandardResolutions()
{
	StandardResolutionList->SetMaxCount((4 - DetailedResolutionList->GetCount()) * 6 + 8);
	return StandardResolutionList;
}
//---------------------------------------------------------------------------
bool DisplayClass::EstablishedInit()
{
	EstablishedRead();
	EstablishedResolutionList->UpdateUndo();
	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::EstablishedRead()
{
	if (!EstablishedResolutionList->Read(&OverrideData[35], 3))
		return false;

	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::EstablishedWrite()
{
	if (!EstablishedResolutionList->Write(&OverrideData[35], 3))
		return false;

	return true;
}
//---------------------------------------------------------------------------
EstablishedResolutionListClass *DisplayClass::EstablishedResolutions()
{
	return EstablishedResolutionList;
}
//---------------------------------------------------------------------------
bool DisplayClass::ExtensionInit()
{
	ExtensionRead();
	ExtensionBlockList->UpdateUndo();
	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::ExtensionRead()
{
	if (!ExtensionBlockList->Read(&OverrideData[126], MAX_EDID_EXTENSION_BLOCKS * 128 + 2))
		return false;

	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::ExtensionWrite()
{
	if (!ExtensionBlockList->Write(&OverrideData[126], MAX_EDID_EXTENSION_BLOCKS * 128 + 2))
		return false;

	return true;
}
//---------------------------------------------------------------------------
ExtensionBlockListClass *DisplayClass::ExtensionBlocks()
{
	return ExtensionBlockList;
}
//---------------------------------------------------------------------------
bool DisplayClass::ClearChecksums()
{
	int Block;

	for (Block = 0; Block < MAX_EDID_BLOCKS; Block++)
		OverrideData[Block * 128 + 127] = 0;

	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::CalculateChecksums()
{
	int Block;
	int Index;

	for (Block = 0; Block < MAX_EDID_BLOCKS; Block++)
	{
		OverrideData[Block * 128 + 127] = 0;

		for (Index = 0; Index < 127; Index++)
			OverrideData[Block * 128 + 127] -= OverrideData[Block * 128 + Index];
	}

	return true;
}
//---------------------------------------------------------------------------
bool DisplayClass::Compare(DisplayClass *Display1, DisplayClass *Display2)
{
	char Text1[TEXTSIZE];
	char Text2[TEXTSIZE];

	if (Display1->Active && !Display2->Active)
		return true;

	if (!Display1->Active && Display2->Active)
		return false;

	Display1->GetText(Text1, TEXTSIZE, false);
	Display2->GetText(Text2, TEXTSIZE, false);

	if (strnicmp(Text1, Text2, TEXTSIZE) < 0)
		return true;

	if (strnicmp(Text1, Text2, TEXTSIZE) > 0)
		return false;

	if (Display1->InstanceID < Display2->InstanceID)
		return true;

	if (Display1->InstanceID > Display2->InstanceID)
		return false;

	return false;
}
//---------------------------------------------------------------------------
